/**
 * JavaScript for the ALERTSettings page.
 * Defines alert_init() for page initialization along with Show(), Validate()
 * and Apply() functions
 * Note: include it last.
 *
 * Copyright (C) 2016 McAfee, Inc. All rights reserved
 */

var vframeworks = new Object();
var dataBindings = new Array();
var qparams = queryParameters();
var newInstance = false;
var newId = null;
var rm = new ResourceManager("DefaultAlerts");
var labelMap = new Object();

var basePP = "Settings.ALERTSettings";
var alert_pb = null;
var origName="";
var settingsId = qparams["id"];
var alertOptsForm = "alert_options";

var currFmt = "";
var browserInfo = IdentifyBrowser();

var hfDefaults = {};

var g_changeNotifyFn = new Function("top.gPageStack.Active().SetModified(true);return true;");

// Rich-text controls for footer and header editors
var g_rtcAH = null, g_rtcAF = null;

// Input restrictions for the alert file name
function fnCheckInput(evt) {
    // Mixed case ASCII, numbers, underscores, periods
    return cb_restrictInput(evt, ["AZ", "az", "09", "_", "."]);
}

// Gets the default value of either an alert header or footer
function ah_getDefault(key, pt, rm) {
    var resKey = "DEFAULT" + key;
    if (pt)
        resKey += "_FORMATTEDPLAIN";
    var text = rm.getResource("DefaultHeadersAndFooters", resKey);
    if (!text)
        return "";
    // The text may contain tokens which need to be replaced
    var re = /%(.*)%/g;
    var match;
    var section = "TokenReplacements";
    var tokens = [];
    while ((match = re.exec(text)) != null) {
        var tok = match[1];
        if (tokens[tok])
            continue;
        tokens.push(tok);
        tokens[tok] = 1;
    }
    for (var i = 0; i < tokens.length; i++) {
        var token = tokens[i]
        re = new RegExp("%" + token + "%", "g");
        text = text.replace(re, rm.getResource(section, token));
    }
    return text;
}

// Gets default alert header and footer values
function ah_getDefaults() {
    var rm = new ResourceManager("DefaultAlerts");
    hfDefaults.defH = ah_getDefault("ALERTHEADER", false, rm);
    hfDefaults.defHP = ah_getDefault("ALERTHEADER", true, rm);
    hfDefaults.defF = ah_getDefault("ALERTFOOTER", false, rm);
    hfDefaults.defFP = ah_getDefault("ALERTFOOTER", true, rm);
}

/**
 * Data binding object for the alert header / footer fields
 * @param container the element used to house the associated
 * rich-text control
 * @param key either "ALERTHEADER" or "ALERTFOOTER"
 * @param rtc a rich-text control
 * @param html true if alerts are in HTML format, false otherwise
 * @param defH the default HTML value of the header / footer
 * @param defP the default plain value of the header / footer
 */
function HFDataBinding(container, key, rtc, html, defH, defP) {
    this.container = container;
    this.initialValue = "";
    this.key = key;
    this.rtc = rtc;
    this.html = html;
    this.defH = defH;
    this.defP = defP;
    this.def = html ? defH : defP;
    this.initialized = false;
}

HFDataBinding.prototype = new DataBinding("", "", "");

/**
 * Sets the field value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
HFDataBinding.prototype.set = function(pp, pbIn) {
    var ppT = null;
    if ("string" == typeof(pp)) {
        ppT = new PropertyPath(pp);
    }
    else {
        ppT = pp;
    }
    ppT.SetTempLeaf(this.key);
    this.initialValue = pbIn.GetPropertyDef(ppT, "");
    if (!this.html) {
        // Convert <#0A> sequences to \r\n
        this.initialValue = this.initialValue.replace(/<#0A>/g, "\r\n");
    }
    if (!this.initialValue) {
        this.initialValue = this.def;
    }
    ppT.SetTempLeaf("");
    var htmlContent = this.html ? this.initialValue : this.defH;
    var plainContent = this.html ? this.defP : this.initialValue;

    if (this.initialized) {
        this.rtc.sync_r(htmlContent);
        this.rtc.setText(plainContent, false);
    }
    else {
        this.rtc.makeControl(this.container, htmlContent, plainContent, true);
        this.initialized = true;
    }

    // Switch to plain-text mode if required
    if (!this.html)
        this.rtc.setPT();
}

/**
 * Gets the field value and places it in a PropertyBag object
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
HFDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var v = this.html ? this.rtc.getValue() : this.rtc.getText();
    if (!this.html)
        v = fixNewLines(v);
    if (v != this.initialValue) {
        ret = true;
    }
    // Do not store a value if it as the same as the default
    /*if (v == this.def)
        v = "";*/
    var ppT = null;
    if ("string" == typeof(pp)) {
        ppT = new PropertyPath(pp);
    }
    else {
        ppT = pp;
    }
    ppT.SetTempLeaf(this.key);
    pbOut.SetProperty(ppT, v);
    ppT.SetTempLeaf("");
    return ret;
}

/**
 * Toggles the data binding object between plain-text and HTML
 * @param html true if HTML-format alerts are to be used
 */
HFDataBinding.prototype.toggleHTML = function(html) {
    this.html = html;
    this.def = html ? this.defH : this.defP;
    if (this.initialized) {
        if (this.html)
            this.rtc.setWYSIWYG();
        else
            this.rtc.setPT();
    }
}


// Handles the user changing format
function alert_changeFormat(html) {
    var testFmt = html ? "H" : "P";
    if (currFmt == testFmt)
        return;
    setEncodingChoices(html);
    setAlertFileName(html);
    // Toggle alert header and footer between HTML and plain-text
    var hfFields = ["ALERTHEADER", "ALERTFOOTER"];
    for (var i = 0; i < hfFields.length; i++) {
        var dbObj = dataBindings[hfFields[i]];
        if (dbObj && dbObj.toggleHTML)
            dbObj.toggleHTML(html);
    }
    // Keep a record of the chosen format so that we know when to bail early
    currFmt = testFmt;
}

// Sets the options on the character encoding select control
function setEncodingChoices(html) {
    var f = document.forms[alertOptsForm];
    if (!f)
        return;
    var selCtl = f.elements["Charset"];
    if (!selCtl)
        return;
    var plainOpts = ["unicode", "shift_jis", "us-ascii", "utf-8"];
    var htmlOpts = pp_cse_encodings;
    if ("NCR" != htmlOpts[0]) {
        // Add an additional encoding to the standard list: numeric character references
        htmlOpts.unshift("NCR");
    }

    var currSel = "";
    var newSelIdx = 0;
    if (selCtl.length > 0) {
        currSel = selCtl.value;
        // Remove any current options
        while(selCtl.length > 0)
            selCtl.remove(0);
    }
    // Add new options
    var opts = html ? htmlOpts : plainOpts;
    var nOpts = opts.length;
    for (var i = 0; i < nOpts; i++) {
        var option = document.createElement("option");
        var encoding = opts[i];
        if (encoding == currSel)
            newSelIdx = i;
        option.value = encoding;
        option.text = resolveString(cse_pp_st, encoding);
        if (browserInfo.isIE) {
            selCtl.add(option);
        }
        else {
            selCtl.add(option, null);
        }
    }
    // Set the selection
    selCtl.selectedIndex = newSelIdx;
}

// Changes the alert filename according to whether HTML or plain-text
// format is selected
function setAlertFileName(html) {
    var f = document.forms[alertOptsForm];
    var field = "ASCIIFILENAME";
    if (!f)
        return;
    var fnCtl = f.elements[field];
    if (!fnCtl)
        return;
    var currVal = fnCtl.value;
    var defFN = "warning.htm";
    // Do an ending replacement:
    // Plain text - ".html" or ".htm" to ".txt"
    // HTML - ".txt" to ".htm"
    if (html) {
        var re = /\.txt$/i;
        var repStr = ".htm";
        if (re.test(currVal)) {
            currVal = currVal.replace(re, repStr);
        }
        else {
            currVal += repStr;
        }
    }
    else {
        defFN = "warning.txt";
        var re = /\.html$/i;
        var re2 = /\.htm$/i;
        var repStr = ".txt";
        if (re.test(currVal)) {
            currVal = currVal.replace(re, repStr);
        }
        else if (re2.test(currVal)) {
            currVal = currVal.replace(re2, repStr);
        }
        else {
            currVal += repStr;
        }
    }
    fnCtl.value = currVal;
    // Set the default value on the data binding object; this will affect
    // whether the value is stored
    var dbKey = alertOptsForm + "_" + field;
    var dbObj = dataBindings[dbKey];
    if (dbObj)
        dbObj.def = defFN;
}

// Initializes the page's forms
function alert_forms_init() {
    var forms = [alertOptsForm];
	labelMap["ASCIIFILENAME"] = gsErrAlertFileName;

	vframeworks["InstanceSettings"]= new ValidationFramework("InstanceSettings");
	vframeworks["InstanceSettings"].formAction = std_val_form_action;
    vframeworks["InstanceSettings"].fieldAction = std_val_field_action;
    vframeworks["InstanceSettings"].addValidator("displayName", new RequiredValidator());
    //vframeworks["InstanceSettings"].addValidator("displayName", new ScriptValidator());
	vframeworks["InstanceSettings"].addValidator("displayName", new InputTextValidator());
	vframeworks["InstanceSettings"].labelMap["displayName"] =  resolveString(cse_pp_field_st, "__instancename");

    var inputArray= new Array();
	inputArray["SECTION"]="Scanners";
	inputArray["origName"]=origName;
	inputArray["basePP"]="Settings.ALERTSettings";
	vframeworks["InstanceSettings"].addValidator("displayName",new DuplicatesValidator(alert_pb,inputArray));
	vframeworks["InstanceSettings"].addValidator("displayName",new LengthValidator());

    for (var i = 0; i < forms.length; i++) {
        var form = forms[i];
        if (document.forms[form]) {
            vframeworks[form] = new ValidationFramework(form);
			var asciiValidator = new ASCIIValidator();
			vframeworks[form].addValidator("ASCIIFILENAME", asciiValidator);
			vframeworks[form].labelMap = labelMap;
            vframeworks[form].formAction = std_val_form_action;
            vframeworks[form].fieldAction = std_val_field_action;
        }
    }

    shs_affectedPolicyReport(newInstance, "ALERT", settingsId, alert_pb);

    var form = alertOptsForm;
    if (document.forms[form]) {
        var fnCtl = document.forms[form].elements["ASCIIFILENAME"];
        if (fnCtl)
            fnCtl.onkeypress = fnCheckInput;
    }

    ah_getDefaults();

    // Initialize the rich-text library
    initRTE("../../images/rte/", "", "");
    var div = document.getElementById("_ah");
    var editWidth = 788;
    if (div) {
        editWidth = div.offsetWidth ? div.offsetWidth : div.clientWidth;
        editWidth -= 80;
    }

    // Create rich-text objects to handle alert header and footer
    var rtcProps = { width:editWidth, height:125, richText:true, plainTextOption:true,
        plainTextButton:false, buttons:false, changeNotifyFn:g_changeNotifyFn};
    g_rtcAH = new RTControl(resolveString(cse_pp_st, "__alert_header:"),
        "ah", "g_rtcAH", rtcProps);
    g_rtcAF = new RTControl(resolveString(cse_pp_st, "__alert_footer:"),
        "af", "g_rtcAF", rtcProps);
    // Create data binding objects for the editors
    var dbObj = new HFDataBinding("_ah", "ALERTHEADER", g_rtcAH, true,
        hfDefaults.defH, hfDefaults.defHP);
    dataBindings.push(dbObj);
    dataBindings["ALERTHEADER"] = dbObj;
    dbObj = new HFDataBinding("_af", "ALERTFOOTER", g_rtcAF, true,
        hfDefaults.defF, hfDefaults.defFP);
    dataBindings.push(dbObj);
    dataBindings["ALERTFOOTER"] = dbObj;
    addChangeNotification(g_changeNotifyFn);
}

// Sets form data and enables / disables controls as appropriate
function alert_init() {
    alert_forms_init();
    Reset();
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var defValues = top.defValues["ALERT"];
    var alert_pp = new PropertyPath(basePP);
    // Peek ahead to see what the alert format is; this affects the choices
    // available for character encoding and the default value of the alert
    // filename
    var fmt = alert_pb.GetPropertyDef(basePP + ".PlainText", "0");
    var html = ("0" == fmt);
    alert_changeFormat(html);

    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(alert_pp, alert_pb);
    }

    if (document.forms[alertOptsForm]) {
        // Check to see whether the current encoding is present in the
        // charset selector; add it if not
        var propName = "Charset";
        var selCtl = document.forms[alertOptsForm].elements[propName];
        var dbObj = dataBindings[alertOptsForm + "_" + propName];
        if (selCtl && dbObj && dbObj.initialValue) {
            var currVal = dbObj.initialValue;
            var found = false;
            for (var i = selCtl.options.length - 1; i >= 0; i--) {
                if (currVal == selCtl.options[i].value) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                var option = document.createElement("option");
                option.value = currVal;
                option.text = resolveString(cse_pp_st, currVal);
                if (browserInfo.isIE) {
                    selCtl.add(option);
                }
                else {
                    selCtl.add(option, null);
                }
                selCtl.selectedIndex = selCtl.length - 1;
            }
        }
    }
    top.gPageStack.Active().SetModified(false);
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    newInstance = shs_initPB(pbIn, pp, "ALERT");
    alert_pb = pbIn;
    if (settingsId) {
        // Search for the settings blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", settingsId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
        // Assume that we've been set up with the correct property path
        // and property bag
        basePP = pp.GetPath();
        settingsId = pbIn.GetPropertyDef(basePP + ".id", "");
        origName=pbIn.GetPropertyDef(basePP + ".displayName", "");
    }
    alert_init();
}


/**
 * Checks for the presence of "Script" keyword in the Alert Header and footer.
 */
function ValidateAlertFooterAndHeader( dataBindings )
{
	for (var i = 0; i < dataBindings.length; i++)
	{
        var db = dataBindings[i];
		if( "ALERTHEADER" == db.key || "ALERTFOOTER" == db.key )
		{
			if(null != db.rtc)
			{
				//both html value ( one extra arguement) and text value are checked for the presence of script keyword.
				if(db.rtc.getMode() == -1)
				{
					if(  ! ScriptInputValidation( db.rtc.getValue(), true , false)  || ! ScriptInputValidation( db.rtc.getText(), true)   )
					{
							return false ;
					}
					if(  ! ScriptInputValidation( db.rtc.getValue(), true , true)  || ! ScriptInputValidation( db.rtc.getText(), true)   )
					{	
							return false ;
					}
				}
					if(!ValidateAlertTextInput(db.rtc.getValue()))
					{
							alert(gsScriptNotAllowed);
								return false;
					}
					if(!ValidateXMLInput(db.rtc.getValue()) || !ValidateXMLInput(db.rtc.getText()))
					{
							alert(gsXMLCommentTagsNotAllowed);
								return false;
					}
					if(!ValidateLimitedHtmlInput(db.rtc.getValue()) || !ValidateLimitedHtmlInput(db.rtc.getText()) )
					{
							alert(gsHTMLTagsNotAllowed);
								return false;
					}
					
			}
		}
	}
	return true;
}
/**
 * Asynchronous data validation function
 */
function Validate() {
  										
					if( ! ValidateAlertFooterAndHeader( dataBindings ) )
						return false;
						return shs_Validate();
					
						
}


/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */

function CheckFileName()
{
    var f = document.forms[alertOptsForm];
    var field = "ASCIIFILENAME";
    var fnCtl = f.elements[field];
    var currVal = fnCtl.value;
    if(currVal == "."  || currVal == ".txt" || currVal == ".htm" || currVal == ".html")
        return 0;
    var fileExtension = currVal.substring(currVal.lastIndexOf("."),currVal.length)
    if(document.getElementById("radio_0").checked == true )
    {
        if(fileExtension != ".htm" &&  fileExtension != ".html" )
            return 0;
    }
    if(document.getElementById("radio_1").checked == true )
    {
        if(fileExtension != ".txt" )
            return 0;
    }
}	

function Apply(pb) {
    if (null == pb) {
        pb = alert_pb;
    }
    var fileValid = CheckFileName()
    if (fileValid == 0 )
    {
        alert(gsInvalidFilename)
        Cancel()
        top.gPageStack.Active().SetModified(false);
        return;
    }
    shs_Apply(pb, basePP, dataBindings, newInstance);
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        alert_pb.DeleteElement(basePP);
    }
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["DefaultAlerts"];
}
