/**
 * JavaScript for the AVSettings page; this file needs to be included
 * if any of the following components are used:
 * AVLevelDlg.xml, AVOptionsDlg.xml, PackerOptionsDlg.xml, PupOptionsDlg.xml
 * Defines avs_init() for page initialization along with Show(), Validate()
 * and Apply() functions
 * Note: include it last.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

includeJS("../js/Locale.js");
includeJS("../js/PolicyRes.js");
includeJS("../js/VFrameworkStrings.js");
includeJS("../../js/PropertySource.js");
includeJS("../../js/PropertyPath.js");
includeJS("../../js/PropertyBag.js");
includeJS("../../js/PointProductSpecific.js");
includeJS("../../js/OnlineHelp.js");
includeJS("../../js/ValidationFramework.js");
includeJS("../../js/StdValidationActions.js");
includeJS("../../js/Rollups.js");
includeJS("../../js/Tabs.js");
includeJS("../../js/InputList.js");
includeJS("../../js/DefaultValues.js");
includeJS("../../js/SharedSettings.js");

var vframeworks = new Object();
var dataBindings = new Array();
var inputLists = new Object();
var qparams = queryParameters();
var newInstance = false;
var newId = null;
var g_changeNotifyFn = "top.gPageStack.Active().SetModified(true);";

var PUP_SPYWARE = 1;
var PUP_ADWARE = 2;
var PUP_REMOTE_ADMIN = 4;
var PUP_DIALERS = 8;
var PUP_PWD_CRACKERS = 0x10;
var PUP_JOKE_PROGRAMS = 0x20;
var PUP_COOKIES = 0x40;
var PUP_OTHER = 0x80;

var basePP = "Settings.AVSettings";
var avs_pb = null;
var origName="";
var settingsId = qparams["id"];
var polSet = "";
try {
    polSet = top.polctx.myPolicySet.pp_key;
}
catch(e) {}

// Certain PUP types may be hidden for a given protocol but this settings
// object may be shared among many protocols; we need to keep a record of
// the hidden PUP types that have been selected for detection elsewhere
var pupTypesRegister = 0;

// Filter function for file extensions input list
function toUpper(val) {
    return val.toUpperCase();
}

// Input restrictions for the file extensions
function fextCheckOnPaste(evt) {
	return false;
}

// Input restrictions for the file extensions
function fextCheckInput(evt) {
	if ( evt != undefined )
    {
		if ( evt.ctrlKey && evt.which == 118 )
		{
			return false;
		}
    }
    // Only allow characters permissible in filenames (but also no
    // spaces or periods);
    // Forbidden characters are: /\:*?"<>|
    return cb_restrictInput(evt, ["AZ", "az", "09", "!", "#)", "+-",
        ";", "=", "@", "[", "]`", "{", "}", "~"]);
}

// Specialized data binding class for the PUP options bitfield
function PUPOptionsDataBinding(form, def) {
    this.form = form;
    this.propName = "PupTypes";
    this.def = def;
    this.initialValue = null;
}

PUPOptionsDataBinding.prototype = new DataBinding("", "");

PUPOptionsDataBinding.prototype.set = function(pp, pbIn) {
    pp.SetTempLeaf(this.propName);
    var selected = pbIn.GetPropertyDef(pp, this.def);
    pp.SetTempLeaf("");
    var f = document.forms[this.form];
    if (f) {
        var elems = f.elements;
        for (var i = 0; i < elems.length; i++) {
            var e = elems[i];
            if (e.type == "checkbox" && e.name == "pup_type") {
                e.checked = (0 != (selected & e.value));
            }
        }
    }
    this.initialValue = selected | 0xFF00;
}

PUPOptionsDataBinding.prototype.get = function(pp, pbOut) {
    var ret = false;
    var selected = 0xFF00;
    var f = document.forms[this.form];
    if (f) {
        var elems = f.elements;
        for (var i = 0; i < elems.length; i++) {
            var e = elems[i];
            if (e.type == "checkbox" && e.name == "pup_type" && e.checked) {
                selected |= e.value;
            }
        }
    }
    // OR in the states for the hidden PUP types
    selected |= pupTypesRegister;
    pp.SetTempLeaf(this.propName);
    pbOut.SetProperty(pp, selected.toString());
    pp.SetTempLeaf("");
    if (selected != this.initialValue) {
        ret = true;
    }
    return ret;
}

function showFileTypesList(show) {
    inputLists["ft"].draw(show);
    if (show) {
        inputLists["ft"].focus();
    }
}

function handleSpecificVirusProtectionCheck(checked) {
    inputLists["svt"].draw(checked);
    if (checked) {
        inputLists["svt"].focus();
    }
}

function enablePackerDetection(checked) {
    var f = document.getElementById("pd_row2");
    var disabled = true;
    var className = "disabled";
    if (checked) {
        disabled = false;
        className = "";
    }
    f.className = className;
    var e = document.forms["packer_options"].elements;
    for (var i = 0; i < e.length; i++) {
        if (e[i].type == "radio")
            e[i].disabled = disabled;
    }
    inputLists["pn"].draw(!disabled);
}

function makePUPForm() {
    var PupTypesHidden = getUIFlag(polSet, "PupTypesHidden", 0);

    var pTypes = ["PUP_SPYWARE", "PUP_ADWARE", "PUP_REMOTE_ADMIN", "PUP_DIALERS",
        "PUP_PWD_CRACKERS", "PUP_JOKE_PROGRAMS", "PUP_COOKIES", "PUP_OTHER"];
    var inputs = new Array();
    for (var i = 0; i < pTypes.length; i++) {
        var mask = 1 << i;
        if (0 == (PupTypesHidden & mask)) {
            var cb = "<input type=\"checkbox\" name=\"pup_type\" value=\"" + mask + "\"";
            if (mask == PUP_COOKIES) {
                cb += " onclick=\"enableCookieRemoval(this.checked);\"";
            }
            cb += "id=\"cb_" + pTypes[i] + "\"> " + resolveString(page_st, pTypes[i]);
            inputs[inputs.length] = cb;
        }
    }

    if (!inputs.length) {
        return;
    }

    var html = "<span class=\"st_label\">" + resolveString(page_st, "PUP_Directive") + "</span><br>";
    html += "<table border=\"0\" id=\"PUPTypesTbl\" cellpadding=\"0\" cellspacing=\"0\"><tr>";
    var splitPoint = Math.ceil(inputs.length / 2);
    for (var i = 0; i < 2; i++) {
        html += "<td nowrap class=\"st\">";
        for (var j = 0; j < splitPoint; j++) {
            var index = j + splitPoint * i;
            if (index < inputs.length) {
                html += inputs[index] + "<br>";
            }
        }
        html += "</td>";
    }
    html += "<td nowrap width=\"100%\">&nbsp;</td>";
    html += "</tr></table>";

    if (0 == (PupTypesHidden & PUP_COOKIES)) {
        // Scanning for cookies is possible; show the remove detected cookies option
        var div = document.getElementById("CookieOptsDiv");
        if (div)
            div.style.display = "block";
    }

    var div = document.getElementById("PupTypeDiv");
    if (div) {
        div.innerHTML = html;
    }
}

// Enables / disables the remove cookies option when the
// detect cookies checkbox is clicked
function enableCookieRemoval(checked) {
    var f = document.forms["pup_options"];
    if (f) {
        var e = f.elements["RemoveDetectedCookies"];
        if (e)
		{
            e.disabled = !checked;
			if(!checked)
			{
				e.checked = false;
			}
		}
    }
    var div = document.getElementById("CookieOptsDiv");
    if (div) {
        var cn = div.className;
        if (checked)
            cn = removeClass(cn, "disabled");
        else
            cn = addClass(cn, "disabled");
        div.className = cn;
    }
}

function enablePupDetection(checked) {
    var f = document.getElementById("pupd_row2");
    var disabled = true;
    var cn = f.className;
    var cnElem = "disabled";
    if (checked) {
        disabled = false;
        cn = removeClass(cn, cnElem);
    }
    else {
        cn = addClass(cn, cnElem);
    }

    f.className = cn;

    // Grey out the PUP types table
    var tbl = document.getElementById("PUPTypesTbl");
    if (tbl) {
        var rows = tbl.rows;
        for (var i = rows.length - 1; i >= 0; i--) {
            var cells = rows[i].cells;
            for (var j = cells.length - 1; j >= 0; j--) {
                cn = cells[j].className;
                if (checked)
                    cells[j].className = removeClass(cn, cnElem);
                else
                    cells[j].className = addClass(cn, cnElem);
            }
        }
    }

    var e = document.forms["pup_options"].elements;
    for (var i = 0; i < e.length; i++) {
        e[i].disabled = (e[i].name == "AVP_PROGRAM") ? false : disabled;
    }
    if (checked) {
        var e = document.getElementById("cb_PUP_COOKIES");
        if (e) {
            // Cookie detection possible
            enableCookieRemoval(e.checked);
        }
    }
    inputLists["pup"].draw(!disabled);
}

function avs_forms_init() {
    var useTabs = (undefined !== this.av_tabs);
    var forms = ["custom_av_options", "custom_malware_options",
        "packer_options", "pup_options"];
	vframeworks["InstanceSettings"]= new ValidationFramework("InstanceSettings");
	vframeworks["InstanceSettings"].formAction = std_val_form_action;
    vframeworks["InstanceSettings"].fieldAction = std_val_field_action;
    vframeworks["InstanceSettings"].addValidator("displayName", new RequiredValidator());
    //vframeworks["InstanceSettings"].addValidator("displayName", new ScriptValidator());
	vframeworks["InstanceSettings"].addValidator("displayName", new InputTextValidator());
	vframeworks["InstanceSettings"].labelMap["displayName"] = resolveString(cse_pp_field_st, "__instancename");

    var inputArray= new Array();
	inputArray["SECTION"]="Scanners";
	inputArray["origName"]=origName;
	inputArray["basePP"]="Settings.AVSettings";
	vframeworks["InstanceSettings"].addValidator("displayName",new DuplicatesValidator(avs_pb,inputArray));
	vframeworks["InstanceSettings"].addValidator("displayName",new LengthValidator());

	for (var i = 0; i < forms.length; i++) {
        var form = forms[i];
        if (document.forms[form]) {
            vframeworks[form] = new ValidationFramework(form);
            vframeworks[form].formAction = std_val_form_action;
            vframeworks[form].fieldAction = std_val_field_action;
        }
    }

    shs_affectedPolicyReport(newInstance, "AV", settingsId, avs_pb);

    var form = "custom_av_options";
    var tabIdx = 0;
    if (document.forms[form]) {
        var fileTypesList = new InputList([], "file_types_div", "inputLists['ft']",
            { baseFieldName:"extension", maxlength:3, filter:toUpper,
            directive:resolveString(page_st, "ftl_directive"),
            vframework:vframeworks[form], compulsory:true,
            displayName:resolveString(page_st, "ftl_displayName"),
            inputNamePattern:resolveString(page_st, "ftl_inputNamePattern"),
            inputFilter:fextCheckInput,inputOnPaste:fextCheckOnPaste,changeNotify:g_changeNotifyFn });
        inputLists["ft"] = fileTypesList;
        dataBindings.push(new InputListDataBinding(inputLists["ft"], "AVP_EXTENSIONS", true, " "));
        if (useTabs) {
            av_tabs.addValidation(tabIdx++, vframeworks[form]);
        }
    }

    form = "custom_malware_options";
    if (document.forms[form]) {
        var virusNamesList = new InputList([], "virus_names_div", "inputLists['svt']",
            { baseFieldName:"virus", vframework:vframeworks[form], compulsory:true,
            displayName:resolveString(page_st, "svt_displayName"),
            inputNamePattern:resolveString(page_st, "lm_vNamePattern"),
            changeNotify:g_changeNotifyFn });
        inputLists["svt"] = virusNamesList;
        dataBindings.push(new InputListDataBinding(inputLists["svt"], "SpecificVirusName"));
        if (useTabs) {
            av_tabs.addValidation(tabIdx++, vframeworks[form]);
        }
    }

    form = "packer_options";
    if (document.forms[form]) {
        var packerList = new InputList([], "packer_names_div", "inputLists['pn']",
            { baseFieldName:"packer", vframework:vframeworks[form],
            inputNamePattern:resolveString(page_st, "pnl_inputNamePattern"),
            changeNotify:g_changeNotifyFn });
        inputLists["pn"] = packerList;
        dataBindings.push(new InputListDataBinding(inputLists["pn"], "PackerNamePattern"));
        if (useTabs) {
            av_tabs.addValidation(tabIdx++, vframeworks[form]);
        }
    }

    form = "pup_options";
    if (document.forms[form]) {
        pupTypesRegister = fillPupTypesRegister();
        makePUPForm();
        dataBindings.push(new PUPOptionsDataBinding(form, top.defValues["AV"]["PupTypes"]));
        var pupList = new InputList([], "pup_names_div", "inputLists['pup']",
            { baseFieldName:"pup", vframework:vframeworks[form],
            inputNamePattern:resolveString(page_st,"pupl_inputNamePattern"),
            changeNotify:g_changeNotifyFn });
        inputLists["pup"] = pupList;
        dataBindings.push(new InputListDataBinding(inputLists["pup"], "PupNamePattern"));
        if (useTabs) {
            av_tabs.addValidation(tabIdx++, vframeworks[form]);
        }
    }
    //shs_enableSave();
    addChangeNotification(g_changeNotifyFn);
}

function avs_init() {
    avs_forms_init();
    Reset();
}

function fillPupTypesRegister() {
    var PupTypesHidden = getUIFlag(polSet, "PupTypesHidden", 0);
    var pTypes = ["PUP_SPYWARE", "PUP_ADWARE", "PUP_REMOTE_ADMIN", "PUP_DIALERS",
        "PUP_PWD_CRACKERS", "PUP_JOKE_PROGRAMS", "PUP_COOKIES", "PUP_OTHER"];
    var ret = 0;
    var selected = avs_pb.GetPropertyDef(basePP + ".PupTypes", 0xffff);
    for (var i = 0; i < pTypes.length; i++) {
        var mask = 1 << i;
        if (0 != (PupTypesHidden & mask)) {
            if (0 != (selected & mask)) {
                // Save the selected state for this PUP type
                ret |= mask;
            }
        }
    }
    return ret;
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var avs_pp = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(avs_pp, avs_pb);
    }

    if (document.forms["protection_level"]) {
        // AV protection level dialog included
        // There is a PointProduct.js option to display or hide the protection
        // level dialog
        document.getElementById("protection_level_div").style.display =
            pp_cse_ShowProtectionLevel ? "inline" : "none";
        if (pp_cse_ShowProtectionLevel) {
            var div = document.getElementById("av_options_div");
            if (div) {
                document.getElementById("tbl_custom_av_options").style.marginTop = "10px";
            }
        }
        else {
            // Force protection level to "Custom"
            avs_pb.SetProperty(basePP + ".ScanLevel", "Custom");
        }
    }

    if (document.forms["custom_av_options"]) {
        // AV options dialog included
        var div = document.getElementById("av_options_div");
        div.style.display = (avs_pb.GetPropertyDef(basePP + ".ScanLevel",
            top.defValues["AV"]["ScanLevel"]) == "Custom") ?
            "inline" : "none";
        inputLists["ft"].draw(true);
        showFileTypesList("SELECTED" == avs_pb.GetPropertyDef(basePP + ".AVP_SCANWHAT",
            top.defValues["AV"]["AVP_SCANWHAT"]));
		enableCMBSENSLEVEL(document.getElementById("chkbox_ArtemisSensitivityLevel").checked);
    }

    if (document.forms["custom_malware_options"]) {
        inputLists["svt"].draw(
            document.forms["custom_malware_options"].elements["SpecificVirusProtection"].checked);
    }

    if (document.getElementById("clean_options_div")) {
        if (document.forms["custom_malware_options"]) {
            // Create vertical space for the clean options dialog
            document.getElementById("tbl_clean_options").style.marginTop = "10px";
        }
    }

    if (document.forms["packer_options"]) {
        enablePackerDetection("1" == avs_pb.GetPropertyDef(basePP + ".AVP_SHOWCOMPRESSED",
            top.defValues["AV"]["AVP_SHOWCOMPRESSED"]));
    }

    if (document.forms["pup_options"]) {
        var pdEnabled = ("1" == avs_pb.GetPropertyDef(basePP + ".AVP_PROGRAM",
            top.defValues["AV"]["AVP_PROGRAM"]));
        enablePupDetection(pdEnabled);
        var e = document.getElementById("cb_PUP_COOKIES");
        if (e) {
            // Cookie detection possible
            enableCookieRemoval(e.checked);
        }
    }
    top.gPageStack.Active().SetModified(false);
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    newInstance = shs_initPB(pbIn, pp, "AV");
    avs_pb = pbIn;
    if (settingsId) {
        // Search for the settings blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", settingsId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
        // Assume that we've been set up with the correct property path
        // and property bag
        basePP = pp.GetPath();
        settingsId = pbIn.GetPropertyDef(basePP + ".id", "");
        origName=pbIn.GetPropertyDef(basePP + ".displayName", "");
    }
    avs_init();
	//to ensure that the cookie removal checkbox is greyed out when the cookies checkbox is unchecked.
	var e = document.getElementById("cb_PUP_COOKIES");
	if (e) {
		// Cookie detection possible
		enableCookieRemoval(e.checked);
	}
	var eA = document.getElementById("chkbox_ArtemisSensitivityLevel");
	if (eA) {
		// disable CMBSENSLEVEL
		if(eA.checked == false)
			document.getElementById("CMBSENSLEVEL").disabled = true;
	}
}

/**
 * Asynchronous data validation function
 */
function Validate() {
    return shs_Validate(this.av_tabs);
}

/**
 * Enable/Disable CMBSENSLEVEL combobox for artemis
 */

function enableCMBSENSLEVEL(obj)
{
	var eA = document.getElementById("CMBSENSLEVEL");
	if (eA) {
		// disable CMBSENSLEVEL
		document.getElementById("CMBSENSLEVEL").disabled = !obj
	}
	
}
/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb) {
    if (null == pb) {
        pb = avs_pb;
    }
    shs_Apply(pb, basePP, dataBindings, newInstance);
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        avs_pb.DeleteElement(basePP);
    }
}
