/**
 * ActionHelper.js
 *
 * JavaScript support for the actions component
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

// Flags to indicate whether an action list includes a replacement option
var replacementOption = {};
// Flag to indicate whether at least one alert is displayed on the page
var g_showAlerts = false;
var ACT_PROP_PREFIX = "Actions";
var qparams = queryParameters();
fixQParams(qparams, "type", "");
fixQParams(qparams, "fromPB", "1");
fixQParams(qparams, "actionKey", "");
fixQParams(qparams, "insert", "");
fixQParams(qparams, "showAlerts", "1");
var g_onLoadPB = null;
var g_currPolicySettings = new PropertyBag();

function ga_usePB() {
    return ("1" == qparams["fromPB"][0]);
}

function ga_getActionKey() {
    return qparams["actionKey"][0];
}

function ga_showAlerts(actName) {
    return ("1" == qparams["showAlerts"][0] && replacementOption[actName]);
}

var baseSettingsType = qparams["type"][0];
var settingsType = baseSettingsType;
var actionList = [];
var actionsPB = null, actionsPP = null;
var dataBindings = [];

/**
 * Replacement function for the onclick method in the primary actions form
 */
function PolicyAction_onclick(the_ctl, alert_id) {
    // Get the name of the action from the enclosing form
    var actName = "Action";
    if (the_ctl.form && the_ctl.form.name) {
        var formName = the_ctl.form.name;
        actName = formName.replace("_actions_f_p", "");
    }
    top.gPageStack.Active().SetModified(true);
    var alDivDisplayStyle = "none";
    if (ga_showAlerts(actName) && "1" == pp_cse_alertActions[the_ctl.value]) {
        alDivDisplayStyle = "block";
    }
    var e = document.getElementById(alert_id);
    if (e)
        e.style.display = alDivDisplayStyle;
}

/**
 * Functions from PolSettings required by AlertHelper
 */
function PolSetting_GetPb() {
    return actionsPB;
}

function PolSetting_GetPPStr() {
    return actionsPP.toString();
}

function PolSetting_GetPPNode() {
    return actionsPP;
}



/**
 * Helper function to stick text in an element
 */
function ga_setEleText(elem, text) {
    var e = document.getElementById(elem);
    if (e) {
        while(e.firstChild) {
            e.removeChild(e.firstChild);
        }
        e.appendChild(document.createTextNode(text));
    }
}

/**
 * Sets the header title of the actions document
 */
function ga_setHeaderTitle() {
    var title = "";
    var key = baseSettingsType + "_ACT_PAGE_TITLE";
    if (null != cse_pp_st[key]) {
        title = resolveString(cse_pp_st, key);
    }
    else {
        var inserts = [resolveString(cse_pp_st, baseSettingsType)];
        title = resolveString(cse_pp_st, "ACT_PAGE_TITLE", inserts);
    }

    ga_setEleText("_cm_hdr_text", title);
}

/**
 * Sets the header of the primary action list ("Take the following action...")
 */
function ga_setPrimaryHeader(text, actName) {
    if (!text)
        text = resolveString(cse_pp_st, "__def_primary_text");
    var spanId = actName + "_label";
    if (null == document.getElementById(spanId))
        spanId = "Action_label";
    ga_setEleText(spanId, text);
}

/**
 * Sets the header of the secondary action list ("And also...")
 */
function ga_setSecondaryHeader(text, actName) {
    if (!text)
        text = resolveString(cse_pp_st, "__def_secondary_text");
    var spanId = actName + "f_sec_header";
    if (null == document.getElementById(spanId))
        spanId = "Actionf_sec_header";
    ga_setEleText(spanId, text);
}

/**
 * Sets the directive at the top of the control ("When x is found...")
 */
function ga_setActionsDirective(actName) {
    var stKey = this.baseSettingsType + "_" + actName;
    if (null == cse_pp_st[stKey])
        stKey = "ACT_" + stKey;
    var directive = resolveString(cse_pp_st, stKey, qparams["insert"]);
    var spanId = actName + "_directive";
    if (null == document.getElementById(spanId))
        spanId = "Action_directive";
    ga_setEleText(spanId, directive);
}

/**
 * Gets the list of actions from the property bag
 */
function ga_getActionList(type) {
    var ret = pp_cse_actionNames["STD"];
    if (null != pp_cse_actionNames[type]) {
        ret = pp_cse_actionNames[type];
    }
    return ret;
}

/**
 * Gets the most specific default action list for a settings type (e.g. given a
 * settings type of "AV.smtp", an action list labelled "AV.smtp" would be
 * used in preference to one labelled "AV"
 * @return an action list (or an empty string if none can be found)
 */
function ga_getDefaultActionString(actName) {
    var ret = [];
    var parts = settingsType.split(".");
    while (parts.length > 0) {
        var testKey = parts.join(".");
        if (top.defValues["Actions"][testKey]  &&
            top.defValues["Actions"][testKey][actName]) {
            ret = top.defValues["Actions"][testKey][actName];
            break;
        }
        parts.shift();
    }
    return ret;
}

/**
 * Initializes the action and alert forms
 */
function ga_init(pbIn, pp) {
    actionList = ga_getActionList(baseSettingsType);
    ga_initPB(pbIn, pp);
    ga_setHeaderTitle();
    // Width and height of the alert preview
    var apWidth = AlertPreview.getDefaultWidth();
    var apHeight = AlertPreview.getDefaultHeight();
    for (var i = 0; i < actionList.length; i++) {
        var actName = actionList[i];
        replacementOption[actName] = a_hasReplacementOption(settingsType, actName);
        if (1 == actionList.length) {
            // Register replacementOption["Action"] so that the alert form displays
            replacementOption["Action"] = replacementOption[actName];
        }
        ga_setActionsDirective(actName);
        ga_setPrimaryHeader(null, actName);
        ga_setSecondaryHeader(null, actName);
        // Create a data binding object for each action
        var formPfx = actName;
        if (!document.forms[actName + "_actions_f_p"]) {
            formPfx = "Action";
        }
        dataBindings.push(new ActionListDataBinding(formPfx,
            ACT_PROP_PREFIX + "." + actName, ga_getDefaultActionString(actName)));
        if (ga_showAlerts(actName)) {
            g_showAlerts = true;
            // Create an alerts form
            // Generate a uid for the alerts forms and lookup the alert
            // property corresponding to this action together with the alert
            // category
            var uid = baseSettingsType + "_" + actName;
            var alertProps = pp_cse_actionAlertMap[uid];
            if (null == alertProps)
                continue;
            var alertProp = alertProps[0];
            var alertCat = "";
            if (alertProps.length > 1) {
                alertCat = alertProps[1];
            }
            else {
                alertCat = pp_cse_alertTypeMap[baseSettingsType];
            }
            if (!alertCat)
                continue;
            var divId = formPfx + "_alert";
            alh_makeAlertSelector(uid, alertProp, alertCat, divId, apWidth, apHeight, false);
        }
    }
    if (g_showAlerts)
        alh_init(pbIn, pp);
}

/**
 * PropertyBag initialization; if the action value is being passed
 * through polctx (as is the case with file filtering, for example)
 * the function creates a dummy PropertyBag
 */
function ga_initPB(pbIn, pp) {
    var ppStr = pp.toString();
    if (ga_usePB()) {
        if (null == pbIn)
            pbIn = top.gPB;

        actionsPB = pbIn;
        actionsPP = pp;
        var polSettingKey = "current_policy_settings";
        if (null != top.polctx && null != top.polctx.polSetting &&
            null != top.polctx.polSetting[polSettingKey]) {
            g_currPolicySettings = PropertyBag.Recreate(top.polctx.polSetting[polSettingKey]);
            if (null == actionsPB.GetProperty(ppStr)) {
                actionsPB = actionsPB.Clone();
                g_currPolicySettings.GetElement(ppStr, actionsPB);
            }
            top.polctx.polSetting[polSettingKey] = null;
        }
        if (null == g_onLoadPB)
            g_onLoadPB = actionsPB;
    }
    else {
        // Get the action string from the policy context
        var actionKey = ga_getActionKey();
        var actionStr = "";
        if (top.polctx && top.polctx[actionKey]) {
            actionStr = top.polctx[actionKey];
            delete top.polctx[actionKey];
        }
        // Create a dummy property path and property bag
        var ppT = "AK" + actionKey;
        actionsPB = new PropertyBag();
        actionsPB.SetProperty(ppT);
        actionsPB.SetProperty(ppT + ".Actions");
        actionsPB.SetProperty(ppT + ".Actions." + actionList[0], actionStr);
        actionsPP = new PropertyPath(ppT);
    }
	if(ppStr != "")
	    top[ppStr] = null;
}

/**
 * Callback to save context temporarily
 */
function ga_TempSave(pb, pp) {
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].get(pp, pb);
    }
    for (var i = 0; i < actionList.length; i++) {
        alh_getSel(baseSettingsType, actionList[i], pb, pp);
    }
}

/**
 * Page reset function: sets form values and shows / hides parts
 * of the page as required
 * @param onShow true if this function is being called to display
 * the page; false if it is being called to reset the page to its
 * initial state
 */
function ga_Reset(onShow) {
    if (null == onShow)
        onShow = false;
    actionsPB = alh_getPBToUse(false, !onShow);
    if (!onShow)
        actionsPB = alh_getInitial();
    var pp = new PropertyPath(actionsPP.toString());
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(pp, actionsPB);
    }
    if (g_showAlerts) {
        alh_reset(actionsPB, pp, false, !onShow);
        for (var i = 0; i < actionList.length; i++) {
            var actName = actionList[i];
            var formPfx = actName;
            var actForm = formPfx + "_actions_f_p";
            if (null == document.forms[actForm]) {
                formPfx = "Action";
                actForm = formPfx + "_actions_f_p";
            }
            var f = document.forms[actForm];
            if (!f)
                continue;
            var sel = f.elements["pri_actions"];
            if (sel)
                PolicyAction_onclick(sel, formPfx + "_alert");
            alh_setSel(baseSettingsType, actName, actionsPB, pp);
        }
    }
    top.gPageStack.Active().SetModified(alh_hasChanged());
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function ga_Apply(pb) {
    var changed = false;
    var pp = new PropertyPath(actionsPP.toString());
    for (var i = 0; i < dataBindings.length; i++) {
        if (dataBindings[i].get(pp, pb))
            changed = true;
    }
    if (!ga_usePB()) {
        var actionKey = ga_getActionKey();
        if (actionKey) {
            // Save the new action in policy context
            var actionStr = pb.GetPropertyDef(actionsPP.toString() + ".Actions." +
                actionList[0], ga_getDefaultActionString(actionList[0]));
            top.polctx[actionKey] = actionStr;
        }
        changed = false;
    }
    else {
        if (g_showAlerts) {
            var pbT = pb.Clone();
			if( baseSettingsType == "MAILURL" ) //special case for Mail URL
			{
				alh_getMailURLSel(actionList, pb, pp);
			}
			else //for everything else
			{
				for (var i = 0; i < actionList.length; i++) {
					alh_getSel(baseSettingsType, actionList[i], pb, pp);
				}
			}
            if (!changed)
                changed = (!pb.equals(pbT));
            alh_apply(pb, pp);
        }
        pb.GetElement(pp, g_currPolicySettings);
        top.polctx.polSetting["current_policy_settings"] = g_currPolicySettings;
    }
    top.gPageStack.Active().SetModified(changed);
    return changed;
}
