/*
 * Actions.js
 * Action lists for various CSE settings.
 * Depends on the JavaScript file CSEStrings.js
 *
 * Copyright (C) 2005 McAfee, Inc. All rights reserved.
 */

// Actions are stored in a JavaScript object; they are keyed by settings name 
// (e.g. "AV" for anti-virus) and then by "p" for primary and "s" for
// secondary

var a_actionsList = new Object();

// Default action list
a_actionsList["STD_LIST"] = new Object();
a_actionsList["STD_LIST"]["p"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["STD_LIST"]["s"] = ["LOG", "QUARANTINE", "FORWARD", "NOTIFYADM","NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT"];
a_actionsList["AV"] = a_actionsList["FF"] =  a_actionsList["CS"] = a_actionsList["SS"] = a_actionsList["EM"] =  
					  a_actionsList["PM"] =  a_actionsList["CM"] = a_actionsList["PPF"] = a_actionsList["STD_LIST"];

// Scanner/Filter specific action lists
a_actionsList["AS"] = new Object();
a_actionsList["AS"]["p"] = ["SYSJUNKFOLDER", "USERJUNKFOLDER", "REJECT", "REMOVECONTAINER", "ESERVICES:ALLOWTHRU"];
a_actionsList["AS"]["s"] = ["LOG", "QUARANTINE", "NOTIFYADM"];

a_actionsList["AF"] = new Object();
a_actionsList["AF"]["p"] = ["REMOVECONTAINER", "ESERVICES:ALLOWTHRU"];
a_actionsList["AF"]["s"] = ["LOG", "QUARANTINE", "NOTIFYADM"];
//a_actionsList["AF"] = a_actionsList["AS"];

a_actionsList["SM"] = new Object();
a_actionsList["SM"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:DONTBREAKSIG", "REMOVECONTAINER", "ESERVICES:BREAKSIG", "ESERVICES:ALLOWTHRU"];
a_actionsList["SM"]["s"] = ["LOG", "QUARANTINE","FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["MF"] = new Object();
a_actionsList["MF"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:REMOVE", "ESERVICES:ALLOWTHRU"];
a_actionsList["MF"]["p"]["MaxSizeAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MF"]["p"]["BiggerThanAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MF"]["p"]["MoreThanAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MF"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["MAILURL"] = new Object();
a_actionsList["MAILURL"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:REMOVE", "ESERVICES:ALLOWTHRU"];
a_actionsList["MAILURL"]["p"]["HighThresholdAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MAILURL"]["p"]["LowThresholdAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MAILURL"]["p"]["MAXURLExceededAction"] = ["ESERVICES:REPLACE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["MAILURL"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["MIME"] = new Object();
a_actionsList["MIME"]["p"] = ["REMOVECONTAINER", "ESERVICES:REPLACE", "ESERVICES:ALLOWTHRU"];
a_actionsList["MIME"]["s"] = a_actionsList["STD_LIST"]["s"];
a_actionsList["PX"] = {};
a_actionsList["PX"]["p"] = ["REMOVECONTAINER", "ESERVICES:ALLOWTHRU"];
a_actionsList["PX"]["s"] = a_actionsList["STD_LIST"]["s"];

// Policy specific sec actions
a_actionsList["on_access.AV"] = new Object();
a_actionsList["on_access.AV"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:REMOVE", "REMOVECONTAINER","ESERVICES:ALLOWTHRU"];
a_actionsList["on_access.AV"]["s"] = ["LOG", "QUARANTINE", "FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["on_access.FF"] = a_actionsList["on_access.CS"] = a_actionsList["on_access.SS"] = a_actionsList["on_access.EM"] = 
								a_actionsList["on_access.PM"] = a_actionsList["on_access.CM"] = a_actionsList["on_access.PPF"] = 
								a_actionsList["on_access.AV"];

a_actionsList["on_access.SM"] = new Object();
a_actionsList["on_access.SM"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:DONTBREAKSIG", "ESERVICES:REMOVE", "ESERVICES:BREAKSIG", "ESERVICES:ALLOWTHRU"];
a_actionsList["on_access.SM"]["s"] = a_actionsList["SM"]["s"];

a_actionsList["on_access.MF"] = new Object();
a_actionsList["on_access.MF"]["p"] = a_actionsList["MF"]["p"];
a_actionsList["on_access.MF"]["p"]["MaxSizeAction"] = a_actionsList["MF"]["p"]["MaxSizeAction"];
a_actionsList["on_access.MF"]["s"] = ["LOG", "QUARANTINE","FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["on_access.MIME"] = new Object();
a_actionsList["on_access.MIME"]["p"] = a_actionsList["MIME"]["p"];
a_actionsList["on_access.MIME"]["s"] = ["LOG", "QUARANTINE","FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["od_default.AV"] = new Object();
a_actionsList["od_default.AV"]["p"] = a_actionsList["STD_LIST"]["p"];
a_actionsList["od_default.AV"]["s"] = ["LOG", "QUARANTINE", "FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["od_default.FF"] = a_actionsList["od_default.CS"] = a_actionsList["od_default.SS"] = a_actionsList["od_default.EM"] = 
								a_actionsList["od_default.PM"] = a_actionsList["od_default.CM"] = a_actionsList["od_default.PPF"] = 
								a_actionsList["od_default.AV"];

a_actionsList["od_default.SM"] = new Object();
a_actionsList["od_default.SM"]["p"] = a_actionsList["SM"]["p"]
a_actionsList["od_default.SM"]["s"] = a_actionsList["SM"]["s"];

a_actionsList["od_default.MF"] = new Object();
a_actionsList["od_default.MF"]["p"] = a_actionsList["MF"]["p"];
a_actionsList["od_default.MF"]["p"]["MaxSizeAction"] = a_actionsList["MF"]["p"]["MaxSizeAction"];
a_actionsList["od_default.MF"]["s"] = ["LOG", "QUARANTINE","FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["od_default.MIME"] = new Object();
a_actionsList["od_default.MIME"]["p"] = a_actionsList["MIME"]["p"];
a_actionsList["od_default.MIME"]["s"] = a_actionsList["od_default.MF"]["s"];

a_actionsList["outbreakmanager.AV"] = new Object();
a_actionsList["outbreakmanager.AV"]["p"] = a_actionsList["STD_LIST"]["p"];
a_actionsList["outbreakmanager.AV"]["s"] = ["LOG", "QUARANTINE", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["outbreakmanager.FF"] = a_actionsList["on_access.CS"] = a_actionsList["on_access.SS"] = a_actionsList["on_access.EM"] = 
								a_actionsList["on_access.PM"] = a_actionsList["on_access.CM"] = a_actionsList["on_access.PPF"] = 
								a_actionsList["on_access.AV"];

a_actionsList["outbreakmanager.SM"] = new Object();
a_actionsList["outbreakmanager.SM"]["p"] = a_actionsList["SM"]["p"]
a_actionsList["outbreakmanager.SM"]["s"] = a_actionsList["SM"]["s"];

a_actionsList["outbreakmanager.MF"] = new Object();
a_actionsList["outbreakmanager.MF"]["p"] = a_actionsList["MF"]["p"];
a_actionsList["outbreakmanager.MF"]["p"]["MaxSizeAction"] = a_actionsList["MF"]["p"]["MaxSizeAction"];
a_actionsList["outbreakmanager.MF"]["s"] = ["LOG", "QUARANTINE","FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["outbreakmanager.MIME"] = new Object();
a_actionsList["outbreakmanager.MIME"]["p"] = a_actionsList["MIME"]["p"];
a_actionsList["outbreakmanager.MIME"]["s"] = a_actionsList["outbreakmanager.MF"]["s"];


a_actionsList["gateway.MF"] = new Object();
a_actionsList["gateway.MF"]["p"] = a_actionsList["MF"]["p"];
a_actionsList["gateway.MF"]["p"]["MaxSizeAction"] = a_actionsList["MF"]["p"]["MaxSizeAction"];
a_actionsList["gateway.MF"]["s"] = ["LOG", "QUARANTINE",  "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["gateway.MIME"] = new Object();
a_actionsList["gateway.MIME"]["p"] = a_actionsList["MIME"]["p"];
a_actionsList["gateway.MIME"]["p"]["MaxSizeAction"] = a_actionsList["MIME"]["p"]["MaxSizeAction"];
a_actionsList["gateway.MIME"]["s"] = ["LOG", "QUARANTINE",  "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];

a_actionsList["gateway.SS"] = new Object();
a_actionsList["gateway.SS"]["p"] = a_actionsList["SS"]["p"];
a_actionsList["gateway.SS"]["p"]["MaxSizeAction"] = a_actionsList["MIME"]["p"]["MaxSizeAction"];
a_actionsList["gateway.SS"]["s"] = ["LOG", "QUARANTINE",  "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];


// Compound actions for Corrupted, Encrypted, Pwd Protected & Protected Cnt
a_actionsList["CM"] = new Object();
a_actionsList["CM"]["p"] = ["ESERVICES:REPLACE[SAFE:CORRUPTED]", "REMOVECONTAINER[SAFE:CORRUPTED]", "ESERVICES:ALLOWTHRU[SAFE:CORRUPTED]"];
a_actionsList["CM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["EM"] = new Object();
a_actionsList["EM"]["p"] = ["ESERVICES:REPLACE[SAFE:ENCRYPTED]", "REMOVECONTAINER[SAFE:ENCRYPTED]", "ESERVICES:ALLOWTHRU[SAFE:ENCRYPTED]"];
a_actionsList["EM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["PPF"] = new Object();
a_actionsList["PPF"]["p"] = ["ESERVICES:REPLACE[SAFE:PWDPROTECTED]", "REMOVECONTAINER[SAFE:PWDPROTECTED]","ESERVICES:ALLOWTHRU[SAFE:PWDPROTECTED]"];
a_actionsList["PPF"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["PM"] = new Object();
a_actionsList["PM"]["p"] = ["ESERVICES:REPLACE[SAFE:PROTECTEDCNT]", "REMOVECONTAINER[SAFE:PROTECTEDCNT]", "ESERVICES:ALLOWTHRU[SAFE:PROTECTEDCNT]"];
a_actionsList["PM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["on_access.CM"] = new Object();
a_actionsList["on_access.CM"]["p"] = ["ESERVICES:REPLACE[SAFE:CORRUPTED]", "REMOVECONTAINER[SAFE:CORRUPTED]", "ESERVICES:REMOVE[SAFE:CORRUPTED]", "ESERVICES:ALLOWTHRU[SAFE:CORRUPTED]"];
a_actionsList["on_access.CM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["on_access.EM"] = new Object();
a_actionsList["on_access.EM"]["p"] = ["ESERVICES:REPLACE[SAFE:ENCRYPTED]", "REMOVECONTAINER[SAFE:ENCRYPTED]", "ESERVICES:REMOVE[SAFE:ENCRYPTED]", "ESERVICES:ALLOWTHRU[SAFE:ENCRYPTED]"];
a_actionsList["on_access.EM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["on_access.PPF"] = new Object();
a_actionsList["on_access.PPF"]["p"] = ["ESERVICES:REPLACE[SAFE:PWDPROTECTED]", "REMOVECONTAINER[SAFE:PWDPROTECTED]", "ESERVICES:REMOVE[SAFE:PWDPROTECTED]", "ESERVICES:ALLOWTHRU[SAFE:PWDPROTECTED]"];
a_actionsList["on_access.PPF"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["on_access.PM"] = new Object();
a_actionsList["on_access.PM"]["p"] = ["ESERVICES:REPLACE[SAFE:PROTECTEDCNT]", "REMOVECONTAINER[SAFE:PROTECTEDCNT]", "ESERVICES:REMOVE[SAFE:PROTECTEDCNT]", "ESERVICES:ALLOWTHRU[SAFE:PROTECTEDCNT]"];
a_actionsList["on_access.PM"]["s"] = a_actionsList["STD_LIST"]["s"];

a_actionsList["od_default.CM"] = new Object();
a_actionsList["od_default.CM"]["p"] = a_actionsList["CM"]["p"];
a_actionsList["od_default.CM"]["s"] = a_actionsList["od_default.AV"]["s"];

a_actionsList["od_default.EM"] = new Object();
a_actionsList["od_default.EM"]["p"] = a_actionsList["EM"]["p"];
a_actionsList["od_default.EM"]["s"] = a_actionsList["od_default.AV"]["s"];

a_actionsList["od_default.PPF"] = new Object();
a_actionsList["od_default.PPF"]["p"] = a_actionsList["PPF"]["p"];
a_actionsList["od_default.PPF"]["s"] = a_actionsList["od_default.AV"]["s"];

a_actionsList["od_default.PM"] = new Object();
a_actionsList["od_default.PM"]["p"] = a_actionsList["PM"]["p"];
a_actionsList["od_default.PM"]["s"] = a_actionsList["od_default.AV"]["s"];

a_actionsList["TIEACTIONS"] = new Object();
a_actionsList["TIEACTIONS"]["p"] = ["ESERVICES:REPLACE", "ESERVICES:REMOVE", "REMOVECONTAINER"];
a_actionsList["TIEACTIONS"]["s"] = ["LOG", "QUARANTINE", "FORWARD", "NOTIFYADM", "NOTIFYINTSDR", "NOTIFYEXTSDR", "NOTIFYINTRPT", "NOTIFYEXTRPT" ];
/**
 * Creates a primary action list in a form (through document.write calls).
 * The output is a collection of radio buttons.
 * @param settingsType the settings type (e.g. "AV")
 * @param formName the name of the form
 * @param alert_id the id string for the alert placeholder
 * @param actionName the name of the action; may affect the label displayed
 */
function a_makePrimaryActionsForm(settingsType, formName, alert_id, actionName) {
    document.write("<form name=\"" + formName + "\" style=\"display:inline\">");
    var list = a_getActionList(settingsType, "p", actionName);
    document.write('<select name="pri_actions" style="margin-left:10px;" ' +
        'onclick="top.gPageStack.Active().SetModified(true);PolicyAction_onclick(this,\'' + alert_id + '\');">');
    for (var i = 0; i < list.length; i++) {
        document.write('<option value="' + list[i] + '"');
        if (0 == i)
            document.write(" selected");
        document.write('>' + ReplaceSpecialHTMLChars(a_getLabel(settingsType, list[i], actionName)) + '</option>');
    }
    document.write("</select>");
    document.write("<input type='hidden' id='init' value='' >");
    document.write("</form>");
}


/**
 * Creates a secondary action list in a form (through document.write calls).
 * The output is a collection of checkboxes.
 * @param settingsType the settings type (e.g. "AV")
 * @param formName the name of the form
 * @param actionName the name of the action; may affect the options offered
 */
function a_makeSecondaryActionsForm(settingsType, formName, actionName) {
    var actionName = "";
    // "Success / failure" flag; for AV only there are actions to take when cleaning succeeds and
    // actions to take when cleaning fails; this flag differentiates between the two sets of actions
    var sfFlag = "";
    var pattern = /(.*)_actions_(.)_s/;
    var matchInfo = pattern.exec(formName);
    if (matchInfo) {
        actionName = matchInfo[1];
        sfFlag = matchInfo[2];
    }

    var list = a_getActionList(settingsType, "s", actionName);
    var n = list.length;
    document.write('<form name="' + formName + '"');
    if (0 == n)
        document.write(' style="display:none"');
    document.write('>');
    document.write('<table border="0" cellpadding="0" cellspacing="0" ><tr>');
    document.write('<td style="padding:10px 0px 5px 0px;" class="st_label" id="' +
        actionName + sfFlag + '_sec_header">' + resolveString(cse_pp_st, "__def_secondary_text") + '</td>');
    document.write('</tr><tr><td valign="top">');
	
    for (var i = 0; i < n; i++) {

		if(list[i] == "FORWARD")
		{
				if(settingsType != "AV")
				{
    				document.write('&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type="checkbox" ID="frd' + formName + '" name="a2" value="' +
					list[i] + '" style="vertical-align:middle;" ' +
					'onclick="top.gPageStack.Active().SetModified(true);"><text id="frd1"> ' +
					ReplaceSpecialHTMLChars(a_getLabel(settingsType, list[i])) + '</text><br>');
				}
		}
		else if(list[i] == "QUARANTINE")
		{
			document.write('<input type="checkbox" ID="qtn' + formName + '" name="a2" value="' +
				list[i] + '" style="vertical-align:middle;" ' +
				'onclick="top.gPageStack.Active().SetModified(true);QtnCheck(\'' + formName + '\')"><text id="frd1"> ' +
				ReplaceSpecialHTMLChars(a_getLabel(settingsType, list[i])) + '</text><br>');

			
		}
		else
		{
			
			document.write('<input type="checkbox" name="a2" value="' +
				list[i] + '" style="vertical-align:middle;" ' +
				'onclick="top.gPageStack.Active().SetModified(true);QtnCheck()"> ' +
				ReplaceSpecialHTMLChars(a_getLabel(settingsType, list[i])) + '<br>');
		}
		
		
    }
    document.write('</td></tr></table>');
    document.write('</form>');
}

function QtnCheck(formName)
{
	
	var id = "qtn"+ formName;
	
	var chkqtn = document.getElementById(id);

    if (chkqtn)
	{
		
        if(chkqtn.checked)
		{
        	var frdid = "frd"+ formName;
			var frdchk = document.getElementById(frdid);
			
			 if (frdchk)
				frdchk.disabled = false;
		}
		else
		{
			var frdid = "frd"+ formName;
		    var frdchk = document.getElementById(frdid);
			
			if (frdchk)
			{
				frdchk.checked = false;
				frdchk.disabled = true;
			}
		}
	}


  
}
/**
 * Gets the most specific action list for a settings type (e.g. given a
 * settings type of "AV", and a policy-set called "smtp" an action list
 * labelled "smtp.AV" would be used in preference to one labelled "AV"
 * @param settingsType the lookup key for the action list (e.g. "AV")
 * @param psFlag - "p" for the primary list, "s" for the secondary
 * @return an action list (or an empty array if none can be found)
 * @param actionName the particular action being requested
 */
function a_getActionList(settingsType, psFlag, actionName) {
    var ret = [];
    var polSet = "";
    try {
        polSet = top.polctx.myPolicySet.pp_key;
    }
    catch(e) {}
    if (polSet) {
        // The key for available action lists is a concatenation of the
        // base settings type (such as "AF") and the current policy set
        // (e.g. "smtp")
        settingsType = polSet + "." + settingsType;
    }
    var parts = settingsType.split(".");
    while (parts.length > 0) {
        var testKey = parts.join(".");
        if (a_actionsList[testKey] && a_actionsList[testKey][psFlag]) {
            ret = a_actionsList[testKey][psFlag];
            if (actionName && a_actionsList[testKey][psFlag][actionName])
                ret = a_actionsList[testKey][psFlag][actionName];
            break;
        }
        parts.shift();
    }
    return ret;
}

/**
 * Gets the label for a form control
 * @param settingsType the settings type (e.g. "AV")
 * @param key the lookup key
 * @param actionName the name of the action (some actions have type-
 * specific label, e.g. mail size filtering actions)
 * @return the corresponding form label
 */
function a_getLabel(settingsType, key, actionName) {
    var ret = key;
    var parts = [];
    if (settingsType) {
        // The settingsType (e.g. "AV.smtp" creates a hierarchical
        // namespace for actions
        parts = settingsType.split(".");
    }
    parts.push(key);
    while (parts.length > 0) {
        var testKey = parts.join(".");
        var keys = [testKey];
        if (actionName) {
            // There may be an action-specific label
            keys.unshift(testKey + "_" + actionName);
        }
        for (var i = 0; i < keys.length; i++) {
            if (cse_pp_actions_st[keys[i]] != null) {
                return cse_pp_actions_st[keys[i]];
            }
        }
        parts.shift();
    }
    return ret;
}

/**
 * Determines whether a settings type has an action that involves
 * replacement alerts
 */
function a_hasReplacementOption(settingsType, actionName) {
    var actList = a_getActionList(settingsType, "p", actionName);
    var ret = false;
    for (var i = 0; i < actList.length; i++) {
        if ("1" == pp_cse_alertActions[actList[i]]) {
            ret = true;
            break;
        }
    }
    return ret;
}
