/**
 * Alert Editor wrapper for the richtext control
 * Dependent on RichText.js and ResourceManager.js
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var basePPName = "Alerts.Alert";
// Memo cache for alert paths, keyed by ID
var alertPaths = {};

var g_changeNotifyFn = new Function("top.gPageStack.Active().SetModified(true);return true;");

/**
 * Data binding object for the HTML control
 * @param container the element used to house the associated
 * rich-text control
 * @param category the alert category
 * @param rtc a rich-text control
 * @param rm a resource manager object for getting default text
 * @param pt whether plain-text alerts are the default
 */
function AlertDataBinding(container, category, rtc, rm, pt) {
    this.container = container;
    this.initialValueH = "";
    this.initialValueP = "";
    this.category = category;
    this.rtc = rtc;
    this.rm = rm;
    this.html = "";
    this.text = "";
    this.initialized = false;
    this.elemH = "htmlText";
    this.elemP = "text";
    this.pt = (null != pt) ? pt : false;
}

AlertDataBinding.prototype = new DataBinding("", "", "");

AlertDataBinding.prototype.init = function() {
    if (this.initialized)
        return;
    // Get the default texts
    var defHComplete = this.rm.getResource("AlertMessages", this.category);
    var defPComplete = this.rm.getResource("AlertMessages", this.category + "_FORMATTEDPLAIN");

    this.defP = this.extractContent(defPComplete);
    this.templateP = defPComplete.replace(this.defP, "%CONTENT%");
    
	var section = "TokenReplacements";
	var newToks = ["REASON_START", "REASON_END", "DESCRIPTION_START", "DESCRIPTION_END",
		"FURTHER_INFO_START", "FURTHER_INFO_END"];
		
	for (var i = 0; i < newToks.length; i++) 
	{
		var rep = "%" + newToks[i] + "%";
		defHComplete = defHComplete.replace(rep, this.rm.getResource(section, newToks[i]));
	}
	this.defH = this.extractContent(defHComplete);
    var _templateH = defHComplete.replace(this.defH, "%CONTENT%");
    // The HTML template needs further processing
	var toks = ["HTMLSTART", "HEADSTART", "DEFAULTMETATAGS", "TITLESTART",
			"TITLEEND", "HEADEND", "BODYSTART", "BODYEND", "HTMLEND"];
		
	for (var i = 0; i < toks.length; i++) 
	{
		var rep = "%" + toks[i] + "%";
		_templateH = _templateH.replace(rep, this.rm.getResource(section, toks[i]));
	}
	
    this.templateH = _templateH;
}

AlertDataBinding.prototype.extractContent = function(text) {
    var startT = "%ALERTHEADER%";
    var endT = "%ALERTFOOTER";
    var startTIdx = text.indexOf(startT);
    if (-1 == startTIdx)
        return text;
    var startIdx = startTIdx + startT.length;
    var endIdx = text.lastIndexOf(endT);
    if (-1 == endIdx)
        return text.substring(startIdx);
    return text.substring(startIdx, endIdx);
}

/**
 * Sets the field value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
AlertDataBinding.prototype.set = function(pp, pbIn) {
    this.init();
    var ppT = null;
    if (pp) {
        if ("string" == typeof(pp)) {
            ppT = new PropertyPath(pp);
        }
        else {
            ppT = pp;
        }
    }
    this.initialValueH = this._set(this.elemH, ppT, pbIn,
        this.defH, this.templateH, true);
    this.initialValueP = this._set(this.elemP, ppT, pbIn,
        this.defP, this.templateP, false);
    if (this.initialized) {
        this.rtc.sync_r(this.html);
        this.rtc.setText(this.text, false);
    }
    else {
        this.rtc.makeControl(this.container, this.html, this.text, true);
        if (this.pt) {
            // Switch to plain-text mode
            this.rtc.setPT();
            var csCtl = document.getElementById("chkPt" + this.rtc.name);
            if (csCtl)
                csCtl.checked = true;
        }
        this.initialized = true;
    }
}

AlertDataBinding.prototype._set = function(elem, pp, pb, def, template, html) {
    var ret = "";
    var v = null;
    var returnInitValue = true;
    if (pp) {
        pp.SetTempLeaf(elem);
        v = pb.GetProperty(pp);
    }
    if (null == v) {
        // New alert; return empty string as the initial value to ensure that
        // the value gets saved, even if the user does not change it
        v = this.mergeWithTemplate(def, template);
        returnInitValue = false;
    }
    // Remove the surrounding crud because we're only interested in text
    // between the header and footer
    v = this.extractContent(v);
    if (html) {
        this.html = v;
    }
    else {
        this.text = v;
    }
    if (returnInitValue)
        ret = v;
    if (pp)
        pp.SetTempLeaf("");
    return ret;
}

AlertDataBinding.prototype.mergeWithTemplate = function(text, template) {
    return template.replace("%CONTENT%", text);
}

/**
 * Gets the field value and places it in a PropertyBag object
 * @param pp the base property path to use
 * @param pbOut the output property bag
 * @return true if the value has changed, false otherwise
 */
AlertDataBinding.prototype.get = function(pp, pbOut) {
    var ppT = null;
    if (pp) {
        if ("string" == typeof(pp))
            ppT = new PropertyPath(pp);
        else
            ppT = pp;
    }
    var ret1 = this._get(this.elemH, ppT, pbOut, this.initialValueH, this.templateH, true);
    var ret2 = this._get(this.elemP, ppT, pbOut, this.initialValueP, this.templateP, false);
    return ret1 || ret2;
}

AlertDataBinding.prototype._get = function(elem, pp, pb, initial, template, html) {
    var ret = false;
    if (pp) {
        var v = html ? this.rtc.getValue() : fixNewLines(this.rtc.getText());
        if (v != initial) {
            ret = true;
        }
        // The content must be placed inside its surrounding template prior to
        // storage
        v = template.replace("%CONTENT%", v);
        pp.SetTempLeaf(elem);
        pb.SetProperty(pp, v);
        pp.SetTempLeaf("");
    }
    return ret;
}

/**
 * Constructor for the AlertEditor
 * @param category the alert category (e.g. "AV_REPLACEMENT")
 * @param name the name of the control; used for creating element IDs
 * @param ctlName the name by which the control has been declared in the HTML
 * @param rtcProps properties for the underlying rich text control; see
 * RichText.js for details of the permitted properties; note that only height
 * width and readOnly will not be overwritten
 */
function AlertEditor(category, name, ctlName, rtcProps) {
    // Initialize the rich-text library
    initRTE("../../images/rte/", "", "");
    // Fix up the properties object so that it is in a known state
    if (null == rtcProps)
        rtcProps = {};
    if (null == rtcProps["height"])
        rtcProps["height"] = 275;
    rtcProps["richText"] = true;
    rtcProps["tokens"] = true;
    rtcProps["plainTextOption"] = true;
    rtcProps["htmlControl"] = "htmlText";
    rtcProps["textControl"] = "text";
    rtcProps["changeNotifyFn"] = g_changeNotifyFn;
    initControl(rtcProps, this);
    this.headerText = "";
    this.name = name;
    this.ctlName = (null != ctlName) ? ctlName : "alertEditor";
    this.html = "";
    this.text = "";
    this.created = false;
    this.mode = WYSIWYG_MODE;
    this.initialContent = "";
    this.initialText = "";

    // Create a resource manager object to handle resources
    this.rm = new ResourceManager("DefaultAlerts");
    this.setCategory(category);
}

AlertEditor.prototype = new RTControl("", "");
AlertEditor.prototype.makeControlA = AlertEditor.prototype.makeControl;

/**
 * Sets the alert category
 * @param category the alert category (e.g. "AV_REPLACEMENT")
 */
AlertEditor.prototype.setCategory = function(category) {
    // Get the tokens appropriate to the category
    var toks1 = this.rm.getTokens(category);
    // Convert the return value to an array of IToken objects
    this.tokenList = [];
    for (var i = 0; i < toks1.length; i++) {
        this.tokenList.push(new IToken(toks1[i].token, "%" + toks1[i].token + "%", toks1[i].value));
    }
    this.category = category;
}

/**
 * Writes the alert editor in the page
 */
AlertEditor.prototype.makeControl = function(container, html, text, clearParent) {
    /*if (this.richText) {
        // Import the stylesheet for the control
        document.writeln('<style type="text/css">@import "../../css/rte.css";</style>');
    }*/
    this.headerText = resolveString(cse_pp_st,
        this.readOnly ? "__alert_header_text_read_only" : "__alert_header_text",
        [resolveString(cse_pp_alerts_st, this.category)]);
    this.makeControlA(container, html, text, clearParent);
}

/**
 * Shows any tokens that can be inserted
 */
AlertEditor.prototype.showTokens = function() {
    this.makeTokenList(this.tokenList);
}

/**
 * Creates a data binding object for the alert editor
 * @param container the ID of the element containing the editor control
 * @param pt (optional) whether the alert editor should be initialized in
 * plain-text mode
 * @return an AlertDataBinding object
 */
AlertEditor.prototype.makeDataBinding = function(container, pt) {
    return new AlertDataBinding(container, this.category, this, this.rm, pt);
}

/**
 * Utility function for getting a custom alert's property path
 * @param id the ID of the custom alert
 * @param pb the property bag to use (may be null - defaults to the global bag)
 * @return the property path for looking up the value of the custom alert
 */
AlertEditor.lookupPath = function(id, pb) {
    var ret = "";
    if (null != alertPaths[id])
        return alertPaths[id];
    var basePP = new PropertyPath(basePPName);
    if (pb == null) {
        pb = top.gPB;
    }
    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("id");
        if (id == pb.GetProperty(basePP)) {
            basePP.SetTempLeaf("");
            ret = basePP.GetPath();
            break;
        }
        basePP.SetTempLeaf("");
    }
    alertPaths[id] = ret;
    return ret;
}

/**
 * Utility function for getting an attribute of a custom alert (name, text, htmlText)
 * @param id the ID of the custom alert
 * @param attr the attribute required
 * @param pb the property bag to use (may be null - defaults to the global bag)
 * @return the value of the attribute (empty string if not found)
 */
AlertEditor.getAlertAttribute = function(id, attr, pb) {
    var ret = "";
    if (pb == null) {
        pb = top.gPB;
    }
    var path = AlertEditor.lookupPath(id, pb);
    if (path) {
        var pp = path + "." + attr;
        if (null != pb.GetProperty(pp)) {
            ret = pb.GetProperty(pp);
        }
    }
    return ret;
}

/**
 * Gets a list of alert choices for a given category (that is the default plus
 * any customized texts);
 * @param category the alert category
 * @param pb the property bag to use (may be null - defaults to the global bag)
 * @return a list if Objects keyed by displayName, id and pp, where id is the ID
 * of the customized alert and pp its property path
 */
AlertEditor.getList = function(category, pb) {
    var insStr = resolveString(cse_pp_alerts_st, category + "_S");
    var ret = [{displayName:resolveString(cse_pp_st, "__defalert", [insStr]), id:"", pp:""}];
    var basePP = new PropertyPath(basePPName);
    if (pb == null) {
        pb = top.gPB;
    }
    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("type");
        if (category == pb.GetProperty(basePP)) {
            basePP.SetTempLeaf("name");
            var name = pb.GetProperty(basePP);
            basePP.SetTempLeaf("id");
            var alertId = pb.GetProperty(basePP);
            basePP.SetTempLeaf("");
            ret.push({displayName:name, id:alertId, pp:basePP.GetPath()});
        }
        basePP.SetTempLeaf("");
    }

    return ret;
}

/**
 * Creates a property bag containing all alert data
 * @param newAlertPB a property bag containing a newly defined alert (may be null)
 * @return a property bag with alert data
 */
AlertEditor.makePB = function(newAlertPB) {
    var pb = new PropertyBag();
    top.gPB.GetElement("Alerts", pb);
    if (null != newAlertPB) {
        pb.Merge([], newAlertPB);
    }
    return pb;
}

/*
 * Class for generating alert previews
 */

/**
 * Constructor for the AlertPreview
 * @param category the alert category (e.g. "AV_REPLACEMENT")
 * @param name the name of the control; used for creating element IDs
 * @param ctlName the name by which the control has been declared in the HTML
 * @param rtcProps properties for the underlying rich text control; see
 * RichText.js for details of the permitted properties; note that only height
 * and width will not be overwritten
 */
function AlertPreview(category, name, ctlName, rtcProps) {
    // Initialize the rich-text library
    var pvStyle = "<style>body, td {font-size:x-small;} body {background-color:white;} " +
        "h1, h2, h3 {font-size:medium;}</style>";
    initRTE("../../images/rte/", "", pvStyle);
    // Fix up the properties object so that it is in a known state
    if (null == rtcProps)
        rtcProps = {};
    if (null == rtcProps["height"])
        rtcProps["height"] = AlertPreview.getDefaultHeight();
    if (null == rtcProps["width"])
        rtcProps["width"] = AlertPreview.getDefaultWidth();
    rtcProps["richText"] = true;
    rtcProps["tokens"] = false;
    rtcProps["plainTextOption"] = true;
    rtcProps["htmlControl"] = "htmlText";
    rtcProps["textControl"] = "text";
    rtcProps["readOnly"] = true;
    rtcProps["srcCtl"] = false;
    //rtcProps["selector"] = false;

    initControl(rtcProps, this);
    this.headerText = "";
    this.name = name;
    this.ctlName = (null != ctlName) ? ctlName : "alertEditor";
    this.html = "";
    this.text = "";
    this.created = false;
    this.mode = WYSIWYG_MODE;
    this.initialContent = "";
    this.initialText = "";

    // Create a resource manager object to handle resources
    this.rm = new ResourceManager("DefaultAlerts");
    this.category = category;
}

AlertPreview.prototype = new RTControl("", "");
AlertPreview.prototype.makeControlA = AlertPreview.prototype.makeControl;

/**
 * Writes the alert preview in the page
 */
AlertPreview.prototype.makeControl = function(container, html, text) {
    this.headerText = resolveString(cse_pp_st, "__preview");
    this.makeControlA(container, html, text, true);
    var rte = this.name;
    // Fix the font-size of the text areas
    var elemId = rte + "Src";
    var e = document.getElementById(elemId);
    if (e)
        e.style.fontSize = "xx-small";
    elemId = rte + "Plain";
    e = document.getElementById(elemId);
    if (e)
        e.style.fontSize = "xx-small";
}

/**
 * Creates a data binding object for the alert editor
 * @param container the ID of the element containing the editor control
 * @param pt (optional) whether the alert editor should be initialized in
 * plain-text mode
 * @return an AlertDataBinding object
 */
AlertPreview.prototype.makeDataBinding = function(container, pt) {
    return new AlertDataBinding(container, this.category, this, this.rm, pt);
}

/**
 * Shows the previewer in a disabled state
 */
AlertPreview.prototype.showDisabled = function(disabled) {
    var clD = "disabled";
    var elIds = [this.name + "_header_text", this.name + "_b3_left"];
    for (var i = 0; i < elIds.length; i++) {
        var e = document.getElementById(elIds[i]);
        if (!e)
            continue;
        var cl = (null == e.className) ? "" : e.className;
        if (disabled) {
            e.className = addClass(cl, clD);
        }
        else {
            e.className = removeClass(cl, clD);
        }
    }
    // Change state of the plain-text checkbox
    var ptE = document.getElementById("chkPt" + this.name);
    var ptChecked = false;
    if (ptE) {
        ptE.disabled = disabled;
        ptChecked = ptE.checked;
    }
    // Change state of the view-source checkbox
    var srcE = document.getElementById("chkSrc" + this.name);
    if (srcE) {
        srcE.disabled = ptChecked ? true : disabled;
    }
    // Disable the source and plain-text textarea controls
    /*var mode = this.getMode();
    if (WYSIWYG_MODE == mode) {

    }
    else {
        var eleId = (SOURCE_MODE == mode) ? (this.name + "Src") : (this.name + "Plain");
        var e = document.getElementById(eleId);
        if (e)
            e.disabled = disabled;
    }*/
}

AlertPreview.getDefaultWidth = function() {
    return "400px";
}

AlertPreview.getDefaultHeight = function() {
    return "150px";
}
