/**
 * AlertHelper.js
 *
 * JavaScript support for the alerts component
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

var g_initialState = null;
var g_currentState = null;

var newAlertId = "";

/**
 * Creates the infrastructure for the alert selection form; innerHTML used
 * because of IE bugs in creating form elements with DOM
 * @param uid a unique reference for the alert
 * @param prop the name of the Alert property being controlled
 * @param cat the alert category (e.g. "FILE_FILTER_BLOCK")
 * @param divId the ID of a div that this structure will go into
 * @param width the width of the alert previewer (null for default)
 * @param height the height of the alert previewer (null for default)
 * @param showView whether the view / hide controls should be displayed
 * @param editFn the name of the function to call when editing / creating alerts
 */
function alh_makeAlertSelector(uid, prop, cat, divId, width, height, showView, editFn) {
    if (null == showView) {
        showView = true;
    }
    if (null == width)
        width = AlertPreview.getDefaultWidth();
    if (null == height)
        height = AlertPreview.getDefaultHeight();
    if (null == editFn)
        editFn = "alh_EditAlert";
    var e = document.getElementById(divId);
    if (!e)
        return;

    var html = "";
    var stKey = cat + "_" + prop + "_directive";
    if (null == cse_pp_st[stKey])
        stKey = "__use_following_alert";
    html += '<span id=\"' + uid + "_text" +'">' +
        ReplaceSpecialHTMLChars(resolveString(cse_pp_st, stKey)) +
        '</span>\n';
    html += '<form name="' + uid + "_fld_form" + '" style="margin-top:7px;">\n';
    // Placeholder for select control
    html += '<span id=\"' + uid + "_name_div" +'"></span>&nbsp;&nbsp;\n';

    // View, edit, create links
    html += '<span id="' + uid + '_links">';
    var fnStr = "";
    if (showView) {
        fnStr = "javascript:AlertSummaryPop('" + uid + "');";
        html += '<a href="' + fnStr + '" class="blueLink">' +
            resolveString(cse_pp_st, "__view_hide") + '</a> | \n';
    }
    html += '<span id="edLink_' + uid + '">';
    fnStr = "javascript:" + editFn + "(gPolicyAlerts, '" + uid + "', false);";
    html += '<a href="' + fnStr + '" class="blueLink">' +
        resolveString(cse_pp_st, "__edit") + '</a> | \n';
    html += '</span>';

    fnStr = "javascript:" + editFn + "(gPolicyAlerts, '" + uid + "', true);";
    html += '<a href="' + fnStr + '" class="blueLink">' +
        resolveString(cse_pp_st, "__create") + '</a>\n';
    html += '</span>';

    // Alert preview container
    var displayStyle = showView ? "none" : "block";
    html += '<div id="' + uid + '" style="margin-top:10px;display:' + displayStyle +
        '">\n';
    html += '<div id="box_' + uid + '">';
    html += '</div>\n';
    html += '</div>\n';


    var hFields = ["id", "type", "ndx", "name"];
    for (var i = 0; i < hFields.length; i++) {
        html += '<input type="hidden" name="' + hFields[i] + '" id="' +
            uid + "_" + hFields[i] + '">\n';
    }
    html += '</form>';
    e.innerHTML = html;

    // Register the alert
    gPolicyAlerts.Register(uid, cat, prop, width, height);
}

/**
 * Called from a page's Show function
 */
function alh_onShow() {
    // Null out the state variables so they may be updated
    g_currentState = g_initialState = null;
    if (top.polctx && top.polctx.newAlert) {
        // New alert defined - use it
        newAlertId = top.polctx.newAlert;
    }
}

/**
 * Restores any temporarily saved context
 */
function alh_tRestore() {
    var ret = false;
    var ppT = PolSetting_GetPPNode("").toString();
    var ctx = top.retrieveContext(location.href);
    if (ctx) {
        g_initialState = ctx.initial ? PropertyBag.Recreate(ctx.initial) : null;
        g_currentState = ctx.current ? PropertyBag.Recreate(ctx.current) : null;
        if (top.CONTENT_FRAME.Restore) {
            top.CONTENT_FRAME.Restore(g_currentState, new PropertyPath(ppT));
        }
        // Check to see whether values have changed
        if (!g_currentState.equals(g_initialState)) {
            // Set modified flag on the page
            top.gPageStack.Active().SetModified(true);
        }
        ret = true;
    }
    if(this.gPolicyAlerts && top.polctx.polSetting["tmp_alerts"]) {
        gPolicyAlerts.LocalRestore(top.polctx.polSetting["tmp_alerts"]);
        delete top.polctx.polSetting["tmp_alerts"];
    }
    return ret;
}

/**
 * Temporarily saves context
 */
function alh_tSave() {
    var ppT = PolSetting_GetPPNode("").toString();
    var pp = new PropertyPath(ppT);
    var pb = g_initialState.Clone();
    if (top.CONTENT_FRAME.TempSave) {
        top.CONTENT_FRAME.TempSave(pb, pp);
    }
    top.saveContext(location.href, pb, g_initialState);
    if(gPolicyAlerts) {
        top.polctx.polSetting["tmp_alerts"]=[];
        gPolicyAlerts.LocalStore(top.polctx.polSetting["tmp_alerts"]);
    }
    if (top.CONTENT_FRAME.PolSetting_StoreLocalCopy) {
        PolSetting_StoreLocalCopy(true);
    }
}

/**
 * Gets the property bag to use when setting data in the page; will set
 * g_initialState if it is null
 * @param inherited true if the inherited box is checked
 * @param init true if the page is being set to its initial state
 */
function alh_getPBToUse(inherited, init) {
    var pb = PolSetting_GetPb();
    if (inherited) {
        g_currentState = pb;
        return pb;
    }

    // See whether there's a temporary save to restore
    if (alh_tRestore()) {
        pb = init ? g_initialState : g_currentState;
    }

    if (!g_initialState)
        g_initialState = pb;
    if (!g_currentState)
        g_currentState = pb;

    return pb;
}

function alh_getInitial() {
    return g_initialState;
}

function alh_getCurrent() {
    return g_currentState;
}

/**
 * Edits the alert
 * @param alertList the list of globally defined alert objects for this page
 * @param alertId an ID for the alert being edited
 * @param create true if a new alert is being created
 */
function alh_EditAlert(alertList, alertId, create) {
    if (g_initialState)
        alh_tSave();
    if (alertList) {
        alertList.EditAlert(alertId, create);
    }
}

/**
 * Called during the initialization phase
 */
function alh_init(pb, pp) {
    gPolicyAlerts.Init(pb, pp, true);
}

/**
 * Called during the set/reset phase
 * @param pb the property bag to read from
 * @param pp the property path to use
 * @param inherited true if the inherited control is checked
 * @param init true if this is an initialization phase
 */
function alh_reset(pb, pp, inherited, init) {
    gPolicyAlerts.ShowAll();
    if (init)
        g_currentState = g_initialState;
}

/**
 * Called during the apply phase
 */
function alh_apply(pb, pp) {
    // Update the reset point
    g_initialState = pb;
    gPolicyAlerts.Apply();
}

/**
 * Determines whether the page has changed
 */
function alh_hasChanged() {
    return (!g_currentState.equals(g_initialState));
}

/**
 * Sets state property bags
 */
function alh_setState(initial, current) {
    g_initialState = initial;
    g_currentState = current;
}

/**
 * Sets selection of the alert selector
 */
function alh_setSel(settingsType, actName, pb, pp) {
    var uid = settingsType + "_" + actName;
    var alertProps = pp_cse_actionAlertMap[uid];
    if (null == alertProps)
        return;
    var alertProp = alertProps[0];
    var alertCat = alertProps[1];
    alh_setSel2(uid, alertProp, alertCat, pb, pp);
}

function alh_setSel2(uid, alertProp, alertCat, pb, pp) {
    var form = document.forms[uid + "_fld_form"];
    if (!form)
        return;
    var sel = form.elements["alert_sel"];
    if (!sel)
        return;
    var oldSelIdx = sel.selectedIndex;

    var alertList = gPolicyAlerts.getAlertList(alertCat);
    var ppK = "Alerts." + alertProp;
    var alertId = pb.GetPropertyDef(pp.toString() + "." + ppK, "");
    // See whether any new alert is for this particular alert settings
    if (newAlertId) {
        var useNewAlert = false;
        for (var i = 0; i < alertList.length; i++) {
            var aInfo = alertList[i];
            if (aInfo.id == newAlertId) {
                useNewAlert = true;
                break;
            }
        }
        if (useNewAlert) {
            // New alert defined - use it
            sel.selectedIndex = sel.length - 1;
            pb.SetProperty(pp.toString() + "." + ppK, newAlertId);
            newAlertId = "";
            // Set modified flag on the page
            top.gPageStack.Active().SetModified(true);
        }
    }
    else {
        var nAlerts = alertList.length;
        for (var i = 0; i < nAlerts; i++) {
            var aInfo = alertList[i];
            if (alertId == aInfo.id) {
                sel.selectedIndex = i;
                break;
            }
        }
    }

    var newSelIdx = sel.selectedIndex;
    if (newSelIdx != oldSelIdx)
        gPolicyAlerts.SelAlert(sel);
}

/**
 * Gets selection of the alert selector
 */
function alh_getSel(settingsType, actName, pb, pp) {
    var uid = settingsType + "_" + actName;
    var alertProps = pp_cse_actionAlertMap[uid];
    if (null == alertProps)
        return;
    var alertProp = alertProps[0];
    var alertCat = alertProps[1];
    alh_getSel2(uid, alertProp, alertCat, pb, pp);
}

function alh_getSel2(uid, alertProp, alertCat, pb, pp) {
    var form = document.forms[uid + "_fld_form"];
    if (!form)
        return;
    var sel = form.elements["alert_sel"];
    if (!sel)
        return;

    var selIdx = sel.selectedIndex;
    var alertList = gPolicyAlerts.getAlertList(alertCat);
    var alertId = alertList[selIdx].id;
    var ppK = "Alerts." + alertProp;
    pb.SetProperty(pp.toString() + "." + ppK, alertId);
}

function alh_getMailURLSel(actList, pb, pp)
{
	var oldAlertId = pb.GetPropertyDef(pp.toString() + ".Alerts.ThresholdAlert", "");
	var newAlertId = "";
	var newSelIdx = "";
	var alertChanged = false;
	for (var i = 0; i < actList.length; i++) 
	{
		var uid = settingsType + "_" + actList[i];
		var alertProps = pp_cse_actionAlertMap[uid];
		if (null == alertProps)
			return;
		var alertProp = alertProps[0];
		var alertCat = alertProps[1];
		if( alertProp == "ThresholdAlert" )
		{
			var form = document.forms[uid + "_fld_form"];
			if (!form)
				return;
			var sel = form.elements["alert_sel"];
			if (!sel)
				return;

			var selIdx = sel.selectedIndex;
			var alertList = gPolicyAlerts.getAlertList(alertCat);
			var alertId = alertList[selIdx].id;
			if( alertId != oldAlertId )
			{
				newAlertId = alertId;
				newSelIdx = selIdx;
				alertChanged = true;
			}
		}
		else
		{
			alh_getSel2(uid, alertProp, alertCat, pb, pp)
		}
	}
	if( alertChanged )
	{	
		var formHTA = document.forms["MAILURL_HighThresholdAction_fld_form"];
		var formLTA = document.forms["MAILURL_LowThresholdAction_fld_form"];
		if( !formHTA )
			return;
		var selHTA = formHTA.elements["alert_sel"];
		if (!selHTA)
			return;
		selHTA.selectedIndex = newSelIdx;
		formHTA.id.value = newAlertId;
		
		if( !formLTA )
			return;
		var selLTA = formLTA.elements["alert_sel"];
		if (!selLTA)
			return;
		selLTA.selectedIndex = newSelIdx;
		formLTA.id.value = newAlertId;
		pb.SetProperty(pp.toString() + ".Alerts.ThresholdAlert", newAlertId);
	}
}
