/**
 * JavaScript for the Alerts management panel in shared resources
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var a_pb = null;
var aListWidget = null;

// The resource management driver class for alerts
function AlertsDriver() {
}

// Gets the header text: "Alerts"
AlertsDriver.prototype.getHeader = function() {
    return resolveString(cse_pp_st, "__alerts");
}

// Gets the select directive: "Category:"
AlertsDriver.prototype.getSelDirective = function() {
    return resolveString(cse_pp_st, "__cat_sel_directive");
}

// Gets category list for displaying in a dropdown list
AlertsDriver.prototype.getCategoryList = function() {
    var ret = [];

    var catList = alerts_getOrderedTypes();
    for (var i = 0; i < catList.length; i++) {
        ret.push({value:catList[i], text:resolveString(cse_pp_alerts_st, catList[i])});
    }

    return ret;
}

// Creates a custom alert
AlertsDriver.prototype.newInstance = function(cat) {
    CustomAlert(cat);
}

// Lists alerts in a given category
AlertsDriver.prototype.drawList = function(category, tbody, listWidget) {
    if (null == category) {
        var row = tbody.insertRow(i);
        var td = row.insertCell(cellIdx++);
        td.colSpan = 3;
        td.style.textAlign = "center";
        td.appendChild(document.createTextNode(resolveString(cse_pp_st, "__nothing_to_display")));
        return;
    }
    var links = alerts_getListInfo(category);
    for (var i = 0; i < links.length; i++) {
        var link = links[i];
        var row = tbody.insertRow(i);
        var cellIdx = 0;
        var td = row.insertCell(cellIdx++);
        td.appendChild(document.createTextNode(link.text));

        td = row.insertCell(cellIdx++);
        //td.style.textAlign = "center";
        var usedByStr = (1 == link.refCount) ? resolveString(cse_pp_st, "__one_policy") :
            resolveString(cse_pp_st, "__n_policies", [link.refCount]);
        td.appendChild(document.createTextNode(usedByStr));

        td = row.insertCell(cellIdx++);
        td.appendChild(SharedResourceManager.createLink(link.viewLink, resolveString(cse_pp_st, "__view")));
        if (!link.isDefault) {
            td.appendChild(document.createTextNode(" | "));
            td.appendChild(SharedResourceManager.createLink(link.editLink, resolveString(cse_pp_st, "__edit")));
            if (0 == link.refCount) {
                td.appendChild(document.createTextNode(" | "));
                td.appendChild(SharedResourceManager.createLink(link.delLink, resolveString(cse_pp_st, "__delete")));
            }
        }
    }

    if (listWidget)
        listWidget.setHeaderText(0, resolveString(cse_pp_alerts_st, category));
}

function DeleteAlert(pp, type) {
    if (confirm(resolveString(cse_pp_st, "__del_alert_prompt"))) {
        a_pb.DeleteElement(pp);
        top.SetModified(true);
        aListWidget.drawInner(type);
    }
}

function LoadAlert(url, title, pp) {
    var page = new DialogPage(url, title);
    top.gPageStack.Forward(page, pp);
}

function CustomAlert(type) {
    var basePP = "Alerts.Alert";
    // Get the next enumeration
    var newPP = a_pb.GetNextElementEnum(basePP);
    var target = uriAppend("EditAlert.html", "type", type);
    var title = resolveString(cse_pp_alerts_st, type);
    top.gPageStack.Forward(new DialogPage(target, title), newPP);
}

function alerts_Show(pbIn) {
    a_pb = pbIn;

    if (top.polctx && top.polctx.newAlert) {
        // We don't care about this item because we'll pick up
        // the alert by reading the property bag
        delete top.polctx.newAlert;
    }

    aListWidget = new SharedResourceManager("aListsDiv", "aListWidget", new AlertsDriver(), {verticalSpace:"10px"});
    aListWidget.draw();
}

/**
 * Gets an ordered list of alert types to display; the order is
 * obtained from a variable in PointProductSpecific.js called
 * pp_cse_alertsOrder
 */
function alerts_getOrderedTypes() {
    var types = alerts_getAvailableTypes();
    if (this.pp_cse_alertsOrder && types.length > 0) {
        // Display listings in preferred order
        var ordered = [];
        for (var i = 0; i < pp_cse_alertsOrder.length; i++) {
            var type = pp_cse_alertsOrder[i];
            for (var j = 0; j < types.length; j++) {
                if (type == types[j]) {
                    types.splice(j, 1);
                    ordered.push(type);
                    break;
                }
            }
        }
        // Add any remaining types onto the ordered list
        for (var i = 0; i < types.length; i++) {
            ordered.push(types[i]);
        }
        types = ordered;
    }
    return types;
}

function alerts_getListInfo(type) {
    var editPage = uriAppend("EditAlert.html", "type", type);
    var viewPage = uriAppend(editPage, "readOnly", "1");
    // Start with an entry for the default alert
    var defViewLink = "javascript:LoadAlert('" + uriAppend(viewPage, "def", "1") +
        "','" + resolveString(cse_pp_alerts_st, type) + "','');";
    var insStr = resolveString(cse_pp_alerts_st, type + "_S");
	var countOfDefaultAlerts = RefCounter.getRefCount(type, "", null,
        RefCounter.getAlertRefCount, RefCounter.alertTypeToSettingsType);
    var list = [{"text":resolveString(cse_pp_st, "__defalert", [insStr]), "isDefault":1,
        viewLink:defViewLink, refCount:countOfDefaultAlerts.refCount}];
    var pp = new PropertyPath("Alerts.Alert");

    for (pp.BeginEnum(); a_pb.GetProperty(pp) !== undefined; pp.IncrementEnum()) {
        pp.SetTempLeaf("type");
        if (type == a_pb.GetProperty(pp))
        {
            pp.SetTempLeaf("id");
            var id = a_pb.GetProperty(pp);
            pp.SetTempLeaf("name");
            var text = a_pb.GetProperty(pp);
            if (!text) {
                text = id;
            }
            pp.SetTempLeaf("");
            var viewLink = "javascript:LoadAlert('" + viewPage +
                "','" + resolveString(cse_pp_alerts_st, type) + "','" + pp.GetPath() + "');";
            var editLink = "javascript:LoadAlert('" + editPage +
                "','" + resolveString(cse_pp_alerts_st, type) + "','" + pp.GetPath() + "');";
            var delLink = "javascript:DeleteAlert('" + pp.GetPath() + "', '" + type + "');";
            var refCounter = RefCounter.getRefCount(type, id, null,
                RefCounter.getAlertRefCount, RefCounter.alertTypeToSettingsType);
            list.push({"viewLink":viewLink, "text":text, "refCount":refCounter.refCount,
                "isDefault":0, "editLink":editLink, "delLink":delLink});
        }
        else {
            pp.SetTempLeaf("");
        }
    }

    return list;
}

/**
 * Gets a list of available alert categories
 */
function alerts_getAvailableTypes() {
    var rm = new ResourceManager("DefaultAlerts");
    var ret = [];
    var types = rm.getAlertTypes();
    // There is a list of excluded alert types defined in
    // PointProductSpecific.js called pp_cse_alertExclusions
    if (this.pp_cse_alertExclusions && types.length > 0) {
        // Index the excluded types
        var tmp = [];
        var excludedTypes = {};
        for (var i = pp_cse_alertExclusions.length - 1; i >= 0 ; i--) {
            excludedTypes[pp_cse_alertExclusions[i]] = i;
        }
        for (var i = 0; i < types.length; i++) {
            var type = types[i];
            if (null == excludedTypes[type])
                tmp.push(type);
        }
        types = tmp;
    }

    // Some alerts belong to optional software components; do not display them
    // if the corresponding component is not installed / activated
    for (var i = types.length - 1; i >= 0; i--) {
        var alertType = types[i];
        var sType = pp_cse_alertTypeMap[alertType];
        if (pp_componentInstalled(sType)) {
            ret.push(alertType);
        }
    }

    return ret;
}
