// Copyright (C) 2009 McAfee, Inc.  All rights reserved.
//
// Manage reading and writing content rules
//
var gContentRules;
var K_PPBase="RuleSet";

//to be obsoleted
var CSRules_PhraseTypes =[ ["Match", "MatchPattern"], ["NotNear", "NearAll", "AndRule", "NearAny"] ];

var CSRules_PhraseStruct =[ {node:"Match",       path:"",      logic:"",               dist:false, wildcard:false},
                           {node:"MatchPattern", path:"",      logic:"" ,              dist:false, wildcard:true},
                           {node:"NearAll",      path:"List",  logic:"ALL",            dist:true,  wildcard:false},
                           {node:"NearAny",      path:"List",  logic:"ANY",            dist:true,  wildcard:false},
                           {node:"NotNear",      path:"List",  logic:"NONE",           dist:true,  wildcard:false},
                           {node:"AndRule",      path:"",                logic:"ALL",  dist:false, wildcard:false},
                           {node:"AndRule",      path:"OrRule",          logic:"ANY",  dist:false, wildcard:false},
                           {node:"AndRule",      path:"NotRule.OrRule",  logic:"NONE", dist:false, wildcard:false}]


function  CSRules_GetTopPb()
{
	return top.getCustomPB("CSRules");
}

//Function creating a writable property bag with rules in.
function  CSRules_MakePb(pbIn, path)
{
	 var pbOut = new PropertyBag();
	 var pp_str = "RuleSet";
	 if(path) pp_str = path;

     if (undefined == pbIn || undefined == pbIn.GetProperty(pp_str))
     {
         var pbT=CSRules_GetTopPb();
         pbT.GetElement(pp_str, pbOut);
     }
     else
     {
        pbIn.GetElement(pp_str, pbOut);
     }

	 return pbOut;
}


function  CSRules_IsStorePb(ndx)
{

	if(undefined == top.polctx)             return null
	if(undefined == top.polctx.cspb)        return null
    if(undefined == top.polctx.cspb[ndx])   return null

	if(top.polctx.cspb[ndx] == CSRules_GetTopPb())
	{   return null;
	}

	return top.polctx.cspb[ndx];
}

function  CSRules_SetStorePb(pbIn,ndx)
{
	if(undefined == top.polctx)  top.polctx = new Object();
    if(undefined == top.polctx.cspb)  top.polctx.cspb = new Object();

	top.polctx.cspb[ndx]=pbIn;

}

//returns the reactivated locally stored pb if any, else get the top pb
//reinit store place.
//
function  CSRules_GetStorePb(ndx, clear, path)
{

	if(undefined == top.polctx)
	{   return CSRules_GetTopPb();
	}
	if(undefined == top.polctx.cspb)
	{   return CSRules_GetTopPb();
	}
    if(undefined == top.polctx.cspb[ndx])
	{   return CSRules_GetTopPb();
	}

   // Now do a deep copy from stored pb
	var pbOut= new PropertyBag();
	var pbIn = top.polctx.cspb[ndx];

    for (var prop in pbIn.propValues) {
            pbOut.propValues[prop] = pbIn.propValues[prop];
    }

    var pp_str = "RuleSet";
	if(path) pp_str = path;
	if (undefined == pbOut.GetProperty(pp_str))
	{   return CSRules_GetTopPb();
	}


	if(clear)top.polctx.cspb[ndx]=null;
	else top.polctx.cspb[ndx]=pbOut;

	return pbOut;
}



//extract data from local property bag if they are there,
//or use global otherwise
//members:
//rules_pb = property bag that includes rules
//rulesTable = structure storing the rules registered in policy
//rulesList = structure storing the complete list of rules and rules groups

function CSRules()
{
   this.rules_pb= CSRules_GetTopPb();
   this.rulesTable=[];
   this.rules;
   this.readonly=true;
}


CSRules.prototype.InitPb = function(pbIn)
{   this.rules_pb=CSRules_MakePb(pbIn);
	this.readonly=false;
}


//Retrieve the pp of a single rule
//gr_pp_str: path of the Rule group to examine
CSRules.prototype.GetPPFromRuleId = function(gr_pp_str, id)
{
    var loc_name="";
    var ppT =  new PropertyPath(gr_pp_str);
    ppT.Append("Rule");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
        var loc_id;
        ppT.SetTempLeaf("ID")
        loc_id=this.rules_pb.GetProperty(ppT);
		ppT.SetTempLeaf("");

        if (loc_id==id)
        {    return ppT.GetPath();
        }
        ppT.IncrementEnum();
    }

    ppT.EndEnum();
    return "";
}

//type : either "RuleGroup" or "Rule"
//id = rule or rule group
CSRules.prototype.GetPPFromId = function(type, id)
{
    var pp_str="";

    var ppT =  new PropertyPath(K_PPBase);
    ppT.Append("RuleGroup");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
	   pp_str=ppT.GetPath();

       if (type=="RefRuleGroup")
       {
           var loc_id;
           ppT.SetTempLeaf("ID")
           loc_id=this.rules_pb.GetProperty(ppT);
	       ppT.SetTempLeaf("");

           if (loc_id==id)
           {    return pp_str;
           }
       }
       else
       {   res=this.GetPPFromRuleId(pp_str, id)
	       if (res!="")
	       {   return res;
	       }
       }

       ppT.IncrementEnum();
    }
    ppT.EndEnum();
    return "";
}//fn

//create a new PP for the given rule group
CSRules.prototype.GetNextRulePP = function(gr_id)
{
	    var parent_pp=this.GetPPFromId("RefRuleGroup", gr_id);
		var pp_str = this.rules_pb.GetNextElementEnum(parent_pp+".Rule");
		return pp_str;
}


//Retrieve a the name of a single rule
//pp_str: path of the Rule group to examine
//id:id of the rule to look for
CSRules.prototype.GetNameFromRuleId = function(pp_str, id)
{
    var loc_name="";
    var ppT =  new PropertyPath(pp_str);
    ppT.Append("Rule");
    ppT.BeginEnum();

    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
        var loc_id;
        ppT.SetTempLeaf("ID")
        loc_id=this.rules_pb.GetProperty(ppT);
        ppT.SetTempLeaf("name");
        loc_name=this.rules_pb.GetProperty(ppT);

        if (loc_id==id)
        {    return loc_name;
        }
        ppT.IncrementEnum();
    }

    ppT.EndEnum();
    return "";
}

//Retrieve the property path of a rule or rule group



//Retrieve a the name of a rule group or a rule
//type : either "RuleGroup" or "Rule"
CSRules.prototype.GetNameFromId = function(type, id)
{
    var group_name="";
    var loc_name="";

    var ppT =  new PropertyPath(K_PPBase);
    ppT.Append("RuleGroup");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
       ppT.SetTempLeaf("name");
       group_name=this.rules_pb.GetProperty(ppT);


       if (type=="RefRuleGroup")
       {
           var loc_id;
           ppT.SetTempLeaf("ID")
           loc_id=this.rules_pb.GetProperty(ppT);

           if (loc_id==id)
           {
               return group_name;
           }
       }
       else
       {   ppT.SetTempLeaf("");
           loc_name=this.GetNameFromRuleId(ppT.GetPath(), id)
           if(loc_name!="")
           {   return group_name+ " - " + loc_name;
           }
       }
	   ppT.SetTempLeaf("");
       ppT.IncrementEnum();
    }
    ppT.EndEnum();
    return "";
}//fn


//Retrieve a the ID of a rule group or a rule
//type : either "RuleGroup" or "Rule"
CSRules.prototype.GetScoreFromId = function(type, id)
{
    var group_name="";
    var loc_name="";

    var ppT =  new PropertyPath(K_PPBase);
    ppT.Append("RuleGroup");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
       ppT.SetTempLeaf("recommended_threshold");
       group_name=this.rules_pb.GetProperty(ppT);


       if (type=="RefRuleGroup")
       {
           var loc_id;
           ppT.SetTempLeaf("ID")
           loc_id=this.rules_pb.GetProperty(ppT);

           if (loc_id==id)
           {
               return group_name;
           }
       }
       else
       {   ppT.SetTempLeaf("");
           loc_name=this.GetNameFromRuleId(ppT.GetPath(), id)
           if(loc_name!="")
           {   return group_name+ " - " + loc_name;
           }
       }
	   ppT.SetTempLeaf("");
       ppT.IncrementEnum();
    }
    ppT.EndEnum();
    return "";
}//fn

//Retrieve a the name of a rule group or a rule
//type : either "RuleGroup" or "Rule"
CSRules.prototype.GetScoreFromName = function(type, id)
{
    var group_name="";
    var loc_name="";
//alert(id)
    var ppT =  new PropertyPath(K_PPBase);
    ppT.Append("RuleGroup");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
       ppT.SetTempLeaf("recommended_threshold");
       group_name=this.rules_pb.GetProperty(ppT);


       if (type=="RefRuleGroup")
       {
           var loc_id;
           ppT.SetTempLeaf("name")
           loc_id=this.rules_pb.GetProperty(ppT);

           if (loc_id==id)
           {
               return group_name;
           }
       }
       else
       {   ppT.SetTempLeaf("");
           loc_name=this.GetNameFromRuleId(ppT.GetPath(), id)
           if(loc_name!="")
           {   return group_name+ " - " + loc_name;
           }
       }
	   ppT.SetTempLeaf("");
       ppT.IncrementEnum();
    }
    ppT.EndEnum();
    return "";
}//fn

//creates the list of rules as an array of rulesgroup
CSRules.prototype.BuildRulesList = function(pp_str)
{
    var r_name="";
    var r_id="";

    var r_lst= [];


    var ppT =  new PropertyPath(pp_str);
    ppT.Append("Rule");
    ppT.BeginEnum();

    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {
        var loc_id;
        ppT.SetTempLeaf("ID")
        r_id=this.rules_pb.GetProperty(ppT);
        ppT.SetTempLeaf("name");
        r_name=this.rules_pb.GetProperty(ppT);
		ppT.SetTempLeaf("enabled");
		var r_enabled ;
		if(this.rules_pb.GetProperty(ppT) == McUndefined)
		{
			
			r_enabled = 1;
			this.rules_pb.SetProperty(ppT, r_enabled);
			
		}
		else
		{
			
			r_enabled= this.rules_pb.GetProperty(ppT);
			this.rules_pb.SetProperty(ppT, r_enabled);


		}
		
		
        ppT.SetTempLeaf("");
		
        r_lst.push({name:r_name, id:r_id,enabled11:r_enabled});
        ppT.IncrementEnum();
    }

    ppT.EndEnum();
    return r_lst;
}

//creates the list of rules as an array of rulesgroup
CSRules.prototype.BuildGrpRulesList = function()
{
    var grp_name="";
    var grp_id="";
	var rul_type = false;
	var langID = "neutral"
	var TScore = "";

    this.rules= [];

    var ppT =  new PropertyPath(K_PPBase);
    ppT.Append("RuleGroup");
    ppT.BeginEnum();
    while(this.rules_pb.GetProperty(ppT) !== undefined)
    {

       ppT.SetTempLeaf("name");
       grp_name=this.rules_pb.GetProperty(ppT);
       ppT.SetTempLeaf("ID");
       grp_id=this.rules_pb.GetProperty(ppT);

	   ppT.SetTempLeaf("builtin");
       rul_type=this.rules_pb.GetProperty(ppT);

	   if(rul_type == McUndefined)
		rul_type = "1";

	   ppT.SetTempLeaf("lang");
       langID=this.rules_pb.GetProperty(ppT);

	   if(langID == McUndefined)
		langID = "neutral";

	   ppT.SetTempLeaf("recommended_threshold");
       TScore=this.rules_pb.GetProperty(ppT);

	   if(TScore == McUndefined)
		TScore = "";
	   

       ppT.SetTempLeaf("");
       lst=this.BuildRulesList(ppT.GetPath())
	
       this.rules.push({name:grp_name, id:grp_id,  lst:lst,  rultype:rul_type, lang:langID,score:TScore})

		
       ppT.IncrementEnum();
    }
    ppT.EndEnum();

}

//returns array of rules ids from a rule group
CSRules.prototype.GetGrpRules = function(ndx)
{
    var ret = [];
    for(var i=0; i<this.rules[ndx].lst.length;i++)
    {    ret.push(this.rules[ndx].lst[i].id)
	}

    return ret;
}

//create a new category
CSRules.prototype.CreateGrpRules = function( grp_name,id)
{
    var grp_id= MakeGuid("RGNAI")


    var pp_str=this.rules_pb.GetNextElementEnum(K_PPBase+"."+"RuleGroup")
    var ppT=new PropertyPath(pp_str);


    this.rules_pb.SetProperty(ppT);

	ppT.SetTempLeaf("ID")
    this.rules_pb.SetProperty(ppT, grp_id);

    ppT.SetTempLeaf("name")
    this.rules_pb.SetProperty(ppT, grp_name);

    ppT.SetTempLeaf("type")
    this.rules_pb.SetProperty(ppT, "ContentRule");

    ppT.SetTempLeaf("package")
    this.rules_pb.SetProperty(ppT, "__safe4_custom");
    top.gPageStack.Active().SetModified(true);

    this.rules.push({name:grp_name, id:grp_id,  lst:[]})
}

//rename rule group
CSRules.prototype.RenameGrpRules = function( grp_id, new_name)
{
    //change propertybag
    var pp_str=this.GetPPFromId("RefRuleGroup", grp_id)
    var ppT =  new PropertyPath(pp_str);
    if(ppT)
    {   ppT.SetTempLeaf("name")
        this.rules_pb.SetProperty(ppT, new_name);
    }
    else
    {    return;
    }

    //change list
    var lst = this.rules;
    for (var i = lst.length - 1; i >= 0; i--)
    {
        if (lst[i].id == grp_id)
        {
            lst[i].name=new_name;
            break;
        }
    }

}

//delete rule group then rebuild list.
CSRules.prototype.DeleteGrpRules = function( grp_id)
{
    var pp_str=this.GetPPFromId("RefRuleGroup", grp_id)
    var ppT =  new PropertyPath(pp_str);
    if(ppT)
    {   this.rules_pb.DeleteElementEx(ppT);
        this.BuildGrpRulesList();
    }

}

//Function returns boolean true if a phrase is simple
//(there is no logic)
function  CSRules_IsSimplePhrase(node)
{
  for(var i=0;i<CSRules_PhraseStruct.length;i++)
  {
	   if(node == CSRules_PhraseStruct[i].node && CSRules_PhraseStruct[i].logic=="")
       {  return true;
       }
  }
  return false;
}

//Function retrieving the name of a phrase
//pbIn: property bag that would include the rules
//pp_str: property path to the phrase node
//node: nde label=Match|MatchPattern|NearAll|AndRule|NotNear
//cut : boolean specifying if the phrase should be shortened
function  CSRules_GetPhraseTxt(pbIn, pp_str,  node, cut)
{
    var ppT=new PropertyPath(pp_str);
	var prop;

	if(CSRules_IsSimplePhrase(node))
    {   	ppT.SetTempLeaf("text");
    }
    else
	{   	ppT.SetTempLeaf("name");
	}

    var prop=pbIn.GetProperty(ppT)

	var phrase=prop;
    if(cut && prop.length>30)
    {
        phrase=prop.substr(0,25);
        phrase+='[...]'

    }
    return phrase;

}

//Retrieve "found in " infromation from the propertybag
//@pbIn = propertybag
//pp_str: pp to the condition node
function CSRules_FoundIn(pbIn, pp_str)
{
   var ppT= new PropertyPath(pp_str);
   ppT.Append("Applies")
   ppT.SetTempLeaf("section");
   var txt=resolveString(cse_pp_cs_st,"__anywere");

   var res=pbIn.GetProperty(ppT);
   //if(res!="All")

   return txt;
}

//look for all property path for simple or complex phrases
//eg, Rule.ConditionRule.CFRule.Match
//or  Rule.ConditionRule.CFRule.MatchPattern
//or  Rule.ConditionRule.CFRule.NearAll
//or  Rule.ConditionRule.CFRule.NotNear
//or  Rule.ConditionRule.CFRule.AndRule
//@pbIn : property bag that includes rules
//@ppIn: pp to either the condition node


//Build table of all phrases in this condition
function  CSRules_GetAllPhrasesPP(pbIn, pp_str, key)
{
   var node = "(\\w+)";
   if (key != undefined && key != "")
   {  node="("+key+")";
   }

   //find list of words
   var regexStr='^' + pp_str + "\.(CFRule|RERule)\."+node+"(:\\d{8})*$";
   var pattern=new RegExp(regexStr);
   var res=pbIn.GetMatchingPaths(pattern);

   return res;
}


// Sorting functions for phrases
function CSRules_CSPhrasesCompare(a, b) {
    return a.txt.localeCompare(b.txt);
}

//Build table of all phrases in this condition
function  CSRules_GetAllSortedPhrases(pbIn, pp_str, key)
{
   var res = CSRules_GetAllPhrasesPP(pbIn, pp_str, key);

   //sort array
   for (var i=0;i<res.length ;i++ )
   {
        res[i].txt= CSRules_GetPhraseTxt(pbIn, res[i][0], res[i][2], false)
   }
   res.sort(CSRules_CSPhrasesCompare);

   return res;
}


/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["CSRules", "DefaultAlerts"];
}
