/**
 * JavaScript for the generic settings view page
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

// Top level property nodes that need to be added to the page's property bag
var g_tlNodes = ["Settings", "Alerts"];
var dpsv_pp = "";
var gPagePb = null;

var qparams = queryParameters();
fixQParams(qparams, "type", "");
var settingsType = qparams["type"][0];

function dpsv_addTLNode(pbIn, nodeName) {
    if (null == pbIn.GetProperty(nodeName)) {
        top.gPB.GetElement(nodeName, pbIn);
    }
}

// Store local configuration temporarily
function dpsv_LocalStore() {
    PolSetting_StoreLocalCopy( true, null);

    var ppT = PolSetting_GetPPNode("");
    var pbT = new PropertyBag();
    gPagePb.GetElement(ppT, pbT);
    var settingsRef = dpsv_getCurrentSettingsRef();
    if (settingsRef)
        pbT.SetProperty(ppT + ".RefSettings", settingsRef);
    top.polctx.polSetting["current_policy_settings"] = pbT;

    gPolicyAlerts.Apply(pbT);

    // Save current alert selection
    top.polctx.polSetting["tmp_alerts"]=[];
    if (gPolicyAlerts) {
        gPolicyAlerts.LocalStore(top.polctx.polSetting["tmp_alerts"]);
    }
}

// Gets the currently selected settings ref
function dpsv_getCurrentSettingsRef() {
    var ret = null;
    var sel = document.getElementById("OPTIONS_SEL");
    if (null != sel && (sel.options.length - 1) != sel.selectedIndex) {
        ret = sel.value;
    }
    return ret;
}

// Restore local configuration if any
function dpsv_LocalRestore() {
    PolSetting_RestoreLocalCopy();

    // Restore local property bag
    var ppT = PolSetting_GetPPNode("");
    var pbT = PolSetting_GetPb();
    gPagePb = new PropertyBag();
    pbT.GetElement(ppT, gPagePb);

    var ppStr = ppT.toString();
    var polSettingKey = "current_policy_settings";
    if (null != top.polctx.polSetting[polSettingKey]) {
        var pbT = PropertyBag.Recreate(top.polctx.polSetting[polSettingKey]);
        copyCurrentPolicySettings(pbT, gPagePb, ppStr);
        top.polctx.polSetting[polSettingKey] = null;
    }
}

/**
 * Edits the alert
 * @param alertList the list of globally defined alert objects for this page
 * @param alertId an ID for the alert being edited
 * @param create true if a new alert is being created
 */
function dpsv_EditAlert(alertList, alertId, create) {
    dpsv_LocalStore();
    if(alertList) {
         alertList.EditAlert(alertId, create);
    }
}

/**
 * Shows the options for a given settings type
 */
function dpsv_ShowOptions() {
    var settingsViewer = SettingsViewerFactory.getViewer(settingsType);
    if (!settingsViewer.hasOptions()) {
        // Hide the option selector
        var e = document.getElementById("OPTIONS_AREA");
        if (e)
            e.style.display = "none";
        return false;
    }

    if (!document.getElementById("OPTIONS_SEL")) {
        PolSetting_DrawOptionsCombo(settingsType, "OPTIONSLST_DIV");
        if (document.getElementById("OPTIONS_SEL")) {
            document.getElementById("OPTIONS_SEL").value =
                gPagePb.GetProperty(PolSetting_GetPPStr() + ".RefSettings");
        }
    }

    var table = dpsv_MakeSettingsViewer(settingsViewer.getPreferredHeight());
    // Get the options data from the settingsViewer object
    var optionList = settingsViewer.getOptions();
    var nOptions = optionList.length;

    // Scan through the option list looking for the data row with the most
    // elements
    var maxElems = 0;
    for (var i = 0; i < nOptions; i++) {
        if (optionList[i]) {
            var n = optionList[i].length;
            if (n > maxElems)
                maxElems = n;
        }
    }
    for (var i = 0; i < nOptions; i++) {
        dpsv_DrawRow(table, optionList[i], i - 1, maxElems);
    }
    dpsv_FixOptionsTable(table);
    return true;
}

/**
 * Creates the settings viewer table
 * @param height the height of the div in which to view the settings
 * @return a table object to populate with content
 */
function dpsv_MakeSettingsViewer(height) {
    if (null == height)
        height = "78px";

    var td = document.getElementById("OPTIONS_CONTAINER");
    if (!td)
        return null;
    while(td.firstChild)
        td.removeChild(td.firstChild);
    var div = document.createElement("div");
    if (height) {
        div.style.height = height;
    }
    div.className = "optionsTable optContent";
    td.appendChild(div);

    var table = document.createElement("table");
    table.border = table.cellSpacing = 0;
    table.cellPadding = 5;
    div.appendChild(table);
    return table;
}

/**
 * Fixes styles on the settings table
 */
function dpsv_FixOptionsTable(table) {
    if (null == table || null == table.rows)
        return;
    table.width = "100%";
    var rows = table.rows;
    var nRows = rows.length;
    for (var i = 0; i < nRows; i++) {
        var cells = rows[i].cells;
        var nCells = cells.length;
        for (var j = 0; j < nCells; j++) {
            var cell = cells[j];
            cell.className = addClass(cell.className, "optItem");
            if (0 == i) {
                cell.className = addClass(cell.className, "optTop");
            }
        }
        cells[nCells - 1].className = addClass(cells[nCells - 1].className, "eor");
    }
}

/**
 * Sets the edit options link
 */
function dpsv_SetEditLink() {
    var link = document.getElementById("OPTIONS_LNK");
    if (!link)
        return;
    var fnStr = "javascript:dpsv_EditOptions();";
    link.href = fnStr;
}

/**
 * Edits options
 */
function dpsv_EditOptions() {
    dpsv_LocalStore();
    PolSetting_EditOptions(settingsType);
}

/**
 * Fixes the onchange function for the configuration selector
 */
function dpsv_FixOptOnChange() {
    var sel = document.getElementById("OPTIONS_SEL");
    if (!sel) {
        return;
    }
    sel.onchange = new Function("dpsv_SelectFromOptions(this);");
}

/**
 * Shows the actions for a given settings type
 * @param globalProperties the global property bag
 * @param inherited whether settings are inherited from a parent property
 */
function dpsv_ShowActions(globalProperties, inherited) {
    var settingsViewer = SettingsViewerFactory.getViewer(settingsType);
    if (!settingsViewer.hasActions()) {
        return;
    }
    var div = document.getElementById("act_div");
    if (!div)
        return;
    while(div.firstChild)
        div.removeChild(div.firstChild);
    var table = dpsv_CreateActionTable(settingsType, settingsViewer, inherited);
    // Get the action data from the settingsViewer object
    var actionList = settingsViewer.getActions(globalProperties);
    var nActions = actionList.length;
    // Scan through the action list looking for the data row with the most
    // elements
    var maxElems = 0;
    for (var i = 0; i < nActions; i++) {
        if (actionList[i]) {
            var n = actionList[i].length;
            if (n > maxElems)
                maxElems = n;
        }
    }
    for (var i = 0; i < nActions; i++) {
        dpsv_DrawRow(table, actionList[i], i, maxElems);
    }
    dpsv_FixActionsTable(table);
    div.appendChild(table);
}

/**
 * Creates an action table
 */
function dpsv_CreateActionTable(settingsType, settingsViewer, inherited) {
    var table = document.createElement("table");
    table.border = table.cellPadding = table.cellSpacing = 0;
    if (settingsViewer.hasOptions() || settingsViewer.hasAlertSettings())
        table.style.marginTop = "10px";
    var cellIdx = 0;

    var row = table.insertRow(0);
    var cell = row.insertCell(cellIdx++);
    cell.noWrap = true;
    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    cell.appendChild(span);
    var stKey = "ACTTITLE_" + settingsType;
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, stKey)));

    cell = row.insertCell(cellIdx++);
    cell.style.textAlign = "right";
    if (inherited)
        cell.appendChild(document.createTextNode(" "));
    var a = document.createElement("a");
    a.className = "blueLink";
    var generic = (pp_cse_genActionMap[settingsType]) ? true : false;
    a.href = "javascript:dpsv_EditActions(" + generic + ");";
    a.appendChild(document.createTextNode(resolveString(cse_pp_st, "__edit")));
    a.style.display = inherited ? "none" : "inline";
    cell.appendChild(a);
    return table;
}

/**
 * Edits actions
 * @param settingsType the settings type of the object being viewed
 * @param generic whether the generic actions page should be used
 */
function dpsv_EditActions(generic) {
    dpsv_LocalStore();
    PolSettings_Actions(settingsType, generic);
}

/**
 * Creates a row in an options or actions table
 */
function dpsv_DrawRow(table, rowData, idx, maxElems) {
    var row = table.insertRow(idx + 1);
    var cellIdx = 0;
    var nElems = rowData.length;
    // if the elements in this row is not equal to the maxmimum number of elements,
    // the final cell needs to span multiple columns
    var colSpan = maxElems - nElems + 1;
    for (var i = 0; i < nElems; i++) {
        var cell = row.insertCell(cellIdx++);
        var data = rowData[i];
        if ("string" == typeof(data)) {
            cell.appendChild(document.createTextNode(data));
        }
        else {
            try {
                cell.appendChild(data);
            }
            catch(e){}
        }
    }
    if (colSpan > 1) {
        row.cells[nElems - 1].colSpan = colSpan;
    }
}

/**
 * Fixes styles on the action table
 */
function dpsv_FixActionsTable(table) {
    table.className = "dataSubTable";
    table.width = "100%";
    var rows = table.rows;
    var nRows = rows.length;
    if (nRows) {
        rows[0].className = addClass(rows[0].className, "header");
        var cells = rows[0].cells;
        if (cells.length > 1) {
            var cell = cells[cells.length - 1];
            cell.className = addClass(cell.className, "noLeft");
        }
        for (var i = 0; i < nRows; i++) {
            rows[i].className = addClass(rows[i].className, "row1");
            var cells = rows[i].cells;
            var nCells = cells.length;
            for (var j = 0; j < nCells; j++) {
                cells[j].className = addClass(cells[j].className, "dst");
            }
            var cell = cells[nCells - 1];
            cell.className = addClass(cell.className, "eor");
        }
        var row = rows[nRows - 1];
        row.className = addClass(row.className, "lastRow");
    }
}

/**
 * Shows the alert settings for this instance
 */
function dpsv_ShowAlerts() {
    var settingsViewer = SettingsViewerFactory.getViewer(settingsType);
    if (!settingsViewer.hasAlertSettings()) {
        return;
    }

    var alertData = settingsViewer.getAlertList();
    var nAlerts = alertData.length;
    if (0 == nAlerts)
        return;
    if (!document.getElementById(alertData[0].prop + "_alert_settings")) {
        var div = document.getElementById("alert_div");
        if (!div)
            return;
        while(div.firstChild)
            div.removeChild(div.firstChild);

        var table = dpsv_createAlertsTable(settingsViewer);
        div.appendChild(table);
        var rowIdx = 1;

        for (var i = 0; i < alertData.length; i++) {
            var cellIdx = 0;
            var row = table.insertRow(rowIdx++);
            var cell = row.insertCell(cellIdx++);
            var alertInfo = alertData[i];
            var uid = alertInfo.prop;
            cell.id = uid + "_alert_settings";

            alh_makeAlertSelector(uid, alertInfo.prop,
                alertInfo.cat, uid + "_alert_settings", null, null, null, "dpsv_EditAlert");
        }

        dpsv_FixActionsTable(table);
    }
}

/**
 * Edits the alert
 * @param alertList the list of globally defined alert objects for this page
 * @param alertId an ID for the alert being edited
 * @param create true if a new alert is being created
 */
function dpsv_EditAlert(alertList, alertId, create) {
    dpsv_LocalStore();
    if (alertList)
        alertList.EditAlert(alertId, create);
}

/**
 * Creates alerts settings table
 */
function dpsv_createAlertsTable(settingsViewer) {
    var table = document.createElement("table");
    table.border = table.cellPadding = table.cellSpacing = 0;
    if (settingsViewer.hasOptions())
        table.style.marginTop = "10px";
    var cellIdx = 0;

    var row = table.insertRow(0);
    var cell = row.insertCell(cellIdx++);
    cell.noWrap = true;
    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    cell.appendChild(span);
    span.appendChild(document.createTextNode(resolveString(cse_pp_st, "__alert_selection")));
    return table;
}


/**
 * Option select callback
 */
function dpsv_SelectFromOptions(c) {
    dpsv_LocalStore();
    if (PolSetting_OnOptionsSel(c, settingsType)) {
        dpsv_ShowOptions();
    }
}

/**
 * Inherit checkbox callback
 */
function OnclickInherit(val) {
    PolSetting_SetModify();
    Reset();
}

/**
 * Show callback
 */
function Show(pbIn, pp) {
    dpsv_SetEditLink();
    PolSetting_Show(pbIn);
    dpsv_ShowAlerts();
    dpsv_LocalRestore();
    dpsv_Draw(pbIn);
    dpsv_FixOptOnChange();
}

function Reset() {
    var inherit = PolSetting_CheckInherit();
    // Reset the local property bag
    var pbT = PolSetting_GetPb();
    var pp_str = PolSetting_GetPPStr();
    gPagePb = new PropertyBag();
    pbT.GetElement(pp_str, gPagePb);

    // Set the correct selection in the options dropdown
    if (document.getElementById("OPTIONS_SEL")) {
        document.getElementById("OPTIONS_SEL").value =
            gPagePb.GetProperty(pp_str + ".RefSettings");
    }

    dpsv_ShowOptions();
    dpsv_ShowActions(gPagePb, inherit);
    gPolicyAlerts.InitAll(gPolicyAlerts.alert_pb, gPagePb, pp_str);
    gPolicyAlerts.ShowAll();
    gPolicyAlerts.DisableAlertSel(inherit);
}

function dpsv_Draw(pbIn) {
    var pp_str = PolSetting_GetPPStr();

    // Draw alerts - reinit if new alert was created
    if (gPolicyAlerts) {
        gPolicyAlerts.InitAll(pbIn, gPagePb, pp_str);
        gPolicyAlerts.ShowAll();
        if (top.polctx.polSetting["tmp_alerts"])
        {
            gPolicyAlerts.LocalRestore(top.polctx.polSetting["tmp_alerts"]);
            top.polctx.polSetting["tmp_alerts"] = null;
        }
    }
    dpsv_ShowOptions();
    var inherit = PolSetting_CheckInherit();
    dpsv_ShowActions(gPagePb, inherit);
    gPolicyAlerts.DisableAlertSel(inherit);
}

/**
 * Cancel callback
 */
function Cancel() {
    PolSetting_Draw();
    Reset();
    top.gPageStack.Active().SetModified(false);
}

/**
 * Validate callback
 */
function Validate() {
    return true;
}

/**
 * Save callback
 */
function Apply(pb, pp, save) {
    var pbT = PolSetting_GetPb();
    var ppT = PolSetting_GetPPNode("");

    // If this is a new policy settings instance, gPagePb may have some settings we need
    if (!PolSetting_CheckInherit()) {
        copyCurrentPolicySettings(gPagePb, pbT, ppT);
    }

    // Ensure that the Alerts node is actually created if required
    var settingsViewer = SettingsViewerFactory.getViewer(settingsType);
    var ppN = ppT + ".Alerts";
    if (settingsViewer.hasAlertSettings() && null == pbT.GetProperty(ppN)) {
        pbT.SetProperty(ppN);
        var alertList = settingsViewer.getAlertList();
        for (var i = 0; i < alertList.length; i++) {
            pbT.SetProperty(ppN + "." + alertList[i], "");
        }
    }

    // Save alert id and other polsetting data (e.g. inherit, time)
    gPolicyAlerts.Apply(pbT);

    // Now apply everything for good and refresh the page
    var pbFinal = PolSetting_Apply(settingsType, true);

    // Refresh the page by reloading it.
    top.gPageStack.Active().SetModified(false);

    if(!gIsEPO4)
    {
		 top.SetModified(true);
		 var sel=document.getElementById("SETTING_SEL");
		 if(sel)
			 PolSetting_SelectFromCombo(sel)
	}

    return pbFinal;
}

// Copies action and alert nodes between property bags
function copyCurrentPolicySettings(pbSrc, pbDest, pp) {
    var ppN = pp + ".Alerts";
    pbSrc.GetElement(ppN, pbDest);
    ppN = pp + ".Actions";
    pbSrc.GetElement(ppN, pbDest);
    /*ppN = pp + ".RefSettings";
    if (null != pbSrc.GetProperty(ppN))
        pbDest.SetProperty(ppN, pbSrc.GetProperty(ppN));*/
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["DefaultAlerts"];
}
