/**
 * Disc Editor wrapper for the richtext control
 * Dependent on RichText.js
 * Disclaimer edit
 * Copyright (C) 2010 McAfee, Inc. All rights reserved
 */
var g_changeNotifyFn = new Function("top.gPageStack.Active().SetModified(true);return true;");


// Converts <#0A> sequences to \r\n
function fixNewLinesA(s) {
	if(gEPOMode)
		return s.replace(/~e/g, "\r\n");
	else
		return s.replace(/<#0A>/g, "\r\n");
}

/**
 * Data binding for disclaimer text
 */
function DiscTDataBinding(rtc) {
    this.form = discOptsForm;
	this.elemH = "DisclaimerHTML";
    this.elemP = "DisclaimerText";
    this.initialText = null;
	this.initialHTML = null;
	this.initialized = false;
	this.rtc = rtc;
}

//DiscTDataBinding.prototype = new CFDataBinding("", "", "", null, fixNewLines);

/**
 * Sets the field value from a PropertyBag object
 * @param pp the base property path to use
 * @param pbIn the property bag to use
 */
DiscTDataBinding.prototype.set = function(pp, pbIn) {

	pp.SetTempLeaf(this.elemP);
	var txt = pbIn.GetPropertyDef(pp, "");
	this.initialText = fixNewLinesA(txt);
	pp.SetTempLeaf("");
	
	pp.SetTempLeaf(this.elemH);
	this.initialHTML = pbIn.GetPropertyDef(pp, "");
	pp.SetTempLeaf("");

	if(this.initialized)
	{
		this.rtc.sync_r(this.initialHTML);
		this.rtc.setText(this.initialText, false);
	}
	else
	{
		this.rtc.makeControl(discContainer, this.initialHTML, this.initialText, true);
		this.initialized = true;
	}
}

DiscTDataBinding.prototype.get = function(pp, pbOut) {

	var html = this.rtc.getValue();
	var txt = this.rtc.getText();
	
	if(document.getElementById("radio_0").checked) // If HTML radio button selected then make Plain text content otherwise if it contains any script non-allowed chars it may corrupt McAfeeConfig.xml
		txt = "";
		
	txt = txt.replace(/\r\n/g,"~e");
	
	pp.SetTempLeaf(this.elemH);
	pbOut.SetProperty(pp, html );
	pp.SetTempLeaf("");
	
	pp.SetTempLeaf(this.elemP);
	pbOut.SetProperty(pp, txt);
	pp.SetTempLeaf("");
	
	if( html != this.initialHTML || txt != this.initialText   )
		return true;

	return false;
}

/**
 * Constructor for the DiscEditor
 * @param name the name of the control; used for creating element IDs
 * @param ctlName the name by which the control has been declared in the HTML
 * @param rtcProps properties for the underlying rich text control; see
 * RichText.js for details of the permitted properties; note that only height
 * width and readOnly will not be overwritten
 */
function DiscEditor(name, ctlName, rtcProps) {
    // Initialize the rich-text library
    initRTE("../../images/rte/", "", "");
    // Fix up the properties object so that it is in a known state
    if (null == rtcProps)
        rtcProps = {};
    if (null == rtcProps["height"])
        rtcProps["height"] = 275;
    rtcProps["richText"] = true;
    rtcProps["tokens"] = false;
    rtcProps["plainTextOption"] = true;
    rtcProps["htmlControl"] = "htmlText";
    rtcProps["textControl"] = "text";
    rtcProps["changeNotifyFn"] = g_changeNotifyFn;
    initControl(rtcProps, this);
    this.headerText = "";
    this.name = name;
    this.ctlName = (null != ctlName) ? ctlName : "discEditor";
    this.html = "";
    this.text = "";
    this.created = false;
    this.mode = WYSIWYG_MODE;
    this.initialContent = "";
    this.initialText = "";

    // Create a resource manager object to handle resources
    //this.rm = new ResourceManager("DefaultAlerts");
}

DiscEditor.prototype = new RTControl("", "");
DiscEditor.prototype.makeControlA = DiscEditor.prototype.makeControl;

DiscEditor.prototype.makeControl = function(container, html, text, clearParent) {
    this.headerText = resolveString(cse_pp_st, "__disclaimer_header" );
    this.makeControlA(container, html, text, clearParent);
}
