/**
 * JavaScript for the EditAlert page
 *
 * Copyright (C) 2016 McAfee, Inc. All rights reserved
 */

var qparams = queryParameters();
// Fix up the query parameters to ensure that all expected parameters are present
fixQParams(qparams, "readOnly", "0");
fixQParams(qparams, "pt", "0");
fixQParams(qparams, "def", "0");
fixQParams(qparams, "id", "");
fixQParams(qparams, "type", "");

function ea_isReadOnly() {
    return "1" == qparams["readOnly"][0];
}

function ea_isDefault() {
    return "1" == qparams["def"][0];
}

function ea_getId() {
    return qparams["id"][0];
}

function ea_getType() {
    return qparams["type"][0];
}

function ea_isPlainText() {
    return "1" == qparams["pt"][0];
}

var vframework = null;
var dataBindings = new Array();
var newInstance = false;
var newId = null;
var rm = null;
var basePP = "Alerts.Alert";
var alert_pb = null;
var origType = "";
var origName = "";
var alertId = ea_getId();
var alertForm = "_alert";
var alertContainer = "_ae";
var aeProps = {readOnly:ea_isReadOnly(), resetBtn:false, width:788};
var alertEditor = null;

/**
 * Fixes up the input property bag and ensures that it is in a correct state
 * @param pbIn the input property bag
 * @param pp the property path for the page
 * @return true if a new settings instance is created
 */
function ea_initPB(pbIn, pp) {
    var newInstance = false;
    if (!ea_isDefault()) {
        // Check that the alerts properties are loaded
        if (undefined === pbIn.GetProperty("Alerts")) {
            // Copy the Alerts node into the property bag that we've been passed
            top.gPB.GetElement("Alerts", pbIn);
        }
        if (null != pp) {
            // Check for presence of property path in property bag
            if (undefined === pbIn.GetProperty(pp)) {
                // New instance
                // Generate a random ID
                newId = MakeGuid("A");
                var ppT = PropertyPath.MakePP(pp);
                pbIn.SetProperty(ppT);
                ppT.SetTempLeaf("id");
                pbIn.SetProperty(ppT, newId);
                ppT.SetTempLeaf("type");
                pbIn.SetProperty(ppT, ea_getType());
                ppT.SetTempLeaf("");
                newInstance = true;
            }
        }
    }
    return newInstance;
}

// Initializes the page's forms
function ea_forms_init() {
    if (document.getElementById("ap_report_area")) {
        if (!newInstance && !ea_isDefault()) {
             var refCounter = RefCounter.getRefCount(ea_getType(), alertId, alert_pb,
                RefCounter.getAlertRefCount, RefCounter.alertTypeToSettingsType);
            if (0 != refCounter.getDependentPolicyCount())
                refCounter.briefReport("ap_report_area");
        }
    }

    rm = new ResourceManager("DefaultAlerts");
    var e = document.getElementById(alertContainer);
    if (e) {
        var width = e.offsetWidth ? e.offsetWidth : e.clientWidth;
        if (e.clientWidth) {
            // Reported width is two pixels too wide
            width -= 2;
        }
        aeProps.width = width;
    }

	// Disable the change notifications so that the 'Save' button will never be enabled
	if( ea_isDefault() || ea_isReadOnly() ) {
		g_changeNotifyFn = new Function("return true;");
	}

    alertEditor = new AlertEditor(ea_getType(),
        "ae1", "alertEditor", aeProps);
    if (document.forms[alertForm]) {
        vframework = new ValidationFramework(alertForm);
        vframework.formAction = std_val_form_action;
        vframework.fieldAction = std_val_field_action;
        vframework.labelMap["name"] =
            resolveString(page_st, "lm_alert_name");
        vframework.addValidator("name", new RequiredValidator());
        //vframework.addValidator("name", new ScriptValidator());
		vframework.addValidator("name", new InputTextValidator());

        var inputArray= new Array();
		inputArray["SECTION"]="Alerts";
		inputArray["origName"]=origName;
        inputArray["basePP"]="Alerts.Alert";
        inputArray["origType"]=origType;
        vframework.addValidator("name",new DuplicatesValidator(alert_pb,inputArray));
        vframework.addValidator("name",new LengthValidator());

        // If we're read only or showing the default, the name
        // field needs to be read-only
        if (ea_isReadOnly()) {
            var e = document.forms[alertForm].elements["name"];
            if (e)
                e.disabled = true;
            e = document.getElementById("ea_reset");
            if (e)
                e.style.visibility = "hidden";
        }

        // Create the data binding for the alert editor
        var aeDB = alertEditor.makeDataBinding(alertContainer, ea_isPlainText());
        dataBindings.push(aeDB);
        dataBindings["alertEditor"] = aeDB;
    }
    addChangeNotification(g_changeNotifyFn);
}

// Sets form data
function ea_init() {
    ea_forms_init();
    Reset();
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var alert_pp = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(alert_pp, alert_pb);
    }
    // Show the token list
    alertEditor.showTokens();
    if (ea_isDefault()) {
        var e = document.forms[alertForm].elements["name"];
        if (e)
            e.value = resolveString(cse_pp_st, "__default");
    }
    top.gPageStack.Active().SetModified(false);
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    else {
        // We need to ensure that the PolicySet node is included to do
        // reference counting
        if (null == pbIn.GetProperty("Policies")) {
            top.gPB.GetElement("Policies", pbIn);
        }
    }
    newInstance = ea_initPB(pbIn, pp);
    alert_pb = pbIn;
    if (alertId) {
        // Search for the alert blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", alertId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
		// Assume that we've been set up with the correct property path
		// and property bag
		basePP = pp.GetPath();
		alertId = pbIn.GetPropertyDef(basePP + ".id", "");
		origType= pbIn.GetPropertyDef(basePP + ".type", "");     //we store the type of the alert we are about to edit
		origName= pbIn.GetPropertyDef(basePP + ".name", "");     //we store the name of the alert we are about to edit
		 }
    ea_init();
}

/**
 * Checks for the presence of "Script" keyword and HTML tags in the Alert Header and footer.
 */
function ValidateAlertText( dataBindings )
{
	for (var i = 0; i < dataBindings.length; i++)
	{
        var db = dataBindings[i];
		if( null != db.rtc )
			{
				//both html value ( one extra arguement) and text value are checked for the presence of script keyword.
				if(  ! ScriptInputValidation( db.rtc.getValue(), true , true) || ! ScriptInputValidation( db.rtc.getText(), true) )
					{
						return false ;
					}
					
				if(!ValidateLimitedHtmlInput(db.rtc.getValue()) || !ValidateLimitedHtmlInput(db.rtc.getText()) )
					{
							alert(gsHTMLTagsNotAllowed);
								return false;
					}
				if(!ValidateXMLInput(db.rtc.getValue()) || !ValidateXMLInput(db.rtc.getText()) )
					{
							alert(gsXMLCommentTagsNotAllowed);
								return false;
					}
			}
			
	}
	return true;
}

/**
 * Asynchronous data validation function
 */
function Validate() {
    if(  !ValidateAlertText(dataBindings) )
	{
		return false;
	}
	return vframework.onsubmit();
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb) {
    var changed = false;
    if (null == pb) {
        pb = alert_pb;
    }
    var ppT = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        if (dataBindings[i].get(ppT, pb)) {
            changed = true;
        }
    }
    if (changed && newInstance) {
        // Notify any interested caller about the new instance
        if (top.polctx) {
            top.polctx.newAlert = newId;
        }
    }
    top.gPageStack.Active().SetModified(changed);
    return changed;
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        alert_pb.DeleteElement(basePP);
    }
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["DefaultAlerts"];
}

