/**
 * Interface to the engine.xml configuration file
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

/**
 * Determines whether network test are enabled
 * @param def the value to return if the data cannot be found
 * @return true if spam network tests are enabled, false otherwise
 */
function engine_SpamNetworkTestsEnabled(def) {
    var ret = def;

    var pb = top.getCustomPB("Engine");
    if (pb.IsEmpty()) {
        return def;
    }

    var pp = new PropertyPath("EngineSet.Engine");
    for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
        var s = pp.GetPath();
        if ("SpamScanner" == pb.GetProperty(s + ".name") &&
            "SpamScanner" == pb.GetProperty(s + ".type")) {
            ret = ("1" == engine_getAttribute(pb, s, "EnableNetworkTests", def ? "1" : "0"));
            break;
        }
    }

    return ret;
}

/**
 * Gets an attribute from an engine.xml settings blob
 */
function engine_getAttribute(pb, pp, attrName, def) {
    var ret = def;
    var ppT = new PropertyPath(pp.toString() + ".Attr");
    for (ppT.BeginEnum(); null != pb.GetProperty(ppT); ppT.IncrementEnum()) {
        if (attrName == pb.GetProperty(ppT.toString() + ".name")) {
            ret = pb.GetProperty(ppT.toString() + ".value");
            break;
        }
    }
    return ret;
}

/**
 * Gets the maximum number of Bayesian databases supported by the engine
 * param def the value to return if the data cannot be found
 * @return the
 */
function SpamGetMaximumBayes(def) {
    var ret = def;

    var pb = top.getCustomPB("Engine");
    if (pb.IsEmpty()) {
        return def;
    }

    var pp = new PropertyPath("EngineSet.Engine");
    for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
        var s = pp.GetPath();
        if ("SpamScanner" == pb.GetProperty(s + ".name") &&
            "SpamScanner" == pb.GetProperty(s + ".type")) {
            ret = engine_getAttribute(pb, s, "MaxBayesDatabaseCount", def);
            break;
        }
    }

    return ret;
}

