//Copyright (C) 2009 McAfee, Inc.  All rights reserved.
/////////////////////////////////////////////////
//
// Maintains envelope rule, mapping to/from
// plain english and rendering in HTML for editing
//
/////////////////////////////////////////////////

/////////////////////////////////////////////////
//
// Constructor
//
function EnvelopeRule(natural,rule,values)
{
	// Member variables
	this.natural = natural
	this.rule = rule
	this.editedValue = -1

	// Member functions
	this.GetValue = GetValue
	this.StoreValue = StoreValue
	this.RenderDescription = RenderDescription
	this.Render = Render
	this.InitValues = InitValues
	this.RenderValue = RenderValue
	this.Defined = Defined
	this.FromLiteral = FromLiteral

	// Build default set of values and types
	this.InitValues()

	// Override with instantiated list if provided
	if (values != McUndefined)
		this.values =values
}

/////////////////////////////////////////////////////
//
// Reads rule definition from given property bag
//
function FromLiteral(literal, templates,templateNames)
{
	// Attempt to find matching template in given list
	for (var i = 0; i < templates.length; i++)
	{
		var values = new Array
		if (ExtractValues(templates[i],literal,values))
		{
			this.rule = templates[i]
			this.natural = templateNames[i]
			this.InitValues() // to get value types
			this.values = values
			return i
		}
	}
	return -1
}


/////////////////////////////////////////////////
//
// Renders value at given index so that it follows
// the syntax for a constant in an envelope rule
//
function RenderValue(iIndex)
{
	var value = this.values[iIndex]
	switch (typeof value)
	{
		case "boolean":
			return value ? "true" : "false"
		break

		case "number":
			return value

		default:
			var reg = /\^/g
			str = value .replace(reg,"^^")
			reg = /\"/g
			str = str.replace(reg,"^\"")
			return "\"" + str +"\""
		break

	}
}

/////////////////////////////////////////////////
//
// Get the value at the given index
//
function GetValue(iIndex)
{
	var val = ReadValue(this.values[iIndex],0,true)
	return val[1]
}

//////////////////////////////////////
//
// Returns (index of next token, type of next token, token number)
// as three value array.
//
// Input param "tokenIndex" is optional.
//
function FindNextToken(str, tokenIndex)
{
	var index = str.indexOf("%",index)
	while (index != -1)
	{

		// Check that next char is lower case alpha
		if (str.charAt(index+2) >= 'a' && str.charAt(index+2) <= 'z')
		{
			// Check that next char is numeric or equals requested token
			if (str.charAt(index+1) > '0' && str.charAt(index+1) <= '9')
			{
			 	if (tokenIndex == McUndefined || str.charAt(index+1) == tokenIndex)
					return new Array(index,str.charAt(index+2), str.charAt(index+1))
			}
		}

		// Look for next %
		index = str.indexOf("%",++index)
	}
	return McUndefined
}

/////////////////////////////////////////////////
//
// Attempts to render natural language description,
// defaulting to raw rule if none is available
//
function RenderDescription()
{
	if (this.natural == McUndefined)
		return this.Render(this.rule,true)
	else
		return this.Render(this.natural,false)
}
/////////////////////////////////////////////////
//
// Inserts values into natural language or rule
// template and returns as one string
//
function Render(template, bLit)
{
	var strOut = ""

	// Loop until we cant find any more tokens
	var token = FindNextToken(template)
	while (token != McUndefined)
	{

		// Add text up to this point
		var iValue = token[2]-1
		strOut += template.substr(0,token[0])

		// Add value from array
		if (this.values[iValue] == McUndefined)
		{
			strOut += gDataTypeNames[token[1]]
		}
		else
		{
		     if (bLit)
			strOut += this.RenderValue(iValue)
		     else
			strOut += this.values[iValue]
		}

		// Look for next token
		template = template.substr(token[0]+3)
		token = FindNextToken(template)
	}

	// Add fag end of string
	strOut += template
	return strOut
}

/////////////////////////////////////////////////
//
// Extracts values from the given literal rule
// , assuming that it was built from the given
// template. Returns false if the rule and template
// do not match
//
function ExtractValues(templateIn,ruleIn,values)
{
	// Prepare params
	var template = templateIn
	var rule = ruleIn

	// Look for all tokens in template
	var count= 1
	var token = FindNextToken(template,count)
	while (token != McUndefined)
	{

		// Check that rule and template match up
		// to this point
		if (template.substr(0,token[0]) != rule.substr(0,token[0]))
			return false

		// Attempt to read unparsed value from rule
		var valRes = ReadValue(rule,token[0],true)
		if (valRes[0] < 0)
			return false

		// Add to value array
		values[token[2]-1] = valRes[1]

		// Remove processed section of both rule and template
		rule = rule.substr(valRes[0])
		template = template.substr(token[0]+3)

		// Look for next
		token = FindNextToken(template)
	}

	// Check that fag end of rule and template match
	if (rule != template)
		return false

	// Return success
	return true
}

//////////////////////////////////////////////////////
//
// Attempts to read a value from the given rule at
// the given position.
// Returns (endofvalindex, value) as an array. Value is parsed
// or unparsed depending on input param
//
var McESCAPECHAR = '^'
var McWILDCARD = '?'
var McMULTIWILDCARD = '*'
var gBoolKeywords = new Array("yes", "no", "true", "false")
function IsNumberChar(c)
{
	return ((c >= '0' && c <= '9')  || (c == '.') || (c == '+') || (c == '-'))
}
function ReadValue(rule, iIndex, bParse)
{
	var operand = ""

	// Check for end of string
	if (iIndex >= rule.length)
		return McUndefined
	var iIndexIn = iIndex

	// See if first character is numeric and keep reading until
	// we get a non-numeric
	if (IsNumberChar(rule.charAt(iIndex)))
	{
		while (IsNumberChar(rule.charAt(iIndex)) && iIndex < rule.length)
			operand+= rule.charAt(iIndex++)
		return new Array(iIndex, bParse ? operand : rule.substr(iIndexIn,iIndex))
	}

	// See if first char is a double quote and attempt to parse a string
	if (rule.charAt(iIndex) == '\"')
	{
		iIndex++;
		while (iIndex < rule.length && rule.charAt(iIndex) != '\"')
		{
			// Unescape all chars ...
			if (rule.charAt(iIndex) == McESCAPECHAR)
			{
				iIndex++
				// ... except wildcards
				if (iIndex < rule.length &&
					(rule.charAt(iIndex) == McWILDCARD || rule.charAt(iIndex) == McMULTIWILDCARD))
					iIndex--
			}

			// Check for end of string
			if (iIndex >= rule.length)
				return new Array(-1,"")

			// Add to string
			operand += rule.charAt(iIndex++)
		}

		// Return error if necessary
		if (rule.charAt(iIndex) != '\"')
			return new Array(-1,"")
		iIndex++

		// Add parsed value or raw string
		return new Array(iIndex, bParse ? operand : rule.substring(iIndexIn,iIndex))
	}

	// Boolean keyword ("yes", "no", "true" or "false")
	for (var i = 0; i < gBoolKeywords.length ; i++)
	{
		if (rule.substr(iIndex,gBoolKeywords[i].length).toLowerCase() == gBoolKeywords[i])
		{
			iIndex += gBoolKeywords[i].length
			return new Array(iIndex, bParse ? gBoolKeywords[i] : rule.substr(iIndexIn,iIndex))
		}
	}

	// Return failure
	return new Array(-1,"")
}

/////////////////////////////////////////////////
//
// Inits values based on raw template info
// (Note all are undefined initially)
//
function InitValues()
{
	// Retrieve base template
	var template = this.natural == McUndefined ? this.rule : this.natural
	if (template == McUndefined)
		return

	this.values = new Array
	this.types = new Array

	// Loop until we cant find any more tokens
	var count= 1
	var token = FindNextToken(template,count)
	while (token != McUndefined)
	{

		// Add type and empty value to lists
		this.values[this.values.length] = McUndefined
		this.types[this.types.length] = token[1]

		// Find next
		token = FindNextToken(template,++count)
	}
}

/////////////////////////////////////////////////
//
// Should be called by value edit box to update
// value in rule. Note value is stored parsed
//
function StoreValue(val)
{
	this.values[this.editedValue] = val
	this.editedValue = -1
}

/////////////////////////////////////////////////
//
// Returns true if all values defined
//
function Defined()
{
	for (var i=0; i<this.values.length; i++)
		if (this.values[i] == McUndefined)
			return false;
	return true;
}
