/**
 * JavaScript for the File filter rules management panel in shared resources
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var ffr_pb = null;
var ffrListWidget = null;

// The resource management driver class for file filtering rules
function FFRulesDriver() {
}

// Gets the header text: "File Filtering Rules"
FFRulesDriver.prototype.getHeader = function() {
    return resolveString(cse_pp_st, "File_Hdr");
}

// No category selector for file filtering rules
FFRulesDriver.prototype.getSelDirective = function() {
    return null;
}

// No category selector for file filtering rules
FFRulesDriver.prototype.getCategoryList = function() {
    return [];
}

// Creates a new file filtering rule
FFRulesDriver.prototype.newInstance = function(cat) {
    ffr_NewRule();
}

// Lists file filtering rules
FFRulesDriver.prototype.drawList = function(category, tbody, listWidget) {
    var links = ffr_getListInfo();
    for (var i = 0; i < links.length; i++) {
        var link = links[i];
        var row = tbody.insertRow(i);
        var cellIdx = 0;
        var td = row.insertCell(cellIdx++);
        td.appendChild(document.createTextNode(link.text));

        td = row.insertCell(cellIdx++);
        var usedByStr = (1 == link.refCount) ? resolveString(cse_pp_st, "__one_policy") :
            resolveString(cse_pp_st, "__n_policies", [link.refCount]);
        td.appendChild(document.createTextNode(usedByStr));

        td = row.insertCell(cellIdx++);
        td.appendChild(SharedResourceManager.createLink(link.editLink, resolveString(cse_pp_st, "__edit")));
        if (0 == link.refCount && link.text!=resolveString(cse_pp_st,"__ff_sample_rule")) {
            td.appendChild(document.createTextNode(" | "));
            td.appendChild(SharedResourceManager.createLink(link.delLink, resolveString(cse_pp_st, "__delete")));
        }
    }
    if (0 == links.length) {
        var row = tbody.insertRow(i);
        var td = row.insertCell(cellIdx++);
        td.colSpan = 3;
        td.style.textAlign = "center";
        td.appendChild(document.createTextNode(resolveString(cse_pp_st, "__nothing_to_display")));
    }
}

function ffr_DeleteRule(pp) {
    if (confirm(resolveString(cse_pp_st, "__del_rule_prompt"))) {
        ffr_pb.DeleteElement(pp);
        top.SetModified(true);
        ffrListWidget.drawInner(null);
    }
}

function ffr_LoadRule(url, title, pp) {
    var page = new DialogPage(url, title);
    top.gPageStack.Forward(page, pp);
}

function ffr_NewRule() {
    var basePP = "FileRules.Rule";
    // Get the next enumeration
    var newPP = ffr_pb.GetNextElementEnum(basePP);
    var target = "FileRule.html";
    var title = resolveString("File");
    top.gPageStack.Forward(new DialogPage(target, title), newPP);
}

function ffr_getListInfo() {
    var settingsPage = "FileRule.html";
    var list = [];
    var pp = new PropertyPath("FileRules.Rule");
    var type = "File";
    for (pp.BeginEnum(); ffr_pb.GetProperty(pp) !== undefined; pp.IncrementEnum()) {
        pp.SetTempLeaf("builtin");
        var builtIn = ("1" == ffr_pb.GetProperty(pp));
        if (builtIn) {
            // Can't be edited or deleted
            pp.SetTempLeaf("");
            continue;
        }
        pp.SetTempLeaf("id");
        var id = ffr_pb.GetProperty(pp);
        pp.SetTempLeaf("name");
        var text = resolveString(cse_pp_st, ffr_pb.GetProperty(pp));
        if (!text) {
            text = id;
        }
        pp.SetTempLeaf("");
        var href = "javascript:ffr_LoadRule('" + settingsPage +
            "','" +	resolveString(cse_pp_st, type) + "','" + pp.GetPath() + "');";
        var delLink = "javascript:ffr_DeleteRule('" + pp.GetPath() + "');";
        var refCounter = RefCounter.getRefCount(type, id, null,
            RefCounter.getRuleRefCount, RefCounter.ruleTypeToSettingsType);
        list.push({"editLink":href, "text":text, "refCount":refCounter.refCount,
            "delLink":delLink, "root":0});
    }

    return list;
}

function ffr_Show(pbIn) {
    ffr_pb = pbIn;
    if (top.polctx && top.polctx.newFRule) {
        // We don't care about this item because we'll pick up
        // the filtering rule by reading the property bag
        delete top.polctx.newFRule;
    }

    ffrListWidget = new SharedResourceManager("rListsDiv", "ffrListWidget", new FFRulesDriver(), {verticalSpace:"10px"});
    ffrListWidget.draw();
}

