// Copyright (C) 2009 McAfee, Inc.  All rights reserved.

// Table of support macros, to be inserted infront of a relative path
var gMacroTable = new Array("McDESKTOP","McINSTALLROOT","McOSDRIVE","McPROGRAMFILES","McWINDOWSDIR", "McHOME")

////////////////////////////////////////////////
//
// Confirms if given string is a valid win32 file
// path. If bRelative is false then it checks for
// a full UNC path or standard path, otherwise a relative path.
//
function IsValidFilePath(pathIn, bRelative)
{

	// Check for beginning of relative path ...
	var path = pathIn.toLowerCase()
	if (bRelative)
	{
		if (path.indexOf("\\")==0)
		{
			return gsInvalidRelativePath;
		}
	}
	// ... check for beginning of absolute path
	else
	{
		// Standard
		var bValidPrefix = false
		if (path.charAt(0) >='a' &&
		    path.charAt(0) <='z'&&
		    path.charAt(1) ==':' &&
		    path.charAt(2) =="\\")
			bValidPrefix = true
		else
			// UNC
			if (path.substr(0,2) == "\\\\" &&
		 	   path.indexOf("\\",2) > -1 &&
		 	   path.indexOf("\\",2) < path.length -1)
				bValidPrefix = true
		if (!bValidPrefix)
		{
			return gsInvalidAbsolutePath;
		}

		// Remove prefix
		path = path.substr(2)
	}

	// check for path length
	if(path.length > 254)
	{
		return gsInvalidPathTooLong;
	}

	// Check if reserved characters are used.
	var re = /[\/:*?"<>|]/
	if (path.search(re) != -1)
	{
		return gsInvalidCharsInPath;
	}

	// Check for invalid folder names
	var reEmptyFolder = /(^\s{1,})|(\s{1,}$)|(\s{1,}\\)|(\\\\)|(\\\s{1,})/
	if (path.search(reEmptyFolder ) != -1)
	{
		return gsInvalidPath;
	}
}

////////////////////////////////////////////////
//
// Looks for a macro at the beginning of
// the given path and retunrs an index to
// it in gMacroTable
//
function GetFilePathMacro(path)
{
	// Check for presence of macro
	var index = path.indexOf("\\")
	var macroIndex = -1
	if (index !=0 && index != -1)
	{
		// Look for macro in table
		for (var i= 0; i< gMacroTable.length; i++)
			if (path.substr(0,index) ==gMacroTable[i])
				macroIndex = i
	}
	return macroIndex
}

////////////////////////////////////////////////
//
// Validates a file name
//
function IsValidFilename(filename)
{
	// Check for empty
	if (filename.length == 0)
	{
		return gsInvalidFilename;
	}

	// Check for file name length ( max 256 chars.)
	if (filename.length > 255)
	{
		return gsInvalidFileTooLong;
	}

	// Check for invalid chars
	var re = /[\\\/:*?"<>|]/
	if (filename.search(re) != -1)
	{
		return gsInvalidCharsInName;
	}
}

try
{
	function FilePathValidator(bRelative)
	{
    		this.name = "FilePathValidator";
    		this.bRelative = bRelative;
	}

	FilePathValidator.prototype = new Validator();

	FilePathValidator.prototype.validate = function(value)
	{
  		this.message = IsValidFilePath(value,this.bRelative);
		return this.message == null;
	}

	FilePathValidator.prototype.errorMessage = function()
	{
		return this.message;
	}

	function FileNameValidator()
	{
    		this.name = "FileNameValidator";
	}

	FileNameValidator.prototype = new Validator();

	FileNameValidator.prototype.validate = function(value)
	{
  		this.message = IsValidFilename(value);
		return this.message == null;
	}

	FileNameValidator.prototype.errorMessage = function()
	{
		return this.message;
	}
}
catch(e) {}

