/**
 * JavaScript for the FileRule page
 *
 * Copyright (C) 2016 McAfee, Inc. All rights reserved
 */

var FN_FILTER = "f_filename_filter";
var FT_FILTER = "f_filetype_filter";
var FS_FILTER = "f_filesize_filter";
var LIMIT_DP = 3;
var MAX_F_SIZE = 2147483648; // 2GB
var KB = 1024, MB = 1048576, GB = 1073741824;
var dimensions = [1, KB, MB, GB];
var currDim = dimensions[0];

var qparams = queryParameters();
fixQParams(qparams, "id", "");

var g_changeNotifyFn = "top.gPageStack.Active().SetModified(true);";

function ffr_getId() {
    return qparams["id"][0];
}

var vframeworks = new Object();
var dataBindings = new Array();
var inputLists = new Object();
var newInstance = false;
var newId = null;
var basePP = "FileRules.Rule";
var ffr_pb = null;
var origName="";
var ffrId = ffr_getId();
var slFloatV = null;
var slCrossV = null;
var ffr_cs = null;

// Input restrictions for the file name patterns
function fnCheckInput(evt) {
    // Only allow characters permissible in filenames (and wildcards)
    // Forbidden characters are: /\:"<>| and space
    return cb_restrictInput(evt, ["AZ", "az", "09", "!", "#.",
        ";", "=", "?", "@", "[", "]`", "{", "}", "~", " "]);
}

// Checks to see whether one of the file filtering options is selected
function frCheckForFFOption() {
    var optSelected = false;
    var checkboxes = {}
    checkboxes[FN_FILTER] = "FilenameCheck";
    checkboxes[FT_FILTER] = "FormatCheck";
    checkboxes[FS_FILTER] = "SizeCheck";
    for (var formName in checkboxes) {
        var cbName = checkboxes[formName];
        var f = document.forms[formName];
        if (!f)
            continue;
        var e = f.elements[cbName];
        if (e && e.checked) {
            optSelected = true;
            break;
        }
    }
    return optSelected;
}

// Handles the user checking or unchecking the filename filtering checkbox
function handleFNFCheck(checked, focus) {
    if (null == focus)
        focus = true;
    var div = document.getElementById("f_filename_filter_opts");
    if (div) {
        var clName = div.className;
        if (null == clName)
            clName = "";
        clName = checked ? removeClass(clName, "disabled") : addClass(clName, "disabled");
        div.className = clName;
    }
    inputLists["ffn"].draw(checked);
    if (checked && focus) {
        try {inputLists["ffn"].focus();} catch(e){}
    }
}

// Handles the user checking or unchecking the file type filtering checkbox
function handleFTFCheck(checked) {
    var div = document.getElementById("f_filetype_filter_opts");
    if (div) {
        var clName = div.className;
        if (null == clName)
            clName = "";
        clName = checked ? removeClass(clName, "disabled") : addClass(clName, "disabled");
        div.className = clName;
    }
    var e = document.forms[FT_FILTER].elements;
    for (var i = 0; i < e.length; i++) {
        e[i].disabled = (e[i].name == "FormatCheck") ? false : !checked;
		if(e[i].name == "ThirdSelect")
			e[i].disabled  = true;
    }
    ffr_cs.enable(checked);
}

// Handles the user checking or unchecking the filesize filtering checkbox
function handleFSFCheck(checked) {
    var div = document.getElementById("f_filesize_filter_opts");
    if (div) {
        var clName = div.className;
        if (null == clName)
            clName = "";
        clName = checked ? removeClass(clName, "disabled") : addClass(clName, "disabled");
        div.className = clName;
    }
    var e = document.forms[FS_FILTER].elements;
    for (var i = 0; i < e.length; i++) {
        e[i].disabled = (e[i].name == "SizeCheck") ? false : !checked;
    }
    setSLValidators(checked);
}

// Initializes the dimension drop down list from the size threshold value
function initDimSel() {
    var threshold = ffr_pb.GetPropertyDef(basePP + ".SizeLimit",
        top.defValues["FF"]["SizeLimit"]);
    var sel = document.forms[FS_FILTER].elements["dim"];
    if (null != sel) {
        for (var i = dimensions.length - 1; i >= 0 ; i--) {
            if (threshold >= dimensions[i]) {
                sel.options[i].selected = true;
                currDim = dimensions[i];
                break;
            }
        }
    }
}

// Handles changes to the dimension selector
function handleDimSelChange(value) {
    var oldDim = currDim;

    var multiplier = oldDim / value;
    var e = document.forms[FS_FILTER].elements["SizeLimit"];
    if (e && 1 != multiplier) {
        var v = e.value * multiplier;
        if (value > 1)
            e.value = v.toFixed(LIMIT_DP);
        else
            e.value = Math.round(v);
    }

    currDim = value;
}

// "Set" function for the SizeLimit data binding
function sizeLimitSet(x) {
    var divisor = dimensions[0];
    for (var i = dimensions.length - 1; i > 0; i--) {
        if (x >= dimensions[i]) {
            divisor = dimensions[i];
            break;
        }
    }
    if (divisor > 1) {
        x /= divisor;
    }
    return (divisor > 1) ? (x - 0.0).toFixed(LIMIT_DP) : x;
}

// "Get" function for the SizeLimit data binding
function sizeLimitGet(x) {
    var e = document.forms[FS_FILTER].elements["SizeLimit"];
    return (e) ? Math.round(e.value * currDim) : x;
}

/**
 * Fixes up the input property bag and ensures that it is in a correct state
 * @param pbIn the input property bag
 * @param pp the property path for the page
 * @return true if a new settings instance is created
 */
function ffr_initPB(pbIn, pp) {
    var newInstance = false;
    // Check that the file rules properties are loaded
    if (undefined === pbIn.GetProperty("FileRules")) {
        // Copy the FileRules node into the property bag that we've been passed
        top.gPB.GetElement("FileRules", pbIn);
    }
    if (null != pp) {
        // Check for presence of property path in property bag
        if (undefined === pbIn.GetProperty(pp)) {
            // New instance
            // Generate a random ID
            newId = MakeGuid("FFR");
            var ppT = PropertyPath.MakePP(pp);
            pbIn.SetProperty(ppT);
            ppT.SetTempLeaf("id");
            pbIn.SetProperty(ppT, newId);
            ppT.SetTempLeaf("");
            newInstance = true;
        }
    }
    return newInstance;
}

// Initializes the page's forms
function ffr_forms_init() {
    var forms = ["_rule_name", FN_FILTER, FT_FILTER, FS_FILTER];
    for (var i = 0; i < forms.length; i++) {
        var form = forms[i];
        if (document.forms[form]) {
            var vframework = new ValidationFramework(form);
            vframework.formAction = std_val_form_action;
            vframework.fieldAction = std_val_field_action;
            vframeworks[form] = vframework;
        }
    }

    if (document.getElementById("ap_report_area")) {
        if (!newInstance) {
             var refCounter = RefCounter.getRefCount("File", ffrId, ffr_pb,
                RefCounter.getRuleRefCount, RefCounter.ruleTypeToSettingsType);
            if (0 != refCounter.getDependentPolicyCount())
                refCounter.briefReport("ap_report_area");
        }
    }

    var reqV = new RequiredValidator();
    //var scriptV = new ScriptValidator();
	var scriptV = new InputTextValidator();
    var form = "_rule_name";
    if (document.forms[form]) {
        var vframework = vframeworks[form];
        var field = "name";
        vframework.labelMap[field] = resolveString(page_st, "lm_rule_name");
        vframework.addValidator(field, reqV);
        vframework.addValidator(field, scriptV);

        var inputArray= new Array();
		inputArray["SECTION"]="File Filter Rules";
		inputArray["origName"]=origName;
		inputArray["basePP"]="FileRules.Rule";
		vframework.addValidator("name",new DuplicatesValidator(ffr_pb,inputArray));
		vframework.addValidator("name",new LengthValidator());

        // The rule name may be a stringtable key so use an STKeyDataBinding object
        dataBindings.push(new STKeyDataBinding(form, field, cse_pp_st));
    }

    form = FN_FILTER;
    if (document.forms[form]) {
        var e = document.getElementById("ap_report_div");
        if (e && "none" != e.style.display) {
            document.getElementById("tbl_" + form).style.marginTop = "10px";
        }
        var fileNamesList = new InputList([], "ff_file_names_div", "inputLists['ffn']",
            { baseFieldName:"filename", vframework:vframeworks[form], compulsory:true,
            displayName:resolveString(page_st, "fnl_displayName"),
            inputNamePattern:resolveString(page_st, "lm_fNamePattern"),
            inputFilter:fnCheckInput, changeNotify:g_changeNotifyFn});
        inputLists["ffn"] = fileNamesList;
        dataBindings.push(new InputListDataBinding(inputLists["ffn"], "FILENAME"));
    }

    form = FT_FILTER;
    if (document.forms[form]) {
        if (document.forms[FN_FILTER]) {
            document.getElementById("tbl_" + form).style.marginTop = "10px";
        }
        var fcsProps = {SelectAllCheck:false,width:300,width2:280};
        ffr_cs = new FileCatSelector(fileCats, "ff_file_types_div", "ffr_cs", fcsProps, "F");
        ffr_cs.draw();

        var vframework = vframeworks[form];
        var CatSelCheckVal=new CatSelectionCheckValidator(ffr_cs); //FileCategoryValidator
        var field = "FormatCheck"; //name of the field in the FileRule.html (selectallcheck is false for this FileCatSelector.)
        vframework.addLabeledValidator(field,"FileFormat",CatSelCheckVal); //adding the CatSelection validator.

        dataBindings.push(new FCSDataBinding(ffr_cs, "FORMAT"));
    }

    form = FS_FILTER;
    if (document.forms[form]) {
        if (document.forms[FT_FILTER] || document.forms[FN_FILTER]) {
            document.getElementById("tbl_" + form).style.marginTop = "10px";
        }
        var fieldName = "SizeLimit";
        dataBindings.push(new CFDataBinding(FS_FILTER, fieldName, top.defValues["FF"][fieldName],
            sizeLimitSet, sizeLimitGet));
        var vf = vframeworks[form];
        vf.labelMap[fieldName] = resolveString(page_st, "lm_fSize");
        slFloatV = new FloatValidator();
        var slValCode = "value > 0 && value * currDim <= " + MAX_F_SIZE;
        var slErr = resolveString(cse_pp_st, "size_limit_err");
        slCrossV = new CrossValidator(slValCode, slErr);
        // Restrict the field to decimal input > 0
        setNumeric(form, [fieldName], true, false);
    }
    addChangeNotification(g_changeNotifyFn);
}

// Sets / unsets field validators on the size limit field
function setSLValidators(b) {
    var fieldName = "SizeLimit";
    var vframework = vframeworks[FS_FILTER];
    var e = document.forms[FS_FILTER].elements[fieldName];
    if (!e)
        return;
    if (b) {
        vframework.addValidator(fieldName, slFloatV);
        vframework.addValidator(fieldName, slCrossV);
    }
    else {
        if (!vframework.check(fieldName)) {
            e.value = ffr_pb.GetProperty(basePP + "." + fieldName);
            vframework.onchange(fieldName);
        }
        vframework.clearValidators(fieldName);
    }
}

// Sets form data
function ffr_init() {
    ffr_forms_init();
    Reset();
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var ffr_pp = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(ffr_pp, ffr_pb);
    }

    if (document.forms[FN_FILTER]) {
        handleFNFCheck(document.forms[FN_FILTER].elements["FilenameCheck"].checked, false);
    }

    if (document.forms[FT_FILTER]) {
        handleFTFCheck(document.forms[FT_FILTER].elements["FormatCheck"].checked);
    }

    if (document.forms[FS_FILTER]) {
        initDimSel();
        handleFSFCheck(document.forms[FS_FILTER].elements["SizeCheck"].checked);
    }
    top.gPageStack.Active().SetModified(false);
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    else {
        // We need to ensure that the PolicySet node is included to do
        // reference counting
        if (null == pbIn.GetProperty("Policies")) {
            top.gPB.GetElement("Policies", pbIn);
        }
    }
    newInstance = ffr_initPB(pbIn, pp);
    ffr_pb = pbIn;
    if (ffrId) {
        // Search for the alert blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", ffrId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
        // Assume that we've been set up with the correct property path
        // and property bag
        basePP = pp.GetPath();
        ffrId = pbIn.GetPropertyDef(basePP + ".id", "");
		origName=pbIn.GetPropertyDef(basePP + ".name", "");
    }
    ffr_init();
}

/**
 * Asynchronous data validation function
 */
function Validate() {
    var ok = true;
    ok = frCheckForFFOption();
    if (!ok) {
        alert(resolveString(cse_pp_st, "ff_option_select_1"));
    }
    else {
        for (var vf in vframeworks) {
            ok = vframeworks[vf].onsubmit();
            if (!ok) {
                break;
            }
        }
    }
    return ok;
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb) {
    var changed = false;
    if (null == pb) {
        pb = ffr_pb;
    }
    var ppT = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        if (dataBindings[i].get(ppT, pb)) {
            changed = true;
        }
    }
    if (changed && newInstance) {
        // Notify any interested caller about the new instance
        if (top.polctx) {
            top.polctx.newFRule = newId;
			newInstance = false;
        }
    }
    top.gPageStack.Active().SetModified(changed);
    return changed;
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        ffr_pb.DeleteElement(basePP);
		newInstance = false;
  }
}

