// Copyright (C) 2009 Networks Associates Technology Inc. All rights reserved.
//

// Constructor for history page class
function HistoryGraphPage()
{
	// Initialize member functions
	this.Show 						= Show
	this.EnableAdvancedTimeSpan 	= EnableAdvancedTimeSpan
	this.EnableDateTime 			= EnableDateTime
	this.SetStateDateTime 			= SetStateDateTime
	this.UpdateQuerySpec 			= UpdateQuerySpec
	this.ChangeState 				= ChangeState
	this.UpdateStateTables			= UpdateStateTables
	this.Query						= Query
	this.GetSelectedFilterRule		= GetSelectedFilterRule
	this.QuerySetDates				= QuerySetDates
	this.OnGraphTypeChange			= OnGraphTypeChange
	this.Validate					= Validate
	this.IsFilterMatch				= IsFilterMatch
	this.JumpToConfigPage			= JumpToConfigPage

	// Default to show the simple state
	this.bSimpleState 				= true
	this.previousFilter 			= null
	this.bDateOptionsChanged		= true
	this.sPreviousDateQuery			= ""
}

////////////////////////////////////////////////
//
// Writes current config settings into controls
// and then reset modified flag
//
function Show(pbIn)
{
	// Make sure page name is displayed in title area and highlighted in task bar
	parent.tasks.UpdateTitle()

	// Update button bar
	InitButtonBar("DetectionStatistics.gif")
	AddButtonToBar(gsRefresh, gsRefreshShortcut, "parent.mcmainpage.thisPage.Query()")

	PopulateFilterRules();

	SetStateDateTime("fr");
	SetStateDateTime("to");

	// Update query specification
	this.UpdateQuerySpec()

	// Make sure the display is correct
	this.UpdateStateTables()

	this.Query()
}

function IsFilterMatch( filter1, filter2 )
{
	if( filter1 == null || filter2 == null )
		return(false)

	if( filter1.nID == filter2.nID &&
		filter1.sGraphType == filter2.sGraphType &&
		filter1.sFilter == filter2.sFilter &&
		filter1.sQueryOn == filter2.sQueryOn &&
		filter1.nNumber == filter2.nNumber )
	{
		return(true)
	}

	return(false)
}

function GetPersonalPeferences(nWidth, nHeight)
{
	var sQueryString = ""

	if( bGraph3D )
		sQueryString += "&ad=3"
	else
		sQueryString += "&ad=2"

	if( bGraphTrans )
		sQueryString += "&tg=1"
	else
		sQueryString += "&tg=0"

	if( bGraphAntiAlias )
		sQueryString += "&aa=1"
	else
		sQueryString += "&aa=0"

	if( bGraphExplodePie )
		sQueryString += "&pe=1"
	else
		sQueryString += "&pe=0"

	var sAngle = sPieAngle
	if( sAngle != "" )
		sQueryString += "&pa="+sAngle

	if(nWidth == McUndefined && nHeight == McUndefined)
	{
		if( !bGraph3D || bGraphExplodePie )
			sQueryString += "&rd=120";
		else
			sQueryString += "&rd=160";
	}
	return(sQueryString)
}

////////////////////////////////////////////////
//
// Performs the query, resulting in a grah ouputted
// to the iFrame.
//
//
function Query()
{
	var queryString = "UIGraph&of=0&pb=1&lx=225&ly=15&lf=" + fontName + "&ls=8&fn=" + fontName + "&ts=14"
	var filterRule	= this.GetSelectedFilterRule()
	var previousQuery = false

	if( filterRule != null && this.Validate() )
	{
		var graph_iframe = document.getElementById("graph")

		// Get the size of the iFrame used to display the graph
		queryString += "&wi="+graph_iframe.width;
		queryString += "&he="+graph_iframe.height;

		// Get the settings for the graph, found in the personal preferences
		queryString += GetPersonalPeferences()
		queryString += GetFilterColour()

		if (filterRule.nGraphType=="bar")
			queryString+="&pl=0";

		// Add the filter & graph type to the query string
		if( filterRule.sFilter != "" ) queryString += "&f=" + filterRule.sFilter
		queryString += "&gf=" + filterRule.nGraphType

		// If a query on is defined then add that to the query string
		if( filterRule.sQueryOn != "" )
			queryString += "&qo=" + filterRule.sQueryOn;

		if( filterRule.nNumber > -1 )
			queryString += "&nr=" + filterRule.nNumber;

		// Set the dates of the query
		var date_query_string = this.QuerySetDates()
		if( date_query_string != "0" )
		{
			queryString += date_query_string;

			// Was there a previous query? If so is it the same?
			if( this.previousFilter != null )
			{
				if( this.IsFilterMatch( this.previousFilter, filterRule ) && this.bDateOptionsChanged == false )
					queryString += "&qr=1"
			}

			// Store the filter we are about to use
			this.previousFilter = filterRule;
			
			//encode the chars - &, > and <
			queryString = EncodeXSSChars(queryString);
			// Run the query
			//queryString = PSEscapeQueryString(queryString);						
			graph.location.href = GetDynamicPageUrl(queryString, "png")
		}
	}
	else
		return false
	return true
}

function QuerySetDates(  )
{
	var 	queryStart
	var 	queryEnd
	var 	interval
	var 	now 		= new Date()
	var 	bDefinedEnd	= true
	var		queryString	= ""

	if( this.bSimpleState == true )
	{
		setCookieValue("McHistGraphTimeSpanIndex", "")
		setCookieValue("McHistGraphTimeSpanIndex", document.getElementById("cmnTimeSpan").selectedIndex)

		queryStart = new Date()
		switch(document.getElementById("cmnTimeSpan").selectedIndex)
		{
			case 0: // Today's infections
			{
				// today's infections
				queryStart.setHours(0)
				queryStart.setMinutes(0)
				queryStart.setSeconds(0)

				bDefinedEnd = false
				interval="hours"

				// Only required if a bar graph
				queryString += "&yt="+gsNumDetections
				queryString += "&xt="+gsHours24
			}
			break;
			case 1:
			{
				// this week
				while(queryStart.getDay())
				{
					queryStart.setTime(queryStart.getTime()-24*60*60*1000)
				}
				queryStart.setHours(0)
				queryStart.setMinutes(0)
				queryStart.setSeconds(0)
				interval="days"
				bDefinedEnd=false

				// Only required if a bar graph
				queryString += "&yt="+gsNumDetections
				queryString += "&xt="+gsDayOfMonth
			}
			break;

			case 2:
			{
				// this month
				queryStart.setDate(1)
				queryStart.setHours(0)
				queryStart.setMinutes(0)
				queryStart.setSeconds(0)
				interval="days"
				bDefinedEnd=false

				// Only required if a bar graph
				queryString += "&yt="+gsNumDetections
				queryString += "&xt="+gsDayOfMonth
			}
			break;

			default:
			break;
		}
	}
	else // Advance Query
	{
		// clear the cookies from previous queries
		setCookieValue("McHistFrom", "")
		setCookieValue("McHistTo", "")

		if(document.getElementById("dtRange").checked)
		{
			queryStart=FromControls("fr")
			setCookieValue("McHistFrom", DateToXMLUTC(queryStart))
		}

		if(document.getElementById("dtRange").checked)
		{
			queryEnd=FromControls("to")
			setCookieValue("McHistTo", DateToXMLUTC(queryEnd))
		}

		queryString += "&yt="+gsNumDetections
		queryString += "&xt="+gsDayOfMonth
	}

	// start of query can have UTC date or "all" to query for all entries in the repository
	if(queryStart != McUndefined)
	{
		queryString += "&qs="
		queryString += DateToXMLUTC(queryStart)
	}
	else
	{
	//	queryStart = new Date(0)
	//	queryString += "&qs="
	//	queryString += DateToXMLUTC(queryStart)
	}

	if(queryEnd != McUndefined && bDefinedEnd==true)
	{
		queryString += "&qe="
		queryString += DateToXMLUTC(queryEnd)
	}

	if(interval != McUndefined)
	{
		queryString += "&du=" // units of division
		queryString += interval
	} else {
		queryString += "&du=days" // units of division
	}

	if( this.sPreviousDateQuery != "" &&
		this.sPreviousDateQuery == queryString )
		this.bDateOptionsChanged = false
	else
		this.bDateOptionsChanged = true

	this.sPreviousDateQuery = queryString;


	return(queryString)
}

////////////////////////////////////////////////
//
// Populates the list of filter rules which
// are defined in PointProductSpecific.js
//
function PopulateFilterRules()
{
	var filter_combo = document.getElementById("cmnFilterRule")

	for( var i = 0; i < gGraphFilterRules.length; i++ )
	{
		if( gGraphFilterRules[i].sRuleDescription != "" )
		{
			// Create a new option element
			var newelement = document.createElement("option");

			// Fill in the details
			newelement.value = gGraphFilterRules[i].nID;
			newelement.text = gGraphFilterRules[i].sRuleDescription;


			// Add the the combo box

			addOption(filter_combo,newelement);
		}
	}
}


function GetFilterColour()
{
	return "&cl="+gGraphColours;
}

function GetSelectedFilterRule()
{
	if( this.bSimpleState )
	{
		var filter_combo 	= document.getElementById("cmnFilterRule")
		var	filter_id		= filter_combo.options[filter_combo.selectedIndex].value;

		// Lookup the ID
		for( var nIndex = 0; nIndex < gGraphFilterRules.length; nIndex++ )
		{
			if( gGraphFilterRules[nIndex].nID == filter_id )
			{
				setCookieValue("McHistGraphFilterIndex", "" );
				setCookieValue("McHistGraphFilterIndex", nIndex)
				return( gGraphFilterRules[nIndex] );
			}
		}
	}
	else
	{
		// Build an GraphFilterRule based on user input, get the type, filter, query on and maximum results
		var sDescription = gsRuleDescriptionAdvance
		var nRuleID		 = 1 // Not used, so does not matter what this is
		var sType		 = "pie" // pie or bar
		var sFilter		 = "" // filter
		var sQueryOn	 = ""
		var nMaxResults  = -1

		if( document.getElementById("pie").checked )
			sType = "pie";
		else if( document.getElementById("bar").checked )
			sType = "bar";


		sFilter = CreateFilterString()
		if (sFilter==null || sFilter.length==0)
			return (null);

		if( sType == "pie")
		{
			sQueryOn = document.getElementById("qQueryOn").value;

			if( document.getElementById("maxResults").value != "" )
				nMaxResults = parseInt( document.getElementById("maxResults").value, 10 )
		}

		setCookieValue("McHistGraphQueryOn", "")
		setCookieValue("McHistGraphQueryFilter", "")
		setCookieValue("McHistGraphMaxResults", "" )
		setCookieValue("McHistGraphMaxType", "" )
		setCookieValue("McHistGraphQueryOn", sQueryOn)
		setCookieValue("McHistGraphQueryFilter", sFilter)
		if( nMaxResults != -1 ) setCookieValue("McHistGraphMaxResults", nMaxResults )
		setCookieValue("McHistGraphMaxType", sType )

		return(new GraphFilterRule( nRuleID, sDescription, sType, sFilter, sQueryOn, nMaxResults ))
	}

	return(null);
}

function checkfor_rpt_filter(filtername, searchfieldvalue)
{
	var filter;
	if(filtername == "rpt")
	{
		filter = filtername + " LIKE \"%"+ searchfieldvalue +"%\"";
	}
	else
	{
		filter = filtername + "=\"" + searchfieldvalue + "\"";
	}
	return filter;
}

function CreateFilterString()
{
	var f1 = document.getElementById("fieldList1").value
	var f2 = document.getElementById("fieldList2").value
	var f3 = document.getElementById("fieldList3").value


	var s1 = document.getElementById("searchField1").value
	var s2 = document.getElementById("searchField2").value
	var s3 = document.getElementById("searchField3").value
	var filterList = new Array()
	var filterString


	var i=0;

	if (s1 != "")
	{
		var filter = checkfor_rpt_filter(f1,s1);
		filterList[i]=filter
		i++
	}

	if (s2 != "")
	{
		var filter = checkfor_rpt_filter(f2,s2);
		filterList[i]=filter
		i++
	}

	if (s3 != "")
	{
		var filter = checkfor_rpt_filter(f3,s3);
		filterList[i]=filter
		i++
	}

	if (filterList.length==0)
	{
		alert(gsBadFilter)
	}else
	{
		filterString=""
		if (filterList.length>1)
		{
			for (var i=0;i<filterList.length-1;i++)
			{
				filterString+=filterList[i];
				if (filterList.length>1)
					filterString+=" AND "
			}
		}else
		{
			filterString+=filterList[0];
		}

		if (filterList.length>1)
		{
			filterString+=filterList[filterList.length-1];
		}


	}

	return filterString
}

function EnableAdvancedTimeSpan(bEnable)
{
	document.getElementById("toTimeOn").disabled = !bEnable
	document.getElementById("frTimeOn").disabled = !bEnable
	EnableDateTime("to", bEnable && document.getElementById("toTimeOn").checked)
	EnableDateTime("fr", bEnable && document.getElementById("frTimeOn").checked)
}

function SetStateDateTime(idRoot)
{
	EnableDateTime(idRoot,document.getElementById(idRoot+"TimeOn").checked)
}

function Validate()
{
	if( this.bSimpleState == false && document.getElementById("pie").checked )
	{
		if( document.getElementById("qFilter").value == "" )
		{
			alert(gsFilter)
			return(false)
		}

		if( document.getElementById("qQueryOn").value == "" )
		{
			alert( gsNoQueryEntered )
			return(false)
		}
	}

	return(true)
}

////////////////////////////////////////////////////
//
// Updates query specification display from cookie values
//
function UpdateQuerySpec()
{
	var yesterday = new Date()
	yesterday.setTime(yesterday.getTime() - 86400000)
	var from = getCookieValue("McHistFrom")
	if (from != "")
	{
		document.getElementById("frTimeOn").checked = true
		ToControls(XMLToDateUTC(from), "fr")
	}
	else
	{
		document.getElementById("frTimeOn").checked = false
		ToControls(yesterday, "fr")
	}

	var to = getCookieValue("McHistTo")
	if (to != "")
	{
		document.getElementById("toTimeOn").checked = true
 		ToControls(XMLToDateUTC(to), "to")
	}
	else
	{
		document.getElementById("toTimeOn").checked = false
		var today = new Date()
  		ToControls(today, "to")

	}

	var sSimpleState = getCookieValue( "McHistGraphSimple" )
	if( sSimpleState != "" )
	{
		this.bSimpleState = parseInt( sSimpleState, 10 )

		// Select the filter index
		var filterIndex = getCookieValue("McHistGraphFilterIndex")
		if( filterIndex != "" )
			document.getElementById("cmnFilterRule").selectedIndex = filterIndex

		// Set the time span index
		var spanIndex = getCookieValue("McHistGraphTimeSpanIndex")
		if( spanIndex != "" )
			document.getElementById("cmnTimeSpan").selectedIndex = spanIndex

		var graphType = getCookieValue("McHistGraphGraphType")
		if( graphType != "" )
		{
			if( graphType == "pie" )
				document.getElementById("pie").checked = true
			else
				document.getElementById("pie").checked = false

				if( graphType == "bar" )
				document.getElementById("bar").checked = true
			else
				document.getElementById("bar").checked = false
		}

		document.getElementById("qFilter").value = getCookieValue("McHistGraphQueryFilter")
		document.getElementById("qQueryOn").value = getCookieValue("McHistGraphQueryOn")

		var maxResults = getCookieValue("McHistGraphMaxResults")
		if( maxResults != "" )
			document.getElementById("maxResults").value = maxResults
	}
}

function UpdateStateTables()
{
	var stateDiv 	= document.getElementById("aChangeState")
	var simpletable = document.getElementById("simpleTable")
	var advtable	= document.getElementById("advanceTable")
	var sHtml = "<a onclick=\"thisPage.ChangeState()\">";

	if( this.bSimpleState == true )
	{
		sHtml += gsAdvanceOptions;

		simpletable.style.display = ""
		advtable.style.display = 'none'

		setCookieValue( "McHistGraphSimple", "1" )
	}
	else
	{
		sHtml += gsSimpleOptions;
		simpletable.style.display = 'none'
		advtable.style.display = ""

		setCookieValue( "McHistGraphSimple", "0" )
	}

	sHtml += "</a>";
	stateDiv.innerHTML = sHtml;
}

function ChangeState()
{
	this.bSimpleState = !this.bSimpleState
	this.UpdateStateTables()
}

function OnGraphTypeChange(sType)
{
	if( sType == "pie" )
	{
		EnableControl("qQueryOn", true )
		EnableControl("maxResults", true )
	}
	else
	{
		EnableControl("qQueryOn", false )
		EnableControl("maxResults", false )
	}
}

function JumpToConfigPage()
{
	parent.tasks.LoadPage("preferencespage.htm", "grapicalreportin")
}
