/**
 * JavaScript for the MailURLSettings page.
 * Defines mailURL_init() for page initialization along with Show(), Validate()
 * and Apply() functions
 * Note: include it last.
 *
 * Copyright (C) 2013 McAfee, Inc. All rights reserved
 */

var vframeworks = new Object();
var dataBindings = new Array();
var qparams = queryParameters();
var newInstance = false;
var newId = null;

var basePP = "Settings.MailURLSettings";
var mailURL_pb = null;
var origName="";
var settingsId = qparams["id"];
var mailURLOptsForm = "mailURL_options";

var g_changeNotifyFn = "top.gPageStack.Active().SetModified(true);";


// Initializes the page's forms
function mailURL_forms_init() {
    shs_affectedPolicyReport(newInstance, "MailURL", settingsId, mailURL_pb);
    vframeworks["InstanceSettings"] = new ValidationFramework("InstanceSettings");	
    vframeworks["InstanceSettings"].formAction = std_val_form_action;
    vframeworks["InstanceSettings"].fieldAction = std_val_field_action;
    vframeworks["InstanceSettings"].addValidator("displayName", new RequiredValidator());
    //vframeworks["InstanceSettings"].addValidator("displayName", new ScriptValidator());	
	vframeworks["InstanceSettings"].addValidator("displayName", new InputTextValidator());
    vframeworks["InstanceSettings"].labelMap["displayName"] =  resolveString(cse_pp_field_st, "__instancename");
	vframeworks["mailURL_options"] = new ValidationFramework("mailURL_options");
	vframeworks["mailURL_options"].formAction = std_val_form_action;
    vframeworks["mailURL_options"].fieldAction = std_val_field_action;
	vframeworks["mailURL_options"].addValidator("HighThresholdValue", new RangeValidator(0,127));
	vframeworks["mailURL_options"].addValidator("HighThresholdValue", new NumericValidator());
	vframeworks["mailURL_options"].addValidator("LowThresholdValue", new RangeValidator(0,127));
	vframeworks["mailURL_options"].addValidator("LowThresholdValue", new NumericValidator());
	vframeworks["mailURL_options"].addValidator("MaxOverallURLLookup", new RangeValidator(101,1000));
	vframeworks["mailURL_options"].addValidator("MaxOverallURLLookup", new NumericValidator());

    var inputArray= new Array();
	inputArray["SECTION"]="Scanners";
	inputArray["origName"]=origName;
	inputArray["basePP"]="Settings.MailURLSettings";
	vframeworks["InstanceSettings"].addValidator("displayName",new DuplicatesValidator(mailURL_pb,inputArray));
	vframeworks["InstanceSettings"].addValidator("displayName",new LengthValidator());

    var form = mailURLOptsForm;
    if (document.forms[form]) {
    }
    addChangeNotification(g_changeNotifyFn);
}

// Sets form data and enables / disables controls as appropriate
function mailURL_init() {
    mailURL_forms_init();
    Reset();
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var mailURL_pp = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(mailURL_pp, mailURL_pb);
    }    
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    newInstance = shs_initPB(pbIn, pp, "MailURL");
    mailURL_pb = pbIn;
    if (settingsId) {
        // Search for the settings blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", settingsId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
        // Assume that we've been set up with the correct property path
        // and property bag
        basePP = pp.GetPath();
        settingsId = pbIn.GetPropertyDef(basePP + ".id", "");
        origName=pbIn.GetPropertyDef(basePP + ".displayName", "");
    }
	
	// Displaying the selected URLThresholdtype based on the property URLThresholdType in Config
	// Enabling the text field for entering highthreshold value in case of URLThresholdType being custom
	var pp = new PropertyPath("Settings.MailURLSettings");
	pp.SetTempLeaf("URLThresholdType");
	document.getElementById("URLThresholdType").selectedIndex = pbIn.GetProperty(pp);
	if (pbIn.GetProperty(pp) == 2) {
		document.getElementById("HighThresholdValue").disabled = false;
	}else {
		document.getElementById("HighThresholdValue").disabled = true;
	}
	
	pp.SetTempLeaf("LURLThresholdType");
	document.getElementById("LURLThresholdType").selectedIndex = pbIn.GetProperty(pp);
	if (pbIn.GetProperty(pp) == 2) {
		document.getElementById("LowThresholdValue").disabled = false;
	}else {
		document.getElementById("LowThresholdValue").disabled = true;
	}
	
    mailURL_init();
}

/**
 * Asynchronous data validation function
 */
function Validate() {
	//validate if high threshold value is greater than low threshold value
	var ctlHighThreshold = document.getElementById("HighThresholdValue");
	var ctlLowThreshold = document.getElementById("LowThresholdValue");
	
	var highThreshold = parseInt(ctlHighThreshold.value);	
	var lowThreshold = parseInt(ctlLowThreshold.value);
	
	if( highThreshold <= lowThreshold )
	{
		alert(gsErrMailURLScore);
		return false;
	}
    return shs_Validate();
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb) {
    if (null == pb) {
        pb = mailURL_pb;
    }
    shs_Apply(pb, basePP, dataBindings, newInstance);
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        mailURL_pb.DeleteElement(basePP);
    }
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
    return [];
}

function SetHighThresholdCtl(val)
{
	if( val == 0 )
	{
		document.getElementById("HighThresholdValue").disabled = true;
		document.getElementById("HighThresholdValue").value = 80;
	}
	else if( val == 1 )
	{
		document.getElementById("HighThresholdValue").disabled = true;
		document.getElementById("HighThresholdValue").value = 50;
	}
	else if( val == 2 )
	{
		document.getElementById("HighThresholdValue").disabled = false;
	}
}

function SetLowThresholdCtl(val)
{
	if( val == 0 )
	{
		document.getElementById("LowThresholdValue").disabled = true;
		document.getElementById("LowThresholdValue").value = 80;
	}
	else if( val == 1 )
	{
		document.getElementById("LowThresholdValue").disabled = true;
		document.getElementById("LowThresholdValue").value = 50;
	}
	else if( val == 2 )
	{
		document.getElementById("LowThresholdValue").disabled = false;
	}
}