
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// McGrid uses the ActiveWidgets grid control to display data, the McGrid.html file should be placed with an
// iFrame and the parent (html page containing the iFrame) should provide at least the following functions:
//
// * indicates a required function
//
//	SetMulipleSelection()	- Return true for multiple selection, or false for single selection
//	*SetMcColumnNames()		- Return an array of columns to display within the grid
//  *SetMcGridData()		- Return an array of data you wish to display
//	OnMcGridContext(event)	- Called when a right click made on the grid
//  OnMcGridDblClick(selection)	- Called when a double click is made on the grid

var gMcGrid					= new AW.UI.Grid; // The ActiveWidgets control
var gMcColumns;		// Array of columns from the parent
var gMcGridData;	// Array of data from the parent
var bMcMultipleSelection	= false;
var gAltRowColours			= true;
var gHighlightOnMouseOver	= true;
var gUseIconColumn			= false;

document.onmousemove=mouseTrack;

function McGetGridData(column,row)
{
	if( gMcGridData != McUndefined )
	{
		var nRows = gMcGridData.length;
		
		if( row <= nRows )
		{
			var nCols = gMcGridData[row].length;
			
			if( column <= nCols )
			{
				var virtualColumn = column;
				
				if( gUseIconColumn )
					virtualColumn = column-1;
			
				if( (gUseIconColumn && column == 0) )
				{
					if(  parent.OnSetRowIcon )
					{
						var sIconInfo = parent.OnSetRowIcon( row );
					
						if( sIconInfo )
						{
							var sReturn;
							var val = getCookieValue("McDynamic")
							if ( val == "NoIISMoz")
							{
								var fnStrShow = "TooltipControl(true,'" + EscapeForJS(sIconInfo["alttext"]) + "');";
								var fnStrVanish = "TooltipControl(false,'" + EscapeForJS(sIconInfo["alttext"]) + "');";
								sReturn = "<img src=\'"+sIconInfo["filename"] + "\' onmouseover = \"" + fnStrShow + "\" onmouseout = \"" + fnStrVanish  + "\"  />";
							}
							else
							{
								sReturn = "<img src=\'"+sIconInfo["filename"]+"\' title=\""+ sIconInfo["alttext"] +"\"\ />";

							}
							
							return(sReturn);
						}
					}
					
					return("");
				}
				else
				{
				
					if( gMcGridData[row][virtualColumn] != McUndefined )
						return( gMcGridData[row][virtualColumn] );
					else
						return("");
				}
			}
		}
	}
	
	return("");
}

function McInitializeGrid()
{
	// Reinitialize the grid control
	gMcGrid = new AW.UI.Grid;
	
	// Check that the parent has the functions we require
	if( parent.SetMcColumnNames == McUndefined || parent.SetMcGridData == McUndefined )
		return(false);
	
	// Get an array of column names from the parent
	gMcColumns = parent.SetMcColumnNames();
	
	// Ask parent if we should enable icon column
	if( parent.EnableIconColumn != McUndefined )
		gUseIconColumn = parent.EnableIconColumn();
	
	if( gUseIconColumn )
	{
		var array = new Array();
		array[0] = " ";
		var combinedArray = array.concat( gMcColumns );
		gMcColumns = combinedArray;
	}
	
	// Ask parent if we should allow multiple selection
	if( parent.SetMulipleSelection != McUndefined )
		bMcMultipleSelection = parent.SetMulipleSelection();
	
	// Get data array from the parent
	gMcGridData = parent.SetMcGridData();
	
	if( gMcGridData == McUndefined )
		return(false);
	
	// Check the return of columns from the parent.
	if( gMcColumns == McUndefined )
		return(false);
	
	// Set column names
	gMcGrid.setHeaderText(gMcColumns);

	// Hide the row count column, normally displayed on the far left.
	//gMcGrid.setRowHeaderWidth("0px");
	
	if( gUseIconColumn )
		gMcGrid.setColumnWidth(30,0);
	
	// Based on the given data set the number rows/columns
	gMcGrid.setColumnCount(gMcColumns.length);
	gMcGrid.setRowCount(gMcGridData.length);
	
	// Set multiple selection property
	if(bMcMultipleSelection)
		gMcGrid.setSelectionMode("multi-row");
	else
		gMcGrid.setSelectionMode("single-row");
	
	// Set the function which will be called to get data for the grid
	gMcGrid.setCellData(McGetGridData);
	gMcGrid.refresh();

	// Render the grid
	McRenderGrid();

}

function McSetColumnSizes(colSizes)
{
	var nDelta = 0;
	
	if( gUseIconColumn ) 
		nDelta = 1;
	
	var stylesheet = document.styleSheets[document.styleSheets.length-1];
	for (var i=0; i < colSizes.length; i++)
	{
		gMcGrid.setColumnWidth(colSizes[i],i+nDelta);
	}
}

function McGetColumnSizes()
{
	var nDelta = 0;
	
	if( gUseIconColumn ) 
		nDelta = 1;
	
	var i, widths = [], count = gMcGrid.getColumnProperty("count"); 
   	for (i=nDelta; i<count;i++)
	{ 
        	widths[i-nDelta] = gMcGrid.getColumnWidth(i);
    	} 
	return widths
}

function McSelectAll()
{
	if( gMcGrid != McUndefined )
	{
		var selected = []
		for (var i=0;i <gMcGridData.length; i++)
		{
			selected[selected.length]=i;
		}	
		gMcGrid.setSelectedRows(selected);
		
		// Fake a click, this allows the parent to do as they wish to set-up the grid
		if( parent.OnMcGridClicked )
			parent.OnMcGridClicked(selected);
	}
}

function McSelectNone()
{
	gMcGrid.setSelectedRows([]);
	
	// Fake a click, this allows the parent to do as they wish to set-up the grid
	if( parent.OnMcGridClicked )
		parent.OnMcGridClicked(new Array());
}

function McSelect( index )
{
	if( gMcGrid != McUndefined ) {
		var selected = [];
		
		for (var i = 0; i < gMcGridData.length; i++ ) {
			if( i == index ) 
				selected[ selected.length ] = i;
		}

		gMcGrid.setSelectedRows(selected);
		
		// Fake a click, this allows the parent to do as they wish to set-up the grid
		if( parent.OnMcGridClicked )
			parent.OnMcGridClicked(selected);
	}
}

function OnGridCM( event )
{
	// Pass event onto parent, if interested
	if( parent.OnMcGridContext != McUndefined )
		parent.OnMcGridContext( event );
}

function OnGridSelStart( event )
{
}

function OnDblClick(event)
{
	if( gMcGrid != McUndefined && parent.OnMcGridDblClick != McUndefined )
	{
		var selectionArray = gMcGrid.getSelectedRows();
		
		if( selectionArray != McUndefined )
		{
			if( selectionArray.length > 0 )
				parent.OnMcGridDblClick(selectionArray);
		}
	}
}

function OnClicked(event)
{
	if( gMcGrid != McUndefined && parent.OnMcGridClicked != McUndefined )
	{
		var selectionArray = gMcGrid.getSelectedRows();
		
		if( selectionArray != McUndefined )
		{
			if( selectionArray.length > 0 )
				parent.OnMcGridClicked(selectionArray);
		}
	}
}

function McGetSelected()
{
	var selected;
	if( gMcGrid != McUndefined )
	{
		selected = gMcGrid.getSelectedRows();
		
		//if the index -1 is selected it probably means the SHIFY key was held down
		//before the user clicked on an entry. Either way, it will end up being passed
		//through to the server side UI as a request to delete item 0, which will always
		//fail, so just remove it from the array
		if(selected[0] == -1 || selected[0] == "-1")
			selected.shift();
			
		return selected;
	}

	return(new Array);
}

function McGetFirstSelected()
{
	var selection = McGetSelected();
	
	if( selection != McUndefined &&
		selection.length > 0 )
	{
		return(selection[0]);
	}
	
	return(-1);
}

function McGetSelectedCount()
{
	var selection	= McGetSelected();
	var nCount		= 0;
	
	if( selection != McUndefined )
	{
		nCount = selection.length;
	}
	
	return(nCount);
}

function McGetRecordNumbers( nPageNumber, nPageSize )
{
	var selection	= McGetSelected();
	var recNums		= "";
	
	if( selection != McUndefined )
	{
		// Build list of record numbers
		for( var row = 0; row < selection.length; row++ )
		{
			var nRecNum = (((nPageNumber-1)*nPageSize) + parseInt(selection[row],10))+1;
			
			if( recNums.length > 0 )
				recNums = recNums + "," +nRecNum;
			else
				recNums = ""+nRecNum;
		}
	}
	
	return(recNums);
}

function McRenderGrid()
{
	var theDiv = document.getElementById("McGridCtrl");
	
	if( theDiv != McUndefined && gMcGrid != McUndefined )
	{
		theDiv.innerHTML = gMcGrid.toString();
	}
}

function OnSetRowBckGrndColour()
{
	if( gAltRowColours )
		return(this.getProperty("row/order") % 2 ? "#FFFFFF" : "#F2F2F2");
		
	return("#FFFFFF");
}

function SetColumnWidth( nCol, nWidth )
{
	var stylesheet = document.styleSheets[document.styleSheets.length-1];
	
	if( stylesheet != McUndefined )
	{
		var sClass = ".active-column-" + nCol;
		if(stylesheet.addRule)
			stylesheet.addRule(sClass, "width:"+nWidth+"px");
		else
			stylesheet.insertRule(sClass + "{width:"+nWidth+"px}", stylesheet.cssRules.length);
	}
}
