// Copyright (C) 2009 McAfee, Inc.  All rights reserved.
var McUndefined

if (null == top.pageCtx)
    top.pageCtx = {};

//////////////////////////////////////////////////
//
// Class for managing embedded pages
//
//////////////////////////////////////////////////
function PageStack()
{
	this.pages = new Array()
}

PageStack.prototype.NewPropertyBag = function() // Use to create new propery bags in top frame, so they are not lost when moving down the stack
{
	return new PropertyBag()
}

PageStack.prototype.Reset = function(page,pb,pp) // Call after navigating to a new top level page using the left menu
{
	if (typeof page == "string")
		page = new EmbeddedPage(page);
	page.pp = pp;
	page.pb = pb;
	this.pages = new Array;
	this.pages[0] = page;
	this.pages[0].pb = pb;
	this.pages[0].pp = pp;
	this.UpdateBreadcrumb();
	this.Active().UpdateButtonBar();
}


PageStack.prototype.SetupPage = function(page,pp,pb)
{
    // Clone the page object so as to avoid "freed script" errors in IE
    var newPage = page;
    if (page.constructor != Wizard)
    {
	newPage= page.Clone();
    	newPage.Copy(page);
    }

    // Initialize other values
    newPage.returnPage = this.pages[this.pages.length-1].name;
    newPage.pp = new PropertyPath(pp);
    newPage.pb = pb?pb:this.pages[0].pb;
    return newPage;
}

PageStack.prototype.Forward = function(page,pp,pb,noWarning) // Call "top.gPageStack.Forward(...)" from parent page navigation link
{
    // Make sure we can close current page
    if (!this.Active().CanClose(noWarning))
	return;

    // Call callback on page
    if (CONTENT_FRAME.Forward != McUndefined)
		CONTENT_FRAME.Forward();

    // Add to page stack
    var newPage = this.SetupPage(page,pp,pb);
    this.pages[this.pages.length] = newPage;
    this.popped = McUndefined;

    // Perform navigation
    CONTENT_FRAME.location.href = newPage.name;
    this.UpdateBreadcrumb();
    this.Active().UpdateButtonBar();
}

PageStack.prototype.Sideways = function(page,pp,pb) // Call "top.gPageStack.Sideways(...)" from parent page navigation link
{
    // Make sure we can close current page
    if (!this.Active().CanClose())
	return;

    // Add to page stack
    var newPage = this.SetupPage(page,pp,pb);
    this.pages[this.pages.length-1] =newPage;
    this.popped = McUndefined;

    // Perform navigation
    CONTENT_FRAME.location.href = newPage.name;
    this.UpdateBreadcrumb();
    this.Active().UpdateButtonBar();
}

PageStack.prototype.Back = function(noWarning)	// Call "top.gPageStack.Back()" from child page when navigating back to parent
{
    	// Make sure we can close current page
    	if (!this.Active().CanClose(noWarning))
		return;

	// Call callback on page
	if (CONTENT_FRAME.Back != McUndefined)
		CONTENT_FRAME.Back();

	// Update page stack
	this.popped = this.pages.pop();

	// Perform navigation
	CONTENT_FRAME.location.href =  this.pages[this.pages.length-1].name;
	this.UpdateBreadcrumb();
	this.Active().UpdateButtonBar();
}

PageStack.prototype.ReturnedFrom = function() // Call "top.gPageStack.ReturnedFrom()" from parent page to retrieve data from closed child page
{
	return this.popped;
}

PageStack.prototype.Goto = function(level,nowarn)	// Called from breadcrumb when jumping to another page in the stack
{
    	// Make sure we can close current page
    	if (!this.Active().CanClose())
		return;

	// Update page stack
	while (this.pages.length > level+1)
		this.popped = this.pages.pop();

	// Perform navigation
	CONTENT_FRAME.location.href = this.pages[level].name;
	this.UpdateBreadcrumb();
	this.Active().UpdateButtonBar();
}

PageStack.prototype.Active = function() // Call "top.gPageStack.Active()" from child page to access property path and bag
{
    if (this.pages.length > 0) {
	return this.pages[this.pages.length - 1];
    }
    return null;
}

//This function returns the page that encapsulates the PropertyBag containing the comitted data from the global property bag and
//the uncommited data except the one under creation.
PageStack.prototype.CachedDataPage = function()
{
    if (this.pages.length > 0) {
		return this.pages[0];
    }
    return null;
}


PageStack.prototype.OnLoad = function() // Call "top.gPageStack.OnLoad()" from onload of child page
{
    // Protection against the reload button which destroys the page stack
    if (!this.Active()) {
	   window.location = "index.html";
    }
    else {
        this.Active().SetModified(this.Active().Modified());
        top.ShowPage();
    }
}

PageStack.prototype.UpdateBreadcrumb = function()
{
	var newHTML = ""

	// page stack
	if (this.pages.length > 0)
	{
		newHTML = BREADCRUMB_DELIMINATOR + this.pages[this.pages.length-1].title
		if (this.pages[this.pages.length-1].Modified())
			newHTML += "*"
		var i = this.pages.length-2
		while ( i >= 0 )
		{
			var temp = this.pages[i].title
			if (this.pages[i].Modified())
				temp += "*"
			newHTML = BREADCRUMB_DELIMINATOR + "<a href=\"javascript:top.gPageStack.Goto(" + i + ");\">" + temp + "</a> " + newHTML;
			i--
		}
	}

	// top level menu item if there is one
	var page = gIsEPO4 ? (top.iframeID_NAP.Page.Items[top.page]):(top.Page.Items[top.page]);
	if (page.ParentKey)
	{
		var page = gIsEPO4 ? (top.iframeID_NAP.Page.Items[ page.ParentKey ]):(top.Page.Items[ page.ParentKey ]);
		newHTML = BREADCRUMB_DELIMINATOR + "<a href=\"javascript:top.SetContent('" + page.URL + "');\">" + page.DisplayName + "</a> " + newHTML;
	}
	TOPNAV_DOCUMENT.getElementById("crumbtrail").innerHTML = newHTML
}

PageStack.prototype.toString = function() {
    var pNames = [];
    for (var i = 0; i < this.pages.length; i++) {
	pNames.push(this.pages[i].name);
    }
    return pNames.join(" -> ");
}

PageStack.prototype.Level = function()
{
	return this.pages.length;
}

//////////////////////////////////////////////////
//
// Class representing an embedded page
//
//////////////////////////////////////////////////
function EmbeddedPage(fileName,title)
{
	this.name = fileName;
	this.title = title;
	this.modified=false;
}
EmbeddedPage.New = Function("return new EmbeddedPage;");
EmbeddedPage.prototype.Copy = function(ep)
{
	this.prototype=ep.prototype;
	this.name = ep.name
	this.title = ep.title
	this.modified= ep.modified
	this.returnPage = ep.returnPage
	this.pb = ep.pb
	this.pp = ep.pp
}
EmbeddedPage.prototype.Clone=function()
{
	return gIsEPO4 ? (top.iframeID_NAP.EmbeddedPage.New()):(top.EmbeddedPage.New());
}
EmbeddedPage.prototype.SetModified = function(modified)
{
	this.modified = (modified == McUndefined) ? true : modified
	if (this.modified)
		top.SetModified(this.modified);
	top.gPageStack.UpdateBreadcrumb();
}
EmbeddedPage.prototype.Modified = function()
{
	return this.modified
}
EmbeddedPage.prototype.toString = function()
{
        return this.name;
}
EmbeddedPage.prototype.CanClose= function()
{
	if (this.modified)
	{
		if(this.DoSave() == false)
		{
			return false;
		}
	}
	return true;
}
EmbeddedPage.prototype.DoSave = function()
{
	if (CONTENT_FRAME.Validate != McUndefined)
	{
		var ok = CONTENT_FRAME.Validate();
		if (!ok)
			return false;
	}
	if (CONTENT_FRAME.Apply != McUndefined)
		CONTENT_FRAME.Apply(this.pb,this.pp,true)

  	if (!top.gbModified)
        	top.SetModified(this.modified);
	this.SetModified(false)
	return true;
}
EmbeddedPage.prototype.DoCancel = function()
{
	// Call callback on page
	if (CONTENT_FRAME.Cancel != McUndefined)
		CONTENT_FRAME.Cancel()
	this.SetModified(false)
}
EmbeddedPage.prototype.UpdateButtonBar = function()
{
	BUTTON_FRAME.document.getElementById("wizButtons").style.display = "none";
	BUTTON_FRAME.document.getElementById("configButtons").style.display = "none";
	BUTTON_FRAME.document.getElementById("closeButton").style.display = "none";
	if(gIsEPO4)
	{
		top.iframeID_NAP.document.getElementById("fsButtons").rows = "*,0";
	}
	else
	{
		top.document.getElementById("fsButtons").rows = "*,0";
	}
	TOPNAV_DOCUMENT.getElementById("idBack").style.display=(top.gPageStack.Level() > 1)?"inline":"none";
}

//////////////////////////////////////////////////
//
// Class representing a dialog page
//
//////////////////////////////////////////////////
DialogPage.prototype = new EmbeddedPage
DialogPage.New = Function("return new DialogPage;");
function DialogPage(fileName,title)
{
	this.name = fileName;
	this.title = title;
	this.modified=false;
}
DialogPage.prototype.Save = function()
{
	if ( gIsEPO4 ? (top.iframeID_NAP.EmbeddedPage.prototype.DoSave.call(this)):(top.EmbeddedPage.prototype.DoSave.call(this)) )
		top.gPageStack.Back(true);
}
DialogPage.prototype.Cancel = function()
{
	gIsEPO4 ? (top.iframeID_NAP.EmbeddedPage.prototype.DoCancel.call(this)):(top.EmbeddedPage.prototype.DoCancel.call(this));
	top.gPageStack.Back();
}
DialogPage.prototype.SetModified = function(modified)
{
	this.modified = (modified == McUndefined) ? true : modified
	BUTTON_FRAME.document.getElementById("configSave").disabled = !this.modified;
	BUTTON_FRAME.document.getElementById("configCancel").disabled = false;
	top.gPageStack.UpdateBreadcrumb();
}

// CanClose method for a dialog page object; under certain circumstances we can navigate away
// from a modified page without prompting to save; a typical example would be editing the alert
// text from an actions page.  Parameter has an inverted logical sense (if true, don't warn) so
// that an absent parameter behaves correctly.
DialogPage.prototype.CanClose= function(noWarning)
{
    if (noWarning)
        return true;
	if (this.modified)
	{
		if (confirm(gsSubPagesModified))
			this.DoCancel();
		else
			return false;
	}
	else
	{
		this.DoCancel();
	}
	return true;
}
DialogPage.prototype.UpdateButtonBar = function()
{
	EmbeddedPage.prototype.UpdateButtonBar.call(this);
	BUTTON_FRAME.document.getElementById("configButtons").style.display = "block";
	if(gIsEPO4)
	{
		top.iframeID_NAP.document.getElementById("fsButtons").rows = "*,30"
	}
	else
	{
		top.document.getElementById("fsButtons").rows = "*,30"
	}
}
DialogPage.prototype.Clone=function()
{
	if(gIsEPO4)
	{
		return top.iframeID_NAP.DialogPage.New();
	}
	else
	{
		return top.DialogPage.New();
	}
}
//////////////////////////////////////////////////
//
// Class representing a status page
//
//////////////////////////////////////////////////
StatusPage.prototype = new EmbeddedPage
StatusPage.New = Function("return new StatusPage;");
function StatusPage(fileName,title)
{
	this.name = fileName;
	this.title = title;
	this.modified=false;
}
StatusPage.prototype.UpdateButtonBar = function()
{

	EmbeddedPage.prototype.UpdateButtonBar.call(this);
	BUTTON_FRAME.document.getElementById("closeButton").style.display = "block";
	if(gIsEPO4)
	{
		top.iframeID_NAP.document.getElementById("fsButtons").rows = "*,30"
	}
	else
	{
		top.document.getElementById("fsButtons").rows = "*,30"
	}
}
StatusPage.prototype.Clone=function()
{
	if(gIsEPO4)
	{
		return top.iframeID_NAP.StatusPage.New();
	}
	else
	{
		return top.StatusPage.New();
	}
}
//////////////////////////////////////////////////
//
// Class representing a page in a wizard
//
//////////////////////////////////////////////////
function WizardPage(name,title,helpID,helpSubID)
{
	this.name = name
	this.title = title
	this.helpID = helpID
	this.helpSubID = helpSubID;
}

//////////////////////////////////////////////////
//
// Class representing a wizard
//
//////////////////////////////////////////////////
Wizard.prototype = new EmbeddedPage
Wizard.prototype.constructor = Wizard

function McWizBegin() // called from onload of Wizard.html
{
	// Check we have an active wizard
	var wiz = top.gPageStack.Active()
	if (wiz == McUndefined)
		return

	// Initialize first page
	wiz.wizardPage = CONTENT_FRAME
	wiz.contentPane = CONTENT_FRAME.window.frames["wizardContent"]
	wiz.finished = false;
	wiz.modified = false;
	wiz.wizardPage.document.getElementById("wizName").innerHTML = wiz.title

	// Update
	wiz.ChangePage(0)
}

function Wizard(pages,title)
{
	this.pages = pages
	this.currentPage = 0
	this.name = "Wizard.html"
	this.bModified = false
	this.title = title
}

Wizard.prototype.UpdateButtonBar = function()
{
	EmbeddedPage.prototype.UpdateButtonBar.call(this);
	BUTTON_FRAME.document.getElementById("wizButtons").style.display = "block";
	gIsEPO4 ? (top.iframeID_NAP.document.getElementById("fsButtons").rows = "*,30"):(top.document.getElementById("fsButtons").rows = "*,30")
}

Wizard.prototype.OnLoad = function () // call "top.gPageStack.Active().OnLoad()" from onLoad of each wizard page
{
    this.contentPane.Show(this.pb,this.pp);
    top.PageFormsSubmit(this.contentPane);

}

Wizard.prototype.ChangePage = function( nNewPageNumber )
{
	// Update the wizard page index
	this.currentPage = nNewPageNumber
	this.currentModified = false;

	/// Make sure the quick menu is re-rendered
	this.RenderQuickNav()

	// Make sure controls (back, next, cancel, finish) are displayed as expected.
	BUTTON_FRAME.document.getElementById("wizFinishButton").disabled =  this.currentPage!=this.pages.length-1
	BUTTON_FRAME.document.getElementById("wizNextButton").disabled = this.currentPage>=this.pages.length-1
	BUTTON_FRAME.document.getElementById("wizBackButton").disabled = this.currentPage ==0

	// Show the correct pageon the wizard page
	this.contentPane.location.href = this.pages[this.currentPage].name

	// Update the help button to display help for the current page
	if( this.pages[this.currentPage].helpID )
	{
	    if( this.wizardPage.document.getElementById( "wizHelpButton" ) )
	    {
	        var sSubId = 'global_controls';

	        if( this.pages[this.currentPage].helpSubID )
	            sSubId = this.pages[this.currentPage].helpSubID;

	        this.wizardPage.document.getElementById( "wizHelpButton" ).href = "javascript:parent.help.openHelp('"+this.pages[this.currentPage].helpID+"','"+sSubId+"')";
	    }
	 }
}

Wizard.prototype.Next = function()
{
	if (this.contentPane.Validate())
	{
		this.contentPane.Apply(this.pb, this.pp)
		this.ChangePage( this.currentPage+1)
	}
}

Wizard.prototype.Back = function()
{
	if (this.currentModified)
	{
		if (this.contentPane.Validate())
			this.contentPane.Apply(this.pb,this.pp)
		else
			return;
	}
	this.ChangePage( this.currentPage-1)
}

Wizard.prototype.Cancel = function()
{
	top.gPageStack.Back()
}

Wizard.prototype.Finish = function()
{
	if (this.contentPane.Validate())
	{
		this.contentPane.Apply(this.pb, this.pp);
		this.SetModified(false);
		top.gPageStack.Back();
		this.finished=true;
	}
}

Wizard.prototype.CanClose= function()
{
	if (this.modified)
	{
		if (!confirm(gsSubPagesModified))
			return false;
	}
	return true;
}

Wizard.prototype.Navigate = function(index)
{
	if (index > this.currentPage || this.currentModified)
	{
		if (!this.contentPane.Validate())
			return;
		this.contentPane.Apply(this.pb,this.pp)
	}
	this.ChangePage(index)
}

Wizard.prototype.SetModified = function(modified)
{
	this.modified = modified?true:false;
	this.currentModified=modified?true:false;
	top.gPageStack.UpdateBreadcrumb();
}

Wizard.prototype.RenderQuickNav = function()
{
	// Get the division where we will render the quick nav
	var cWizQuickNav = this.wizardPage.document.getElementById( "wizQuickNav" );

	var sHtml = "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" class=\"btn\">";

	for( var nIndex = 0; nIndex < this.pages.length; nIndex++ )
	{
		var sClass = "btn_bg";

		// If first item, then class for item is different
		if( 0 == nIndex )
			sClass = "btn_lt";
		title=(nIndex+1) + ". " + this.pages[nIndex].title;
		sHtml += "<td class=\""+sClass+"\" nowrap>"
		if(nIndex==this.currentPage)

			sHtml += "<em>"+ title +"</em>";

		else
			if (nIndex<= this.currentPage+1)

			 	sHtml+="<a href=\"javascript:top.gPageStack.Active().Navigate("+nIndex+")\">"+title+"</a>";
			else
				sHtml+=title;
		sHtml += "</td>";

		// If not the last entry place a seperator
		if( this.pages.length-1 != nIndex )
			sHtml += "<td class=\"btn_sep\" nowrap></td>";
	}

	// Close the table and buttons
	sHtml += "<td class=\"btn_rt\" nowrap></td>"
	sHtml += "</table>";

	// Set the inner HTML for the division
	cWizQuickNav.innerHTML = sHtml;
}

/**
 * Performs a context save
 * @param pp the property path of the page whose context is being saved
 * @param currentState a PropertyBag containing the current state of the page
 * @param initialState a PropertyBag containing the initial state of the page
 */
function saveContext(pp, currentState, initialState) {
    var ppT = pp.toString();
    if (null == top.pageCtx[ppT])
        top.pageCtx[ppT] = {};
    // To ensure that the saved property bags are declared in the top
    // context AND immune to being inadvertently changed, we first the
    // clone them and then copy their contents into new bags
    if (currentState.Clone) {
        top.pageCtx[ppT].current = new PropertyBag();
        top.pageCtx[ppT].current.Copy(currentState.Clone());
    }
    else {
        top.pageCtx[ppT].current = currentState;
    }
    if (initialState.Clone) {
        top.pageCtx[ppT].initial = new PropertyBag();
        top.pageCtx[ppT].initial.Copy(initialState.Clone());
    }
    else {
        top.pageCtx[ppT].initial = initialState;
    }
}

/**
 * Performs a context restore
 * @param pp the property path of the page whose context is being restored
 * @return the page context (two property bags keyed as current and initial)
 */
function retrieveContext(pp) {
    var ppT = pp.toString();
    var ret = null;

    // Only retrieve context if there's a popped page on the stack, since the
    // context information will be useless otherwise
    if (top.gPageStack.popped && top.pageCtx[ppT]) {
        ret = top.pageCtx[ppT];
    }

    // Always delete the page context to help ensure that it won't become stale
    delete top.pageCtx[ppT];

    return ret;
}
