/**
 * JavaScript for the file filtering policy settings page
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var g_tlNodes = ["Alerts", "FileRules"];
var ALERT_PROP = "Alert";
var ALERT_CAT = "FILE_FILTER_BLOCK";
var RULE_ADD_SEL_ID = "ff_rules_available";
var RULE_ADD_BTN_ID = "ff_rule_add";
var RULES_TABLE_ID = "ff_rules_tbl";
var NBSP = 160;

var DEF_FF_ACTION = "(ESERVICES:ALLOWTHRU)";
if (top.defValues["FF"] && top.defValues["FF"]["Action"]) {
    DEF_FF_ACTION = top.defValues["FF"]["Action"];
}

// Get the language for alerts
var lang = "en_US";
try {
    if (top.polctx && top.polctx.lang)
        lang = top.polctx.lang;
    else
        lang = t_locale_id;
}
catch(e) {}

function ffsv_addTLNode(pbIn, nodeName) {
    if (null == pbIn.GetProperty(nodeName)) {
        top.gPB.GetElement(nodeName, pbIn);
    }
}

var g_ruleList = null;
var dataBindings = [];
var browserInfo = null;
var ffs_pb = null;
var g_psRules = null;
var g_disabled = false;

/**
 * Callback to restores any temporarily saved context
 */
function Restore() {
    getNewActions();
}

/**
 * Gets actions defined for a rule in the actions page
 */
function getNewActions() {
    var pb = g_currentState;
    if (!pb)
        return;
    var pp = new PropertyPath(PolSetting_GetPPNode("") + ".RuleRef");
    for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
        pp.SetTempLeaf("id");
        var rId = pb.GetPropertyDef(pp, "");
        if (top.polctx[rId]) {
            // An action has been saved in the policy context
            var actionStr = top.polctx[rId];
            delete top.polctx[rId];
            if (actionStr) {
                pp.SetTempLeaf("Actions.Action");
                pb.SetProperty(pp, actionStr);
                break;
            }
        }
        pp.SetTempLeaf("");
    }
}

/**
 * Callback to save context temporarily
 */
function TempSave(pb, pp) {
    ff_apply(pb, pp);
}

/**
 * Gets a list of defined file filtering rules;
 * @param pb the property bag to use (may be null - defaults to the global bag)
 * @return a list if Objects keyed by displayName, id and pp, where id is the ID
 * of the rule and pp its property path
 */
function ff_getRuleList(pb) {
    var ret = [];
    var basePP = new PropertyPath("FileRules.Rule");
    if (pb == null) {
        pb = top.gPB;
    }

    for (basePP.BeginEnum(); null != pb.GetProperty(basePP); basePP.IncrementEnum()) {
        basePP.SetTempLeaf("name");
        var name = resolveString(cse_pp_st, pb.GetPropertyDef(basePP, ""));
        basePP.SetTempLeaf("id");
        var ruleId = pb.GetProperty(basePP);
        basePP.SetTempLeaf("builtin");
        var builtIn = ("1" == pb.GetProperty(basePP));
        if (!name)
            name = ruleId;
        basePP.SetTempLeaf("");
        ret.push({displayName:name, id:ruleId, builtIn:builtIn, pp:basePP.GetPath()});
    }

    // Key the global rule list by ID
    var nRules = ret.length;
    for (var i = 0; i < nRules; i++) {
        var rInfo = ret[i];
        var id = rInfo.id;
        ret[id] = rInfo;
    }

    return ret;
}

//Sort function for rules
function ff_sortFn(a, b) {
    if ("final" == a.order || "first" == b.order) {
        // a comes last
        return 1;
    }
    else if ("final" == b.order || "first" == a.order) {
        // b comes last
        return -1;
    }
    return a.order - b.order;
}

/**
 * Gets the list of rules included in this settings instance
 */
function ff_getIncludedRules(init) {
    var ppT = PolSetting_GetPPNode("") + ".RuleRef";
    var pp = new PropertyPath(ppT.toString());
    var pb = init ? g_initialState : g_currentState;
    var ret = [];

    for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
        pp.SetTempLeaf("id");
        var id = pb.GetPropertyDef(pp, "");
        pp.SetTempLeaf("Order");
        var order = pb.GetPropertyDef(pp, "0");
        pp.SetTempLeaf("Actions.Action");
        var actVal = pb.GetPropertyDef(pp, DEF_FF_ACTION);
        pp.SetTempLeaf("");
        var actObj = new PolicyAction(pb, pp, false);
        actObj.BuildTable("Action", 0);
        var actList = actObj.GetVal(0);
        ret.push({id:id, order:order, actVal:actVal, actions:actList});
    }

    // Sort the rule list
    ret.sort(ff_sortFn);

    if (top.polctx.newFRule) {
        // A new rule has just been created - add it to the selected list
        var newRuleId = top.polctx.newFRule;
        ff_addRuleToSelected(newRuleId, ret);
        delete top.polctx.newFRule;
        top.gPageStack.Active().SetModified(true);
    }

    return ret;
}

/**
 * Creates the structure of the page
 */
function ff_makePage(pbIn) {
    // Create the structure for the alerts section
    ff_ShowAlerts();
    // Now create the alerts form
    alh_makeAlertSelector("ff", ALERT_PROP, ALERT_CAT, "ff_alert_settings");
    alh_init(pbIn, PolSetting_GetPPNode(""));
}

/**
 * Initializes data for the file filtering settings
 */
function ff_init(pbIn, disabled) {
    ffs_pb = pbIn;
    browserInfo = IdentifyBrowser();
    // Get the rules available
    g_ruleList = ff_getRuleList(pbIn);

    if (null == document.getElementById("ff_alert_settings")) {
        ff_makePage(pbIn);
    }
    var asCell = document.getElementById("ff_alert_settings");
    var cl = asCell.className;
    if (null == cl)
        cl = "";
    if (disabled)
        cl = addClass(cl, "disabled");
    else
        cl = removeClass(cl, "disabled");
    asCell.className = cl;
}

/**
 * Shows the alert settings for this instance
 */
function ff_ShowAlerts() {
    var div = document.getElementById("alert_div");
    if (!div)
        return;
    while(div.firstChild)
        div.removeChild(div.firstChild);

    var table = ff_createTable(resolveString(cse_pp_st, "__alert_selection"));
    var rowIdx = 1;
    var cellIdx = 0;
    var row = table.insertRow(rowIdx++);
    var cell = row.insertCell(cellIdx++);

    cell.id = "ff_alert_settings";

    ff_FixTable(table);
    div.appendChild(table);
}

/**
 * Creates a standardized table object
 */
function ff_createTable(headerText) {
    var table = document.createElement("table");
    table.border = table.cellPadding = table.cellSpacing = 0;
    var cellIdx = 0;

    var row = table.insertRow(0);
    var cell = row.insertCell(cellIdx++);
    cell.noWrap = true;
    var span = document.createElement("span");
    span.style.fontWeight = "bold";
    cell.appendChild(span);
    span.appendChild(document.createTextNode(headerText));
    return table;
}

/**
 * Fixes styles on the tables
 */
function ff_FixTable(table) {
    table.className = "dataSubTable";
    table.width = "100%";
    var rows = table.rows;
    var nRows = rows.length;
    if (nRows) {
        rows[0].className = addClass(rows[0].className, "header");
        var cells = rows[0].cells;
        if (cells.length > 1) {
            var cell = cells[cells.length - 1];
            cell.className = addClass(cell.className, "noLeft");
        }
        for (var i = 0; i < nRows; i++) {
            rows[i].className = addClass(rows[i].className, "row1");
            var cells = rows[i].cells;
            var nCells = cells.length;
            for (var j = 0; j < nCells; j++) {
                cells[j].className = addClass(cells[j].className, "dst");
            }
            var cell = cells[nCells - 1];
            cell.className = addClass(cell.className, "eor");
        }
        var row = rows[nRows - 1];
        row.className = addClass(row.className, "lastRow");
    }
}

/**
 * Shows the file filtering rules included in this instance
 */
function ff_ShowRules(disabled, globalProperties) {
    var div = document.getElementById("rules_div");
    if (!div)
        return;
    while(div.firstChild)
        div.removeChild(div.firstChild);

    var table = ff_createTable(resolveString(cse_pp_st, "__ff_rules_table_header"));
    table.style.marginTop = "10px";
    table.id = RULES_TABLE_ID;

    var row = table.rows[0];
    var cells = row.cells;
    var cellIdx = 0;
    // The table header text spans two or three columns, depending on whether the
    // page is in a read-only state
    var cell = cells[cellIdx++];
    cell.colSpan = 2;

    if (!disabled) {
        // Add the rule selector
        cell = row.insertCell(cellIdx++);
        cell.className = "rightAligned";
        cell.colSpan = 2;
        cell.noWrap = true;
        ff_createRuleSelector(cell);
    }

    // Add a row for when there are no rules to display
    var rowIdx = 1;
    row = table.insertRow(rowIdx++);
    row.style.display = "none";
    // Specify the classname here rather than depending on ff_fixTable
    // because this row is only displayed when there are no rules selected
    row.className = "lastRow";
    cell = row.insertCell(0);
    // This cell needs to span all columns
    cell.colSpan = disabled ? 2 : 4;
    cell.appendChild(document.createTextNode(resolveString(cse_pp_st, "__no_rules_selected")));

    // Create as many rows are there are available rules
    var nRules = g_ruleList.length;
    for (var i = 0; i < nRules; i++) {
        row = table.insertRow(rowIdx++);
        row.style.display = "none";
        var cellIdx = 0;
        // Name
        var cell = row.insertCell(cellIdx++);
        cell.noWrap = true;
        // Actions
        row.insertCell(cellIdx++);
        if (!disabled) {
            // Up-down controls
            cell = row.insertCell(cellIdx++);
            cell.style.width = "7%";
            cell.align = "center";
            // Options
            cell = row.insertCell(cellIdx++);
            row.style.width = "18%";
        }
    }

    ff_FixTable(table);
    div.appendChild(table);
    g_disabled = disabled;
    ff_drawRules();
}

/**
 * Creates the rule select controls
 * @param e the element that will house the controls
 */
function ff_createRuleSelector(e) {
    e.appendChild(document.createTextNode(resolveString(cse_pp_st, "__ff_rulelist")));
    var select = document.createElement("select");
    select.style.marginLeft = "5px";
    select.style.width = "200px";
    select.id = RULE_ADD_SEL_ID;
    var fnStr = "ff_addRule();";
    select.onchange = new Function(fnStr);
    // Create a dummy option
    var opt = document.createElement("option");
    opt.value = "";
    opt.text = resolveString(cse_pp_st, "__ff_selectOne");
    //opt.disabled = true;
    if (browserInfo.isIE)
        select.add(opt);
    else
        select.add(opt, null);
    select.selectedIndex = 0;

    // Add only the rules that are not currently included to the pick list
    var nRules = g_psRules.length;
    var rMap = {};
    for (var i = 0; i < nRules; i++) {
        var rInfo = g_psRules[i];
        rMap[rInfo.id] = 1;
    }

    nRules = g_ruleList.length;
    for (var i = 0; i < nRules; i++) {
        var rInfo = g_ruleList[i];
        if (rMap[rInfo.id])
            continue;
        opt = document.createElement("option");
        opt.value = rInfo.id;
        opt.text = rInfo.displayName;
        if (browserInfo.isIE)
            select.add(opt);
        else
            select.add(opt, null);
    }
    ff_addCreateNewOption(select);

    e.appendChild(select);
}

/**
 * Adds a "create new" option to the rule selector
 */
function ff_addCreateNewOption(select) {
    // Add a create new option to the list
    var opt = document.createElement("option");
    opt.value = "";
    opt.text = resolveString(cse_pp_st, "__create_new_rule");
    if (browserInfo.isIE)
        select.add(opt);
    else
        select.add(opt, null);
}

function isBuiltIn(ruleId) {
    var ret = false;
    if (g_ruleList[ruleId]) {
        ret = g_ruleList[ruleId].builtIn;
    }
    return ret;
}

/**
 * Draws the currently selected rules
 */
function ff_drawRules() {
    var rulesTable = document.getElementById(RULES_TABLE_ID);
    if (!rulesTable) {
        return;
    }
    var nbspStr = String.fromCharCode(NBSP);
    var disabled = g_disabled;
    var nRows = rulesTable.rows.length;
    var nRules = g_psRules.length;
    if (0 == nRules) {
        // Report that there is nothing to display; this string is in a hidden
        // row at the start of the table
        rulesTable.rows[1].style.display = "";
        for (var i = 2; i < nRows; i++)
            rulesTable.rows[i].style.display = "none";
        return;
    }

    var startIdx = nRows - nRules;
    var currIdx = startIdx;

    // There are "built-in" rules that may not be moved so we need to determine
    // the final index of movable rules which may not be the number of rules
    var maxRule = nRules - 1;
    for (var i = nRules - 1; i >= 0; i--, maxRule--) {
        var rInfo = g_psRules[i];
        if (!isBuiltIn(rInfo.id)) {
            break;
        }
    }

    for (var i = 0; i < nRules; i++) {
        var rInfo = g_psRules[i];
        var row = rulesTable.rows[currIdx++];
        var cells = row.cells;
        ff_clearElement(cells[0]);
        var ruleName = rInfo.id;
        if (g_ruleList[ruleName]) {
            ruleName = g_ruleList[ruleName].displayName;
        }
        var a = document.createElement("a");
        if (disabled || isBuiltIn(rInfo.id)) {
            cells[0].appendChild(document.createTextNode(ruleName));
        }
        else {
            a.className = "blueLink";
            a.href = "javascript:ff_editRule('" + rInfo.id + "');";
            a.appendChild(document.createTextNode(ruleName));
            cells[0].appendChild(a);
        }

        ff_clearElement(cells[1]);
		cells[1].appendChild(document.createTextNode(rInfo.actions));

		var bInheritMasterPolicy = PolSetting_CheckInherit();
        if(false == bInheritMasterPolicy)
        {
			cells[1].appendChild(document.createTextNode(" ("));
			var a = document.createElement("a");
			a.className = "blueLink";
			var url = "GenActions.html";
			url = uriAppend(url, "type", "FF");
			url = uriAppend(url, "lang", lang);
			url = uriAppend(url, "showAlerts", "0");
			url = uriAppend(url, "insert", EscapeForJS(ruleName));
			url = uriAppend(url, "fromPB", "0");
			a.href = "javascript:ff_editActions('" + rInfo.id + "',%20'" +
				rInfo.actVal + "',%20'" + url + "');";
			a.appendChild(document.createTextNode(resolveString(cse_pp_st, "__change")));
			cells[1].appendChild(a);
			cells[1].appendChild(document.createTextNode(")"));
        }
        if (!disabled) {
            ff_clearElement(cells[2]);
            // Up down controls
            var UP = "../../images/icons/move_up.gif";
            var UPD = "../../images/icons/move_up_disabled.gif";
            var DOWN = "../../images/icons/move_down.gif";
            var DOWND = "../../images/icons/move_down_disabled.gif";
            var iW = 11;
            var iH = 9;

            if (isBuiltIn(rInfo.id)) {
                cells[2].appendChild(document.createTextNode(nbspStr));
            }
            else {
                if (0 != i) {
                    a = document.createElement("a");
                    a.href = "javascript:ff_moveRule(" + i + ", true);";
                    a.appendChild(ff_makeImg(UP, iW, iH, resolveString(cse_pp_st, "__move_up")));
                    cells[2].appendChild(a);
                }
                else {
                    cells[2].appendChild(ff_makeImg(UPD, iW, iH, ""));
                }
                cells[2].appendChild(document.createTextNode(" "));
                if (i != (maxRule)) {
                    a = document.createElement("a");
                    a.href = "javascript:ff_moveRule(" + i + ", false);";
                    a.appendChild(ff_makeImg(DOWN, iW, iH, resolveString(cse_pp_st, "__move_down")));
                    cells[2].appendChild(a);
                }
                else {
                    cells[2].appendChild(ff_makeImg(DOWND, iW, iH, ""));
                }
            }

            ff_clearElement(cells[3]);
            if (isBuiltIn(rInfo.id)) {
				a = document.createElement("a");
                a.className = "blueLink";
                a.href = "javascript:ff_deleteRule(" + i + ");";
                a.appendChild(document.createTextNode(resolveString(cse_pp_st, "__delete")));
                cells[3].appendChild(a);
            }
            else {
                a = document.createElement("a");
                a.className = "blueLink";
                a.href = "javascript:ff_deleteRule(" + i + ");";
                a.appendChild(document.createTextNode(resolveString(cse_pp_st, "__delete")));
                cells[3].appendChild(a);
            }
        }
    }

    // Hide the rows that are not used
    for (var i = 1; i < startIdx; i++) {
        rulesTable.rows[i].style.display = "none";
    }
    // Show the remainder
    for (var i = startIdx; i < nRows; i++) {
        rulesTable.rows[i].style.display = "";
    }
}

function ff_clearElement(e) {
    if (!e)
        return;
    while(e.firstChild)
        e.removeChild(e.firstChild);
}

function ff_makeTextSpacer() {
    return document.createTextNode(" " + String.fromCharCode(NBSP) + " ");
}

function ff_makeDisabledText(text) {
    var span = document.createElement("span");
    span.className = "disabled";
    span.appendChild(document.createTextNode(text));
    return span;
}

function ff_makeImg(src, w, h, alt) {
    var img = document.createElement("img");
    img.src = src;
    img.width = w;
    img.height = h;
    img.border = 0;
    img.alt = alt;
    if (alt)
        img.title = alt;
    return img;
}

/**
 * Adds a rule to the included rule list from the available rules
 */
function ff_addRule() {
    var sel = document.getElementById(RULE_ADD_SEL_ID);
    if (!sel)
        return;

    // Check for the create new option
    var selIdx = sel.selectedIndex;
    if (selIdx == (sel.length - 1)) {
        ff_editRule("");
        return;
    }

    var newRuleId = sel.value;
    if (!newRuleId)
        return;

    // Add the new rule to the included list
    ff_addRuleToSelected(newRuleId);

    // Remove the option from the rule picker
    sel.remove(sel.selectedIndex);
    // Reset the selected index
    sel.selectedIndex = 0;

    // Set modified flag on the page
    top.gPageStack.Active().SetModified(true);

    // Redraw the rules
    ff_drawRules();
}

/**
 * Adds a rule to the selected rules
 * @param newRuleId the the ID of the rule to add
 * @param selectedList the list to add to (may be null)
 */
function ff_addRuleToSelected(newRuleId, selectedList) {
    if (!selectedList)
        selectedList = g_psRules;
    if (!newRuleId)
        return;
    var actVal = DEF_FF_ACTION;
    var acts = actVal.substring(1, actVal.length - 1).split(",");
    var nActs = acts.length;
    for (var i = 0; i < nActs; i++) {
        acts[i] = a_getLabel("FF", acts[i]);
    }
    var actList = acts.join(", ");
    var order = selectedList.length;
    selectedList.unshift({id:newRuleId, order:order, actVal:actVal, actions:actList});
}

/**
 * Removes a rule from the included rules (and adds it to the available rule picker
 */
function ff_deleteRule(idx) {
    var nRules = g_psRules.length;
    if (idx < 0 || idx >= nRules)
        return;
    var rInfo = g_psRules[idx];
    // Remove the selected rule
    g_psRules.splice(idx, 1);

    var select = document.getElementById(RULE_ADD_SEL_ID);
    if (select) {
        // Temporarily remove the create new option
        select.remove(select.length - 1);
        // Add the deleted rule to the available rules picker
        var opt = document.createElement("option");
        opt.value = rInfo.id;
        var rName = rInfo.id;
        if (g_ruleList[rName]) {
            rName = g_ruleList[rName].displayName;
        }
        opt.text = rName;
        if (browserInfo.isIE)
            select.add(opt);
        else
            select.add(opt, null);

        // Restore the create new option
        ff_addCreateNewOption(select);
    }

    // Set modified flag on the page
    top.gPageStack.Active().SetModified(true);

    // Redraw the rules
    ff_drawRules();
}

/**
 * Changes the order of the included rules
 */
function ff_moveRule(idx, up) {
    var minIdx = up ? 1 : 0;
    var nRules = g_psRules.length;
    var maxIdx = up ? nRules : (nRules - 1);
    if (idx < minIdx || idx >= maxIdx)
        return;

    // Swap the selected rule with its neighbour
    var newIdx = up ? (idx - 1) : (idx + 1);
    var tmp = g_psRules[newIdx];
    g_psRules[newIdx] = g_psRules[idx];
    g_psRules[idx] = tmp;

    // Set modified flag on the page
    top.gPageStack.Active().SetModified(true);

    // Redraw the rules
    ff_drawRules();
}

/**
 * Edits the actions when a rule is triggered
 * @param id the ID of the RuleRef being edited
 * @param actVal the current action value
 * @param url the URL of the actions page
 */
function ff_editActions(id, actVal, url) {

	url = "GenActions.html";
	url = uriAppend(url, "type", "FF");
	url = uriAppend(url, "lang", lang);
	url = uriAppend(url, "showAlerts", "0");
	url = uriAppend(url, "insert", g_ruleList[id].displayName);
	url = uriAppend(url, "fromPB", "0");
    url = uriAppend(url, "actionKey", id);
    // Save the old action string in policy context
    top.polctx[id] = actVal;
    // Do a temporary save of the current page state
    alh_tSave();
    var title = resolveString(cse_pp_st, "ACT_PAGE_TITLE", [resolveString(cse_pp_st, "FF")]);
    var page = new DialogPage(url, title);
    top.gPageStack.Forward(page, "");
}

/**
 * Edits a filtering rule
 * @param id the ID of the rule to be edited
 */
function ff_editRule(id) {
    // Look up the property path of the rule to be edited
    var rPP = "";
    if (id) {
        var rInfo = g_ruleList[id];
        if (null == rInfo)
            return;
        rPP = rInfo.pp;
    }
    else {
        // New rule
        var basePP = "FileRules.Rule";
        // Get the next enumeration
        var rPP = ffs_pb.GetNextElementEnum(basePP);
    }
    var url = "FileRule.html";
    // Do a temporary save of the current page state
    alh_tSave();
    var title = resolveString(cse_pp_st, "File");
    var page = new DialogPage(url, title);
    top.gPageStack.Forward(page, rPP);
}

/**
 * Option select callback (should, in fact, never be called)
 */
function PolSetting_FF_SelectFromOptions(c) {

}

/**
 * Inherit checkbox callback
 */
function OnclickInherit(val) {
    PolSetting_SetModify();
    var inherited = PolSetting_CheckInherit();
    ff_init(ffs_pb, inherited);
    Reset(inherited, false);
}

/**
 * Show callback
 */
function Show(pbIn, pp) {
    alh_onShow();
    // There are no policy independent options, so hide the options
    // selector
    PolSetting_Show(pbIn);
    for (var i = g_tlNodes.length - 1; i >= 0; i--) {
        ffsv_addTLNode(pbIn, g_tlNodes[i]);
    }
    var inherited = PolSetting_CheckInherit();
    ff_init(pbIn, inherited);
    Reset(inherited, false);
}

function Reset(inherited, init) {
    var ppT = PolSetting_GetPPNode("");
    var pp = new PropertyPath(ppT.toString());
    var pb = alh_getPBToUse(inherited, init);
    // Get the rules included in this instance
    g_psRules = ff_getIncludedRules(init);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(pp, pb);
    }
    ff_ShowRules(inherited, ffs_pb);
    alh_reset(pb, pp, inherited, init);
    // Set the selected alert
    alh_setSel2("ff", ALERT_PROP, ALERT_CAT, pb, pp);
    gPolicyAlerts.DisableAlertSel(inherited);
}

/**
 * Cancel callback
 */
function Cancel() {
    PolSetting_Draw();
    var inherited = PolSetting_CheckInherit();
    Reset(inherited, true);
    top.gPageStack.Active().SetModified(false);
}

/**
 * Validate callback
 */
function Validate() {
    return true;
}

/**
 * Save callback
 */
function Apply(pb, pp, save) {
    if (!pp || !pp.GetPath())
        pp = new PropertyPath(PolSetting_GetPPNode(""));
    pb = PolSetting_GetPb();
    ff_apply(pb, pp);
    PolSetting_Apply("", true);
    alh_apply(pb, pp);
    if (!save)
        return top.gPageStack.Active().pb;
}

/**
 * Local data save pulled out into its own function so that it can be
 * used for temporary saves as well
 */
function ff_apply(pb, pp) {
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].get(pp, pb);
    }
    // Get the selected alert
    alh_getSel2("ff", ALERT_PROP, ALERT_CAT, pb, pp);
    // Get the rules included in this instance
    pp.SetTempLeaf("RuleRef");
    pb.DeleteList(pp);
    var nRules = g_psRules.length;
    for (var i = 0; i < nRules; i++) {
        var ppT = pb.GetNextElementEnum(pp);
        pb.SetProperty(ppT);
        var rInfo = g_psRules[i];
        pb.SetProperty(ppT + ".id", rInfo.id);
        pb.SetProperty(ppT + ".Order", (i + 1));
        pb.SetProperty(ppT + ".Actions");
        pb.SetProperty(ppT + ".Actions.Action", rInfo.actVal);
    }
    pp.SetTempLeaf("");
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["DefaultAlerts"];
}


