// Copyright (C) 2009 McAfee, Inc.  All rights reserved.
//
// Description: Generic Functions to read actions from and to property bag
// used in PolSettingView pages
// Assumes the cse_pp_actions_st stringtable (from PolicyRes.js file) is included in the html page.
//
// History:  (Author/Comment)

//global variables
gActionsDataBindings=[];
var gPolAction;

//class functions
//=======================

//create a display html string out of the actions data from config.
PolicyAction.MakeHtml = function(str, settingsType, actionName) {
	
	var bits = str.split(',')
	var res="";

    for (var i=0; i< bits.length; i++ )
    {   if(bits[i]=="") continue;
		if (res!="") res+=', ';
        if (!settingsType)
        	settingsType = "";
        res += a_getLabel(settingsType, bits[i], actionName);
    }
	if (!res)
	   res=" - "

	
    return res;
}

//constructor
// parameters:
// pp and pb of the policySetting
// setlocal is true if object should check the existence of a local pb (should be true when call
// is from am action page, typically false when call is from a settings page)


function PolicyAction(parent_pb, ppIn, setlocal)
{

   this.pb=parent_pb;
   this.dataBindings=gActionsDataBindings;

   if ("string" == typeof(ppIn))
   {  this.pp=new PropertyPath (ppIn);
   }
   else
   {  this.pp= new PropertyPath (ppIn.GetPath());
   }
   this.pp.Append("Actions");
   if (setlocal)
   {
        if (top.polctx.polSetting["pb"] != undefined)
        {   this.pb= PropertyBag.Recreate(top.polctx.polSetting["pb"]);
            top.polctx.polSetting["pb"]=this.pb;
        }
   }
   this.table=new Array();
}

PolicyAction.Recreate = function(old)
{
    var PA = new PolicyAction(old.pb, old.pp.path, false)
    PA.table = old.table;
    return PA;
}


//reset pp to a new property path
//pp_str is the path to the parent node(settings or rules)
PolicyAction.prototype.SetPP = function(pp_str)
{
   this.pp= new PropertyPath (pp_str);
   this.pp.Append("Actions");
}

PolicyAction.prototype.GetProp = function(leaf)
{
   	this.pp.SetTempLeaf(leaf);
	var prop = this.pb.GetPropertyDef(this.pp, "");
    this.pp.SetTempLeaf("");
    return prop;
}

PolicyAction.prototype.SetProp = function(leaf, val)
{
   	this.pp.SetTempLeaf(leaf);
	this.pb.SetProperty(this.pp.GetPath(), val)
    this.pp.SetTempLeaf("");
}

PolicyAction.prototype.GetLength = function()
{   return this.table.length;
}
PolicyAction.prototype.GetKey = function(pos)
{   return this.table[pos].key;
}
PolicyAction.prototype.GetVal = function(pos)
{   return this.table[pos].val;
}
PolicyAction.prototype.GetPath = function(pos)
{   return this.table[pos].path;
}
PolicyAction.prototype.GetId = function(pos)
{   return this.table[pos].id;
}

//BuildTable
//Create a table of (action property leave, action string) pairs
PolicyAction.prototype.BuildTable = function(leave, pos, settingsType)
{
    this.pp.SetTempLeaf(leave);
    var oProp=this.pb.GetProperty(this.pp);

	var res=oProp.slice(1,oProp.length-1);
	this.table[pos]= {key:leave, val:PolicyAction.MakeHtml(res, settingsType, leave) };

	return this.table.length;
}//BuildTableStd

//BuildTableAV
//Create a table of (action property leave, action string) pairs
//Where the action could be a (more complex) AV action
PolicyAction.prototype.BuildTableAV = function(leave, pos)
{
   this.pp.SetTempLeaf(leave);

   var oProp=this.pb.GetProperty(this.pp);
   var res, clean, right;
   var o = new Object();

   res=oProp.split("(");
   left=res[0].split(",");
   success= res[0].slice(left[0].length, res[0].length-1)
   fail=res[1].slice(0,res[1].length-1);


   if (left[0].length>1)
   {    this.table[pos]=   {key:leave,      val:PolicyAction.MakeHtml( left[0]) };
        this.table[pos+1]= {key:leave+"_1", val:PolicyAction.MakeHtml( success) };
        this.table[pos+2]= {key:leave+"_2", val:PolicyAction.MakeHtml( fail) };
   }
   else
   {    this.table[pos]= {key:leave, val: PolicyAction.MakeHtml(fail)};
   }
   return this.table.length;

}//BuildTableAV



//BuildRulesTable
//Create a table of (rule label, action string, id)
PolicyAction.prototype.BuildRulesTable = function(name, type, pos, id, pp_str)
{
    this.SetPP(pp_str)
    var oProp = this.GetProp("Action");

    var res=oProp.slice(1,oProp.length-1);
	var bits = res.split('{')
	
	if(bits[1] != McUndefined)
		res = bits[0].slice(0,bits[0].length-1);
	
	
	 
    this.table[pos]= {key:resolveString(cse_pp_st, type)+ name, val:PolicyAction.MakeHtml(res, "CS"), id:id, path:pp_str, del:0};
    return this.table.length;

}//BuildRulesTable

PolicyAction.prototype.MarkDelete = function(pos)
{
    this.table[pos].del=1;
}
PolicyAction.prototype.Deleted = function(pos)
{
    return this.table[pos].del;
}


PolicyAction.prototype.Show = function( )
{
    for (var i = 0; i < this.dataBindings.length; i++)
    {   this.dataBindings[i].set(this.pp, this.pb);
    }

    //enable cancel even though nothing has changed yet
    var f = top.frames["frame_buttons"];
    if (f) {
        var b = f.document.getElementById("configCancel");
        if (b)
            b.disabled = false;
    }
}

PolicyAction.prototype.Apply = function()
{
	var changed=false;

    for (var i = 0; i < this.dataBindings.length; i++)
	{
        if (this.dataBindings[i].get(this.pp, this.pb))
		{
            changed = true;
        }
    }
    return changed;

}

PolicyAction.prototype.LocalStore = function(temp_pb)
{
    for (var i = 0; i < this.dataBindings.length; i++)
	{
        this.dataBindings[i].get(this.pp, temp_pb);
    }
}

PolicyAction.prototype.LocalRestore = function(temp_pb)
{
    for (var i = 0; i < this.dataBindings.length; i++)
	{
        this.dataBindings[i].set(this.pp, temp_pb);
    }
}


//check if the_ctl has the required value to display alerts
function PolicyAction_onclick(the_ctl, alert_id)
{
	var alert_div= document.getElementById(alert_id);
	var row_div= document.getElementById(alert_id+"_row");

	if(!alert_id) return;

    if (the_ctl.value=="ESERVICES:REPLACE")
    {   if (alert_div) alert_div.style.display="";
 	    if (row_div) row_div.style.display="";
    }
	else
    {   if (alert_div) alert_div.style.display="none";
	    if (row_div) row_div.style.display="none";
	}
}

//Launch the alert edit page
//params:
//@pol_alerts : a PolicyAlert object (eg gPolicyAlerts)
//@ae_id : the id referencing the alert in the page
//@create : boolean true if the page will edit a new alert

function PolActions_EditAlert( pol_alerts, ae_id, create)
{
     top.polctx.polSetting["tmp_act_pb"]=new PropertyBag();
	 top.polctx.polSetting["tmp_alerts"]=[];

     gPolAction.LocalStore( top.polctx.polSetting["tmp_act_pb"]);
	 if(pol_alerts)
	 {  pol_alerts.LocalStore(top.polctx.polSetting["tmp_alerts"]);
        pol_alerts.EditAlert(ae_id, create);
     }
}

function PolActions_LocalRestore( pol_alerts)
{
	if (top.polctx.polSetting["tmp_act_pb"])
    {  var pbT=PropertyBag.Recreate(top.polctx.polSetting["tmp_act_pb"]);
       gPolAction.LocalRestore(pbT);
       top.polctx.polSetting["tmp_act_pb"]=null;
	}

	if(pol_alerts && top.polctx.polSetting["tmp_alerts"])
    {  pol_alerts.LocalRestore(top.polctx.polSetting["tmp_alerts"]);
	   top.polctx.polSetting["tmp_alerts"]=null;
	}
}


