// Copyright (C) 2009 McAfee, Inc.  All rights reserved.
//
// Description:
// Javascript functions to help displaying the PolicySetPage.html
// Included in:
// - PolicySetPage.html
// Assumes that the page first includes Policy.js
//
//====================================================================================
// Consts
//====================================================================================
var K_DELETED = 2
var K_ROOT = 1
var K_NOROOT  = 0

//====================================================================================
// globals and flags for this page
//====================================================================================
//copy of the property path and property bag, re-initialised to the right policyset in the FindSet function
var gTopNode = "Policies";
var gPolicySet_ppstr = "";
var gPolicies_pb = null;

//====================================================================================
// Data Structures
//====================================================================================

//store policy management related data in the top frame
//re init always at page load
//myPolicySet is created in the html page
top.polctx=new Object();
top.polctx.myPolicySet=null;

/*
 * Constructor for the PolicyStatement class
 * members:
 * id: unique identifier for the policy
 * name : a user defined name for this Policy Statement
 * description : a user defined desription for this policy
 * order : the priority rank (lowest order is lowest priority)
 * parent : the parent policy (should be part of this table!)
 * active : boolean - 0=disabled; 1=enabled
 * root : 0=can be deleted or edited - 1=fixed value
 * deleted : marked as deleted
 * indent : level of depth in the tree (root=0)
 * time_restriction: string indicating what the time restriction is
 * nb_sub : nb of sub policies
 * parent_name : name of parent policy (for display purpose)
 */

function PolicyStatement(id, name, description, order, parent_id, active, root) {
	this.id = id;
    this.name = name;
    this.description = description;
    this.order = order;
    this.parent_id = parent_id;
	this.active = active;
	this.root = root;

	this.deleted=0;
	this.indent=0;
	this.time_restriction="";
	this.nb_sub=0;
	this.parent_name="";
	this.pp_path=""
}//PolicyStatement


/*
 * Constructor for the PolicySet class
 * members:
 * policies: array of policy statements
 * listNdx: array indexing the policies by priority
 * treeNdx: array indexing the policies by inheritance
 *
 * pp_key: name of policyset in the config file (used as key)
 * confirm_delete : callback function in the html page to ask confirmation (returns 1==ok)
 * selection : ndx of selected policystatement (default is 0)
 */

function PolicySet( key )
{
    this.pp_key=key;  //policy set key
	this.display_name = resolveString(cse_pp_ps_st, key);
    this.policies;
    this.listNdx;
	this.treeNdx;
	this.last_tree_item=0;
	this.last_list_item=0;
	this.selection=0;
	this.objectName="top.polctx.myPolicySet";

	this.pop_div="POP_DIV";
    this.tree_div="DATA_tree_div";
    this.tree_title=resolveString(cse_pp_st,"__set_title1");
	this.list_div="DATA_list_div";
	this.list_title=resolveString(cse_pp_st,"__set_title2");
	this.label_name=resolveString(cse_pp_st,"__name");
	this.label_priority=resolveString(cse_pp_st,"__priority");
	this.label_parent=resolveString(cse_pp_st,"__parent_policy");
	this.label_enable=resolveString(cse_pp_st,"__enabled");
	this.label_move=resolveString(cse_pp_st,"__move");
	this.label_delete=resolveString(cse_pp_st,"__delete");

}//PolicySet

//====================================================================================
// utilities - Private Functions - non display
//====================================================================================


/*
 * Insert an element in the inheritance tree array
 * Does not change other indexes or data array.
 * params:
 * insert_pt: position where the value will sit after insertion
 * value: value to be inserted
 */

PolicySet.prototype.insert = function( insert_pt, value )
{
    var len= this.treeNdx.length;
	if (insert_pt>=len) return;

	if (this.treeNdx[insert_pt]<0) this.treeNdx[insert_pt]=value;
	else
    {
	  for (i=1; i<len;i++ )
	  {    j=len-i-1;

		   if (this.treeNdx[j]<0)
		   {  //do nothing
		   }
		   else if(j>=insert_pt)
		   {     this.treeNdx[j+1]=this.treeNdx[j];
           }
	  }
	  this.treeNdx[insert_pt]=value;
    }

}//insert

/*
 * Appends a PolicyStatement to the policySet
 * Does not change the indexes
 */
PolicySet.prototype.add = function(id, name, description, order, parent_id, active, root) {
    var pol = new PolicyStatement(id, name, description, active, order, parent_id, root);
    this.policies.push(pol);
}

/*
 * order the policies by priority by populating the listNdx array
 * if priority is duplicated or invalid, returns an error
 * priority is the reverse order as the "Order" parameter
 */
PolicySet.prototype.order = function() {
    var ndx = 0;
	var pos=0;
	var error=0;
	var len= this.policies.length;

    //init to -1 value helps checking for inconsitencies
    for(ndx=0;ndx<len;ndx++){
		this.listNdx[ndx]=-1;
    }

	//set up the array
    for(ndx=0;ndx<len;ndx++)
	{
        if(this.policies[ndx].deleted==1) continue;

		pos=this.policies[ndx].order;

		if (this.policies[ndx].root==K_ROOT && pos!=1)
		{  //root is not the right order
		   this.policies[ndx].order=0;
		   error=1;
		}
		else if (pos<1 || pos>len)
		{   //value is invalid
			error=1
		}
		else if (this.listNdx[len-pos]>-1)
		{   //value "+pos+" already used
			error=1;
		}
		else
		{
		    this.listNdx[len-pos]=ndx;
        }
    }

	return error;
}//PolicySet.order



/*
 * Populate the TraverseNdx array with indices of policies such that given a policy its parent is above it.
 * TraverseNdx would contain the indices of the Policies array in a particular order.This order ensures that one finds the parent of a given policy only above it.
 */

PolicySet.prototype.fillTraverseNdx = function(TraverseNdx)
{
	var i=0;
	var j=0;
	var k=0;
	var end=this.policies.length;	//keeps a tab on the end index.
    var parent;
	var child;
	end = end-1;  //logic

	for(i=1; i<=end;i++)
		 for(j=i+1;j<=end;j++)
			if(this.policies[TraverseNdx[i]].parent_id==this.policies[TraverseNdx[j]].id)
				{
				   	child=TraverseNdx[i];
					parent=TraverseNdx[j];

					//insertion moving the child below the parent.
				     if(j==end)
						 TraverseNdx[j+1]=child;    //moving the child
					 else
						{
						 for(k=end;k>j;k--)
							TraverseNdx[k+1]=TraverseNdx[k];  //move the array downwards , to make space for the child to be inserted below the parent.

						 TraverseNdx[j+1] = child;       //moving the child
						}

				   	//reshuffle the TraverseNdx Array
					  for(k=i+1;k<=(end+1);k++)
						  TraverseNdx[k-1]=TraverseNdx[k];              //move array upwards to fill the space of the child which has been moved below its parent.

					  i--;  // change the index.
					  break;      //from the inner loop.
				}


}//fillTraverseNdx


/*
 * sort the policies by inheritance
 * by populating the treeNdx array
 */

PolicySet.prototype.tree = function() {


	var ndx = 0;
    var pos=0;
    var parent_id="";
    var error=0;
    var len= this.policies.length;
	var family=""


	var i=0;		//for loops
	var key=0;       //key used for indexing the policies array.

	var TraverseNdx=new Array(); //array of keys(indices) used to traverse the policies array effectively , so that a given a policy its parent is found above it.
	for(i=0;i<len;i++)
	{
		TraverseNdx[i]=i;         //initial filling up of the array
	}


	this.fillTraverseNdx(TraverseNdx);     // changes are made to the TraverseNdx array if necessary.


	//init
    for(ndx=0;ndx<len;ndx++)
    {   this.treeNdx[ndx]=-1;
    }

	//set up the array
    for(ndx=0;ndx<len;ndx++)
    {

		 key=TraverseNdx[ndx];    // indexing the policies using the keys that are the values present in the TraverseNdx array.

		//don't insert element if it is deleted.
		if(this.policies[key].deleted==1) continue;

        //initialise before loop
        parent_id=this.policies[key].parent_id;
		family="-"+parent_id+"-";

		var i;
		var found=0;
		var indent=0;
		var parent_name="";

        for(pos=0;pos<len;pos++)
        {
           if (this.treeNdx[pos]<0)
           {   //append in empty space
			   this.treeNdx[pos]=key;
			   break;
           }
		   else if (this.policies[this.treeNdx[pos]].id == parent_id)
		   {   //parent found
			   found=1;
               this.policies[this.treeNdx[pos]].nb_sub++;
			   indent = this.policies[this.treeNdx[pos]].indent+1;
			   parent_name = this.policies[this.treeNdx[pos]].name;
		   }

           else if ( (family.indexOf("-"+this.policies[this.treeNdx[pos]].parent_id+"-")+1) > 0)
           {   //if pos.parent in family, so add pos to family
               family+=this.policies[this.treeNdx[pos]].id;
			   family+="-";
           }
		   else if (found==1)
		   {   //we passed the parent section and enter a new section
		       //so we insert here
               this.insert(pos, key);
               break;
		   }
        }

        //update indent and parent_name
        this.policies[key].indent=indent;
        this.policies[key].parent_name=parent_name;
    }
}//tree

/*
 * Reset priorities - in case there is corruption in the config file.
 * Resets order values as the order of the policies in the list,
 * root being the lowest order
 */
PolicySet.prototype.resetOrder = function()
{
	var lenTree=0;
	var lenPol= this.treeNdx.length;
	var i

    //init
    for(i=0;i<lenPol;i++)
    {   this.policies[i].order=i+1;
    }

   this.order();
}

/*
 * Reset priorities with values from 1 to len
 */
PolicySet.prototype.normalOrder = function()
{

	alert("normalizing sub-policies priority order...");

	var i;
	var len= this.policies.length;
	var lowest, low, pos,ndx;
	var current=-1;
	var new_order= new Array(len);

    low= 1;
	lowest = 0;
	pos=0;
	new_order[0]=0;

	//for all elements, pick the next lowest one and store it in new_order
	for(i=0;i<len;i++)
    {
       new_order[pos] = -1;
	   //look for the next lowest order
	   for(ndx=0;ndx<len;ndx++)
	   {
		   current =  this.policies[ndx].order;

           //first time round, the very lowest
           if (i==0 &&  (ndx==0 || current<lowest) )
		   {   lowest=current;
		       low=current;
			   new_order[pos]=ndx;
		   }
           else
           {   //initialise new_order with ndx if no valid ndx found yet
			   if (new_order[pos]==-1)
			   {   new_order[pos]=ndx;
			   }

			   //if current is higher than lowest but lower than current low, pick it up
               if (current>=lowest && current<=low )
		       {
                  low=current
                  new_order[pos]=ndx;
		       }
           }
       }

	   this.policies[new_order[pos]].order=lowest-1;
	   low++;
	   pos++;

    }

	//now reset order
	for(pos=0;pos<len;pos++)
    {  this.policies[new_order[pos]].order=pos+1;
    }

    this.order();
}

/*
 * Move up and down  the priority of a PolicyStatement
 * Assumes that the PolicySet is populated
 * parameter:
 *  value: current order [1..len]
 */
PolicySet.prototype.moveUp = function( value )
{
    var pos = this.listNdx.length - value;
	//check we can move Down
	if (pos <1)
	{   return;
	}
    if (pos>=this.listNdx.length)
    { return;
    }


	//find relevant policyStatements index [0..len-1]
    var current_ndx = this.listNdx[pos];
	var next_ndx= this.listNdx[pos-1];


    //swap
	var tmp_value = this.policies[next_ndx].order;
	this.policies[next_ndx].order=value;
	this.policies[current_ndx].order=tmp_value;

	this.listNdx[pos]=next_ndx;
	this.listNdx[pos-1]=current_ndx;

}//moveUp

PolicySet.prototype.moveDown = function( value )
{
    var pos = this.listNdx.length - value;
	//check we can move Down
	if (pos <0)
	{   return;
	}
    if (pos>=this.listNdx.length+1)
    { return;
    }

	//find relevant policyStatements index [0..len-1]
    var current_ndx = this.listNdx[pos];
	var next_ndx= this.listNdx[pos+1];

    //swap
	var tmp_value = this.policies[next_ndx].order;
	this.policies[next_ndx].order=value;
	this.policies[current_ndx].order=tmp_value;

	this.listNdx[pos+1]=current_ndx;
	this.listNdx[pos]=next_ndx;

}//moveDown

/*
 * mark policy as deleted
 * subpolicies parents are set to the deleted policy parent.
 */
PolicySet.prototype.deletePolicy = function(ndx)
{
var order;
var len=this.policies.length;
var id;


    order=parseInt(this.policies[ndx].order)
    id=this.policies[ndx].id

	//delete
	if (this.policies[ndx].deleted==1)
	{  return;
	}
	this.policies[ndx].deleted=1;
    this.policies[ndx].order=0;

    for(i=0;i<len;i++)
	{
		//reset parent for sub policies if any,
		if (id==this.policies[i].parent_id)
		{   this.policies[i].parent_id= this.policies[ndx].parent_id;
		}
		//reset order of other subpolicies
        var tmpOrder=parseInt(this.policies[i].order);
		if (tmpOrder>order)
		{
			tmpOrder--;
			tmpOrder +="";        //converting to string
			this.policies[i].order=tmpOrder;
		}
		//reduce nb_sub for parent
		if (this.policies[i].id==this.policies[ndx].parent_id)
		{   this.policies[i].nb_sub--;
		}
    }

	//populate listNdx and treeNdx again.
    this.order();
    this.tree();

}//deletePolicy

//Draw a combo box for policy set selection
//provided a <div id=""select_policyset_type"> is added in the page
function PolicySet_drawSetsSelection(current)
{
          //page header
          var div = document.getElementById("policiesnode_div");
          if (!div) return;
		  div.innerHTML="";
          div.appendChild(document.createTextNode(resolveString(cse_pp_st, gTopNode)));

          //combo
          div = document.getElementById("policysets_div");
          if (!div) return;
		  div.innerHTML="";
          var sel_label=resolveString(cse_pp_st, "select_policyset_type")+" : ";
          div.appendChild(document.createTextNode(sel_label));

		  var sel=document.createElement("select");
          div.appendChild(sel);
          sel.id="policyset_sel";
		  sel.onchange=Function("javascript:ProtocolCtx_onPolicySetSelection(this)")

          var policySet_pp = new PropertyPath(gTopNode);
          policySet_pp.Append("PolicySet");
          policySet_pp.BeginEnum();

          while(gPolicies_pb.GetProperty(policySet_pp) !== undefined) {
             policySet_pp.SetTempLeaf("name");
             var name = gPolicies_pb.GetProperty(policySet_pp);
			 var opt=document.createElement("option");
             sel.appendChild(opt);

             if(name==current)
             {
                 opt.selected=true;
             }
             opt.value=name;
             opt.appendChild(document.createTextNode(resolveString(cse_pp_st, name)));
             policySet_pp.IncrementEnum();

          }

          policySet_pp.EndEnum();

}//PolicySet_drawSetsSelection()


//====================================================================================
// DHTML and Draw functions for this page
//====================================================================================

/*
   hide/show summary popup
*/
var mouseX = 0;
var mouseY = 0;

function mouseTrack(e) {
		if (e) {
			mouseX = e.pageX;
			mouseY = e.pageY;
		} else {
			if (event) {
				mouseX = event.x;
				mouseY = event.y;
			}
		}
}

document.onmousemove=mouseTrack;

function dhtmlSummaryPop(layerKey, onState) {
		dhtmlSummary = document.getElementById(layerKey);
		dhtmlSummary.style.display = (onState) ? "block" : "none";
		if (onState) {
			dhtmlSummary.style.left = (mouseX+5) + "px";
			dhtmlSummary.style.top = (mouseY+5) + "px";
		}
}

/*
 * hide/show one table or the other
 */
function dhtml_display(layerKey1, layerKey2)
{
		dhtmlPanel = document.getElementById("DATA_" +layerKey1);
		dhtmlPanel.style.display = "";
		dhtmlPanel = document.getElementById("LNK_" +layerKey1);
		dhtmlPanel.style.display = "";

		dhtmlPanel = document.getElementById("DATA_" +layerKey2);
		dhtmlPanel.style.display = "none";
		dhtmlPanel = document.getElementById("LNK_" +layerKey2);
		dhtmlPanel.style.display = "none";
}


/*
 * Show graphically a row for inheritance tree table
 * table is assumed of class report table
 * <tr> tag is assumed done elswhere
 *
 * div_id : relevant pop summary div id
 */

PolicyStatement.prototype.drawPolicySetTreeItem = function(div_id, set_name, set_len)
{
var html = "";
var pol_activate = "";
var pol_root = " &nbsp; ";
var pol_indent="";
var pol_priority=1+set_len-this.order;
var i;

    //check if the policy is active
    if (this.active==1) pol_activate = " checked ";

    pol_activate+=' onclick=javascript:' + set_name + '.drawActivate("' + this.id + '") '

    //add delete button if it is not fixed data
    if (this.root==K_NOROOT)
    {
        pol_root =  '<A class="blueLink" HREF= javascript:' + set_name + '.drawDelete("' + this.id + '")>'
        pol_root +=  resolveString(cse_pp_st, "__delete")
        pol_root +=  '</A>'
    }

    //build indentation depending of nb of ancestors
    for (i=1; i<this.indent; i++)
    {
         pol_indent+="<img src=\"../../images/void.gif\" width=\"8\" height=\"8\" border=\"0\" style=\"vertical-align:middle;margin-right:7px;\"> "
    }
    if (this.indent>0)
    {
         pol_indent+="<img src=\"../../images/bullet_next.gif\" width=\"8\" height=\"8\" border=\"0\" style=\"vertical-align:middle;margin-right:7px;\"> "
    }

    pol_link= ' javascript:PolicySet_SelectPolicy("' + this.id + '") '

    //build html for the row
    html+='<td class="rt label" nowrap>'
    html+=    '<img src="../../images/icons/info_popup.gif" width="14" height="14" border="0" ';
    html+=        'onmouseover=\" dhtmlSummaryPop(' +'\''+ div_id +'_' + this.id + '\'' + ', true)\" ' ;
    html+=        'onmouseout =\" dhtmlSummaryPop(' +'\''+ div_id +'_' + this.id + '\'' + ', false)\" ' ;
    html+=        'style="vertical-align:middle;margin-right:7px" > ';
    html+=     pol_indent + '<a class="blueLink" href=' + pol_link + '> ' + ReplaceSpecialHTMLChars( this.name ) + '</a>';
    html+='</td> <td class="rt label">';
    html+=     pol_priority;
    html+='</td> <td class="rt label">';
    html+=     '<input type="checkbox" ' + pol_activate + 'class="noPadCheck"';
    if( !(this.root==K_NOROOT) && gbDisableMasterPolEnable  )
	{
		html += ' disabled';
	}
	html += '>';
    html+='</td><td class="rt label eor"> ' ;
    html+=     pol_root;
    html+='</td>';

    return html;
}

/*
 * Show graphically a row of the sorted list table
 *
 * parameters:
 * div_id : relevant pop summary div id
 */
PolicyStatement.prototype.drawPolicySetListItem = function(div_id, first_item, last_item, set_name, set_len)
{
var html = "";
var pol_activate = "";
var pol_root = " &nbsp; ";
var pol_indent="";
var pol_moveup="";
var pol_movedown="";
var pol_priority = 1+set_len-this.order;
var i;
var pol_link="";

    //check if the policy is active
    if (this.active==1) pol_activate = " checked ";

        pol_activate+=' onclick=javascript:' + set_name + '.drawActivate("' + this.id + '") '

    //add delete button if it is not fixed data
    pol_root = "";

    if (this.root==K_NOROOT)
    {
        pol_root +=   '<A class="blueLink" HREF= javascript:' + set_name +'.drawDelete("' + this.id + '")>'
        pol_root +=  resolveString(cse_pp_st,"__delete")
        pol_root +=  '</A>'
        pol_root +=  ' | '
    }

    pol_root +=  '<A class="blueLink" HREF=javascript:PolicySet_PolicyDetails("' + this.id + '")>'
    pol_root +=  resolveString(cse_pp_st,"__policy_details")
    pol_root +=  '</A>'



    if (first_item)
    {   pol_moveup="&nbsp;"
    }
    else
    {   pol_moveup =   '<A HREF= javascript:' + set_name +'.drawMove(1,'  + this.order + ')>'
        pol_moveup +=  '<img src="../../images/icons/move_up.gif" width="11" height="9" border="0"> '
        pol_moveup +=  '</A>'
    }

    if (last_item)
    {   pol_movedown="&nbsp;"
    }
    else
    {   pol_movedown =   '<A HREF= javascript:' + set_name +'.drawMove(0,'  + this.order + ')>'
        pol_movedown +=  '<img src="../../images/icons/move_down.gif" width="11" height="9" border="0"> '
        pol_movedown +=  '</A>'
    }

    pol_link= ' javascript:PolicySet_SelectPolicy("'  + this.id + '") '

    //root item don't change priority anyway
    if(this.root==K_ROOT)
    { pol_moveup="&nbsp;"
      pol_movedown="&nbsp;"
    }

    //build html for the row
    html+='<td class="rt label" nowrap>'
    html+=    '<img src="../../images/icons/info_popup.gif" width="14" height="14" border="0" ';
    html+=        'onmouseover=\" dhtmlSummaryPop(' +'\''+ div_id +'_' + this.id + '\'' + ', true)\" ' ;
    html+=        'onmouseout =\" dhtmlSummaryPop(' +'\''+ div_id +'_' + this.id + '\'' + ', false)\" ' ;
    html+=        'style="vertical-align:middle;margin-right:7px" > ';
    html+=     pol_priority + '. ' + '<a class="blueLink" href=' + pol_link + '> ' + ReplaceSpecialHTMLChars( this.name ) + '</a>';
    html+='</td> <td class="rt label">';
    html+=     pol_moveup;
    html+='</td><td class="rt label"> ' ;
    html+=     pol_movedown;
    html+='</td> <td class="rt label" nowrap>';
	html+=     ReplaceSpecialHTMLChars( this.parent_name ) +'&nbsp;';
    html+='</td><td class="rt label"> ' ;
    html+=     '<input type="checkbox" ' + pol_activate + 'class="noPadCheck"';
    if( !(this.root==K_NOROOT) && gbDisableMasterPolEnable  )
	{
		html += ' disabled';
	}
	html += '>';
    html+='</td><td class="rt label eor" nowrap> ' ;
    html+=     pol_root;
    html+='</td>';

    return html;

}//drawPolicySetListItem

/*
 * Show graphically the content of policy set, as an inheritance tree
 * last row is displayed with bottom line
 * except if there is no policy defined (which should never happen)
 */

PolicySet.prototype.drawPolicySetTree = function()
{
   var len=this.treeNdx.length
   var i;
   var ndx;
   var pol_class="";

   var pol_row="";
   var data_html="";
   var last_item=0;
   var max_order=0

   //look for last item (there will be at least one master policy)
   for (i=0; i<len; i++)
   {
       ndx=this.treeNdx[i];
       if (ndx==-1) continue;
       else last_item=i;
       max_order++;
   }

   //table header
   data_html =  '<table border="0" cellpadding="0" cellspacing="0" class="reportTable" width=100%>'
   data_html +=    '<tr class="shaded">'
   data_html += '<td colspan=5 class="rt eor subhead2 label" nowrap><span class="bold">' + this.tree_title + '</span></td>'
   data_html += '</tr>'
   data_html += '<tr class="header">'
   data_html += '<td               class="rt label" nowrap><span class="bold">'      + this.label_name + '</span></td>'
   data_html += '<td width="75px"  class="rt label" nowrap><span class="bold">'      + this.label_priority + '</span></td>'
   data_html += '<td width="75px"  class="rt label" nowrap><span class="bold">'      + this.label_enable + '</span></td>'
   data_html += '<td  width= "75px" class="rt eor label " nowrap>&nbsp;</td>'
   data_html += '</tr>'

   //table rows
   for (i=0; i<len; i++)
   {

       ndx=this.treeNdx[i];
       if (ndx==-1) continue;

       //toggle shade
       if (pol_class=="")
       {   pol_class = "shaded ";
       }
       else
       {   pol_class="";
       }
       //add lastrow if relevant
       if (i==last_item)
       {  pol_class +="lastrow "
       }

       if (pol_class!="")
       {   pol_class="class=\""+pol_class+"\" ";
       }

       //build row
       data_html+= '<TR ' + pol_class + ' height="22">'
       data_html+=this.policies[ndx].drawPolicySetTreeItem(this.pop_div, this.objectName, max_order);
       data_html+='</TR>'

   }

   data_html +='</table>';

   //output on page
   var div = document.getElementById(this.tree_div);
   if (div) {
   div.innerHTML = data_html;
   }

}//drawPolicySetTree

/*
 * Show graphically the content of policy set, sorted by priority
 * last row is displayed with bottom line
 * except if there is no policy defined (which should never happen)
 */

PolicySet.prototype.drawPolicySetList = function()
{
   var len=this.listNdx.length
   var i;
   var is_first=1;
   var is_last=0;
   var ndx;
   var pol_class="";
   var pol_row="";
   var data_html="";
   var last_item=-1;
   var lowest_priority=0;
   var max_order=0;

   //look for last item
   for (i=0; i<len; i++)
   {
       ndx=this.listNdx[i];
       if (ndx==-1) continue;
       lowest_priority=last_item;
       last_item=i;
       max_order++;
   }

   //table header
   data_html =  '<table border="0" cellpadding="0" cellspacing="0" class="reportTable" width=100%>'
   data_html +=    '<tr class="shaded">'
   data_html += '<td colspan=6 class="rt eor subhead2 label"><span class="bold">' + this.list_title + '</span></td>'
   data_html += '</tr>'
   data_html += '<tr class="header">'
   data_html += '<td class="rt label" nowrap><span class="bold">'      + this.label_name + '</span></td>'
   data_html += '<td width="50px"  class="rt label" colspan=2 nowrap><span class="bold">' + this.label_move + '</span></td>'
   data_html += '<td               class="rt label" nowrap><span class="bold">'      + this.label_parent + '</span></td>'
   data_html += '<td width="75px"  class="rt label" nowrap><span class="bold">'      + this.label_enable + '</span></td>'
   data_html += '<td  class="rt eor label " nowrap>&nbsp;</td>'
   data_html += '</tr>'

   is_first=1;

   //table rows
   for (i=0; i<len; i++)
   {
       is_last=0;

       ndx=this.listNdx[i];
       if (ndx==-1) continue;


       //toggle shade
       if (pol_class=="")
       {    pol_class = "shaded ";
       }
       else
       {   pol_class="";
       }

       //Master Policy and the one before are lowest priority
       //add lastrow if relevant
       if (i==lowest_priority)
       {   is_last=1
       }
       if (i==last_item)
       {  pol_class +="lastrow "
          is_last=1
       }


       if (pol_class!="")
       {   pol_class="class=\""+pol_class+"\" ";
       }

       //build row
       data_html+= '<TR ' + pol_class + ' height="22">'
       data_html+=this.policies[ndx].drawPolicySetListItem(this.pop_div, is_first, is_last, this.objectName, max_order);
       data_html+='</TR>'

       //not first item any more
       is_first=0;
   }


   //end of table
   data_html +='</table>';

   //output on page
   var div = document.getElementById(this.list_div);
   if (div) {
   div.innerHTML = data_html;
   }

}//drawPolicySetList

/*
 * output the summary popup windows for later use
 */
PolicySet.prototype.drawSummaryPopups = function()
{
var ndx;
var len=this.policies.length;
var html="";
var div;

    for (ndx=0;ndx<len ;ndx++ )
    {
        html+='<div id=' + this.pop_div+"_"+this.policies[ndx].id +' class="dhtmlSummary" > '
        html+='<table border="0" cellspacing="0" cellpadding="0" class="dhtmlSummary">'
        html+='<tr class="titleRow"><td>' + ReplaceSpecialHTMLChars( this.policies[ndx].name ) + '</td></tr>'
        html+='<tr class="dataRow firstRow lastRow">'
        html+='<td class="eor">' + ReplaceSpecialHTMLChars( this.policies[ndx].description )+ '<BR>'
        html+='</td>'
        html+='</tr>'
        html+='</table>';
        html+='</div>'
    }

    div = document.getElementById(this.pop_div);
    if (div)
    {   div.innerHTML = html;
    }

}//drawSummaryPopups


PolicySet.prototype.drawMove= function(up, value)
{

       if (up) this.moveUp( value);
       else this.moveDown( value);

       this.drawPolicySetTree();
       this.drawPolicySetList();

       top.SetModified(true);
       top.gPageStack.Active().SetModified(true);

}

PolicySet.prototype.drawActivate= function(id)
{
var len=this.policies.length;

    //find policy and current value for activate
    for(i=0;i<len;i++)
    {
        if (id==this.policies[i].id)
        {   ndx=i;
            //break;
        }
    }
    var ok=0;
    if (this.policies[ndx].active==0) ok=1;

   //modify property bag
   //var ppT = new PropertyPath(this.policies[ndx]);
   //ppT.SetTempLeaf("active");
   //gPolicies_pb.SetProperty(ppT.GetPath(), ok);

    //change parameter in the list and redraw
    this.policies[ndx].active=ok;
    this.draw();

    //Activate apply/cancel buttons
    top.SetModified(true);
    top.gPageStack.Active().SetModified(true);

}//drawActivate


PolicySet.prototype.drawDelete= function(id)
{
var ok=0;
var i;
var name;
var ndx;
var len = this.policies.length;

    for(i=0;i<len;i++)
    {
        if (id==this.policies[i].id)
        {   ndx=i;
            name= this.policies[i].name;
        }
    }

     ok= confirm(resolveString(cse_pp_st, "__delete_policy_prompt", [name]));

     if (ok)
     {
       this.deletePolicy(ndx);
       this.drawPolicySetTree();
       this.drawPolicySetList();
       top.SetModified(true);
       top.gPageStack.Active().SetModified(true);
     }
}//drawDelete


PolicySet.prototype.draw= function()
{
       this.drawPolicySetTree();
       this.drawPolicySetList();
       this.drawSummaryPopups();
}


//=========================================================
// Populate and other Mandatory Functions
//=========================================================
/*
 * populate PolicySet
 * param:
 * pp_str = pp to the policy set
 */

PolicySet.prototype.populate = function(pp_str)
{
   this.policies = new Array();

   var pList = this.policies;

   var pp = new PropertyPath (pp_str + '.Policy');

   pp.BeginEnum();

   var leaves = [{lName:"id", def:""}, {lName:"name", def:""},
       {lName:"description", def:""}, {lName:"Order", def:0, oName:"order"},
       {lName:"parent", def:"", oName:"parent_id"}, {lName:"active", def:1},
       {lName:"root", def:0}];

   while(gPolicies_pb.GetProperty(pp) !== undefined)
   {
       var o = new Object();
       for (var i = 0; i < leaves.length; i++)
       {
           var oProp = leaves[i].oName ? leaves[i].oName : leaves[i].lName;
           pp.SetTempLeaf(leaves[i].lName);
           o[oProp] = gPolicies_pb.GetPropertyDef(pp, leaves[i].def);
       }
       pp.SetTempLeaf("");

	   if(o.root == 1)
	   {
	       o.description = resolveString(cse_pp_st, o.description);
		   o.name = resolveString(cse_pp_st,o.name);
	   }

		var pStatement = new PolicyStatement(o.id, o.name,
          o.description, o.order, o.parent_id, o.active, o.root);

       pStatement.pp_path=pp.GetPath()
       pList.push(pStatement);

       pp.IncrementEnum();

   }

   /*create ndx arrays*/
   var len= this.policies.length;
   this.listNdx=new Array(len);
   this.treeNdx=new Array(len);

   if (this.order()) this.normalOrder();

   this.tree();

}//PolicySet.populate


//delete identity with the given id
//except if it is "I:EVERYONE"
//assumes no policy is using it any more.
function PolicySet_DeleteIdentity(id, pbOut)
{
       if (id=="I:EVERYONE")
       {  return;
       }

       var ppT = new PropertyPath("Identities.Identity")
       ppT.BeginEnum();

       while(pbOut.GetProperty(ppT) != undefined)
       {
           ppT.SetTempLeaf("id");
           var the_id = pbOut.GetProperty(ppT);
           ppT.SetTempLeaf("");

           if (the_id == id)
           {   pbOut.DeleteElement(ppT);
               return;
           }
           else
           {   ppT.IncrementEnum();
           }
       }
       ppT.EndEnum();
}


//Save all changes in propertybag
//This function is being called by the Apply function or
//when forwarding to a sub page
function PolicySet_SaveAll()
{

   var pList = top.polctx.myPolicySet.policies;
   var i;
   var ppT = new PropertyPath(gPolicySet_ppstr);
   var deleting=false;

   // modify  policy.attributes
   for (i=0; i<pList.length ; i++)
   {    ppT.SetPath(pList[i].pp_path);

        if (undefined === gPolicies_pb.GetProperty(ppT))
        {
           alert("undefined property path " + ppT.GetPath());
        }
        else
        {
           ppT.SetTempLeaf("active");
           gPolicies_pb.SetProperty(ppT.GetPath(), pList[i].active);
           ppT.SetTempLeaf("parent");
           gPolicies_pb.SetProperty(ppT.GetPath(), pList[i].parent_id);
           ppT.SetTempLeaf("Order");
           gPolicies_pb.SetProperty(ppT.GetPath(), pList[i].order);
           ppT.SetTempLeaf("Language");
           gPolicies_pb.SetProperty(ppT.GetPath(), "");

           if (pList[i].deleted==1)
           {   deleting=true;
               ppT.SetTempLeaf("root");
               gPolicies_pb.SetProperty(ppT.GetPath(), K_DELETED);
           }
        }
   }

   if(deleting)
   {
       var identities_pp = "Identities";

       if (undefined == gPolicies_pb.GetProperty(identities_pp))
       {
            top.gPB.GetElement("Identities", gPolicies_pb);
       }

       ppT.SetPath(gPolicySet_ppstr + '.Policy');
       ppT.BeginEnum();

       while(gPolicies_pb.GetProperty(ppT) !== undefined)
       {
           ppT.SetTempLeaf("root");
           var the_root = gPolicies_pb.GetProperty(ppT);
           ppT.SetTempLeaf("IdentityRef");
           var the_idRef = gPolicies_pb.GetProperty(ppT);
           ppT.SetTempLeaf("");

           if (the_root == K_DELETED)
           {   gPolicies_pb.DeleteElement(ppT)
               PolicySet_DeleteIdentity(the_idRef, gPolicies_pb)
           }
           else
           {   ppT.IncrementEnum();
           }
       }
       ppT.EndEnum();
   }

    //clean populate
    top.polctx.myPolicySet.populate(gPolicySet_ppstr);

    //enable global apply/cancel buttons
    if(top.gPageStack.Active().modified)
    {   top.gPageStack.Active().SetModified(false);
        top.SetModified(true);
    }

}//fn


/*
 * finds the propertypath of the policyset corresponding to tag
 * params:
 * tag = unique identifier for the policyset - name property in the property bag
 */
function PolicySet_findSet( tag )
{
    if (!tag) {
        alert("__build error : policySet name undefined")
		return;
    }

    var policySet_pp = new PropertyPath(gTopNode);
    policySet_pp.Append("PolicySet");
    policySet_pp.BeginEnum();
    while(gPolicies_pb.GetProperty(policySet_pp) !== undefined) {
       policySet_pp.SetTempLeaf("name");
       var name = gPolicies_pb.GetProperty(policySet_pp);
       if (name == tag) {
           policySet_pp.SetTempLeaf("");
           gPolicySet_ppstr = policySet_pp.GetPath();
           break;
       }
       policySet_pp.IncrementEnum();
    }

    policySet_pp.EndEnum();

    //populate and draw.
    top.polctx.myPolicySet.populate(gPolicySet_ppstr);
    top.polctx.myPolicySet.draw();
}

/**
 * Asynchronous data load function
 * enumerate all policyset until the one with the right name is found
 * pbIn : property bag to read data from
 * second parameter is not used
 */
function Show(pbIn, ppIn) {

	// Check for completed wizard and merge in new values
	var returnedFrom = top.gPageStack.ReturnedFrom()
	if (returnedFrom && returnedFrom.finished)
	{
		top.SetModified(true);
		top.gPageStack.Active().pb.SetProperties(returnedFrom.pb);
	}

    //get the right policyset by name
	var policySetName = top.polctx.myPolicySet.pp_key;
    gPolicies_pb=pbIn;
    PolicySet_drawSetsSelection(policySetName);
	var context=PolicySet_findSet( policySetName );


}//Show

/**
 * Asynchronous data validation function : not needed
 */
function Validate() {
    return true;
}//validate

/**
 * Asynchronous data save function
 * @param pbout the property bag to fill
 * other parameters not needed
 */
function Apply(null_pb, null_pp, null_fwd) {
    // Copy all the policyset data into the output bag (all params are ignored)
    //and perform a clean populate
    PolicySet_SaveAll();

    //redraw.
    top.polctx.myPolicySet.draw();
    return gPolicies_pb;

}//Apply


function Cancel()
{   //re-populate from the propertybag
    top.polctx.myPolicySet.populate(gPolicySet_ppstr);
    top.polctx.myPolicySet.draw();
}


//====================================================================================
// Links to other pages
//====================================================================================

/**
 * Create Policy Button
 */
function CreatePolicy()
{
    //first save all changes
    var pg=top.gPageStack.Active();
    if (pg)
    {    if (pg.modified)
         {  PolicySet_SaveAll();
         }
    }

    //use proper property bag
    var pp = new PropertyPath(gPolicySet_ppstr)
    pp.Append("Policy")

    var pb;
    var cur=top.gPageStack.Active()
    if (cur)
    {  pb = top.gPageStack.Active().pb
    }
    else pb=gPolicies_pb;

    //add identities to new propertybag
   var newpb =  top.gPageStack.NewPropertyBag();
   if (pb.GetProperty("Identities"))
   {  pb.GetElement("Identities", newpb);
   }
   else
   {  top.gPB.GetElement("Identities", newpb);
   }

   top.gPageStack.Forward(top.gPolicyCreationWizard,pb.GetNextElementEnum(pp.GetPath()),newpb);

}//CreatePolicy


/*
 * select policy statement and link to Policy Statement page
 */
function PolicySet_SelectPolicy(id)
{

     //first save all changes
     var pg=top.gPageStack.Active();
     if (pg)
     {    if (pg.modified)
          {  PolicySet_SaveAll();
          }
     }

    //find policy from id
    var pList= top.polctx.myPolicySet.policies;
    var len=pList.length;
    var i;
    var ndx=0;
    var name;

    for(i=0;i<len;i++)
	{
		if (id==pList[i].id)
		{   ndx=i;
		    name= pList[i].name;
		}
    }

	top.polctx.myPolicySet.selection = ndx;

	//build breadcrumb and forward

    var url = "PolicySettingsListPage.html"
    var title=resolveString(cse_pp_st, "__policy") + "(" + ReplaceSpecialHTMLChars( name ) + ")"
    top.gPageStack.Forward(new EmbeddedPage(url, title), "");

}//SelectPolicy

//Link to the policy Details page
function PolicySet_PolicyDetails(id)
{

     //first store all changes and rebuild cleanpolicies list
     var pg=top.gPageStack.Active();
     if (pg)
     {    if (pg.modified)
          {  PolicySet_SaveAll();
          }
     }

    //find policy from id
    var pList= top.polctx.myPolicySet.policies;
    var ndx=0;
    for(var i=0;i<pList.length;i++)
	{
		if (id==pList[i].id)
		{   ndx=i;
		}
    }

	//build breadcrumb and forward
    var url = "PolicyDetails.html"
    var title=resolveString(cse_pp_st, "__policy_details")

    top.gPageStack.Forward(new EmbeddedPage(url, title), pList[ndx].pp_path);

}//PolicyDetails
