// Copyright (C) 2009 McAfee, Inc.  All rights reserved.
//
// Description:
// Included in PolicySettingsListPage.html
// Javascript functions to manage data for the policySettingsList page.

//====================================================================================
// Data Structures
//====================================================================================

/*
 * Constructor for the PolicySettingsTable class
 * members:
 * tbl_title = table title, taken from one of the RefTypes.sections
 * items = Array, indexes the list of settings
 *
 */
function PolicySettingsTable(tbl_title)
{
    this.tbl_title=tbl_title;
    this.items = new Array();

}//PolicySettingsTable

/*
 * Constructor for the PolicySettingsListItem class
 * members:
 * id: unique identifier for the settings
 * name : display name
 * enabled : boolean - 0=disabled; 1=enabled
 * time_restriction: string indicating what the time restriction is
 * inherits: boolean 1=inherit parent policy
 * root : 0=can be deleted or edited - 1=fixed value
 * deleted : marked as deleted
 * indent : level of depth in the tree (root=0)

 * nb_alt : nb of time restricted alternatives
 * url : page to lauch to view this setting
 * rank : ranking according to the typeval ordering
 */

function PolicySettingsListItem(ref_id, typeval, name, time, enabled, root)
{
	this.ref_id = ref_id; //reference to parameters
	this.typeval=typeval;
    this.name = name;
	this.time=time;
	this.enabled = enabled;
	this.root = root;
	this.typeval=typeval;

	this.deleted=0;
	this.nb_alt=0;
	this.lnk_url="#";
	this.rank=0;
	this.pp_local="";
	this.pp_ancestor="";
	this.inherits=0;
    this.activatable = true;
}//PolicySettingsListItem

/*
 * Constructor for the PolicySettingsList class
 * members:
 *   tables = index grouping settings by sections
 *   Settings = array of all settings (unordered)
 *   objectName = name of the instance
 *   pop_div = id of the div section for pop summary
 *
 *   confirm_delete : callback function in the html page to ask confirmation (returns 1==ok)
 */

function PolicySettingsList( name )
{
	this.objectName=name;

    this.tables; //Array created in makeTable
	this.index; //Array ordered as in tables
	this.settings; //Array created in Populate
	this.policy_root=0;
	this.id_key="";

}//PolicySettingsList

//====================================================================================
// utilities - Private Functions - non display
//====================================================================================

/*
 * mark settings as deleted
 * associated root node is updated too
 */
PolicySettingsList.prototype.deleteSetting = function(ndx)
{
var i,rank;
var len=this.settings.length;

    //find and delete Setting
	rank = this.settings[ndx].rank;
	this.settings[ndx].deleted=1;

	//nothing more to do if this setting is root
	if (this.settings[ndx].root ==1)
	{   return;
	}

	//decrease nb_alt in root node
	for(i=0;i<len;i++)
	{
		if (this.settings[i].time=="" && this.settings[i].rank == rank)
		{
			this.settings[i].nb_alt--;
			return;
		}
    }

}//deleteSetting

/*
 * insertTableItem
 */
PolicySettingsList.prototype.insertTableItem = function(the_array, the_ndx, the_rank)
{ //inserts an item in the table data in the right place according to its valtype rank
var len = 0;
var insert_pt =0;
var pos;
var prev;
var k;

	   //add a position to the array
	   the_array.push(-1);
	   len=the_array.length;
	   insert_pt = len-1;

       //find previous position of higher priority (lower rank)
       for (k=1;k<len ;k++ )
       {
		    pos=len-k;
	        prev = the_array[pos-1];

			if (this.settings[prev].rank > the_rank)
	        {   the_array[pos]=the_array[pos-1];
			    insert_pt=pos-1;
	        }
			else if (this.settings[prev].rank<the_rank)
			{   //current insert_pt is valid
				break;
			}
			else if ( this.settings[prev].time=="")
			{  //current insert_pt is valid and prev is the "all the time" node
			   this.settings[prev].nb_alt++;
			   break;
			}
       }

	   the_array[insert_pt]=the_ndx;
}

/*
 * Defines the list of tables where settings are grouped
 * Assumes the data are populated and sorted in the order we want them to be displayed !!!
 */
PolicySettingsList.prototype.makeTables = function(reftype)
{
 var i,j,k;
 var len_sec = reftype.sections.length;
 var len_type = reftype.typeArray.length;
 var len_data = this.settings.length;
 var len_table;
 var nb_tables=0;
 var secndx;
 var typeval;
 var insert_pt;
 var prev;
 var rank=0;
 var the_lnk="";
 var activatable = true;

   //figure out the number of tables
   for (i=0;i< len_type; i++)
   {   secndx = reftype.typeArray[i].secndx;
	   if ( secndx >= nb_tables) nb_tables=secndx+1;
   }

   //init the tables titles for all possible secndx values
   this.tables = new Array(nb_tables);
   for (i=0; i<nb_tables ; i++)
   {
	   if (i<len_sec)
       {     this.tables[i] = new PolicySettingsTable(reftype.sections[i]);
       }
       else
       {    this.tables[i] = new PolicySettingsTable ("");
       }
   }

   //add ndx to the settings to tables[secndx].items
   for (i=0;i<len_data; i++)
   {
       typeval=this.settings[i].typeval;
	   secndx=-1;
	   rank=-1;

       for (j=0;j<len_type ; j++)
       {
           if ( typeval==reftype.typeArray[j].typeval)
           {  secndx=reftype.typeArray[j].secndx;
			  rank=j;
			  the_lnk=reftype.typeArray[j].lnk_url;
	          activatable = reftype.typeArray[j].activatable;
           }
       }

	   if(rank>=0)
       {
	      this.settings[i].rank=rank;
		  this.settings[i].lnk_url=the_lnk;
          this.settings[i].activatable = activatable;
	      this.insertTableItem(this.tables[secndx].items, i, rank)
       }
   }

}//makeTables

PolicySettingsList.prototype.makeIndex = function()
{
var nb_table= this.tables.length;
this.index = new Array();

   for (var sec=0;sec<nb_table ;sec++ )
   {  var len = this.tables[sec].items.length;
      for (var i=0;i<len; i++ )
	  {   this.index.push(this.tables[sec].items[i])
	  }
   }
}


/*
 * Appends a PolicySettingsListItem to the policySettingsList
 * layer is 0 is we are dealing with the current policy
 * or >0 if we are reading a parent policy
 * Other parameters are PolicySettingsListItem members
 */

PolicySettingsList.prototype.add = function(id, typeval, name, time, enabled, root, pp_str, layer)
{

    var not_found = true;
    var item = new PolicySettingsListItem(id, typeval, name, time, enabled, root);

	if (layer>0)
	{   item.inherits=1;
	    item.pp_ancestor=pp_str
	}
	else
	{   item.inherits=0;
 	    item.pp_local=pp_str
	}

    for (var i=0; i<this.settings.length; i++ )
    {    if ( this.settings[i].typeval==typeval && this.settings[i].time==time)
         {   not_found=false;
	         if (layer==0) item.pp_ancestor= this.settings[i].pp_ancestor;
		     this.settings[i] = item;
         }
    }

	if (not_found) this.settings.push(item);
}


/*
 * Restructures the res Array (which contains all the matching propertypaths)
 * The restructuring is such that all the enumerated paths of a given property path are sorted(w.r.t the enumeration count) and placed together.
 * The output is the sorted res Array which has the same number of elements as the res Array.
 */
function restructure(res)
{
	var tempIndex = 0;
	var sortIndex = 0;

	var sortedResArray = new Array(res.length);
	var tempArray = new Array();
	var map = new Array(res.length);

	for(var i=0; i < map.length ; i++)
		map[i] = 0 ;

	for (var i=0 ; i < res.length ;i++ )
		{
			 if ( map[i] ) // has been sorted already
			    continue;

			 var tmpType = res[ i ][ 1 ];
			 map[i] = 1;
			 tempArray[ tempIndex ] = res[ i ] ;

			 for (var j = i + 1 ; j < res.length ; j++ )
				{
					if(  tmpType == res[ j ][ 1 ]  )
					{
						tempIndex++;
						tempArray[ tempIndex ] = res[ j ];
						map[j] = 1;  //the jth element in the res array has been sorted.
						continue;
					}
				}

		  //tempArray filled
		  //now call the sort on the array
		  if( tempArray.length > 1)
			sortResArray(tempArray) ;

		//now build the sortedResArray
		  for( var j = 0 ;	j < tempArray.length ; j++)
			{
			   sortedResArray[ sortIndex ] = tempArray[ j ];
			   sortIndex++;
			}

		 tempArray = new Array ; 	 // reinitializing.
		 tempIndex = 0;
		} //outer loop

	return sortedResArray;
}


/*sorts the array from startindex to the end on basis of the enumeration value.
 *See the res Array structure for details.
 */
function sortResArray( tempArray )
{
	//insertion sort on the array
	 for(var j =1; j < tempArray.length ; j++)
		{
		 var i = j-1;
		 var tmpObject = tempArray[ j ] ;
		 var tmpValue = getEnumValue( tempArray[ j ][ 2 ] ) ;

		 while( i >=0 &&  tmpValue < getEnumValue( tempArray[ i ][ 2 ] ) )
			{
			  tempArray[ i+1 ] = tempArray[ i ];
			  i--;
			}
			tempArray[ i +1 ] = tmpObject ;
		}
 }

/* to get the enumValue from the res element */
function getEnumValue( enumValue )
{
	if(undefined == enumValue || "" == enumValue)
		 return 1;

	 var isPresent = enumValue.indexOf(":");
	 if( isPresent != -1)
	 {
		var value = parseInt(enumValue.substring(isPresent+1), 10);
		return value;
	 }
	else
	 return 1;

}

/*
 *  populate PolicySettingsList array
 *  Parameters:
 *  reftypes: PolicySettingsRefTypes object populated with PP specific
 *  settings for this policy
 *  ndx: index of policy to look at
 *  layer: 0=current policy, >0=parent or root
 */
PolicySettingsList.prototype.ReadPolSettings = function(reftypes, the_pb, ndx, layer)
{
    var i;
    var pp = new PropertyPath ("");
    var pp_str= top.polctx.myPolicySet.policies[ndx].pp_path;

   //find all settings pp,
   //looking for policy keys ending with string in format: ".XXXPolicySettings[:00000000]"
   var regexStr='^' + pp_str + "\\.(\\w+)PolicySettings(:\\d{8})*$";
   var pattern=new RegExp(regexStr);
   var res=the_pb.GetMatchingPaths(pattern);

   //Need to restructure the res array if needed
   //Restructuring the array puts all the scanners with their sub-scanners in sorted order with respect to the enumeration count.)
   //this necessary because makeTables(),Assumes the data are populated and sorted in the order we want them to be displayed !!!
   res = restructure( res );

   //for each settings, get the properties.
   for (i=0;i<res.length; i++ )
   {
	  var name_str= res[i][1];
	  if(!pp_componentInstalled(name_str)) continue;

      pp.SetPath(res[i][0]);

      var o = new Object();
	  o["typeval"]=name_str;
	  o["name"] = resolveString(cse_pp_st, name_str);

      //define properties with default values
      //lName is the property key, oName is a replacement key name in this page.
	  id_key = "Ref"+ o.typeval +"Settings";
      var leaves = [{lName:id_key, def:"", oName:"id"},
	                {lName:"RefTimeRestriction", def:"", oName:"time"},
	                {lName:"enabled", def:1},
	                {lName:"root", def:0}];


      for (var j = 0; j < leaves.length; j++)
	  {
		   var oProp = leaves[j].oName ? leaves[j].oName : leaves[j].lName;
		   pp.SetTempLeaf(leaves[j].lName);
		   o[oProp] = the_pb.GetPropertyDef(pp, leaves[j].def);
	  }
      pp.SetTempLeaf("");

	  //add layer as parameter, to possible override settings

	  this.add(o.id, o.typeval, o.name, o.time, o.enabled, o.root, pp.GetPath(), layer);

   }//for

};//ReadPolSettings


/*
 *  populate PolicySettingsList
 *  Parameters:
 *  reftypes: PolicySettingsRefTypes object populated with PP specific
 *  settings for this policy
 *  policy_pb: pb to copy settings from (is really the policySet pb)
 */


PolicySettingsList.prototype.populate = function(polset_pb, polset_pp)
{
var policy_root
var i;
var pp = new PropertyPath (polset_pp);
var type;
var ndx = top.polctx.myPolicySet.selection;
var layer = 0;
var reftypes=top.polctx.myPolRefTypes;

   /*is this policy a master policy?*/
   pp.SetTempLeaf("root");
   var policy_root=polset_pb.GetPropertyDef(pp, 0);
   pp.SetTempLeaf("parent");
   var parent_id=polset_pb.GetPropertyDef(pp, 0);
   pp.SetTempLeaf("");
   this.policy_root=policy_root;

   var pList=top.polctx.myPolicySet.policies;
   for (var i=0; i<pList.length ; i++ )
   {   if(pList[i].pp_path == polset_pp)
	   {   ndx=i;
	   }
   }

   //get the list of parent policies
   var family= new Array();
   family[0]=ndx;
   //keep looking for ancestors in the list of policies
   //stop if no more ancestor is found or if it is root
   while ( policy_root != 1)
   {
        policy_root=1;
        if ( family.length>pList.length)
		{   alert("__configuration error: no root policy found");
		    break;
		}
        var parent_id = pList[ndx].parent_id;
		for (var j=0; j<pList.length ; j++ )
		{   if(pList[j].id == parent_id)
		    {   ndx=j;
			    policy_root = pList[j].root;
			    family.push(ndx);
			}
		}
   }

   //populate by reading settings from the top policy down to the current policy
   //if a sub policy has settings, it overrides settings from parent policy
   this.settings = new Array();

   for (var k=family.length ; k>0 ; k--)
   {  var pos=k-1;
      this.ReadPolSettings(reftypes, polset_pb, family[pos], pos);
   }

   /*create ndx arrays*/
   this.makeTables(reftypes);
   this.makeIndex();

}; //Populate PolicySettingsList

