// Copyright (C) 2009 McAfee, Inc.  All rights reserved.


// Empty class for undefined values
function ValueLess() {}
//ValueLess.prototype.toString = function() { return ""; }
var gValueless = new ValueLess

///////////////////////////////////////////////////////
//
// Constructor
//
function PropertyBag()
{
	// Initialize member data
	this.propValues = new Array;
	this.modified = false;
}


///////////////////////////////////////////////////////
//
//recreate a property bag from a non functional pb datastructure
//
PropertyBag.Recreate = function(old_pb) {
    if(!old_pb) return;
    var pb = new PropertyBag();
    for (var prop in old_pb.propValues) {
        pb.propValues[prop] = old_pb.propValues[prop];
    }
    pb.modified = old_pb.modified;
    return pb;
}

///////////////////////////////////////////////////////
//
// Clones this object, performing a deep copy
//
PropertyBag.prototype.Clone = function() {
    var pb = new PropertyBag();
    for (var prop in this.propValues) {
        pb.propValues[prop] = this.propValues[prop];
    }
    return pb;
}



var pbVerbose = false;
///////////////////////////////////////////////////////
//
// Stores a property value.
//
// IMPORTANT: Name is case-sensitive
//
PropertyBag.prototype.SetProperty = function(name,value)
{
    //if (pbVerbose) alert("\"" + name + "\" (" + (typeof name) + ") : " + value);
    this.propValues[typeof(name)=="string" ? name : name.GetPath()] = value == McUndefined ?  top.gValueless  : value
    this.modified = true;
}

///////////////////////////////////////////////////////
//
// Retrieves a property value.
//
// IMPORTANT: Name is case-sensitive
//
PropertyBag.prototype.GetProperty = function(name)
{
    return this.propValues[typeof(name)=="string" ? name : name.GetPath()]
}

///////////////////////////////////////////////////////
//
// Retrieves a property value, returning a default if no value is found.
//
// IMPORTANT: Name is case-sensitive
//
PropertyBag.prototype.GetPropertyDef = function(name, def)
{
    var val = this.GetProperty(name);
    return (val === undefined || val === gValueless) ? def : val;
}


//////////////////////////////////////////////////////
//
// Deletes all property values
//
PropertyBag.prototype.Empty = function()
{
    this.propValues = new Array
}

//////////////////////////////////////////////////////
//
// Returns true if bag is empty
//
PropertyBag.prototype.IsEmpty = function()
{
	for (var pp in this.propValues)
	{
		return false
	}
	return true
}

///////////////////////////////////////////////////////
//
// Copies contents of given bag into this one,
// deleting all previous values
//
PropertyBag.prototype.Copy = function(pbIn)
{
	this.propValues = pbIn.propValues;
	this.modified = true;
}

///////////////////////////////////////////////////////
//
// Merges this PropertyBag with another; the ppList parameter
// defines one or more nodes to remove prior to the merge so
// as to ensure that enumerated lists are constructed correctly
//
PropertyBag.prototype.Merge = function(ppList, pbIn)
{
    if (this === pbIn) {
        return;
    }
    var pps = [];
    if (ppList instanceof Array) {
        pps = ppList;
    }
    else {
        pps.push(ppList);
    }
    for (var i = 0; i < pps.length; i++) {
        // Use DeleteList rather than DeleteElement because we
        // don't want the enumeration to be fixed up
        this.DeleteList(pps[i]);
    }
    // Now do a deep copy from pbIn
    for (var prop in pbIn.propValues) {
        this.propValues[prop] = pbIn.propValues[prop];
    }
	this.modified = true;
}

///////////////////////////////////////////////////////
//
// Deletes given property (specified as a string) and
// all its descendants.
//
// Variant of the DeleteElement Function.
// Note that if property is part of an enumeration then
// all properties lower down in enumeration are not altered.
// The enumeration is untouched, so this function should only
// be used when one is sure to put the element back at the same
// enumeration level it has been deleted from, otherwise the enumeration
// is broken.
//
//
PropertyBag.prototype.DeleteSingleElement = function(ppIn)
{
	ppIn = typeof(ppIn)=="string" ? ppIn : ppIn.GetPath()
	var pbOut = new PropertyBag()

	// Enumerate all properties
	for (var pp in this.propValues)
	{
		// Check whether property should not be deleted ...
		if (pp != ppIn &&  !IsAncestorPathOf(ppIn,pp))
		{
			var value = this.propValues[pp]
			// Add to set of properties to be retained
			pbOut.SetProperty(pp,value)
		}
	}
	// Replace old with new
	this.Copy(pbOut)
}

///////////////////////////////////////////////////////
//
// Deletes given property (specified as a string) and
// all its descendants.
//
// Note that if property is part of an enumeration then
// all properties lower down in enumeration need to be
// decremented to keep enumeration contiguous
//
PropertyBag.prototype.DeleteElement = function(ppIn)
{
	ppIn = typeof(ppIn)=="string" ? ppIn : ppIn.GetPath()

	var pbOut = new PropertyBag()

	// Retrieve level and enumeration of property to be deleted
	var delLevel = GetPathLevel(ppIn)
	var delLevelEnum = GetPathEnum(ppIn,delLevel)
	var ppRoot = RemoveLeafEnum(ppIn)

	// Enumerate all properties
	for (var pp in this.propValues)
	{
		// Check whether property should not be deleted ...
		if (pp != ppIn &&  !IsAncestorPathOf(ppIn,pp))
		{
            // Shift element upwards at delete level if necessary
			var value = this.propValues[pp]
			var enumThis = GetPathEnum(pp,delLevel)
			var ppTemp = RemoveLeafEnum(pp.substr(0,ppRoot.length))
			if (ppTemp == ppRoot)
			{
				if (enumThis  > delLevelEnum)
				{
					pp = SetPathEnum(pp,delLevel,enumThis -1)
				}
			}

			// Add to set of properties to be retained
			pbOut.SetProperty(pp,value)

		}
	}

	// Replace old with new
	this.Copy(pbOut)
}

///////////////////////////////////////////////////////
//
//Behaves similarly to DeleteElement
//It takes care of not decrementing the enumeration count of 'MatchPattern' tags
//during the deletion of property with the 'Match' tag.
//
//The same would happen incase of other substrings also but as of now
//McAfeeConfig.xml/CSRules.xml do not contain tags that are
//substrings of other tags.
//
//This function is used only during the updation of CSRules, instead of the standard
//DeleteElement to ensure the above mentioned behaviour.
//
//New function had to be written so as to continue using the unmodified
//DeleteElement for all other purposes.
//
PropertyBag.prototype.DeleteElementEx = function(ppIn)
{
	ppIn = typeof(ppIn)=="string" ? ppIn : ppIn.GetPath()

	var pbOut = new PropertyBag()

	// Retrieve level and enumeration of property to be deleted
	var delLevel = GetPathLevel(ppIn)
	var delLevelEnum = GetPathEnum(ppIn,delLevel)
	var ppRoot = RemoveLeafEnum(ppIn)

	var matchflag = false;
	var index = ppRoot.lastIndexOf(".");
	var tail  = ppRoot.substr(index+1);
	if(tail == "Match")
		matchflag = true;

	// Enumerate all properties
	for (var pp in this.propValues)
	{
		// Check whether property should not be deleted ...
		if (pp != ppIn &&  !IsAncestorPathOf(ppIn,pp))
		{
            // Shift element upwards at delete level if necessary
			var value = this.propValues[pp]
			var enumThis = GetPathEnum(pp,delLevel)
			var ppTemp = RemoveLeafEnum(pp.substr(0,ppRoot.length))
			var matchpatternflag = false;
			if(matchflag)
				if( -1 != pp.indexOf("MatchPattern"))
				  matchpatternflag = true;
			if (ppTemp == ppRoot && !matchpatternflag)
			{
				if (enumThis  > delLevelEnum)
				{
					pp = SetPathEnum(pp,delLevel,enumThis -1)
				}
			}
			// Add to set of properties to be retained
			pbOut.SetProperty(pp,value)
		}
	}
	// Replace old with new
	this.Copy(pbOut)
}

///////////////////////////////////////////////////////
//
// Deletes an entire list of properties (x, x:00000002, etc.)
//
//
PropertyBag.prototype.DeleteList = function(ppIn)
{
    var killList = new Array();
    var pattern = (typeof ppIn == "string") ? ppIn : ppIn.GetPath();
    delete this.propValues[pattern];
    var p1 = new RegExp("^" + pattern + "\\.");
    var p2 = new RegExp("^" + pattern + ":\\d\\d\\d\\d\\d\\d\\d\\d");
    for (var prop in this.propValues)
    {
        if (p1.test(prop) || p2.test(prop))
        {
            killList.push(prop);
        }
    }
    for (var i = 0; i < killList.length; i++)
    {
        delete this.propValues[killList[i]];
    }
    this.modified = true;
}

///////////////////////////////////////////////////////
//
// Removes a non-enumerated leaf from the property list;
// will leave the PropertyBag in an inconsistent state if
// used on a node property or the middle of an enumerated
// list.  Use with care.
//
PropertyBag.prototype.RemoveLeaf = function(ppIn)
{
    var pp = ppIn.GetPath();
    if (this.propValues.hasOwnProperty(pp)) {
        delete this.propValues[pp];
    }
    this.modified = true;
}

///////////////////////////////////////////////////////
//
// Retrieves given property (specified as string)
// and all its descendents into given property bag.
//
PropertyBag.prototype.GetElement = function(ppIn,pbOut)
{
	ppIn = typeof(ppIn)=="string" ? ppIn : ppIn.GetPath()
	for (var pp in this.propValues)
	{
		if (pp == ppIn || IsAncestorPathOf(ppIn, pp))
		{
			pbOut.SetProperty(pp, this.propValues[pp])
		}
	}
}

///////////////////////////////////////////////////////
//
// Returns the next available enumerator for the
// given path
//
PropertyBag.prototype.GetNextElementEnum = function(pp)
{
	var pp = new PropertyPath(pp)
	pp.BeginEnum()
	while (this.GetProperty(pp.GetPath()) != McUndefined)
	{
		pp.IncrementEnum()
	}
	return pp.GetPath()
}

///////////////////////////////////////////////////////
//
// Returns match information for property paths matching
// a given pattern
// The return value is an array of arrays
//
PropertyBag.prototype.GetMatchingPaths = function(re)
{
    var ret = [];
    for (var prop in this.propValues)
    {
        var matchInfo = re.exec(prop);
        if (matchInfo)
        {
            ret.push(matchInfo);
        }
    }

    return ret;
}

////////////////////////////////////////////////////////
//
// Gets all paths in a property bag
// Return value is a string array
//
PropertyBag.prototype.GetAllPaths = function() {
    var ret = [];
    for (var path in this.propValues)
    {
        ret.push(path);
    }
    return ret;
}

////////////////////////////////////////////////////////
//
// Determines whether this property bag has the same value
// as another
// Return true if the two are equal, false otherwise
PropertyBag.prototype.equals = function(anotherBag) {
    var ret = true;

    // Test for identity
    if (this === anotherBag) {
        return true;
    }
    var pArray1 = this.GetAllPaths();
    var pArray2 = anotherBag.GetAllPaths();
    var nP1 = pArray1.length, nP2 = pArray2.length;
    if (nP1 != nP2)
        return false;

    pArray1.sort();
    pArray2.sort();
    // Sort the paths before comparing them
    for (var i = nP1 - 1; i >= 0; i--) {
        // Compare property name
        if (pArray1[i] != pArray2[i]) {
            ret = false;
            break;
        }
        // Compare property value
        var p = pArray1[i];
        if (this.GetProperty(p) != anotherBag.GetProperty(p)) {
            ret = false;
            break;
        }
    }
    return ret;
}

///////////////////////////////////////////////////////
//
// Merges given set of properties into member property
// bag
//
PropertyBag.prototype.SetProperties=function(propertySet)
{

	var toDelete = new Array
	for (var path in propertySet.propValues)
	{
		// Check against existing elements to delete
		var bProcessed = false;
		for (var i=0; i< toDelete.length && !bProcessed;i++)
		{

			// If a child skip ...
			if (IsAncestorPathOf(toDelete[i],path))
				bProcessed = true;
			else
			{
				// ... if a parent replace
				if (IsAncestorPathOf(path,toDelete[i]))
				{
					toDelete[i] = path;
					bProcessed = true;
				}
			}

		}

		// Add as new root if neither parent nor child
		if (!bProcessed)
			toDelete[toDelete.length] = path;
	}

	this.Merge(toDelete,propertySet);
}

///////////////////////////////////////////////////////
//
// Displays contents of property bag in a message box.
// For debugging only
//
PropertyBag.prototype.ShowValues = function(title)
{
    var str = ""
    if (title) {
        str += title + "\r\n";
    }
    for (var pp in this.propValues)
    {
        str += pp + "=" + this.propValues[pp] +"\r\n"
    }
    alert(str)
}

////////////////////////////////////////////////////////
//
// Displays contents of property bag matching a given property path
//
PropertyBag.prototype.ShowSubset = function(pp, title) {
    var pbT = new PropertyBag();
    var ppStr = pp.toString();
    this.GetElement(ppStr, pbT);
    if (null == title)
        title = '"' + ppStr + '"';
    pbT.ShowValues(title);
}

PropertyBag.prototype.ShowScoreValues = function(regexrule)
{
    var str = ""
   
	var i=0;
	
    for (var pp in this.propValues)
    {
		str = this.propValues[pp]

		if((i>1 && regexrule) || (i>4 && !regexrule))
		{
			break;
		}
		i++;
    }
	
    return str
}
