// Copyright (C) 2009 McAfee, Inc.  All rights reserved.

///////////////////////////////////////////////////
//
// Constructor for PropertyPath class
//
function PropertyPath(pathIn)
{
	// Member data
	this.path = pathIn
	this.enumerator = 0
	this.tempLeaf = ""

	// Member functions
	this.Append = Append
	this.RemoveLeaf = RemoveLeaf
	this.GetPath = GetPath
	this.BeginEnum = BeginEnum
    this.SetEnum = SetEnum
	this.IncrementEnum = IncrementEnum
	this.EndEnum = EndEnum
	this.SetTempLeaf = SetTempLeaf
	this.Copy = Copy
	this.SetPath = SetPath

    this.toString = GetPath;
}

function GetPaddedEnum(enumVal)
{
	var strEnum = enumVal.toString(10)
	var strPadding = "00000000"
	strPadding = strPadding.substr(0, 8 - strEnum.length)
	return strPadding + strEnum
}

///////////////////////////////////////////////////
//
// Sets path to given string
//
function SetPath(pathIn)
{
	this.path = pathIn
	this.enumerator = 0
	this.tempLeaf = ""
}

///////////////////////////////////////////////////
//
// Adds element name to end of current path
//
function Append(element)
{
	var pathOut = this.GetPath()
	if (pathOut != "")
		pathOut += "."
	this.path = pathOut + element
}

///////////////////////////////////////////////////
//
// Removes last element from current path
//
function RemoveLeaf()
{
	this.path = this.path.substr(0,this.path.lastIndexOf("."))
	this.enumerator = 0
	this.tempLeaf = ""
}

///////////////////////////////////////////////////
//
// Returns full path represented by current
// member data
//
function GetPath()
{
	var strOut = this.path

	if (this.enumerator > 1)
		strOut += ":" + GetPaddedEnum(this.enumerator)

	if (this.tempLeaf != "")
		strOut += "." + this.tempLeaf

	return strOut
}

//////////////////////////////////////////////////
//
// Enumeration
//
function BeginEnum()
{
	this.enumerator = 1
}
function SetEnum(n) {
    this.enumerator = n;
}
function IncrementEnum()
{
	this.enumerator++
}
function EndEnum()
{
	this.enumerator = 0
}

/////////////////////////////////////////////////
//
// Assign a temporary leaf name. Useful during
// enumeration
//
function SetTempLeaf(leaf)
{
	this.tempLeaf = leaf
}

////////////////////////////////////////////////
//
// Copy given instance to local instance
//
function Copy(aPath)
{
	this.path = aPath.path
	this.enumerator = aPath.enumerator
	this.tempLeaf = aPath.tempLeaf
}


///////////////////////////////////////////////
//
// Static helper functions
//
///////////////////////////////////////////////

///////////////////////////////////////////////
//
// Returns true if path1 is an ancestor of path2
// both paths are passed as strings
//
function IsAncestorPathOf(path1,path2)
{
	var str = path1 + "."
	return path2.substr(0,str.length) == str
}

///////////////////////////////////////////////
//
// Returns the enumeration step  (1-based)
// of the given path at the specified 0-based level
//
function GetPathEnum(path,level)
{
	// Move to correct level
	var levelStart= 0
	for (var i = -0; i < level; i++)
		levelStart= path.indexOf(".",levelStart) + 1

	// Look for next delimiter
	var nextColon =  path.indexOf(":",levelStart)
	var nextPeriod = path.indexOf(".",levelStart)

	// Return 1 for no enumerator
	if (nextColon == -1)
		return 1

	// Return 1 for no enumerator at this level
	if (nextPeriod != -1 && nextPeriod <= nextColon)
		return 1

	// Return parsed enum value
	if (nextPeriod != -1)
		return parseInt(path.substring(nextColon+1,nextPeriod), 10)
	else
		return parseInt(path.substring(nextColon+1), 10)
}

///////////////////////////////////////////////
//
// Returns number of levels in given path
// (0-based)
//
function GetPathLevel(path)
{
	var index  = 0
	var level = -1

	do
	{
		level++
		index = path.indexOf(".",index) +1
	}
	while (index != 0)
	return level
}

///////////////////////////////////////////////
//
// Shifts enumerator at given level in given path
// by specified amount
//
function SetPathEnum(path,level,value)
{

	// Move to specified level
	var levelStart= 0
	for (var i = 0; i < level; i++)
		levelStart= path.indexOf(".",levelStart) + 1

	// Look for next delimiter
	var nextColon =  path.indexOf(":",levelStart)
	var nextPeriod = path.indexOf(".",levelStart)

	// Map "not found" to "end of string"
	if (nextColon == -1)
		nextColon = path.length
	if (nextPeriod == -1)
		nextPeriod = path.length

	// Build out path
	var pathOut = path.substring(0, min(nextColon,nextPeriod))
	if (value>1)
		pathOut += ":" + GetPaddedEnum(value)
	if (nextPeriod != path.length)
		pathOut += path.substr(nextPeriod)
	return pathOut
}

///////////////////////////////////////////////
//
// Removes any enumerator leaf element
//
function RemoveLeafEnum(pp)
{
	var index = pp.lastIndexOf(":")
	if (index > pp.lastIndexOf("."))
		return pp.substr(0,index)
	else
		return pp
}

/////////////////////////////////////////////
//
// Class method to return a guaranteed
// PropertyPath object whether or not the input
// is a string or PropertyPath object
PropertyPath.MakePP = function(pp) {
    var ret = pp;
    if ("string" == typeof(pp)) {
	ret = new PropertyPath(pp);
    }
    return pp;
}

/////////////////////////////////////////////
//
// Retrieve leaf
//
PropertyPath.GetLeaf = function(pp,removeDec)
{
	if (removeDec != false)
		pp =RemoveLeafEnum(pp)
 	return pp.substr(pp.lastIndexOf(".") +1);
}

/////////////////////////////////////////////
//
// Retrieve root
//
PropertyPath.GetRoot = function(pp)
{
	var i = pp.indexOf(".")
	return (i <0) ? pp : pp.substr(0,i)
}

