//Copyright (C) 2005 McAfee, Inc.  All rights reserved.
var nMaxValLen		= 230 // maximum length of one value
var nMaxSectionItems	= 10 // maximum number of values within one section
   			
/////////////////////////////////////////////////
//
// Constructor for property source class
//
function PropertySource(bSched)
{
	// Initialize members
	this.Put = Put
	this.Get = Get
	this.bSched = bSched == true
	this.GetString = GetString
	this.SetString = SetString
	this.GetBool = GetBool
	this.SetBool = SetBool	
	this.PutLongString = PutLongString
}

/////////////////////////////////////////////////
//
// Abstraction of 
//
function GetString(section,name)
{
	if(gIsEPO4)
	{
		return this.bSched ? parent.PolicyCore.legacyGetPolicySettingString(section,name) : parent.PolicyCore.legacyGetPolicySettingString(section,name)
	}
	else
	{
		return this.bSched ? window.external.schedule_value_string(section,name) : window.external.policy_value_string(section,name)
	}
}
function SetString(section,name,value)
{
	if (this.bSched)
	{
		gIsEPO4 ? parent.PolicyCore.legacySetPolicySettingString(section,name,value) : window.external.schedule_value_string(section,name) = value
	}
	else
	{
		gIsEPO4 ? parent.PolicyCore.legacySetPolicySettingString(section,name,value) : window.external.policy_value_string(section,name) = value
	}
}
function GetBool(section,name)
{	
	if(gIsEPO4)
	{
		return this.bSched ? parent.PolicyCore.legacyGetPolicySettingBool(section,name) : parent.PolicyCore.legacyGetPolicySettingBool(section,name)
	}
	else
	{
		return this.bSched ? window.external.schedule_value_bool(section,name) : window.external.policy_value_bool(section,name)
	}
}
function SetBool(section,name,value)
{
	if (this.bSched)
	{
		gIsEPO4 ? parent.PolicyCore.legacySetPolicySettingBool(section,name,value) : window.external.schedule_value_bool(section,name) = value
	}
	else
	{
		gIsEPO4 ? parent.PolicyCore.legacySetPolicySettingBool(section,name,value) : window.external.policy_value_bool(section,name) = value
	}
}
/////////////////////////////////////////////////
//
// Rebuilds long string split into parts to 
// avoid ePO limit on policy length
//
function GetLongString(section, valNum,nParts)
{
	
	// Read each part in turn, remove quotes, and append
	var strOut = ""
	var strCurrentSection = ""
	var nCurrentSection = -1

	for (var i=0; i < nParts; i++)
	{
		// update section name if maximum number of items per section is reached
		// 0 modulo x = 0 --> first loop enters following if and sets section name
		if ((i % nMaxSectionItems) == 0)
		{
			nCurrentSection = nCurrentSection + 1
			// section name format is the same as for values: e.g.: [eServices0_0], [eServices0_1], [eServices0_2],...
			strCurrentSection = section + valNum + "_" + nCurrentSection
		}

		var strTemp = this.GetString(strCurrentSection, "value"+valNum+"_"+i)
		strOut += strTemp.substr(1,strTemp.length-2)		
	}

	// Return result
	return strOut
}

/////////////////////////////////////////////////
//
// Writes a long string policy to ePO
//
function PutLongString(name,value, section, valNum)
{
	// Write string in nMaxValLen char chunks
	var iStart = 0
	var iChars = 0
	var iIndex =0
	var strCurrentSection = ""
	var nCurrentSection = -1

	while (iStart < value.length)
	{
		// update section name if maximum number of items per section is reached
		// 0 modulo x = 0 --> first loop enters following if and sets section name
		if ((iIndex % nMaxSectionItems) == 0)
		{
			nCurrentSection = nCurrentSection + 1
			// section name format is the same as for values: e.g.: [eServices0_0], [eServices0_1], [eServices0_2],...
			strCurrentSection = section + valNum + "_" + nCurrentSection

			// additionally write the inherit state for each section
			this.SetBool(strCurrentSection, "Inherit",this.bInherit)		
		}

		iChars = min(nMaxValLen, value.length-iStart )
		this.SetString(strCurrentSection, "value"+valNum+"_"+iIndex,"\"" + value.substr(iStart ,iChars) +"\"")
		iStart += iChars
		iIndex++
	}

	// Write# parts
	this.SetString(section, "value"+valNum, name + "|longstring|"+iIndex)

	// Write type	
	this.SetString(section, "type"+valNum,"longstring")

}

/////////////////////////////////////////////////
//
// Load properties from ePO server into local
// property bag 
//
function Get(aBag,sections)
{
	// Make sure bag is empty
	aBag.Empty()

	// Set sections to retrieve from ePO
	this.sections = (sections == McUndefined) ? gePOCategorySectionsMap[gContentPage] : sections

	// Iterate through sections
	for (var iSection = 0; iSection < this.sections.length; iSection++)
	{
		
		// Read  number of extraSections (needed for ePO 3.5 only)
		var section=this.sections[iSection];
		var iExtra=0;
		var nExtras = Number(this.GetString(section+"+0", "value0"))
		if (isNaN(nExtras)) nExtras=0;
		for (var iExtra=0;iExtra<=nExtras;iExtra++)
		{
			// Build section name
			section=iExtra? (this.sections[iSection]+"+"+iExtra):this.sections[iSection];

			// Retrieve # of props from ePO
			var nValues = Number(this.GetString(section, "NumberOfValues"))

			// Iterate through values for each section
			for (var i = 0; i < nValues; i++)
			{
				// Get property name, value and type
				var raw=this.GetString(section , "value"+i);
				var i1=raw.indexOf("|");
				var i2=raw.indexOf("|",i1+1)
				var policyName = raw.substring(0,i1);
				var policyType = raw.substring(i1+1,i2);
				var policyValue = raw.substring(i2+1);
			
				// Write to property bag
				switch (policyType)
				{
					case "undefined":
						aBag.SetProperty(policyName)
					break
			
					case "boolean":
						aBag.SetProperty(policyName,eval(policyValue))
					break

					case "number":
						aBag.SetProperty(policyName,Number(policyValue))
					break

					case "longstring":
						aBag.SetProperty(policyName,Decompress(GetLongString(section, i,parseInt(policyValue,10))))
					break		
					default:
						// Fix for BZ:533910 & 523488 i.e formatting of Customized alerts and BZ:850973
						//if(raw.indexOf(".name|string|") != -1 || raw.indexOf(".displayName|string|") != -1 || raw.indexOf(".ModSubjPrefix|string|") != -1 || raw.indexOf(".text|string|") != -1 || raw.indexOf(".Subject|string|") != -1 || raw.indexOf(".DisclaimerText|string|") != -1 || raw.indexOf(".DisclaimerHTML|string|") != -1)
						if(raw.indexOf(".name|string|") != -1 || raw.indexOf(".displayName|string|") != -1 || raw.indexOf(".ModSubjPrefix|string|") != -1 || raw.indexOf(".text|string|") != -1 || raw.indexOf(".Subject|string|") != -1 || raw.indexOf(".DisclaimerText|string|") != -1 || raw.indexOf(".DisclaimerHTML|string|") != -1 || raw.indexOf(".ALERTHEADER|string|") != -1 || raw.indexOf(".ALERTFOOTER|string|") != -1)
						{
							aBag.SetProperty(policyName,Decompress(policyValue))
						}
						else
						{
							aBag.SetProperty(policyName,policyValue)
						}
					break
				}
			}
		}
	}
	gbConfigLoaded = true
}

/////////////////////////////////////////////////
//
// Sends the given set of properties to ePO
// server by writing to the hidden form and then
// submitting
//
var EPOMAXSECTIONCHARS=0x10000
function Put(aBag,bInherit)
{
	// Force navigation to top of page stack if running ePO 3.5
	if (gIsePO35) {
		top.gPageStack.Goto(0);
	}
	// Cache inherit state
	this.bInherit = (bInherit == McUndefined) ? tasks.inherit.checked : bInherit

	// Iterate through bag to build list of sections being put
	var sections = new Array

	for (var pp in aBag.propValues)
    	{		
		// Get name and extract section (root element name)
		var section = pp
		if (section.indexOf(".") != -1)
		{
			section = section.substr(0,section.indexOf("."))
		}

		// Add to list of section names if not already there
		if (FindInArray(sections,section) == -1)
			sections[sections.length] = section
	}

	// Write inherit flag for each section (must be done before all properties
	// or it wont work)
	for (var i=0; i < sections.length; i++)
		this.SetBool(sections[i], "Inherit",this.bInherit)		

	// Iterate through each section, writing properties
	var pbSection = new PropertyBag
	for (var i=0; i < sections.length; i++)
	{	
		// Extract all values for current section into property bag
		var section=sections[i];
		var nExtraSections=0;
		var iSectionLength=0;
		pbSection.Empty()
		aBag.GetElement(section,pbSection)

		// write each value in turn
		var i2 = 0;
		for (var pp2 in pbSection.propValues)
		{
			// Build value
			var value = pbSection.GetProperty(pp2);
			var valueStr = (value == top.gValueless) ? (pp2+"|"+undefined+"|") : (pp2+"|"+(typeof value)+"|"+value);

			// Check for a long string 
			if ((typeof value)=="string" && value.length > nMaxValLen )
				this.PutLongString(pp2,Compress(value),section,i2);
			else
			{
				
				// Check for total length exceeding ePO 3.5 limit and start an extra section
				var name="value"+i2;
				if (gIsePO35  && (iSectionLength+name.length+valueStr.length+1)>EPOMAXSECTIONCHARS)
				{
					this.SetString(section, "NumberOfValues",i2);	
					section=sections[i]+"+"+(++nExtraSections);
					this.SetBool(section, "Inherit",this.bInherit);		
					i2=0;
					iSectionLength=0;
					name="value"+i2;
				}		
			
				// Write value
				// Fix for BZ:533910 & 523488 i.e formatting of Customized alerts and BZ:850973
				//if(valueStr.indexOf(".name|string|") != -1 || valueStr.indexOf(".displayName|string|") != -1 || valueStr.indexOf(".ModSubjPrefix|string|") != -1 || valueStr.indexOf(".text|string|") != -1 || valueStr.indexOf(".Subject|string|") != -1 || valueStr.indexOf(".DisclaimerText|string|") != -1 || valueStr.indexOf(".DisclaimerHTML|string|") != -1)
				if(valueStr.indexOf(".name|string|") != -1 || valueStr.indexOf(".displayName|string|") != -1 || valueStr.indexOf(".ModSubjPrefix|string|") != -1 || valueStr.indexOf(".text|string|") != -1 || valueStr.indexOf(".Subject|string|") != -1 || valueStr.indexOf(".DisclaimerText|string|") != -1 || valueStr.indexOf(".DisclaimerHTML|string|") != -1 || valueStr.indexOf(".ALERTFOOTER|string|") != -1 || valueStr.indexOf(".ALERTHEADER|string|") != -1)
				{
					this.SetString(section,name,Compress(valueStr));
				}
				else
				{
					this.SetString(section,name, valueStr);
				}
				iSectionLength+=name.length+valueStr.length+1;
			}
			i2++;
		}

		// Write number of values in this section
		this.SetString(section, "NumberOfValues",i2)	
		
		// If we have written any extra sections that add a stub section to report how many
		if (gIsePO35)
		{
			this.SetString(sections[i]+"+0", "NumberOfValues",1);
			this.SetString(sections[i]+"+0", "value0",nExtraSections);
		}

	}
		
	// Commit changes	
	if (!this.bSched) {	
		if( gIsePO35 ) // A temporary fix for the refresh problem in ePO3.5.
			setTimeout("window.external.SavePolicySettings()", 100)
		else
		{
			if(gIsEPO4)
			{
				parent.PolicyCore.legacySavePolicySettings();
			}
			else
			{
				window.external.SavePolicySettings()
			}
		}

}
}
function Compress(szToCompress)
{
//	szToCompress =  szToCompress.replace(/&/g,"&;") ;	// change our escape char to unique strings
	szToCompress =  szToCompress.replace(/{/g,"&ob;") ; 	// encode opening bracket { to &ob;
	szToCompress =  szToCompress.replace(/}/g,"&cb;") ; 	// encode closing bracket } to &cb;
	szToCompress =  szToCompress.replace(/</g,"{") ;	// translate < to opening bracket {
	szToCompress =  szToCompress.replace(/>/g,"}") ;	// translate > to closing bracket }
	szToCompress =  szToCompress.replace(/~/g,"&e;") ; 	// encode tilde to &e; - our escape character
	szToCompress =  szToCompress.replace(/name=/g,"~n") ;
	szToCompress =  szToCompress.replace(/value=/g,"~v") ;
	szToCompress =  szToCompress.replace(/Attr/g,"~a") ;
	szToCompress =  szToCompress.replace(/Match/g,"~m") ;
	szToCompress =  szToCompress.replace(/case="nocare"/g,"~c0") ;
	szToCompress =  szToCompress.replace(/case="care"/g,"~c1") ;
	szToCompress =  szToCompress.replace(/text=/g,"~t") ;
	szToCompress =  szToCompress.replace(/isend="care"/g,"~e1") ;
	szToCompress =  szToCompress.replace(/isend="nocare"/g,"~e0") ;
	szToCompress =  szToCompress.replace(/isstart="care"/g,"~s1") ;
	szToCompress =  szToCompress.replace(/isstart="nocare"/g,"~s0") ;
	szToCompress =  szToCompress.replace(/CFRule/g,"~r") ;
	szToCompress =  szToCompress.replace(/NearAll/g,"~x") ;
	szToCompress =  szToCompress.replace(/List/g,"~l") ;
	szToCompress =  szToCompress.replace(/\r\n/g,"~e") ;		// for ePO 2.5 support, encode all new lines (enter) to ~e
	szToCompress =  szToCompress.replace(/\n/g,"~d") ;

	return szToCompress ;
}

function Decompress(szToCompress)
{
	szToCompress =  szToCompress.replace(/~n/g,"name=") ;
	szToCompress =  szToCompress.replace(/~v/g,"value=") ;
	szToCompress =  szToCompress.replace(/~a/g,"Attr") ;
	szToCompress =  szToCompress.replace(/~m/g,"Match") ;
	szToCompress =  szToCompress.replace(/~c0/g,"case=\"nocare\"") ;
	szToCompress =  szToCompress.replace(/~c1/g,"case=\"care\"") ;
	szToCompress =  szToCompress.replace(/~t/g,"text=") ;
	szToCompress =  szToCompress.replace(/~e1/g,"isend=\"care\"") ;
	szToCompress =  szToCompress.replace(/~e0/g,"isend=\"nocare\"") ;
	szToCompress =  szToCompress.replace(/~s1/g,"isstart=\"care\"") ;
	szToCompress =  szToCompress.replace(/~s0/g,"isstart=\"nocare\"") ;
	szToCompress =  szToCompress.replace(/~r/g,"CFRule") ;
	szToCompress =  szToCompress.replace(/~x/g,"NearAll") ;
	szToCompress =  szToCompress.replace(/~l/g,"List") ;
	szToCompress =  szToCompress.replace(/~e/g,"\r\n") ;			// ~e (enter) is \r\n
	szToCompress =  szToCompress.replace(/~d/g,"\n") ;
	szToCompress =  szToCompress.replace(/&e;/g,"~") ;
	szToCompress =  szToCompress.replace(/{/g,"<") ;
	szToCompress =  szToCompress.replace(/}/g,">") ;
	szToCompress =  szToCompress.replace(/&ob;/g,"{") ;
	szToCompress =  szToCompress.replace(/&cb;/g,"}") ;
//	szToCompress =  szToCompress.replace(/&;/g,"&") ;

	return szToCompress ;
}