/**
 * JavaScript for the Proxy Settings page.
 *
 * Copyright (C) 2010 McAfee, Inc. All rights reserved
 */

var vfOnProxy		= new ValidationFramework("proxyForm");
var labelMap = new Object();
labelMap["portid"] = gsPortNoString;
/**
 * Asynchronous data load function
 */
function Show(pbIn) {
	var pp = new PropertyPath("ProxySettings")
	
	var rangeVtor = new RangeValidator(1, 65535);
	vfOnProxy.addValidator("portid", rangeVtor);
	vfOnProxy.labelMap = labelMap;
    vfOnProxy.formAction = std_val_form_action;
	vfOnProxy.fieldAction = std_val_field_action;

	var inputTextVtor = new InputTextValidator();
	vfOnProxy.addValidator("usernameid", inputTextVtor);

	// Set proxy configuration settings
	pp.SetTempLeaf("enabled");
	var proxyEnable = pbIn.GetProperty(pp) == 1;
	document.getElementById("noproxyid").checked = !proxyEnable;
	document.getElementById("proxyid").checked = proxyEnable;

	// Set the proxy address
	pp.SetTempLeaf("ProxyAddress");
	var proxyAddress = pbIn.GetProperty(pp);
	proxyAddress = proxyAddress.split(":",2);
	document.getElementById("ipid").value = proxyAddress[0];
	document.getElementById("portid").value = proxyAddress[1];

	// Set the proxy type
	pp.SetTempLeaf("ProxyType");
	var proxyType = pbIn.GetProperty(pp);
	document.getElementById("anonymousid").checked = proxyType == 0;
	document.getElementById("ntlmid").checked = proxyType == 1;
	document.getElementById("basicid").checked = proxyType == 2;

	// Set username and password
	pp.SetTempLeaf("ProxyUserAccountDetails");
	var proxyUserAccountDetails = pbIn.GetProperty(pp);
	proxyUserAccountDetails = proxyUserAccountDetails.split(":",2);
	document.getElementById("usernameid").value = proxyUserAccountDetails[0];
	
	//BZ: 588972, BZ:936624
	//var decodedValue = DecodeBase64(proxyUserAccountDetails[1]);
	var decodedValue = proxyUserAccountDetails[1];
	
	document.getElementById("passwordid").value = decodedValue;
	document.getElementById("repasswordid").value = decodedValue;

	EnableDisableUI(proxyEnable, proxyType);
	
	// Fix for BZ:669141 670893
	if(!(IsBHInstalled() || IsGWInstalled()))
	{
		EnableDisableUI(0, proxyType);
		document.getElementById("noproxyid").disabled = true
		document.getElementById("proxyid").disabled = true
		document.getElementById("proxy_disabledmsg").style.display = ""			
	}	
}

/**
 * Asynchronous data validation function
 */
function Validate()
{
	if (document.getElementById("noproxyid").checked)
	{
		// No need to validate.
		return true;
	}

	// Validate for Proxy IP
	var proxyIP = document.getElementById("ipid").value;
	var ipPattern = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/
	var ipArray = proxyIP.match(ipPattern);
	if (ipArray == null)
	{
		alert(gsValidIPError);
		document.getElementById("ipid").select();
		return false;
	}
	for (i = 1; i < 5; ++i)
	{
		if (ipArray[i] > 255)
		{
			alert(gsValidIPError);
			document.getElementById("ipid").select();
			return false;
		}
	}

	// Validate for Proxy Port
	var proxyPort = document.getElementById("portid").value;
	var proxyArray = proxyPort.match(/^(\d+)$/);
	if (proxyArray == null)
	{
		alert(gsValidPortError);
		document.getElementById("portid").select();
		return false;
	}

	var proxyType = (1 * document.getElementById("ntlmid").checked) + (2 * document.getElementById("basicid").checked)
	if (proxyType == 2)
	{
		// Basic Authentication
		var userName = document.getElementById("usernameid").value;
		var password = document.getElementById("passwordid").value;
		var repassword = document.getElementById("repasswordid").value;

		// trim username
		userName = userName.replace(/^\s*/, '').replace(/\s*$/, '');
		if (userName.length == 0)
		{
			alert(gsValidUserError);
			document.getElementById("usernameid").select();
			return false;
		}

		if (password != repassword)
		{
			alert(gsValidPasswordError);
			document.getElementById("passwordid").select();
			return false;
		}
	}

	return(vfOnProxy.onsubmit());
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb)
{
	var pp = new PropertyPath("ProxySettings")
	pb.SetProperty(pp);
	
	pp.SetTempLeaf("enabled")
	pb.SetProperty(pp, document.getElementById("proxyid").checked)
	
	// Set the proxy address
	var proxyAddress = document.getElementById("ipid").value + ":" + document.getElementById("portid").value
	pp.SetTempLeaf("ProxyAddress")
	pb.SetProperty(pp, proxyAddress)

	// Set the proxy type
	// Anonymous :				0
	// NTLM :					1
	// Basic Authentication :	2
	var proxyType = (1 * document.getElementById("ntlmid").checked) + (2 * document.getElementById("basicid").checked)
	pp.SetTempLeaf("ProxyType")
	pb.SetProperty(pp, proxyType)

	// Set username and password
	if (proxyType != 2)
	{
		// Store empty user name and password if Basic authentication is not selected.
		proxyUserAccountDetails = ":";
	}
	else
	{
		var proxyUserAccountDetails = document.getElementById("usernameid").value + ":";
		var proxyPasswordDetails = document.getElementById("passwordid").value;
		//BZ:936624
		// Encode using Base64 encoding
		//proxyPasswordDetails = EncodeBase64(proxyPasswordDetails);
		proxyUserAccountDetails += proxyPasswordDetails;
	}

	pp.SetTempLeaf("ProxyUserAccountDetails")
	pb.SetProperty(pp, proxyUserAccountDetails)
}

function EnableDisableUI(proxy, proxytype)
{
	document.getElementById("ipid").disabled = !proxy;
	document.getElementById("portid").disabled = !proxy;
	document.getElementById("anonymousid").disabled = !proxy;
	document.getElementById("ntlmid").disabled = true;
	document.getElementById("basicid").disabled = !proxy;
	document.getElementById("usernameid").disabled = !proxy || proxytype != 2;
	document.getElementById("passwordid").disabled = !proxy || proxytype != 2;
	document.getElementById("repasswordid").disabled = !proxy || proxytype != 2;
}

function EnableDisable()
{
	var proxyType = (1 * document.getElementById("ntlmid").checked) + (2 * document.getElementById("basicid").checked)
	EnableDisableUI(document.getElementById("proxyid").checked, proxyType);
}

function EncodeBase64(input)
{
	var _keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
	var output = "";
	var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
	var i = 0;

	while (i < input.length) {

		chr1 = input.charCodeAt(i++);
		chr2 = input.charCodeAt(i++);
		chr3 = input.charCodeAt(i++);

		enc1 = chr1 >> 2;
		enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
		enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
		enc4 = chr3 & 63;

		if (isNaN(chr2)) {
			enc3 = enc4 = 64;
		} else if (isNaN(chr3)) {
			enc4 = 64;
		}

		output = output + _keyStr.charAt(enc1) + _keyStr.charAt(enc2) + _keyStr.charAt(enc3) + _keyStr.charAt(enc4);
	}

	return output;
}
//BZ: 588972
function DecodeBase64(input)
{
    var e={},i,k,v=[],r='',w=String.fromCharCode;
    var n=[[65,91],[97,123],[48,58],[47,48],[43,44]];

    for(z in n){for(i=n[z][0];i<n[z][1];i++){v.push(w(i));}}
    for(i=0;i<64;i++){e[v[i]]=i;}

    for(i=0;i<input.length;i+=72)
	{
		var b=0,c,x,l=0,o=input.substring(i,i+72);
        for(x=0;x<o.length;x++)
		{
               c=e[o.charAt(x)];b=(b<<6)+c;l+=6;
               while(l>=8){r+=w((b>>>(l-=8))%256);}
        }
    }
    return r;
}