/**
 * Interface to backend resources files (e.g. defaultalerts_xx.xml)
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

// Map between resource keys and property paths
if (null == top.rm_ResourceMap) {
    top.rm_ResourceMap = {};
}

/**
 * Creates a ResourceManager object
 * @param source the name of the source (e.g. "DefaultAlerts")
 */
function ResourceManager(source) {
    this.source = source;
}

/**
 * Gets a resource from resources
 * @param section the section of interest (e.g. "AlertMessages")
 * @param key the key you're looking for (e.g. "AV_REPLACEMENT")
 * @return the resource string or "" if not found
 */
ResourceManager.prototype.getResource = function(section, key) {
    var ret = "";
	var pbName = this.source;

    // Don't attempt to look up the resource if the resource file has
    // not been loaded
    if (null != top.gPBContainer[pbName]) {
        var pb = top.getCustomPB(pbName);
        // See whether we've already resolved the path for this
        // resource
        var lookup = section + "." + key;
        if (null != top.rm_ResourceMap[lookup]) {
            var pp = top.rm_ResourceMap[lookup];
            if ("" != pp) {
                ret = pb.GetPropertyDef(pp, "");
            }
        }
        else {
            // We need to find the property path
            var pp = this.getPath(pb, section, key);
            if ("" != pp) {
                ret = pb.GetPropertyDef(pp, "");
            }
            // Store the path regardless of whether it's empty
            top.rm_ResourceMap[lookup] = pp;
        }
    }

    return ret;
}

/**
 * Looks up the path to a given property
 * @param pb the PropertyBag to search
 * @param section the section to look for
 * @param key the name of the resource string to find
 * @return the path or "" if not found
 */
ResourceManager.prototype.getPath = function(pb, section, key) {
    var ret = "";

    var basePP = "Resources.ResourceTable";
    var pp1 = new PropertyPath(basePP);
    for (pp1.BeginEnum(); null != pb.GetProperty(pp1); pp1.IncrementEnum()) {
        var ppT = pp1.GetPath() + ".name";
        if (pb.GetProperty(ppT) == section) {
            // Found the section
            if (null == key) {
                ret = pp1.GetPath();
            }
            else {
                var pp2 = new PropertyPath(pp1.GetPath() + ".ResourceValue");
                for (pp2.BeginEnum(); null != pb.GetProperty(pp2); pp2.IncrementEnum()) {
                    ppT = pp2.GetPath() + ".Id";
                    if (pb.GetProperty(ppT) == key) {
                        // Located the resource
                        ret = pp2.GetPath() + ".value";
                        break;
                    }
                }
            }
            break;
        }
    }

    return ret;
}

/**
 * Gets the available alert types
 * @param pb the PropertyBag to search
 * @return a list of alert types
 */
ResourceManager.prototype.getAlertTypes = function() {
    var ret = [];
	var pbName = this.source;


    if (null != top.gPBContainer[pbName]) {
        var pb = top.getCustomPB(pbName);
        var basePP = this.getPath(pb, "AlertMessages", null);
        if (basePP) {
            var pp = new PropertyPath(basePP += ".ResourceValue");
            var typeMap = new Object();
            var pRE = /_PLAIN$/;
            var fpRE = /_FORMATTEDPLAIN$/;
            for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
                var ppT = pp.GetPath() + ".Id";
                var id = pb.GetProperty(ppT);
                if (!id)
                    continue;
                if (id.match(pRE))
                    id = id.replace(pRE, "");
                else if (id.match(fpRE))
                    id = id.replace(fpRE, "");
                typeMap[id] = 1;
            }
            // Read off the undecorated alert types that we have obtained
            for (var type in typeMap) {
                ret.push(type);
            }
        }
    }
    return ret;
}

/**
 * Gets the tokens appropriate to an alert category; depends on a structure in
 * PointProductSpecific.js called pp_cse_categoryContextMap
 * @param type the alert category
 * @return a list of tokens, keyed as "token:x, value:y"
 */
ResourceManager.prototype.getTokens = function(type) {
    var ret = [];

    var context = pp_cse_categoryContextMap[type]
    if (null == context)
        return ret;

	var pbName = this.source;
    if (null != top.gPBContainer[pbName]) {
        var pb = top.getCustomPB(pbName);
        var basePP = this.getPath(pb, "AvailableTokens", null);
        if (basePP) {
            var pp = new PropertyPath(basePP += ".ResourceValue");
            for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
                var ppT = pp.GetPath() + ".context";
                if (context != pb.GetProperty(ppT))
                    continue;
                pp.SetTempLeaf("token");
                var tok = pb.GetPropertyDef(pp, "");
                pp.SetTempLeaf("value");
                var val = pb.GetPropertyDef(pp, "");
                pp.SetTempLeaf("");
                ret.push({token:tok, value:val});
            }
        }
    }

    return ret;
}