/**
 * JavaScript for the Scanners management panel in shared resources
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var sc_pb = null;
var scListWidget = null;

// The resource management driver class for scanners
function ScannersDriver() {
}

// Gets the header text: "Alerts"
ScannersDriver.prototype.getHeader = function() {
    return resolveString(cse_pp_st, "__scanners");
}

// Gets the select directive: "Category:"
ScannersDriver.prototype.getSelDirective = function() {
    return resolveString(cse_pp_st, "__cat_sel_directive");
}

// Gets category list for displaying in a dropdown list
ScannersDriver.prototype.getCategoryList = function() {
    var ret = [];

    var catList = scanners_getOrderedTypes();
    for (var i = 0; i < catList.length; i++) {
        ret.push({value:catList[i], text:resolveString(cse_pp_st, catList[i])});
    }

    return ret;
}

// Creates a new scanner configuration
ScannersDriver.prototype.newInstance = function(cat) {
    NewInstance(cat);
}

// Lists alerts in a given category
ScannersDriver.prototype.drawList = function(category, tbody, listWidget) {
    var links = scanners_getListInfo(category);
    for (var i = 0; i < links.length; i++) {
        var link = links[i];
        var row = tbody.insertRow(i);
        var cellIdx = 0;
        var td = row.insertCell(cellIdx++);
        td.appendChild(document.createTextNode(link.text));

        td = row.insertCell(cellIdx++);
        var usedByStr = (1 == link.refCount) ? resolveString(cse_pp_st, "__one_policy") :
            resolveString(cse_pp_st, "__n_policies", [link.refCount]);
        td.appendChild(document.createTextNode(usedByStr));

        td = row.insertCell(cellIdx++);
        td.appendChild(SharedResourceManager.createLink(link.editLink, resolveString(cse_pp_st, "__edit")));
        if (!link.root && 0 == link.refCount) {
            td.appendChild(document.createTextNode(" | "));
            td.appendChild(SharedResourceManager.createLink(link.delLink, resolveString(cse_pp_st, "__delete")));
        }
    }

    if (listWidget)
        listWidget.setHeaderText(0, resolveString(cse_pp_st, category));
}

function DeleteSettings(pp, type) {
    if (confirm(resolveString(cse_pp_st, "__del_prompt"))) {
        s_pb.DeleteElement(pp);
        var dependencyList = pp_cse_settingsDependencies[type];
        if (dependencyList) {
            // Remove dependent settings blobs
            var ppStr = pp.toString();
            for (var i = 0; i < dependencyList.length; i++) {
                var dType = dependencyList[i];
                var _ppStr = ppStr.replace(type, dType);
                if (null != s_pb.GetProperty(_ppStr))
                    s_pb.DeleteElement(_ppStr);
            }
        }
        top.SetModified(true);
        scListWidget.drawInner(type);
    }
}

function LoadSettings(url, title, pp) {
    // Initialize a property bag with just the data required for the new page
    //var pb = new PropertyBag();
    //s_pb.GetElement(pp, pb);
    var page = new DialogPage(url, title);
    top.gPageStack.Forward(page, pp);
}

function NewInstance(type) {
    var basePP = "Settings." + type + "Settings";
    // Get the next enumeration
    var newPP = s_pb.GetNextElementEnum(basePP);
    var target = type + "Settings.html";
    var title = resolveString(cse_pp_st, type);
    top.gPageStack.Forward(new DialogPage(target, title), newPP);
}

function scanners_getListInfo(type) {
    var basePP = "Settings";
    var settingsPage = type + "Settings.html";
    var list = [];
    var pp = new PropertyPath(basePP + "." + type + "Settings");
    pp.BeginEnum();
    var pbT = new PropertyBag();
    while(sc_pb.GetProperty(pp) !== undefined) {
        sc_pb.GetElement(pp, pbT);
        pbT.Empty();
        pp.SetTempLeaf("id");
        var id = sc_pb.GetProperty(pp);
        pp.SetTempLeaf("displayName");
        var text = resolveString(cse_pp_st, sc_pb.GetProperty(pp));
        if (!text) {
            text = id;
        }
        pp.SetTempLeaf("root");
        var root = (1 == sc_pb.GetProperty(pp));
        pp.SetTempLeaf("");
        var href = "javascript:LoadSettings(\"" + settingsPage +
            "\",\"" +	resolveString(cse_pp_st, type) + "\",\"" + pp.GetPath() + "\");";
        var delLink = "javascript:DeleteSettings('" + pp.GetPath() + "', '" + type + "');";
        var refCounter = RefCounter.getRefCount(type, id, null, RefCounter.getSettingsRefCount);
        list.push({"editLink":href, "text":text, "refCount":refCounter.refCount,
            "root":root, "delLink":delLink});
        pp.IncrementEnum();
    }

    return list;
}

/**
 * Gets an ordered list of settings types to display; the order is
 * specified in a PointProductSpecific.js file called "pp_cse_settingsOrder"
 */
function scanners_getOrderedTypes() {
    var types = scanners_getAvailableTypes();
    if (this.pp_cse_settingsOrder) {
        // Display listings in preferred order
        var ordered = [];
        for (var i = 0; i < pp_cse_settingsOrder.length; i++) {
            var type = pp_cse_settingsOrder[i];
            for (var j = 0; j < types.length; j++) {
                if (type == types[j]) {
                    types.splice(j, 1);
                    ordered.push(type);
                    break;
                }
            }
        }
        // Add any remaining types onto the ordered list
        for (var i = 0; i < types.length; i++) {
            ordered.push(types[i]);
        }
        types = ordered;
    }
    return types;
}

/**
 * Gets a list of available types of settings
 */
function scanners_getAvailableTypes() {
    var tmp = new Object();
    var ret = [];

    var pattern = /^Settings\.(\w+)Settings\./;
    var matches = top.gPB.GetMatchingPaths(pattern);

    var excludedSettings = pp_cse_settingsExclusions;

    for (var i = 0; i < matches.length; i++) {
        var type = matches[i][1];
        var addType = pp_componentInstalled(type);
        if (addType) {
            for (var j = excludedSettings.length - 1; j >= 0; j--) {
                if (type == excludedSettings[j]) {
                    addType = false;
                    break;
                }
            }
        }
        if (addType && undefined === tmp[type]) {
            tmp[type] = 1;
        }
    }

    for (var type in tmp) {
        ret.push(type);
    }
    return ret;
}

function scanners_Show(pbIn) {
    // If this page is being displayed as the result of a settings page
    // being modified, we need to merge properties and set the modified
    // flag
    sc_pb = pbIn;

    if (top.polctx && top.polctx.newInstance) {
        // We don't care about this item because we'll pick up
        // the scanner configuration by reading the property bag
        delete top.polctx.newInstance;
    }

    scListWidget = new SharedResourceManager("sListsDiv", "scListWidget", new ScannersDriver(), {verticalSpace:"0px"});
    scListWidget.draw();
}



