/**
 * JavaScript widget for managing shared resources
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

// when the page loads clear the local storage
if (getCookieValue("CurrentCategory")!=null) 
{
    if(getCookieValue("CurrentLang") && getCookieValue("CurrentLang")!=GetLang())
	{
		setCookieValue("CurrentCategory",0,true);
	}
	else
	{
		//Do nothing
	}
}

if (getCookieValue("CurrentLang")!=null && getCookieValue("CurrentLang")!=GetLang()) 
{
	setCookieValue("CurrentLang","",false);
}

// Non-breaking space
var NBSP = 160;
var GenericWCount=0;
var undefinedCount=0;
/*
 * Properties for SharedResourceManager object
 * scrolling - whether the control should draw a scrollbar after a certain number
 * of rows; defaults to true
 * scrollAfter - the number of rows after which any scrollbar should be drawn;
 * defaults to 4
 * rowHeight - the row height to use when calculating the height of the scrolling
 * area; defaults to 25px
 * verticalSpace - the amount of vertical space before the widget; defaults to "10px"
 */

/**
 * Class constructor
 * @param divId the ID of the container for the widget
 * @param objectName the name by which this widget is declared in the JavaScript
 * @param driver a resource-specific object used to populate the widget with data
 * @param properties an Object containg display options, as documented above
 */
function SharedResourceManager(divId, objectName, driver, properties) {
    this.divId = divId;
    this.objectName = objectName ? objectName : "srMan";
    this.driver = driver;
    this.listTBody = null;
    this.innerTableDiv = null;
    srm_init(properties, this);
}

// Initializes the object
function srm_init(propsIn, propsOut) {
    if (null == propsIn)
        propsIn = {};
    propsOut.scrolling = (null != propsIn["scrolling"]) ?
        propsIn["scrolling"] : true;
    propsOut.scrollAfter = (null != propsIn["scrollAfter"]) ?
        propsIn["scrollAfter"] : 5;
    propsOut.rowHeight = (null != propsIn["rowHeight"]) ?
        propsIn["rowHeight"] : 26;
    propsOut.verticalSpace = (null != propsIn["verticalSpace"]) ?
        propsIn["verticalSpace"] : "10px";
    propsOut.uniquifier = Math.random();
}

// Draws the widget
SharedResourceManager.prototype.draw = function() {
    var div = document.getElementById(this.divId);

    if (null == div || null == this.driver)
        return;

    // Empty the div of any current content
    while(div.firstChild)
        div.removeChild(div.firstChild);

    // Create a panel to hold the resource list
    var table = document.createElement("table");
    table.id = this.uniquifier + "_panel";
    table.cellSpacing = table.cellPadding = table.border = 0;
    table.className = "shadedTable";
    table.style.width = "100%";
    table.style.marginTop = this.verticalSpace;
    div.appendChild(table);

    // Create the title row
    var row = table.insertRow(0);
    row.className = "noBottom";

    var cellIdx = 0;
    var td = row.insertCell(cellIdx++);
    td.className = "st bold";
    td.noWrap = true;
    td.appendChild(document.createTextNode(this.driver.getHeader()));

    td = row.insertCell(cellIdx++);
    td.className = "st";
    td.noWrap = true;
    td.align = "right";
    td.appendChild(document.createTextNode(String.fromCharCode(NBSP)));
    
	var catList = [];
	
	if (this.driver.getCategoryList)
        catList = this.driver.getCategoryList();
    if (catList && catList.length) {
        var browserInfo = IdentifyBrowser();
		// Create a language category selector
		if( div.id == "csr_lst_div" )
			{
				td.appendChild(document.createTextNode(this.driver.getLanguage()));
				
				//Populate the language options
				var selLang = document.createElement("select");
				selLang.id = this.uniquifier + "_langSel";
				selLang.style.marginLeft = "5px";
				selLang.style.width = "150px";
				var fnStrLang = this.objectName + ".langChange();";
				selLang.onchange = new Function(fnStrLang);
				td.appendChild(selLang);

				//LangList is defined in Strings.js
				for(index in LangList) 
				{
					selLang.options[selLang.options.length] = new Option(LangList[index], index);
				}

				//setting default or selected language in a session here
				if(getCookieValue("CurrentLang"))
				{
					selLang.value=getCookieValue("CurrentLang");
				}
				else
				{
						selLang.value=GetLang();		
						setCookieValue("CurrentLang",selLang.value,true);
				}
			}
		
		
        //Create Category selector
        td.appendChild(document.createTextNode(this.driver.getSelDirective()));
		
		//Populate Category words
        var selCtl = document.createElement("select");
        selCtl.id = this.uniquifier + "_catSel";
        selCtl.style.marginLeft = "5px";
        selCtl.style.width = "400px";
        var fnStr = this.objectName + ".catChange();";
        selCtl.onchange = new Function(fnStr);
        td.appendChild(selCtl);
		
		// This for loop function is called to first generate all the generic dictionary words once so that they don't have to be populated again once the language changes
		//Whenever draw function will be called GenericWCount will be set to avoid repetition
	
        for (var i = 0; i < catList.length; i++) 
		{
			if(catList[i].lang == McUndefined)
			{
				undefinedCount=undefinedCount+1;
				var option = document.createElement("option");
				option.value = catList[i].value;
				option.text = catList[i].text;
				
				if (browserInfo.isIE) 
				{
					selCtl.add(option);
				}
				else 
				{
					selCtl.add(option, null);
				}
			}
			else
			{
				if(catList[i].lang == "neutral")
					{
						GenericWCount=GenericWCount+1;
						var option = document.createElement("option");
						option.value = catList[i].value;
						option.text = catList[i].text;
					
						if (browserInfo.isIE) 
						{
							selCtl.add(option);
						}
						else 
						{
							selCtl.add(option, null);
						}
						
					}
			
						
			}
        }

		//This for loop is called to append all the language specific dictionary words
		for (var i = 0; i < catList.length; i++) 
		  {
			if(catList[i].lang == McUndefined)
			{
				//Do Nothing 
			}
			
			else
			{
				var lang = GetLang()
				if(catList[i].lang == lang )
					{
						var option = document.createElement("option");
						option.value = catList[i].value;
						option.text = catList[i].text;
					
						if (browserInfo.isIE) 
						{
							selCtl.add(option);
						}
						else 
						{
							selCtl.add(option, null);
						}
					}
				
				
			}
         }
		
		 var selIdx = 0;
		 if (top.pageCtx && top.pageCtx[this.objectName + "catSel"])
            selIdx = top.pageCtx[this.objectName + "catSel"];
        if (selIdx >= selCtl.length)
            selIdx = 0;
			selCtl.selectedIndex = selIdx;
		 if(getCookieValue("CurrentCategory"))
		 {
			var currCat = document.getElementById(this.uniquifier + "_catSel");
			currCat.selectedIndex=getCookieValue("CurrentCategory");
		 }
        
		
    }

    //add newCtl
    if(this.driver.otherTitleCtl)
    {
        this.driver.otherTitleCtl(td);
    }

    // Create the content panel
    row = table.insertRow(1);
    cellIdx = 0;
    td = row.insertCell(cellIdx++);
    td.className = "bordered noLeft";
    td.colSpan = 2;

    var listHolder = document.createElement("div");
    listHolder.style.width = "100%";
    listHolder.style.paddingTop = "5px";
    td.appendChild(listHolder);

    var listPanel = document.createElement("table");
    listPanel.cellSpacing = table.cellPadding = table.border = 0;
    listHolder.appendChild(listPanel);

    row = listPanel.insertRow(0);
    cellIdx = 0;
    td = row.insertCell(cellIdx++);
    td.width = "100%";
    var innerDiv = document.createElement("div");
    innerDiv.id = this.uniquifier + "_listDiv";
    innerDiv.style.padding = "5px 10px";
    td.appendChild(innerDiv);

    // New instance button
    td = row.insertCell(cellIdx++);
    td.style.padding = "10px";
    td.vAlign = "top";
    var input = document.createElement("input");
    input.type = "button";
    input.className = "btnSmall";
    input.value = resolveString(cse_pp_st, "__create_new");
    var fnStr = this.objectName + ".makeNew();";
    input.onclick = new Function(fnStr);
    td.appendChild(input);


    // Create a div to hold the list table; this will be the
    // scrolling area if the widget is set to scroll
    this.innerTableDiv = document.createElement("div");
    innerDiv.appendChild(this.innerTableDiv);

    // Finally! create the list table itself
    var innerTable = document.createElement("table");
    innerTable.width = "100%";
    innerTable.id = this.uniquifier + "innerTbl";
    innerTable.cellSpacing = innerTable.cellPadding = innerTable.border = 0;
    this.innerTableDiv.appendChild(innerTable);
    var thead = document.createElement("thead");
    innerTable.appendChild(thead);
    this.listTBody = document.createElement("tbody");
    innerTable.appendChild(this.listTBody);

    // Add title row
    row = thead.insertRow(0);
    var keys = [{key:"__name", width:"50%"},
        {key:"__refcount", width:"20%"}, {key:"__actions", width:"30%"}];
    for (var i = 0; i < keys.length; i++) {
        td = document.createElement("th");
        td.appendChild(document.createTextNode(resolveString(cse_pp_st, keys[i].key)));
        td.style.width = keys[i].width;
        td.noWrap = true;
        td.id = this.objectName + "_list_hdr" + i;
        row.appendChild(td);
    }


    // Finally! add the data
    this.catChange();
}

//Set GenericWCount to 0
// Creates a new instance of a shared resource
SharedResourceManager.prototype.setGWCtoZero = function() {
    GenericWCount=0;
}

SharedResourceManager.prototype.setundefinedCounttoZero = function() {
    undefinedCount=0;
}

// Creates a new instance of a shared resource
SharedResourceManager.prototype.makeNew = function() {
    var cat = null;
    var selCtl = document.getElementById(this.uniquifier + "_catSel");
    if (selCtl)
        cat = selCtl.value;
    this.driver.newInstance(cat);
}

// Handles the user changing category
SharedResourceManager.prototype.catChange = function() {
	var cat = null;
    var selCtl = document.getElementById(this.uniquifier + "_catSel");
    if (null != selCtl) {
		
        cat = selCtl.value;

        if (top.pageCtx) {
            top.pageCtx[this.objectName + "catSel"] = selCtl.selectedIndex;
        }
		if(selCtl.selectedIndex==-1)
		{
			//Do Nothing
		}
		else
		{
			setCookieValue("CurrentCategory", selCtl.selectedIndex,true);
		}
    }
    this.drawInner(cat);
}

// Handles the user changing Language
SharedResourceManager.prototype.langChange = function() {
	var catList = [];
	var browserInfo = IdentifyBrowser();
	if (this.driver.getCategoryList)
        catList = this.driver.getCategoryList();
    var cat = null;
    var selLang = document.getElementById(this.uniquifier + "_langSel");
	setCookieValue("CurrentLang", selLang.value,true);
	var selCtl=document.getElementById(this.uniquifier + "_catSel");
	selCtl.length=GenericWCount;
    if (null != selLang && null!=selCtl) 
	{
        cat = selLang.value;
		
		for (var i = 0; i < catList.length; i++) 
		{
			if(catList[i].lang == McUndefined)
			{
				var option = document.createElement("option");
				option.value = catList[i].value;
				option.text = catList[i].text;
				
				if (browserInfo.isIE) 
				{
					selCtl.add(option);
				}
				else 
				{
					selCtl.add(option, null);
				}
			}
			else
			{
				if(catList[i].lang == cat)
					{
						var option = document.createElement("option");
						option.value = catList[i].value;
						option.text = catList[i].text;
					
						if (browserInfo.isIE) 
						{
							selCtl.add(option);
						}
						else 
						{
							selCtl.add(option, null);
						}
					}
				
				
			}
        }
		       
	}
	this.drawInner(cat);
	
}

// Handles the user changing category
SharedResourceManager.prototype.selLastCat = function() {
    var selCtl = document.getElementById(this.uniquifier + "_catSel");
	var currLang = getCookieValue("CurrentLang");
	var selLang = document.getElementById(this.uniquifier + "_langSel");
	selCtl.selectedIndex = GenericWCount+undefinedCount-1;
	var index=returnIndexLang(LangList[currLang]);
	if(index!=null)
	{
		selLang.selectedIndex=index;
	}
	this.catChange();
}

//returns index of the language selected
function returnIndexLang(currlang)
{
	var index=null;
	if(currlang=="Chinese - Simplified")
	{index=0;}
	else if(currlang=="English")
	{index=1;}
	else if(currlang=="French")
	{index=2;}
	else if(currlang=="German")
	{index=3;}
	else if(currlang=="Japanese")
	{index=4;}
	else if(currlang=="Spanish")
	{index=5;}
	return index;
}

// Displays the resource list
SharedResourceManager.prototype.drawInner = function(cat) {
    if (null == this.listTBody)
        return;
    // Clear the current display
    var tb = this.listTBody;
    while(tb.rows.length > 0) {
        tb.deleteRow(0);
    }
    // Get the driver to display its resource list
    this.driver.drawList(cat, tb, this);
    var tblId = this.uniquifier + "innerTbl";
    makeSDL(tblId, true);

    if (this.scrolling) {
        if (tb.rows.length > this.scrollAfter) {
            var height = this.rowHeight * (1 + this.scrollAfter);
            this.innerTableDiv.style.height = height + "px";
            this.innerTableDiv.style.overflow = "auto";
        }
        else {
            this.innerTableDiv.style.height = "auto";
        }
    }
}

// Changes text in one of the headers
SharedResourceManager.prototype.setHeaderText = function(hdrIdx, text) {
    var hdrCellId = this.objectName + "_list_hdr" + hdrIdx;
    cell = document.getElementById(hdrCellId);
    if (!cell)
        return;
    while (cell.firstChild)
        cell.removeChild(cell.firstChild);
    cell.appendChild(document.createTextNode(text));
}

// Convenience function for creating links
SharedResourceManager.createLink = function(href, text) {
    var link = document.createElement("a");
    link.href = href;
    link.appendChild(document.createTextNode(text));
    return link;
}
