/**
 * JavaScript for processing spam rules
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

/*includeJS("../../js/PropertyPath.js");
includeJS("../../js/PropertyBag.js");*/

// Processed rules are cached in the frame container
if (undefined === top.processedSpamRules) {
    top.processedSpamRules = {std:[], net:[]};
}
var g_usingDoc = (0 == top.processedSpamRules.std.length &&
    0 == top.processedSpamRules.net.length);

// Change notification
var g_changeNotifyFn = new Function("top.gPageStack.Active().SetModified(true);");


var NBSP = 160;
var ELEMENT_NODE = 1;

// Mouse tracking functions

var mouseX = 0;
var mouseY = 0;

function mouseTrack(e) {
    //e = (e) ? e : ((event) ? event : null);
    if (e) {
        // Non-IE - e is non-null
        mouseX = e.pageX;
        mouseY = e.pageY;
    } else {
        // IE - uses global event object
        if (event && document.body) {
            mouseX = event.x;
            mouseY = event.y;
            // Handle IE quirks
            if ("BackCompat" == document.compatMode) {
                mouseX += document.body.scrollLeft;
                mouseY += document.body.scrollTop;
            }
            else if ("CSS1Compat" == document.compatMode) {
                mouseX += document.body.parentNode.scrollLeft;
                mouseY += document.body.parentNode.scrollTop;
            }
            else {
                mouseX += window.scrollX;
                mouseY += window.scrollY;
            }
        }
    }
}

document.onmousemove = mouseTrack;

/**
 * Constructor for SpamRule class
 */
function SpamRule() {
}

/**
 * Turns a series of spam-rule-related property paths into a keyed object
 * Used for old-format spam rules where each attribute is specified as
 * a child <Attr> node
 * This function may disappear in a future software version, but for now
 * old and new rule formats are supported.
 */
function processPreDefRuleA(node, o) {
    var defMappings = {Name:"", tflags:"",
        code:"0", ScoreNetOn:"0", ScoreNetOff:"0"};

	var scoreAttr;
    var attrs = node.childNodes;
    for (var i = 0; i < attrs.length; i++) {
        var attrNode = attrs[i];
        if (ELEMENT_NODE != attrNode.nodeType)
            continue;
        var attrName = attrNode.nodeName;
        if (!attrName)
            continue;
        if (attrName == "scores") { // we will handle it later.
			scoreAttr = attrNode;
			continue;
		}

        var attrVal = attrNode.text;
        // Firefox DOM parser have 'textContent' field rather than 'text'
        if (!attrVal)
            attrVal = attrNode.textContent;

        if (!attrVal)
            attrVal = defMappings[attrName];
        o[attrName] = attrVal;
    }

	var scoreChildes = scoreAttr.childNodes;
	for (var i = 0; i < scoreChildes.length; i++) {
	    if (ELEMENT_NODE != scoreChildes[i].nodeType)
			continue;

		if ((attrVal = scoreChildes[i].getAttribute("on")) && attrVal == 1) {
			attrVal = scoreChildes[i].getAttribute("value");
			if (!attrVal)
				o["ScoreNetOn"] = defMappings["ScoreNetOn"];
			o["ScoreNetOn"] = attrVal;
		}
		else {
			attrVal = scoreChildes[i].getAttribute("value");
			if (!attrVal)
				o["ScoreNetOff"] = defMappings["ScoreNetOff"];
			o["ScoreNetOff"] = attrVal;
		}
	}
}

/**
 * Turns a series of spam-rule-related property paths into a keyed object
 * Used for v4-format spam rules where each attribute is specified as
 * an XML attribute
 */
function processPreDefRuleB(node, o) {
    var defMappings = {Type:"", Name:"", Description:"",
        ScoreNetOff:"0", ScoreNetOn:"0", NetFlag:"0"};

    for (var prop in defMappings) {
        var x = node.getAttribute(prop);
        o[prop] = (null == x) ? defMappings[prop] : x;
    }
}

/**
 * Initializes a spam rule object from the predefined list
 * @param pb the property bag object to use
 * @param pp the base property path for the rule
 * @param o a keyed Object with properties for this rule
 */
SpamRule.prototype.fromPreDef = function(o) {
    this.Custom = false;
    this.NetFlag = false;

    for (var attrName in o) {
        this.setAttribute(attrName, o[attrName]);
    }

    this.Overridden = false;
    // Overridable fields
    this.Enabled = true;

    this.Score = this.NetFlag ? this.ScoreNetOn : this.ScoreNetOff;
}

// Sets an attribute from a property bag
SpamRule.prototype.setAttribute = function(attrName, attrVal) {
    var valToStore = attrVal
    switch (attrName) {
        case "Description":
            // Some rule descriptions have surrounding quotes which look strange
            // in the table; remove them
            valToStore = unquote(attrVal);
            break;

        case "ScoreNetOff":
        case "ScoreNetOn":
            valToStore = parseFloat(attrVal);
            break;

        case "NetFlag":
            valToStore = ("1" == attrVal);
            break;

        default:
            break;
    }

    this[attrName] = valToStore;
}

/**
 * Initializes a spam rule object from main config; used
 * when the rule is not to be found in the predefined list
 * @param name - the rule name
 * @param enabled - whether or not the rule is enabled
 * @param score - the score to assign
 */
SpamRule.prototype.customInit = function(name, enabled, score) {
    this.Custom = true;
    this.Name = name;
    this.Enabled = enabled;
    this.Score = parseFloat(score);

    this.Type = this.Description = "";
    this.ScoreNetOff = this.ScoreNetOn = 0.0;
    this.NetFlag = false;
    this.Overridden = false;
}

/**
 * Overrides fields in the rule from the ASSettings section of the
 * main policy config file
 * @param enabled - true or false to indicate the enabled state of the
 * rule
 * @param score the overriden spam score
 */
SpamRule.prototype.setOverrides = function(enabled, score) {
    this.Overridden = true;
    this.Enabled = enabled;
    this.Score = score;
}

/**
 * Writes the spam rule out to a property bag; only appropriate when the
 * Overridden / Custom field is set to true
 * @param pb the property bag object to use
 * @param pp the base property path for the rule
 */
SpamRule.prototype.toPB = function(pb, pp) {
    var s = pp.GetPath();
    pb.SetProperty(s + ".RuleName", this.Name);
    pb.SetProperty(s + ".Enabled", this.Enabled ? "1" : "0");
    pb.SetProperty(s + ".Score", this.Score.toString());
}

/**
 * Determines whether or not a rule has been changed by user action
 * @param oRule the original rule values
 */
SpamRule.prototype.hasChanged = function(oRule) {
    var changed = false;
    if (this.Name != oRule.Name || this.Enabled != oRule.Enabled ||
        this.Score != oRule.Score) {
        changed = true;
    }
    return changed;
}

/**
 * Gets a canonical score value for sorting purposes
 */
SpamRule.prototype.getCanonicalScore = function(nTestsEnabled) {
    var ret = 0.0;
    if (this.Overridden || this.Custom) {
        ret = this.Score;
    }
    else if (nTestsEnabled || this.NetFlag) {
        ret = this.ScoreNetOn;
    }
    else {
        ret = this.ScoreNetOff;
    }
    return ret;
}

/**
 * Gets a type code for grouping purposes; values are:
 *  - C: custom rule without corresponding rule in the predefeined list
 *  - O: overridden rule: rule exists in predefined list but has been customized
 *  - U: unmodified: rule has not been changed from predefined settings
 */
SpamRule.prototype.getFlag = function() {
    var ret = "U";
    if (this.Overridden) {
        ret = "O";
    }
    else if (this.Custom) {
        ret = "C";
    }
    return ret;
}

// Sorting functions for spam rules
function SpamRuleByName(a, b) {
    return a.Name.localeCompare(b.Name);
}

function SpamRuleByName_r(a, b) {
    return b.Name.localeCompare(a.Name);
}

function SpamRuleByScore(a, b) {
    var x = a.getCanonicalScore(this.networkTestsEnabled);
    var y = b.getCanonicalScore(this.networkTestsEnabled);
    return (x - y);
}

function SpamRuleByScore_r(a, b) {
    var x = a.getCanonicalScore(this.networkTestsEnabled);
    var y = b.getCanonicalScore(this.networkTestsEnabled);
    return (y - x);
}

function SpamRuleByFlag(a, b) {
    var x = a.getFlag();
    var y = b.getFlag();
    return (x == y) ? SpamRuleByName(a, b) : ((x < y) ? -1 : 1);
}

function SpamRuleByFlag_r(a, b) {
    return SpamRuleByFlag(b, a);
}

function SpamRuleByEnabled(a, b) {
    var x = a.Enabled;
    var y = b.Enabled;
    // Disabled comes before enabled
    if (a && !b)
        return -1;
    if (b && !a)
        return 1;
    return (x == y) ? SpamRuleByName(a, b) : ((x < y) ? -1 : 1);
}

function SpamRuleByEnabled_r(a, b) {
    return SpamRuleByEnabled(b, a);
}

// Map for alt fields
var altTexts = new Object();
altTexts["C"] = resolveString(cse_pp_st, "__unknown_rule");
altTexts["O"] = resolveString(cse_pp_st, "__overridden_rule");
altTexts["U"] = resolveString(cse_pp_st, "__unmodified_rule");
altTexts["start"] = resolveString(cse_pp_st, "__start");
altTexts["prev"] = resolveString(cse_pp_st, "__prev");
altTexts["next"] = resolveString(cse_pp_st, "__next");
altTexts["end"] = resolveString(cse_pp_st, "__end");

// Map for getting sorting functions
var sortMap = new Object();
sortMap["A"] = new Object();
sortMap["A"]["Name"] = SpamRuleByName;
sortMap["A"]["Score"] = SpamRuleByScore;
sortMap["A"]["En"] = SpamRuleByEnabled;
sortMap["A"]["Flag"] = SpamRuleByFlag;
sortMap["D"] = new Object();
sortMap["D"]["Name"] = SpamRuleByName_r;
sortMap["D"]["Score"] = SpamRuleByScore_r;
sortMap["D"]["En"] = SpamRuleByEnabled_r;
sortMap["D"]["Flag"] = SpamRuleByFlag_r;

var minRecordsPerPage = 10;
var defRecordsPerPage = 25;

function GetSpamRulesPref(key, def) {
    var ret = def;
    if (top.SRPreferences != null && top.SRPreferences[key] != null) {
        ret = top.SRPreferences[key];
    }
    return ret;
}

function SetSpamRulesPref(key, val) {
    if (null == top.SRPreferences) {
        top.SRPreferences = new Object();
    }
    top.SRPreferences[key] = val;
}

// IDs for the rule panel divs
var ssrDivId = "std_rules";
var nsrDivId = "net_rules";

// IDs for the rule table divs
var ssrListId = "std_rules_list";
var nsrListId = "net_rules_list";

// IDs for the rule tables
var ssrTableId = "std_rules_tbl";
var nsrTableId = "net_rules_tbl";


// Lists to hold predefined rules, overrides, and rules that do not appear in
// the predefined lists
var stdRules = [];
var stdOverrides = [];

var netRules = [];
var netOverrides = [];

// Flag to indicate whether standard display is in operation (status flag indicators
// and inline rule descriptions
var g_stdDisplay = true;
// Use popups in EPO
if (this.gEPOMode)
    g_stdDisplay = false;
var g_showFlyouts = !g_stdDisplay;

// Locates the engine node in the spam rules XML
function _sr_getEngineNode(docNode) {
    var children = docNode.childNodes;
    for (var i = children.length - 1; i >= 0; i--) {
		var node = children[i];
        if ("Engine" == children[i].nodeName) {
			return node;
		}
    }
    return null;
}

/**
 * Counts the number of rules in the document tree; this
 * function does not return an accurate count of rules nodes
 * but rather the number of child nodes under EngineSet/Engine;
 * this is for performance reasons and does not affect the logic
 * since n child nodes are processed at a time rather than n rules.
 */
function countPreDefRules_xml(t_locale, xmlDoc) {
    if (null == xmlDoc) {
        return 0;
    }

    // The rules are under EngineSet/Engine
    var docNode = xmlDoc.documentElement;
    var engineNode = _sr_getEngineNode(docNode);
    var rCount = 0;
    if (engineNode)
        rCount = engineNode.childNodes.length;

    return rCount;
}

/**
 * Counts the number of predefined rules
 */
function countPreDefRules(t_locale, xmlDoc) {
    if (g_usingDoc)
        return countPreDefRules_xml(t_locale, xmlDoc);

    var rCount = top.processedSpamRules.std.length +
        top.processedSpamRules.net.length;
    return rCount;
}

/**
 * Processes the predefined spam rules, using the spam rules PropertyBag
 * @param t_locale a locale string (such as "en_US")
 */
function ProcessPreDefRules_xml(t_locale, start, end, total, xmlDoc) {
    if (0 == total) {
        ShowAddForms();
        return;
    }

    var std = [], net = [];

    // Path ro rules nodes is EngineSet/Engine/
    var docNode = xmlDoc.documentElement;
    var rulesNodes = _sr_getEngineNode(docNode).childNodes;
    var idx = start;
    for (; idx < end; idx++) {
        if (idx >= rulesNodes.length) {
            break;
        }
        var node = rulesNodes[idx];
        if (ELEMENT_NODE != node.nodeType)
            continue;
        // v3.4 format: <SubProperties name="Rule">...</SubProperties>; v4 format: <Rule... />
        var nodeName = node.nodeName;
        var o = {};
        // according to new camelus format
        if ("rule" == nodeName) {
            // Should have an attribute called "type"
            var nTypeValue = node.getAttribute("type");
            if (!nTypeValue) o["type"] = "";
          	o["type"] = nTypeValue;
            processPreDefRuleA(node, o);
        }
        else if ("Rule" == nodeName) {
            processPreDefRuleB(node, o);
        }
        else {
            continue;
        }

        // Turn the object into a SpamRule object
        var rule = new SpamRule();
        rule.fromPreDef(o);
        // Add it to the appropriate rule list
        if (rule.NetFlag) {
            netRules.push(rule);
            netRules[rule.Name] = netRules.length - 1;
            net.push(o)
        }
        else {
			if (!stdRules[rule.Name]) {
				stdRules.push(rule);
				stdRules[rule.Name] = stdRules.length - 1;
				std.push(o);
			}
        }
    }

    // Copy the processed rules into the processed rules cache
    var nRules = std.length;
    for (var i = 0; i < nRules; i++) {
        top.processedSpamRules.std.push(std.shift());
    }
    nRules = net.length;
    for (var i = 0; i < nRules; i++) {
        top.processedSpamRules.net.push(net.shift());
    }
    if (idx >= total) {
        // We've processed all the rules - no need to use the source XML now
        g_usingDoc = false;
    }
}

/**
 * Processes the predefined spam rules; attempts to use cached processed rules,
 * otherwise uses the spam rules PropertyBag
 * @param t_locale a locale string (such as "en_US")
 * @param start the start rule to process
 * @param end the end rule to process
 * @param total the total number of rules to process
 * @param xmlDoc the source XML for the spam rules
 */
function ProcessPreDefRules(t_locale, start, end, total, xmlDoc) {
    if (g_usingDoc) {
        ProcessPreDefRules_xml(t_locale, start, end, total, xmlDoc);
        return;
    }
    var nRulesS = top.processedSpamRules.std.length;
    var nRulesN = top.processedSpamRules.net.length;
    if (0 == start) {
        stdRules = new Array(nRulesS);
        netRules = new Array(nRulesN);
    }
    var rulesToProcess = (end - start);
    var rulesProcessed = 0;
    for (var i = start; i < end; i++, rulesProcessed++) {
        if (i >= nRulesS) {
            break;
        }
        var rule = new SpamRule();
        rule.fromPreDef(top.processedSpamRules.std[i]);
        stdRules[i] = rule;
        stdRules[rule.Name] = i;
    }
    start += rulesProcessed;
    for (var i = start; i < end; i++) {
        if (i >= total) {
            break;
        }
        var rule = new SpamRule();
        var idx = i - nRulesS;
        rule.fromPreDef(top.processedSpamRules.net[idx]);
        netRules[idx] = rule;
        netRules[rule.Name] = idx;
    }
}

function ShowAddForms() {
    var types = {"std":null, "net":null};
    for (var type in types) {
        var form = document.forms[type + "_add_rule_form"];
        if (form) {
            form.style.display = "block";
            var eName = type + "_new_name";
            var e = form.elements[eName];
            if (e)
                e.onkeypress = rnCheckInput;
            eName = type + "_new_score";
            e = form.elements[eName];
            if (e)
                e.onkeypress = cb_decimalOnly;
        }
        if (!g_stdDisplay) {
            // Suppress the warning messages about rules not being found
            var eW = document.getElementById(type + "_no_predef");
            if (eW)
                eW.style.display = "none";
        }
    }
}

/**
 * Processes the Rules and NTRules sections of the ASSettings blob
 * @param pb the property bag to use
 * @param pp the base property path for this settings instance
 */
function ProcessCustomRules(pb, pp) {
    ProcessCustomRules2(pb, pp, "Rules");
    ProcessCustomRules2(pb, pp, "NTRules");
}

function ProcessCustomRules2(pb, pp, section) {
    var rules = stdRules;
    var overrides = stdOverrides;
    if ("NTRules" == section) {
        rules = netRules;
        overrides = netOverrides;
    }
    var basePP = new PropertyPath(pp + "." + section + ".Rule");
    for (basePP.BeginEnum(); undefined !== pb.GetProperty(basePP);
        basePP.IncrementEnum()) {
        s = basePP.GetPath();
        var name = pb.GetPropertyDef(s + ".RuleName", "");
        var en = ("1" == pb.GetPropertyDef(s + ".Enabled", "1"));
        var score = parseFloat(pb.GetPropertyDef(s + ".Score", "0"));
        // Check whether this entry represents an override or a rule
        // not present in the predef list
        if (rules[name] != undefined) {
            rule = rules[rules[name]];
            rule.setOverrides(en, score);
        }
        else {
            var rule = new SpamRule();
            rule.customInit(name, en, score);
            rules.push(rule);
            rules[name] = rules.length - 1;
        }
        // Keep a record of the state of this rule when first loading
        var oRule = new SpamRule();
        oRule.customInit(name, en, score);
        overrides.push(oRule);
    }
    // Sort the overrides list; it will not change again
    overrides.sort(SpamRuleByName);
}

/**
 * Saves custom rules out to a property bag
 * @param pb the property bag object to use
 * @param pp the base property path under which to save the rules
 * @param processNetRules true if the network rules should be processed,
 * false otherwise
 */
function SaveCustomRules(pb, pp, processNetRules) {
    var changed = SaveCustomRules2(pb, pp, "Rules");
    if (processNetRules) {
        changed |= SaveCustomRules2(pb, pp, "NTRules");
    }
    return changed;
}

function SaveCustomRules2(pb, pp, section) {
    var changed = false;
    var rules = stdRules;
    var overrides_o = stdOverrides;
    var overrides = [];
    if ("NTRules" == section) {
        rules = netRules;
        overrides_o = netOverrides;
    }

    // Get the custom / override rules from the current list
    for (var i = rules.length - 1; i >= 0; i--) {
        var rule = rules[i];
        if (rule.Custom || rule.Overridden) {
            overrides.push(rule);
        }
    }

    if (overrides.length != overrides_o.length) {
        changed = true;
    }
    else {
        // Sort the new overrides list
        overrides.sort(SpamRuleByName);
        for (var i = overrides.length - 1; i >= 0; i--) {
            if (overrides_o[i].hasChanged(overrides[i])) {
                changed = true;
                break;
            }
        }
    }

    if (changed) {
        // Remove the current rule list from the PropertyBag
        var basePP = pp + "." + section + ".Rule";
        pb.DeleteList(basePP);
        // Recreate the list
        var ppT = new PropertyPath(basePP);
        ppT.BeginEnum();
        for (var i = 0; i < overrides.length; i++, ppT.IncrementEnum()) {
            pb.SetProperty(ppT);
            overrides[i].toPB(pb, ppT);
        }
    }

    return changed;
}

/**
 * Sorts the rules lists
 * @param sdFlag - the sort direction, "A(scending)" or "D(escending)"
 * @param sortField  - the field to sort on: "Flag", "Name", "Score"
 * @param networkRules - true if the network rules are to be sorted
 */
function SortRuleList(sdFlag, sortField, networkRules) {
    var sortFn = sortMap[sdFlag][sortField];
    if (null == sortFn) {
        return false;
    }
    if (networkRules) {
        SortRules(netRules, sortFn);
    }
    else {
        SortRules(stdRules, sortFn);
    }
    return true;
}

function SortRules(rules, sortFn) {
    rules.sort(sortFn);
    // Regenerate the map
    var nRules = rules.length;
    for (var i = 0; i < nRules; i++) {
        rules[rules[i].Name] = i;
    }
}

/**
 * Creates the initial spam rules table
 * @param divId - the div in which to show the table
 * @param tblId - the ID of the table
 * @param rowCount - the number of rows to display at once
 * @param networkRules - true if this is the network rules table
 * @param nTestsEnabled - true if network tests are enabled
 */
function InitRulesTable(divId, tblId, rowCount, networkRules, nTestsEnabled) {
    var colCount = 5;
    var html = makeNavDiv(tblId, "top");;
    html += "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" " +
        "id=\"" + tblId + "\" style=\"width:100%\">";
    // Create the table header
    html += "<tr>";
    if (g_stdDisplay)
        html += "<th style=\"text-align:center;\" nowrap>&nbsp;</th>";
    html += "<th nowrap>&nbsp;</th>";
/*    if (g_stdDisplay)
        html += "<th>&nbsp;</th>";*/
    html += "<th nowrap style=\"text-align:center;\">&nbsp;</th>";
    html += "<th nowrap>&nbsp;</th>";
    html += "<th>&nbsp;</th>";
    html += "</tr>";
    for (var i = 0; i < rowCount; i++) {
        html += "<tr>";
        if (g_stdDisplay)
            html += "<td align=\"center\">&nbsp;</td>"; // Override flag
        html += "<td nowrap></td>"; // Rule name
/*      if (g_stdDisplay)
            html += "<td></td>";*/ // Description
        html += "<td align=\"center\"></td>"; // Enabled indicator
        html += "<td nowrap></td>"; // Score
        html += "<td nowrap></td>"; // Actions
        html += "</tr>";
    }
    html += "</table>";
    html += makeNavDiv(tblId, "bottom");
    var div = document.getElementById(divId);
    if (div) {
        div.innerHTML = html;
        var sdPref = networkRules ? "sortDir_n" : "sortDir_s";
        var sfPref = networkRules ? "sortField_n" : "sortField_s";
        var sdFlag = GetSpamRulesPref(sdPref, "A");
        var sField = GetSpamRulesPref(sfPref, "Name");
        fillTableHeader(tblId, networkRules, nTestsEnabled, sdFlag, sField);
        makeSDL(tblId, true);
    }
}

document.onmousemove=mouseTrack;

var adImagesIE = {A:'<img src="../../images/sort_ascending.gif" width="7" height="4" border="0" ' +
    'alt="A" title="' + resolveString(cse_pp_st, "__asc") + '">',
    D:'<img src="../../images/sort_descending.gif" width="7" height="4" border="0"' +
    'alt="A" title="' + resolveString(cse_pp_st, "__desc") + '">'};

var fnStrAscShow = "TooltipControl(true,'"  + resolveString(cse_pp_st, "__asc") + "');";
var fnStrAscVanish = "TooltipControl(false,'" + resolveString(cse_pp_st, "__asc") + "');";

var fnStrDescShow = "TooltipControl(true,'"  + resolveString(cse_pp_st, "__desc") + "');";
var fnStrDescValish = "TooltipControl(false,'" + resolveString(cse_pp_st, "__desc") + "');";

var adImagesMOZ = {A:'<img src="../../images/sort_ascending.gif" width="7" height="4" border="0" ' +
						' onmouseover = \"' + fnStrAscShow + '\" onmouseout = \"' + fnStrAscVanish  + '\"  />',
		    D:'<img src="../../images/sort_descending.gif" width="7" height="4" border="0"' +
						' onmouseover = \"' + fnStrDescShow + '\" onmouseout = \"' + fnStrDescValish  + '\"  />'};

var hiddenSortImg = '<img src="../../images/sort_ascending.gif" ' +
   'style="visibility:hidden" width="7" height="4" border="0">';
var tableHeaders = [];
if (g_stdDisplay)
    tableHeaders.push({id:"Flag", sort:true, val:resolveString(cse_pp_st, "__flag")});
tableHeaders.push({id:"Name", sort:true, val:resolveString(cse_pp_st, "__rule_name")});
/*if (g_stdDisplay)
    tableHeaders.push({id:"Desc", sort:false, val:resolveString(cse_pp_st, "__description")});*/
tableHeaders.push({id:"En", sort:true, val:resolveString(cse_pp_st, "__enabled")});
tableHeaders.push({id:"Score", sort:true, val:resolveString(cse_pp_st, "__score")});
tableHeaders.push({id:"Actions", sort:false, val:resolveString(cse_pp_st, "__actions")});

/**
 * Fills in the spam rules table header
 */
function fillTableHeader(tblId, networkRules, nTestsEnabled, sdFlag, sField) {
    var table = document.getElementById(tblId);
    if (null == table)
        return;
    var eHeaders = table.rows[0].cells;
    for (var i = 0; i < tableHeaders.length; i++) {
        var headerInfo = tableHeaders[i];
        if (headerInfo.sort) {
            var isCurrSort = (sField == headerInfo.id);
            var newSDFlag = "A";
            if (isCurrSort && "A" == sdFlag) {
                // Offer descending order option
                newSDFlag = "D";
            }
            var href="javascript:ChangeSortOrder('" + tblId + "', " +
                networkRules + ", " + nTestsEnabled + ", '" + newSDFlag +
                "', '" + headerInfo.id + "');";
            // The header is a JavaScript link
            var html = "<a href=\"" + href + "\">" + headerInfo.val + "</a>";
            if (isCurrSort) {
                // Add a sort indicator to the link
				var val = getCookieValue("McDynamic")
				if ( val == "NoIISMoz")
				{
					html += " " + adImagesMOZ[sdFlag];
				}
				else
				{
					html += " " + adImagesIE[sdFlag];
				}
            }
            else {
                // Use a hidden sort indicator to retain width
                html += " " + hiddenSortImg;
            }
            eHeaders[i].innerHTML = html;
        }
        else {
            eHeaders[i].innerHTML = headerInfo.val;
        }
    }
}

function makeNavDiv(tblId, label) {
    return "<div id=\"" + tblId + "_" + label +
        "\" class=\"rightAligned\" style=\"padding:5px 0px;\"></div>";
}

/**
 * Changes the sort field or order of the rules table
 * @param tblId the table affected
 * @param networkRules true if this is the network rules table
 * @param nTestsEnabled true if network tests are enabled
 * @param sdFlag the sort direction, "A" or "D"
 * @param sField the field to sort on: "Flag", "Name", "Score"
 */
function ChangeSortOrder(tblId, networkRules, nTestsEnabled, sdFlag, sField) {
    if (SortRuleList(sdFlag, sField, networkRules)) {
        fillTableHeader(tblId, networkRules, nTestsEnabled, sdFlag, sField);
        var idxPref = networkRules ? "netIdx" : "stdIdx";
        var currIdx = GetSpamRulesPref(idxPref, 0);
        var perPage = GetSpamRulesPref("perPage", 25);
        ShowSpamRules(tblId, networkRules, currIdx, perPage, nTestsEnabled);
        // Store preferences
        var sdPref = networkRules ? "sortDir_n" : "sortDir_s";
        var sfPref = networkRules ? "sortField_n" : "sortField_s";
        SetSpamRulesPref(sdPref, sdFlag);
        SetSpamRulesPref(sfPref, sField);
    }
}

/**
 * Displays spam rules
 * @param tblId - the ID of the table
 * @param nRules - true if rendering the network rules table
 * @param startIdx - the start index in the rules array
 * @param displayCount - the number of rules to display
 * @param nTestsEnabled - true if network tests are enabled
 */
function ShowSpamRules(tblId, nRules, startIdx, displayCount, nTestsEnabled) {
    var rules = stdRules;
    var idxPref = "stdIdx";
    if (nRules) {
        rules = netRules;
        idxPref = "netIdx";
    }
    if (0 == rules.length) {
        // No rules to display
        var divId = nRules ? nsrListId : ssrListId;
        var content = "<p class=\"bold\">" + resolveString(cse_pp_st, "__no_rules") +
            "</p>";
        document.getElementById(divId).innerHTML = content;
        return;
    }
    var remaining = rules.length - startIdx;
    var toDisplay = (remaining > displayCount) ? displayCount : remaining;
    var table = document.getElementById(tblId);
    if (table) {
        var rows = table.rows;
        for (var i = 0; i < toDisplay; i++) {
            var ruleIdx = startIdx + i;
            var rule = rules[ruleIdx];

            var rowIdx = i + 1; // Allow for the table header
            var row = rows[rowIdx];
            // Remove any bottom bordering that may be present due
            // to displaying a final page
            row.className = removeClass(row.className, "lastRow");
            // Ensure that the row is visible
            row.style.display = "";
            ShowRule(tblId, row, nRules, rule, ruleIdx, rowIdx, nTestsEnabled);
        }
        if (toDisplay < displayCount) {
            for (var i = toDisplay; i < displayCount; i++) {
                // Hide all unused rows
                if ((i + 1) < rows.length)
                    rows[i + 1].style.display = "none";
            }
        }
        // Ensure the bottom row displayed has a bottom border
        rows[toDisplay].className = addClass(rows[toDisplay].className, "lastRow");

        var atStart = (0 == startIdx);
        var atEnd = (remaining <= displayCount);
        var navLinks = "";
        if (!atStart) {
            navLinks += makeNavLink(tblId, nRules, 0,
                displayCount, nTestsEnabled, "start");
            navLinks += makeNavLink(tblId, nRules, startIdx - displayCount,
                displayCount, nTestsEnabled, "prev");

        }
        else {
            navLinks += makeNavLinkDisabled("start") +
                makeNavLinkDisabled("prev");
        }
        if (!atEnd) {
            navLinks += makeNavLink(tblId, nRules, startIdx + displayCount,
                displayCount, nTestsEnabled, "next");
            // Calculate the index of the final table
            var endStartIdx = Math.floor(rules.length / displayCount) * displayCount;
            navLinks += makeNavLink(tblId, nRules, endStartIdx,
                displayCount, nTestsEnabled, "end");
        }
        else {
            navLinks += makeNavLinkDisabled("next") +
                makeNavLinkDisabled("end");
        }
        var pag = resolveString(cse_pp_st, "__spam_showing", [startIdx + 1,
            (startIdx + toDisplay).toString(), rules.length]);
        var sel = makePerPageSel(rules.length, displayCount, nTestsEnabled);
        if (sel)
            sel = " &nbsp; " + sel;
        var nav = " &nbsp; " + navLinks;
        document.getElementById(tblId + "_top").innerHTML = pag + sel + nav;
        document.getElementById(tblId + "_bottom").innerHTML = pag + sel + nav;
    }
    SetSpamRulesPref(idxPref, startIdx);
}

function ShowRule(tblId, row, nRules, rule, ruleIdx, rowIdx, nTestsEnabled) {
    var cells = row.cells;
    var cellIdx = 0;

    if (g_stdDisplay) {
        var flag = rule.getFlag();
        var alt = altTexts[flag];

        var imgSrc = "../../images/icons/";
        if (rule.Overridden) {
            imgSrc += "soso.gif";
        }
        else if (rule.Custom) {
            imgSrc += "notok.gif";
        }
        else {
            imgSrc += "ok.gif";
        }
        var img = document.createElement("img");
        img.height = img.width = 14;
        img.align = "middle";
		img.alt = flag;

		var val = getCookieValue("McDynamic")
		if ( val == "NoIISMoz")
		{
			var fnStrShow = "TooltipControl(true,'" + EscapeForJS(alt) + "');";
			var fnStrVanish = "TooltipControl(false,'" + EscapeForJS(alt) + "');";
			img.onmouseover = new Function(fnStrShow);
			img.onmouseout =  new Function(fnStrVanish);
		}
		else
		{
			img.title = alt;
		}
        img.src = imgSrc;

        clearElement(cells[cellIdx]);
        cells[cellIdx++].appendChild(img);
    }
    makeRuleNameHTML(rule, nRules, ruleIdx, cells[cellIdx++]);

	// remove  because of camelus
/*    if (g_stdDisplay) {
        clearElement(cells[cellIdx]);
        var content = rule.Description ? document.createTextNode(rule.Description) :
            document.createTextNode(String.fromCharCode(NBSP));
        cells[cellIdx++].appendChild(content);
    }*/

    imgSrc = "../../images/icons/";
    imgSrc += (rule.Enabled) ? "checkbox.gif" : "checkbox_unchecked.gif";
    img = document.createElement("img");
    img.height = img.width = 13;
    img.align = "middle";
    img.src = imgSrc;
    clearElement(cells[cellIdx]);
    cells[cellIdx++].appendChild(img);

    clearElement(cells[cellIdx]);
    cells[cellIdx++].appendChild(document.createTextNode(formatSpamScore(rule, nTestsEnabled)));

    clearElement(cells[cellIdx]);
    var editLink = makeEditLink(ruleIdx, nRules, tblId, rowIdx, nTestsEnabled, "EditRule", "__edit");
    if (rule.Overridden) {
        var resetLink = makeEditLink(ruleIdx, nRules, tblId, rowIdx, nTestsEnabled,
            "ResetRule", "__reset");
        cells[cellIdx].appendChild(editLink);
        cells[cellIdx].appendChild(document.createTextNode(" | "));
        cells[cellIdx].appendChild(resetLink);
    }
    else if (rule.Custom) {
        var deleteLink = makeEditLink(ruleIdx, nRules, tblId, rowIdx, nTestsEnabled,
            "DeleteRule", "__delete");
        cells[cellIdx].appendChild(editLink);
        cells[cellIdx].appendChild(document.createTextNode(" | "));
        cells[cellIdx].appendChild(deleteLink);
    }
    else
        cells[cellIdx].appendChild(editLink);
}

function clearElement(e) {
    while(e.firstChild)
        e.removeChild(e.firstChild);
}

function makeRuleNameHTML(rule, networkRules, ruleIdx, cell) {
    clearElement(cell);
    if (g_showFlyouts) {
        var img = document.createElement("img");
        img.height = img.width = 14;
        img.align = "middle";
        img.alt = "pop";
        img.src = "../../images/icons/info_popup.gif";
        var fnStr = "dhtmlSummaryPop('ruleDescPop',false);";
        img.onmouseout = new Function(fnStr);
        fnStr = "showRuleDescription(" + networkRules + "," + ruleIdx + ");";
        img.onmouseover = new Function(fnStr);
        cell.appendChild(img);
        cell.appendChild(document.createTextNode(" "));
    }
    cell.appendChild(document.createTextNode(rule.Name));
}

// Pops up a description of a spam rule if available
function showRuleDescription(networkRules, ruleIdx) {
    var rules = stdRules;
    if (networkRules) {
        rules = netRules;
    }
    if (ruleIdx < 0 || ruleIdx >= rules.length) {
        return;
    }
    var rule = rules[ruleIdx];
    var divId = "ruleDescPop";
    var e = document.getElementById(divId);
    if (!e)
        return;
    var content = rule.Description;
    if (!content) {
        content = resolveString(cse_pp_st, "__no_description");
    }
    clearElement(e);
    e.appendChild(document.createTextNode(content));
    // Show the description
    dhtmlSummaryPop(divId, true);
}

function makeNavLink(tblId, nRules, start, count, nTestsEnabled, gif) {
    var alt = altTexts[gif];
	var val = getCookieValue("McDynamic")
	if ( val == "NoIISMoz")
	{
		var fnStrShow = "TooltipControl(true,'" + EscapeForJS(alt) + "');";
		var fnStrVanish = "TooltipControl(false,'" + EscapeForJS(alt) + "');";

		var link = "<a " + " onmouseover = \"" + fnStrShow + "\" onmouseout = \"" + fnStrVanish  + "\" href=\"javascript:ShowSpamRules('" +
			tblId + "', " + nRules + ", " + start + ", " + count + ", " +
			nTestsEnabled + ");\"><img " + " onmouseover = \"" + fnStrShow + "\" onmouseout = \"" + fnStrVanish + "\" height=\"9\" width=\"13\" " +
			"border=\"0\" src=\"../../images/pag_" + gif + ".gif\"></a>";
		return link;
	}
	else
	{
		var link = "<a title=\"" + alt + "\" href=\"javascript:ShowSpamRules('" +
			tblId + "', " + nRules + ", " + start + ", " + count + ", " +
			nTestsEnabled + ");\"><img alt=\"" + alt + "\" height=\"9\" width=\"13\" " +
			"border=\"0\" src=\"../../images/pag_" + gif + ".gif\"></a>";
		return link;
	}
}

function makeNavLinkDisabled(gif) {
    return "<img alt=\"\" height=\"9\" width=\"13\" " +
        "border=\"0\" src=\"../../images/pag_" + gif + "_disabled.gif\">";
}

function formatSpamScore(rule, nTestsEnabled) {
    return rule.getCanonicalScore(nTestsEnabled).toLocaleString();
}

function makePerPageSel(total, perPage, nTestsEnabled) {
    var ret = "<select "
    ret += "onchange=\"ChangePerPagePref(parseInt(this.value), " +
        nTestsEnabled + ");\">";
    var opts = [minRecordsPerPage, defRecordsPerPage, 50, 100];
    for (var i = 0; i < opts.length; i++) {
        ret += makePerPageSelOption(opts[i], perPage);
    }
    ret += "</select>" + resolveString(cse_pp_st, "__per_page");

    return ret;
}

function makePerPageSelOption(val, perPage) {
    var ret = "<option value=\"" + val + "\"";
    if (perPage == val) {
        ret += " selected";
    }
    ret += ">" + val;
    return ret;
}

/**
 * Responds to the user changing the select value for the number of rules
 * per page
 */
function ChangePerPagePref(newCount, nTestsEnabled) {
    InitRulesTable(ssrListId, ssrTableId, newCount, false, nTestsEnabled);
    ChangePerPagePref2(ssrTableId, "stdIdx", false, newCount, nTestsEnabled);
    if (nTestsEnabled && document.getElementById(nsrDivId)) {
        InitRulesTable(nsrListId, nsrTableId, newCount, true, nTestsEnabled);
        ChangePerPagePref2(nsrTableId, "netIdx", true, newCount, true);
    }
    SetSpamRulesPref("perPage", newCount);
}

function ChangePerPagePref2(tblId, prefId, nRules, newCount, nTestsEnabled) {
    var currIdx = GetSpamRulesPref(prefId);
    // The new index will be the highest multiple of the new per-page count
    // that is less than the current index
    var newIdx = Math.floor(currIdx / newCount) * newCount;
    ShowSpamRules(tblId, nRules, newIdx, newCount, nTestsEnabled);
}

/**
 * Called in response to clicking on an "Edit" link in the rules table
 */
function EditRule(idx, nRules, tblId, row, nTestsEnabled) {
    var rules = stdRules;
    if (nRules) {
        rules = netRules;
    }
    // Get the cells we're going to be editing
    var cells = document.getElementById(tblId).rows[row].cells;
    var rule = rules[idx];
    var fieldId;
    if (rule.Custom) {
        fieldId = "rule_name_" + idx;
        var cellIdx = g_stdDisplay ? 1 : 0;
        // Rule name is editable
        cells[cellIdx].innerHTML = makeEditRuleNameForm(rule, idx, nRules);
    }
    fieldId = "enabled_" + idx;
    var html = "<input type=\"checkbox\"";
    if (rule.Enabled) {
        html += " checked";
    }
    html += " id=\"" + fieldId + "\">";
    cells[cells.length - 3].innerHTML = html;
    var score = cells[cells.length - 2].innerHTML;
    fieldId = "score_" + idx;
    cells[cells.length - 2].innerHTML = makeEditRuleScoreForm(rule, idx, nRules, score);
    var saveLink = makeEditLink(idx, nRules, tblId, row, nTestsEnabled, "SaveEdit", "__save");
    var cancelLink = makeEditLink(idx, nRules, tblId, row, nTestsEnabled, "CancelEdit", "__cancel");
    clearElement(cells[cells.length - 1]);
    cells[cells.length - 1].appendChild(saveLink);
    cells[cells.length - 1].appendChild(document.createTextNode(" | "));
    cells[cells.length - 1].appendChild(cancelLink);

    setInputRestrictions(idx);
}

// Sets field input restrictions when editing spam rules
function setInputRestrictions(idx) {
    var formName = "crn_" + idx;
    var fieldId = "rule_name_" + idx;
    if (document.forms[formName] && document.forms[formName].elements[fieldId]) {
        document.forms[formName].elements[fieldId].onkeypress = rnCheckInput;
    }
    fieldId = "score_" + idx;
    var e = document.getElementById(fieldId);
    if (e)
        e.onkeypress = cb_decimalOnly;
}

// Restricts input in rule name fields
function rnCheckInput(evt) {
    // Only allow alphanumerics and underscores
    return cb_restrictInput(evt, ["AZ", "az", "09", "_"]);
}

function makeEditRuleNameForm(rule, idx, nRule) {
    var pfx = nRule ? "net" : "std";
    var formName = "crn_" + idx;
    var html = '<form style="display:inline" name="' + formName + '">';
    var fieldId = "rule_name_" + idx;
    html += '<input type="text" class="textField" style="width:100%" ' +
        'name="' + fieldId + '" id="' + fieldId + '" value="' + rule.Name + '" ' +
        'onblur="vframeworks[\'' + formName + '\'].onchange(\'' + fieldId + '\');">';
    html += '</form>';
    // Create a validator for this field; must have a value and must be unique
    var reqv = new RequiredValidator();
    var scriptv = new ScriptValidator();
    var code = "(value != \"" + rule.Name + "\") ? (undefined === " + pfx + "Rules[value]) : true";
    var crossv = new CrossValidator(code, resolveString(cse_pp_st, "__rule_exists"));
    var vf = new ValidationFramework(formName);
    vf.formAction = std_val_form_action;
    vf.fieldAction = std_val_field_action;
    vf.addValidator(fieldId, reqv);
    vf.addValidator(fieldId, crossv);
    vf.addValidator(fieldId, scriptv);
    vf.labelMap[fieldId] = resolveString(page_st, "lm_rulename");
    vframeworks[formName] = vf;
    return html;
}

function makeEditRuleScoreForm(rule, idx, nRule, score) {
    var pfx = nRule ? "net" : "std";
    var formName = "crs_" + idx;
    var html = '<form style="display:inline" name="' + formName + '">';
    var fieldId = "score_" + idx;
    html += '<input type="text" class="textField" style="width:100%" ' +
        'name="' + fieldId + '" id="' + fieldId + '" value="' + score + '" ' +
        'onblur="vframeworks[\'' + formName + '\'].onchange(\'' + fieldId + '\');">';
    html += '</form>';
    // Create a validator for this field; must have a value and must be valid decimal
    var reqv = new RequiredValidator();
    var fv = new FloatValidator();
	var rv = new RangeFloatValidator(-1000.00, +1000.00);
	var vf = new ValidationFramework(formName);
    vf.formAction = std_val_form_action;
    vf.fieldAction = std_val_field_action;
    vf.addValidator(fieldId, reqv);
    vf.addValidator(fieldId, fv);
	vf.addValidator(fieldId, rv);
    vf.labelMap[fieldId] = resolveString(page_st, "lm_score");
    vframeworks[formName] = vf;
    return html;
}

function makeEditLink(idx, nRules, tblId, rowIdx, nTestsEnabled, jsFn, stKey) {
    var href = "javascript:" + jsFn + "(" + idx +
        "," + nRules + ",'" + tblId + "'," + rowIdx + "," + nTestsEnabled + ");";
    var link = document.createElement("a");
    link.className = "blueLink";
    link.href = href;
    link.appendChild(document.createTextNode(resolveString(cse_pp_st, stKey)));
    return link;
}

function SaveEdit(idx, nRules, tblId, row, nTestsEnabled) {
    var formName = "crs_" + idx;
	if( !vframeworks[formName].onsubmit() )
		return;

    var rules = stdRules;
    if (nRules) {
        rules = netRules;
    }
    var rule = rules[idx];
    var fieldId, field;
    fieldId = "enabled_" + idx;
    var enState = document.getElementById(fieldId).checked;
    fieldId = "score_" + idx;
    var newScore = document.getElementById(fieldId).value;
    var changed = true;
    if (rule.Custom) {
        var formPfxs = {"crn_":null, "crs_":null};
        var err = false;
        for (var pfx in formPfxs) {
            var formName = pfx + idx;
            if (!vframeworks[formName].onsubmit()) {
                err = true;
            }
        }
        if (err)
            return;
        for (var pfx in formPfxs) {
            var formName = pfx + idx;
            // Remove the validator for this field to free resources
            delete vframeworks[formName];
        }
        fieldId = "rule_name_" + idx;
        field = document.getElementById(fieldId);
        rule.Name = transformRuleName(field.value);
    }
    else {
        // See whether the data has changed
        if (!rule.Overridden) {
            if (enState &&
                newScore == formatSpamScore(rule, nTestsEnabled)) {
                changed = false;
            }
            else {
                rule.Overridden = true;
            }
        }
    }
    if (changed) {
        rule.Enabled = enState;
        rule.Score = parseFloat(newScore);
        g_changeNotifyFn();
    }
    ShowRule(tblId, document.getElementById(tblId).rows[row], nRules,
        rule, idx, row, nTestsEnabled);
}

function transformRuleName(ruleName) {
    // All uppercase; non-alphanumerics turned to underscores
    var t = ruleName.replace(/\W/g, "_");
    return t.toUpperCase();
}

/**
 * Called in response to clicking on a "Cancel" link when editing a rule
 */
function CancelEdit(idx, nRules, tblId, row, nTestsEnabled) {
    var rules = stdRules;
    if (nRules) {
        rules = netRules;
    }
    var rule = rules[idx];
    ShowRule(tblId, document.getElementById(tblId).rows[row], nRules,
        rule, idx, row, nTestsEnabled);
}

/**
 * Called in response to clicking on a "Reset" link in the rules table
 */
function ResetRule(idx, nRules, tblId, row, nTestsEnabled) {
    var rules = stdRules;
    if (nRules) {
        rules = netRules;
    }
    var rule = rules[idx];
    rule.Overridden = false;
    rule.Enabled = true;
    ShowRule(tblId, document.getElementById(tblId).rows[row], nRules,
        rule, idx, row, nTestsEnabled);
    g_changeNotifyFn();
}

/**
 * Called in response to clicking on a "Delete" link in the rules table
 */
function DeleteRule(idx, nRules, tblId, row, nTestsEnabled) {
    if (confirm(resolveString(cse_pp_st, "__del_spam_rule_prompt"))) {
        var rules = stdRules;
        if (nRules) {
            rules = netRules;
        }
        var delRule = rules.splice(idx, 1);
        delete rules[delRule[0].Name];
        // Redraw the current table
        var perPage = GetSpamRulesPref("perPage", defRecordsPerPage);
        if (idx == rules.length && rules.length > 0 && row == 1) {
            // We've deleted a rule in a page of its own
            idx--;
        }
        var newIdx = Math.floor(idx / perPage) * perPage;
        ShowSpamRules(tblId, nRules, newIdx, perPage, nTestsEnabled);
        g_changeNotifyFn();
    }
}

/**
 * Called in response to clicking on the "Add" button in the new rule form
 * @param nRule - true if this is a network rule
 * @param ruleName - the name of the rule
 * @param enabled - whether or not the rule is enabled
 * @param score - the rule score
 * @param nTestsEnabled - whether network tests are enabled (although
 * it doesn't really matter)
 */
function AddRule(nRule, ruleName, enabled, score, nTestsEnabled) {
    var rules = stdRules;
    var tblId = ssrTableId;
    var divId = ssrListId;
    if (nRule) {
        rules = netRules;
        tblId = nsrTableId;
        divId = nsrListId;
    }
    var perPage = GetSpamRulesPref("perPage", defRecordsPerPage);

    if (rules.length == 0) {
        // The rules table needs to be recreated
        InitRulesTable(divId, tblId, perPage, nRule, nTestsEnabled);
    }

    var rule = new SpamRule();
    rule.customInit(ruleName, enabled, score);
    rules[ruleName] = rules.length;
    rules.push(rule);
    // Redraw the current table
    var idx = Math.floor(rules.length / perPage) * perPage;
    ShowSpamRules(tblId, nRule, idx, perPage, nTestsEnabled);
    g_changeNotifyFn();
}

/**
 * Determines whether network test are enabled
 * @param def the value to return if the data cannot be found
 * @param lang the language of the policy; may be null
 * @return true if spam network tests are enabled, false otherwise
 */
function SpamNetworkTestsEnabled(def, lang) {
    var ret = def;

    if (lang == null) {
        try {
            lang = t_locale_id;
        }
        catch(e) {
            lang = "en_US";
        }
    }

    var pb = top.getCustomPB("SpamRules_" + lang);
    if (pb.IsEmpty()) {
        return def;
    }

    var pp = new PropertyPath("EngineSet.Engine");
    for (pp.BeginEnum(); null != pb.GetProperty(pp); pp.IncrementEnum()) {
        var s = pp.GetPath();
        if ("SpamScanner" == pb.GetProperty(s + ".Name") &&
            "SpamScanner" == pb.GetProperty(s + ".type")) {
            var ppInner = new PropertyPath(s + ".Attr");
            for (ppInner.BeginEnum(); null != pb.GetProperty(ppInner); ppInner.IncrementEnum()) {
                ppInner.SetTempLeaf("name");
                var attrName = pb.GetProperty(ppInner);
                if ("EnableNetworkTests" == attrName) {
                    ppInner.SetTempLeaf("value");
                    ret = ("1" == pb.GetProperty(ppInner));
                    break;
                }
                ppInner.SetTempLeaf("");
            }
            break;
        }
    }

    return ret;
}




