/*
 * StdValidationActions.js
 * Creates standardized behaviour for form validation
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved.
 */

/**
 * Actions to take when a form is submitted (callback function)
 * @param formName the name of the form currently being processed
 * @param validationResult true if the form validated correctly
 * @param goodFields the list of fields that were OK
 * @param badFields the list of fields that weren't
 * @param errMsgs a list of error messages from the validator objects
 */
function std_val_form_action(formName, validationResult, goodFields, badFields, errMsgs) {
    if (!validationResult) {
        var errStrings = vfr_make_bad_form_err_msg(errMsgs);
        alert(errStrings.join("\n"));

        for (var i = 0; i < badFields.length; i++) {
            var e = document.forms[formName][badFields[i]];
            if (e) {
                e.className = addClass(e.className, "error");
            }
        }
        setTimeout("_set_focus(\"" + formName + "\", \"" + badFields[0] + "\")", 200);
    }
    for (var i = 0; i < goodFields.length; i++) {
        var e = document.forms[formName][goodFields[i]];
        if (e) {
            e.className = removeClass(e.className, "error");
        }
    }
}

/**
 * Callback function for when a form field is changed
 * @param formName the name of the form containing the field
 * @param fieldName the name of the field currently being processed
 * @param validationResult true if the field is OK
 * @param errMsg any error message from an associated validator object
 */
function std_val_field_action(formName, fieldName, validationResult, errMsg) {
    var e = document.forms[formName].elements[fieldName];
    if (!e) {
        return;
    }
    if (!validationResult) {
        e.className = addClass(e.className, "error");
    }
    else {
        e.className = removeClass(e.className, "error");
    }
}
