// Copyright (C) 2016 McAfee, Inc.  All rights reserved.

includeJS("../../js/Browser.js");

///////////////////////////////////////////////////////
//
// Constructor
//
function XMLParser()
{
	this.bIE = (TestForIEorNS() == "1");
}

///////////////////////////////////////////////////////
//
// Convert given XML to a property bag
//
XMLParser.prototype.Load = function(xmlStr,pbOut)
{
	// Parse
	var doc;
	if (this.bIE)
	{
		doc = new ActiveXObject("msxml2.DOMDocument.3.0");
		doc.async = false;
		doc.resolveExternals = false;
		doc.validateOnParse = false;

		// Parse
		doc.loadXML(xmlStr);
		if (doc.parseError.errorCode != 0)
			return;
	}
	else
	{
		var parser = new DOMParser;
		doc = parser.parseFromString(xmlStr, "text/xml");
	}

	// Convert to properties
	try {
	var node = doc.documentElement;
	var pp = new PropertyPath(node.nodeName);
	this.ToPropertyBag(node,pbOut,pp);
	}
	catch(err) {
	}
}

///////////////////////////////////////////////////////
//
// Convert given node into properties and write into propertybag
//
XMLParser.prototype.ToPropertyBag = function(node, pb, pp)
{

	// Add element
	pb.SetProperty(pp)

	// Add attributes
	for (var i=0; i < node.attributes.length; i++)
	{
		pp.SetTempLeaf(node.attributes[i].name);
		pb.SetProperty(pp, node.attributes[i].value);
		pp.SetTempLeaf("");
	}

	// Add children recursively
	var childPaths=[];
	var lastName, name;
	for (var i=0; i < node.childNodes.length; i++)
	{
		switch(node.childNodes[i].nodeType)
		{
			case 1: // NODE_ELEMENT


				// Increment enumerator if element of this name has already been seen
				name = node.childNodes[i].nodeName
				if (childPaths[name] != null)
					childPaths[name].IncrementEnum();
				else
				//  ... otherwise begin new property path
				{
					childPaths[name] = new PropertyPath(pp.GetPath());
					childPaths[name].Append(name);
					childPaths[name].BeginEnum();
				}

				// Recursive call to add properties
				lastName = name;
	 			this.ToPropertyBag(node.childNodes[i],pb,childPaths[name]);

			break;

			//case 3: // NODE_TEXT
			case 4: // NODE_CDATA_SECTION
				pb.SetProperty(pp, node.childNodes[i].nodeValue);
			break;

			default:
			break;
		}
	}
}

///////////////////////////////////////////////////////
//
// Convert property bag to XML
//
XMLParser.prototype.Save = function(pb, typeMap)
{
	if (this.bIE)
	{
		this.doc = new ActiveXObject("msxml2.DOMDocument.3.0");
		this.doc.async = false;
		this.doc.resolveExternals = false;
		this.doc.validateOnParse = false;
		this.BuildDOMTree(pb,this.doc,typeMap);
		return this.doc.xml
	}
	else
	{
		// Create doc object
		var parser =new DOMParser;
		this.doc = parser.parseFromString("<root/>", "text/xml");
		this.BuildDOMTree(pb,this.doc.documentElement,typeMap);
		return (new XMLSerializer()).serializeToString(this.doc.documentElement.childNodes[0]);
	}
}

////////////////////////////////////////////////////////////////////////
//
// Create a node
//
XMLParser.prototype.CreateNode = function(type,name,value)
{
	switch(type)
	{
		case 1:
			return  this.bIE ? this.doc.createNode(type,name,"") : this.doc.createElement(name);
		case 3:
			if (this.bIE)
			{
				var data = this.doc.createNode(type,name,"")
				data.nodeValue = value;
				return data;
			}
			else
			{
				return this.doc.createTextNode(value);
			}
	}
}

////////////////////////////////////////////////////////////////////////
//
// Adds node for given property path to dom tree. Recursively creates
// parent nodes as necessary until it reaches the root node.
//
XMLParser.prototype.CreateDOMNode = function(path, domTree, domNodes)
{

	// Create this node and add to map
	var pathLit =path.GetPath();
	var elementName = PropertyPath.GetLeaf(pathLit);
	var child = this.CreateNode(1,elementName);
	domNodes[pathLit] = child;

	// Check for root node and stop ...
	if (PropertyPath.GetRoot(pathLit) == pathLit)
	{
		domTree.appendChild(child);
	}
	else
	{
		// ... otherwise look for parent and create if it doesnt exists
		path.RemoveLeaf();
		var parent = domNodes[pp.GetPath()];
		if (parent == McUndefined)
			parent = this.CreateDOMNode(path,domTree,domNodes);
		parent.appendChild(child);
	}
	return child;
}

////////////////////////////////////////////////////////////////////////
//
// Build DOM document from complete list of properties
//
XMLParser.prototype.BuildDOMTree = function(pb,domTree,typeMap)
{

	var domNodes = new Array; // existing DOM nodes, keyed on property path

	// Enumerate properties in table
	for (var path in pb.propValues)
	{

		// Get property leaf name and value
		pp = new PropertyPath(path)
		var name
		var value = pb.GetProperty(pp);
		if (value != top.gValueless)
		{
			name = PropertyPath.GetLeaf(path);
			pp.RemoveLeaf();
		}

		// Make sure element node exists
		var currentNode = domNodes[pp.GetPath()];
		if (currentNode == McUndefined)
			currentNode =this.CreateDOMNode(pp,domTree, domNodes);

		// Add property value to node if it exists
		if (value != top.gValueless)
		{
			switch (typeMap[name])
			{

				case 'E':
					var newNode = this.CreateNode(1,name);
					newNode.appendChild(this.CreateNode(3,"",value));
					currentNode.appendChild(newNode);
				break;

				default:
					currentNode.setAttribute(name,value);
				break;
			}

		}
	}
}
