//Copyright (C) 2009 McAfee, Inc.  All rights reserved.

///////////////////////////////////////
//
// Constructor for a node within a
// picker tree
//
function Node(name,id,parent,bHasChildren)
{
	// Member variables
	this.name = name
	this.id  = id
	this.children = new Array
	this.bExpanded = false
	this.elem = McUndefined
	this.bHasChildren = bHasChildren
	this.parent = parent

	// Member functions
	this.DeflateSubTree = DeflateSubTree
	this.AC = AddChild
	this.Render = Render
	this.ExpandDeflate = ExpandDeflate
	this.GetChildren = GetChildren
	this.Level = Level
}

////////////////////////////////////////
//
// Add a child to the current node,
// returning a reference to the new node
//
function AddChild(name,bHasChildren)
{
	var nn = new Node(name, this.id + "_" + this.children.length,this,bHasChildren)
	this.children[this.children.length]=nn
	return nn
}

////////////////////////////////////////
//
// Downloads children of this node if
// they have not already been downloaded
//
function GetChildren()
{
	// Do nothing if tree is not enabled
	if (!gP.bEnabled)
		return

	// Check for children to download ...
	if (this.children.length == 0)
	{
		// Disable tree
		gP.Enable(false)
		var strPath = McEscape(gP.GetPath(this),'&');
		strPath = "(" + strPath + ")";
		strPath = EncodeXSSChars(strPath);
		strPath = strPath.substr(1, strPath.length-2);
		// Download new subtree
		var query= "SAFepicker&n=" + this.id +"&p=" + strPath;
		top.hiddenFrame.location.href = GetDynamicPageUrl( query )
	}
	// ... just update UI otherwise
	else
	{
		this.ExpandDeflate()
	}
}

////////////////////////////////////////////
//
// Show or hide all the immediate children
// of this node
//
function ExpandDeflate()
{
	if (!this.bExpanded)
	{
		for (var i = 0; i < this.children.length; i++)
		{
			this.elem.appendChild(this.children[i].Render(this.Level()))
		}
		document.getElementById("b_" + this.id).src = "../../images/picker/mbut.gif"
		document.getElementById("f_" + this.id).src = "../../images/picker/OpenFolder.gif"
		this.bExpanded = true
	}
	else
	{
		this.DeflateSubTree()
		document.getElementById("b_" + this.id).src = "../../images/picker/pbut.gif"
		document.getElementById("f_" + this.id).src = "../../images/picker/ClosedFolder.gif"
	}
}
function DeflateSubTree()
{
	this.bExpanded = false
	for (var i = 0; i < this.children.length; i++)
	{
		this.children[i].DeflateSubTree()
		if (this.elem != McUndefined && this.children[i].elem != McUndefined)
		{
			this.elem.removeChild(this.children[i].elem)
			this.children[i].elem = McUndefined
		}
	}
}

////////////////////////////////////////////
//
// Returns 1-based level of this node
// within the tree
//
function Level()
{
	var level = 1
	var node = this
	while (node.parent != McUndefined)
	{
		level++
		node = node.parent
	}
	return level
}

////////////////////////////////////////////
//
// Renders this node into html, returned as a
// new element
//
function Render(level)
{

	// Create wrapping DIV
	this.elem=document.createElement("DIV")
	this.elem.className = "pickerNode"
	this.elem.id = "d_" +  this.id
	if (level > 0)
		this.elem.style.marginLeft = "10px"

	// Create expand button if there are children
	var buttonEl=document.createElement("IMG")
  	buttonEl.id = "b_" + this.id
	buttonEl.className = "nodeButton"
	if (this.bHasChildren)
	{
              	buttonEl.onclick = new Function("gP.Find(\'" + this.id + "\').GetChildren();");
		buttonEl.setAttribute("src","../../images/picker/pbut.gif")
	}
	else
		buttonEl.setAttribute("src","../../images/picker/nobut.gif")
	//buttonEl.setAttribute("width","11")
	//buttonEl.setAttribute("height","11")
	this.elem.appendChild(buttonEl)

	// Create folder icon
	var fdrEl=document.createElement("IMG")
	fdrEl.id = "f_" + this.id
	fdrEl.setAttribute("src","../../images/picker/ClosedFolder.gif")
	//fdrEl.setAttribute("width","18")
	//fdrEl.setAttribute("height","13")
	this.elem.appendChild(fdrEl)

	// Create span for folder name
	var nameEl=document.createElement("SPAN")
	nameEl.idRaw=this.id;
	nameEl.onclick = SelectNode;
	nameEl.className = this == gP.selected ? "nodeLabelSelected" : "nodeLabel"
	nameEl.id = "n_" + this.id
	nameEl.style.verticalAlign = "middle"

	var name = document.createTextNode(this.name)
	nameEl.appendChild(name)
	this.elem.appendChild(nameEl)
	return this.elem
}
