// Copyright (C) 2005 McAfee, Inc. All Rights Reserved
/**
 @fileoverview

 This file contains UI widgets for developing Policy Configuration
 pages in the ePO product. Because of the namespace usage, the JSDoc
 loses the constructors, so an automatic init method is added to all
 constructors to help in the display of the methods. For more info,
 please view the Twiki entry.

 */

/**
 * The Policy UI global namespace constructor. This is used to keep the
 * widget names from muddying up or conflicting with a user functions.
 */
var PolicyUI = window.PolicyUI || {};

/**
 * The class object for the "swappable tabstrip"
 * @param {string} div The container div of the tabstrip.
 * @param {array} names The list of names to use for each tab
 */
PolicyUI.SwapTabStrip = function(div,names){
	this.init(div,names);
}
/**
 * The constructor for the "swappable tabstrip". The associative array
 * of names sent will be used for creation of the tab id's. Each tab will
 * have the id of "tab_OBJECTKEY". OBJECTKEY cooresponds to the div that
 * gets swapped by clicking that tab.
 * @constructor
 * @param {string} div The container div of the tabstrip.
 * @param {object} names The list of keys/names to use for each tab ex: "{'divkey':'General'}"
 */
PolicyUI.SwapTabStrip.prototype.init = function(div,names){
	var master = document.getElementById(div);
	var instn = this;

	// Outer Div 1
	var outdiv = document.createElement("DIV");
	outdiv.className="layout_box_24";
	outdiv.setAttribute("id","otsID_Tabs_container");
	// Outer Div 2
	var outdiv2 = document.createElement("DIV");
	outdiv2.className="orionTabStripContainer";
	// Table
	var table = document.createElement("TABLE");
	table.className = "orionTabStrip";
	table.setAttribute("id","otsID_Tabs");
	table.cellSpacing = 0;
	table.cellPadding = 0;
	// TBODY - Needed for IE - gg
	var tbody = document.createElement("TBODY");
	// TR
	var tr = document.createElement("TR");
	// TD LIST
	for(var i in names){
		var td = document.createElement("TD");
		var indiv = document.createElement("DIV");
		indiv.innerHTML = names[i];
		td.appendChild(indiv);
		var myname = "tab_"+i;
		td.setAttribute("id",myname);
		td.className = "orionTab";
		td.onclick = function(evt){
		    evt = evt || event;
            if (! evt.target){
                evt.target = evt.srcElement;
            }
            var p = evt.target.parentNode;
			instn.swap(p);
		}

		tr.appendChild(td);
	}
	tbody.appendChild(tr);
	table.appendChild(tbody);
	outdiv2.appendChild(table);
	outdiv.appendChild(outdiv2);
	master.appendChild(outdiv);

	//Auto swap #1
	this.swap(tr.childNodes[0]);
}
/**
 * The method for swapping the tabs.
 * PRIVATE - This is called automatically, not for use
 * @param {object} me The TD element to be swapped.
 */
PolicyUI.SwapTabStrip.prototype.swap = function(me){
	var key = me.getAttribute("id").replace("tab_","");
	var TABS = me.parentNode.getElementsByTagName("TD");
	for(var i=0;i<=TABS.length;i++){
		if(TABS[i] !== undefined){
			var cls = TABS[i].getAttribute("id").replace("tab_","");
			if(cls == key){
				TABS[i].className = "orionTabSelected";
				document.getElementById(cls).style.display = "block";
			}else{
				TABS[i].className = "orionTab";
				document.getElementById(cls).style.display = "none";
			}
			if(document.getElementById("inclusion_div").style.display == "block")
			{
				if(selector != undefined)
					selector = selectors[0];
			}
			else if(document.getElementById("exclusion_div").style.display == "block")
			{
				if(selector != undefined)
					selector = selectors[1];
			}
        }
	}
}

/**
 * The opening "class" for the AddWidget (+/-) row adder
 * @param {obj} obj	An object holding the default parameters (container/content)
 */
PolicyUI.AddWidget = function(obj){
	var cntrid,cntr,id,list,icons,content;
	this.init(obj);
}
/**
 * The constructor method for the AddWidget (+/-) row adder
 * @constructor
 * @param {obj} obj	An object holding the default parameters (container/content)
 */
PolicyUI.AddWidget.prototype.init = function(obj){
	this.cntrid = obj["container"];
	this.cntr = document.getElementById(obj["container"]);
	this.id = 0;
	this.list = {};
	this.icons = {
		"plus":"<img src='/core/images/button/add-button.gif' border='0' width='27' height='19' align='absmiddle'>",
		"minus":"<img src='/core/images/button/delete-button.gif' border='0' width='27' height='19' align='absmiddle'>"
	};
	this.content = "";
	if(obj["content"] != null){
	    this.content = obj["content"];
	}else{
	    this.content = "<INPUT TYPE='text' size='15' value='@VALUE'>";
	}
}
/**
 * The AddWidget method for adding new rows
 * @param {obj} inval   An object holding any replacements for placeholders in the HTML content
 * @return void
 */
PolicyUI.AddWidget.prototype.add = function(inval){
	this.id++;
	var newbox = document.createElement('div');
	newbox.setAttribute("id",this.cntrid+"_awrow_"+this.id);
	newbox.setAttribute("style","margin-bottom: 3px;");
	if(inval == null){
	    inval = {"@VALUE":""};
	}

	var field = this.content;
	for(var i in inval){
	   field = field.replace(i,inval[i]);
	}

	newbox.innerHTML = field;
	this.cntr.appendChild(newbox);
	this.list[this.cntrid+"_awrow_"+this.id] = true;
	this.toggle();
}
/**
 * The AddWidget method for placing the plus and minus buttons on the screen.
 * PRIVATE. Used by the add and remove methods.
 * @private
 */
PolicyUI.AddWidget.prototype.toggle = function(){
    // Due to scope loss in the add/remove, the buttons are redrawn on every click.
    var btn = this.cntr.getElementsByTagName("A");
    var btncnt = btn.length;
    for(var i=0;i<btncnt;i++){
        btn[0].parentNode.removeChild(btn[0]);
    }
    var btn = this.cntr.getElementsByTagName("SPAN");
    var btncnt = btn.length;
    for(var i=0;i<btncnt;i++){
        btn[0].parentNode.removeChild(btn[0]);
    }

    // Add Minuses
    var instn = this;
    var psave = 0;

    // Ugly roll to accomodate the Mercury style of having no minus on the first row.
    var trues = 0;
    for(var i in this.list){
		if(this.list[i]){
		    trues++;
		}
	}
    for(var i in this.list){
		if(this.list[i]){
		    if(trues > 1){ // ugly from above
                var div = document.getElementById(i);
                var space = document.createElement('SPAN');
                space.innerHTML = "&nbsp;";
                div.appendChild(space);

                var minus = document.createElement('a');
                minus.setAttribute("href","javascript:;");
                minus.onclick=function(evt){
                    evt = evt || event;
                    if (! evt.target){
                        evt.target = evt.srcElement;
                    }
                    instn.remove(evt.target);
                }
                minus.innerHTML = this.icons["minus"];
                div.appendChild(minus);
	        }
	        psave = i;
		}
	}

    // Add plus to last element
    var psave = document.getElementById(psave);
    var space = document.createElement('SPAN');
    space.innerHTML = "&nbsp;";
    psave.appendChild(space);

	var adder = document.createElement('a');
	adder.setAttribute("href","javascript:;");
	adder.onclick=function(){
		instn.add();
	}
	adder.innerHTML = this.icons["plus"];
	psave.appendChild(adder);
	if(psave.style.display == "block"){
		psave.childNodes[0].focus();
    }
}
/**
 * The AddWidget method for removing rows. Called by the minus button
 * @param {obj} me   The row object to remove
 * @return void
 */
PolicyUI.AddWidget.prototype.remove = function(me){
    var mom = me.parentNode.parentNode;
 	if(this.cntr.childNodes.length > 1){
 	    this.list[mom.getAttribute("id")] = false;
		mom.parentNode.removeChild(mom);
	}else{
	    mom.childNodes[0].value = "";
	}
	this.toggle();
}
/**
 * The AddWidget method for retreiving active rows.
 * @return {array} A collection list of DIV tag objects with content in them.
 */
PolicyUI.AddWidget.prototype.getList = function(){
	var its = new Array();
	for(i in this.list){
		if(this.list[i]){
		    its.push(i);
        }
	}
	return its;
}

/**
 * The object for the random UI utilities
 * This is built for namespace encapsulation mostly
 * and is not a real constructor. (Static methods)
 */
PolicyUI.Utils = {

	/**
	 * The shortcut method for toggling sections off and on based on checkbox.
	 * @param {object} me The checkbox element doing the toggling.
	 * @param {string} sect The name of the div element to toggle.
	*/
	toggleSection:function(me,sect){
		this.toggleEnabled(me.checked,sect);
	},

	/**
	 * The shortcut method for toggling sections off and on based on checkbox,
	 * in reverse. Checked disables, unchecked enables.
	 * @param {object} me The checkbox element doing the toggling.
	 * @param {string} sect The name of the div element to toggle.
	*/
	toggleSectionBackwards:function(me,sect){
		var backwards = (me.checked) ? false : true;
		this.toggleEnabled(backwards,sect);
	},

	/**
	 * The method for toggling sections off and on
	 * @param {bool} state The state of the section
	 * @param {string} sect The name of the div element to toggle.
	*/
	toggleEnabled:function(state,sect){
		var sectd = document.getElementById(sect);
		var slistt = new Array();
		var slist = slistt.concat(__arrayme(sectd.getElementsByTagName("LABEL")),
			__arrayme(sectd.getElementsByTagName("INPUT")),
			__arrayme(sectd.getElementsByTagName("SELECT")),
			__arrayme(sectd.getElementsByTagName("TEXTAREA")));
		for(var i=0;i<slist.length;i++){
			OrionCore.setEnabled(slist[i],state);	
		}
		function __arrayme(obj){
			var n = new Array();
			for(var j=0;j<obj.length;j++){
				n.push(obj[j]);
			}
			return n;
		}
	}
}