/**
 * JavaScript for the AFSettings page; this file needs to be included
 * if the PhishOptionsDlg component is used.
 * Defines phish_init() for page initialization along with Show(), Validate()
 * and Apply() functions
 * Note: include it last.
 *
 * Copyright (C) 2009 McAfee, Inc. All rights reserved
 */

var vframeworks = new Object();
var dataBindings = new Array();
var qparams = queryParameters();
var newInstance = false;
var newId = null;
var rm = new ResourceManager("DefaultAlerts");

var basePP = "Settings.AFSettings";
var phish_pb = null;
var origName="";
var settingsId = qparams["id"];
var phishOptsForm = "phish_options";

var g_changeNotifyFn = "top.gPageStack.Active().SetModified(true);";

// Enables / disables the phish prefix text control
function enablePhishPrefixTextCtl(b) {
    var fieldName = "PrefixSubjectText";
    var e = document.forms[phishOptsForm].elements[fieldName];
    var vframework = vframeworks[phishOptsForm];
    if (b) {
        vframework.addValidator(fieldName, new RequiredValidator());
        //vframework.addValidator(fieldName, new ScriptValidator());
		vframework.addValidator(fieldName, new InputTextValidator());
    }
    else {
        if (!vframework.check(fieldName)) {
            e.value = phish_pb.GetPropertyDef(basePP + "." + fieldName, "");
            vframework.onchange(fieldName);
        }
        vframework.clearValidators(fieldName);
    }
    var oldState = e.disabled;
    e.disabled = !b;
    if (b && oldState) {
        e.focus();
    }
}

// Enables and shows / disables and hides the verbose reporting checkbox
function enableVerboseCtl(b) {
    var e = document.getElementById("verbose_ctl");
    var display = "none";
    if (b) {
        display = "";
    }
    e.style.display = display;
}

// Initializes the page's forms
function phish_forms_init() {
    var forms = [phishOptsForm];
	vframeworks["InstanceSettings"]= new ValidationFramework("InstanceSettings");
	vframeworks["InstanceSettings"].formAction = std_val_form_action;
    vframeworks["InstanceSettings"].fieldAction = std_val_field_action;
    vframeworks["InstanceSettings"].addValidator("displayName", new RequiredValidator());
	// vframeworks["InstanceSettings"].addValidator("displayName", new ScriptValidator());
	vframeworks["InstanceSettings"].addValidator("displayName", new InputTextValidator());
	vframeworks["InstanceSettings"].labelMap["displayName"] =  resolveString(cse_pp_field_st, "__instancename");

    var inputArray= new Array();
	inputArray["SECTION"]="Scanners";
	inputArray["origName"]=origName;
	inputArray["basePP"]="Settings.AFSettings";
	vframeworks["InstanceSettings"].addValidator("displayName",new DuplicatesValidator(phish_pb,inputArray));
	vframeworks["InstanceSettings"].addValidator("displayName",new LengthValidator());

    for (var i = 0; i < forms.length; i++) {
        var form = forms[i];
        if (document.forms[form]) {
            vframeworks[form] = new ValidationFramework(form);
            vframeworks[form].formAction = std_val_form_action;
            vframeworks[form].fieldAction = std_val_field_action;
        }
    }

    shs_affectedPolicyReport(newInstance, "AF", settingsId, phish_pb);

    var form = phishOptsForm;
    if (document.forms[form]) {
        vframeworks[form].labelMap["PrefixSubjectText"] =
            resolveString(page_st, "lm_PhishPrefixSubjectText");
    }
    addChangeNotification(g_changeNotifyFn);
}

// Sets form data and enables / disables controls as appropriate
function phish_init() {
    phish_forms_init();
    Reset();
}

/**
 * Asynchronous reset function
 */
function Reset() {
    var defValues = top.defValues["AF"];
    var phish_pp = new PropertyPath(basePP);
    for (var i = 0; i < dataBindings.length; i++) {
        dataBindings[i].set(phish_pp, phish_pb);
    }

    if (document.forms[phishOptsForm]) {
        phish_pp.SetTempLeaf("PrefixSubject");
        enablePhishPrefixTextCtl("1" == phish_pb.GetPropertyDef(phish_pp,
            defValues["PrefixSubject"]));
        phish_pp.SetTempLeaf("AttachReport");
        enableVerboseCtl(0 != phish_pb.GetPropertyDef(phish_pp,
            defValues["AttachReport"]));
    }
}

/**
 * Asynchronous data load function
 */
function Show(pbIn, pp) {
    if (null == pbIn) {
        pbIn = top.gPB;
    }
    newInstance = shs_initPB(pbIn, pp, "AF");
    phish_pb = pbIn;
    if (settingsId) {
        // Search for the settings blob to use
        basePP = top.GetBasePath(pbIn, basePP, "id", settingsId);
        if (null == basePP) {
            top.gPageStack.Active().Cancel();
            return;
        }
    }
    else {
        // Assume that we've been set up with the correct property path
        // and property bag
        basePP = pp.GetPath();
        settingsId = pbIn.GetPropertyDef(basePP + ".id", "");
        origName=pbIn.GetPropertyDef(basePP + ".displayName", "");
    }
    phish_init();
}

/**
 * Asynchronous data validation function
 */
function Validate() {
    return shs_Validate();
}

/**
 * Asynchronous data save function
 * @param pb the property bag to fill
 */
function Apply(pb) {
    if (null == pb) {
        pb = phish_pb;
    }
    shs_Apply(pb, basePP, dataBindings, newInstance);
}

/**
 * Asynchronous cancel function
 */
function Cancel() {
    if (newInstance) {
        // Remove the new instance from the property bag
        phish_pb.DeleteElement(basePP);
    }
}

/**
 * Callback function to list required configuration data
 */
function GetCustomConfigReq() {
	return ["DefaultAlerts"];
}
